package com.cmmap.api.maps;

import android.content.Context;

import com.cmmap.api.maps.model.LatLng;
import com.cmmap.internal.mapcore.MapCore;
import com.cmmap.internal.maps.model.KLatLng;

/**
 * 坐标转换类。
 */
public class CoordinateConverter {
	
	private Context mContext;
	private CoordType mType = null;
	private LatLng mSrcCoord = null;
	
	/**
	 * 构造函数
	 * @param context 应用上下文
	 */
	public CoordinateConverter(Context context) {
		mContext = context;
	}

	/**
	 * 设置源坐标类型
	 * @param coordType 源坐标类型，默认：GPS
	 * @return 该对象本身
	 */
	public CoordinateConverter from(CoordinateConverter.CoordType coordType) {
		mType = coordType;
		return this;
	}

    /**
     * 设置源坐标
     * @param coord 待转换的源坐标
     * @return 对象本身
     */
	public CoordinateConverter coord(LatLng coord) {
		mSrcCoord = coord;
		return this;
	}

    /**
     * 执行转换
     * @return 转换后的坐标
     */
	public LatLng convert() {
		if (mType == null || mSrcCoord == null) {
			return null;
		}
		LatLng dstCoord = null;
		switch (mType) {
		case GPS:
			KLatLng dst = MapCore.StandardToLocatePosition(new KLatLng(mSrcCoord));
			dstCoord = new LatLng(dst.latitude, dst.longitude);
			break;

		default:
			break;
		}
		
		return dstCoord;
	}

	/**
	 * 源坐标类型
	 */
	public static enum CoordType {
		// GPS坐标
		GPS,
	}
}
