package com.cmmap.api.maps;

import com.cmmap.api.maps.model.LatLng;
import com.cmmap.internal.maps.KMapUtils;
import com.cmmap.internal.maps.model.KLatLng;

/**
 * 地图帮助类。
 * <p>用于距离区域计算。</p>
 */
public class MapUtils {
	/**
	 * 计算地图上两点间的距离，此距离为相对较短的距离。
	 * @param startLatlng 起点的坐标
	 * @param endLatlng 终点的坐标
	 * @return 地图两点间的距离，单位：米
	 */
	public static float calculateLineDistance(LatLng startLatlng, LatLng endLatlng){
		return KMapUtils.calculateLineDistance(new KLatLng(startLatlng), new KLatLng(endLatlng));
	}

	/**
	 * 计算地图上矩形区域的面积。
	 * @param leftTopLatlng 矩形区域左上角点坐标
	 * @param rightBottomLatlng 矩形区域右下角点坐标
	 * @return 地图矩形区域的面积，单位：平方米
	 */
	public static float calculateArea(LatLng leftTopLatlng, LatLng rightBottomLatlng){
		return KMapUtils.calculateArea(new KLatLng(leftTopLatlng), new KLatLng(rightBottomLatlng));
	}
}
