package com.cmmap.api.maps;

import android.content.Context;
import android.os.Bundle;
import android.util.AttributeSet;
import android.widget.FrameLayout;

import com.cmmap.internal.maps.KMapView;

/**
 * 地图视图类。
 * <p>地图视图负责从服务端获取并显示地图数据。
 * <br>当屏幕焦点在这个视图上时，它将会捕捉屏幕触控手势事件。
 * <br>使用地图视图类必须遵循Activity的生命周期进行调用。
 * <br>当地图视图初始化完成后，用户可以通过{@link #getMap}方法获得{@link Map}对象，如果初始化失败，将返回null。
 * </p>
 */
public class MapView extends FrameLayout {
	private KMapView mMapView;
	private Map mMap = null;
	/**
	 * 构造函数。
	 * @param context 应用上下文
	 */
	public MapView(Context context) {
		super(context);
		init(context);
	}

	/**
	 * 构造函数。
	 * @param context 应用上下文
	 * @param attrs 视图属性
	 */
	public MapView(Context context, AttributeSet attrs) {
		super(context, attrs);
		init(context);
	}

	/**
	 * 构造函数。
	 * @param context 应用上下文
	 * @param attrs 视图属性
	 * @param defStyleAttr 默认视图风格属性
	 */
	public MapView(Context context, AttributeSet attrs, int defStyleAttr) {
		super(context, attrs, defStyleAttr);
		init(context);
	}

	/**
	 * 构造函数。
	 * @param context 应用上下文
	 * @param attrs 视图属性
	 * @param defStyleAttr 默认视图风格属性
	 * @param defStyleRes 默认视图风格资源
	 */
//	public MapView(Context context, AttributeSet attrs, int defStyleAttr, int defStyleRes) {            //*************未实现***************//  //***********无法实现*************//
//		super(context, attrs, defStyleAttr, defStyleRes);
//		// TODO
//	}

	/**
	 * 返回地图对象。
	 * @return 地图对象。如果这个地图视图没有初始化成功，则返回NULL
	 */
	public Map getMap() {
		return mMap;
	}

	/**
	 * 用户必须重载{@link android.app.Activity#onCreate}函数，调用地图视图的该函数。
	 * @param bundle 上下文参数
	 */
	public final void onCreate(Bundle bundle) {
		mMapView.onCreate(bundle);
	}

	/**
	 * 用户必须重载{@link android.app.Activity#onResume}函数，调用地图视图的该函数。
	 */
	public final void onResume() {
		mMapView.onResume();
	}

	/**
	 * 用户必须重载{@link android.app.Activity#onPause}函数，调用地图视图的该函数。
	 */
	public final void onPause() {
		mMapView.onPause();
	}
	
	/**
	 * 用户必须重载{@link android.app.Activity#onDestroy}函数，调用地图视图的该函数。
	 */
	public final void onDestroy() {
		mMap.clear();
		mMapView.onDestroy();
	}

	/**
	 * 用户必须重载{@link android.app.Activity#onSaveInstanceState}函数，调用地图视图的该函数。
	 * @param bundle 上下文参数
	 */
	public final void onSaveInstanceState(Bundle bundle) {
		mMapView.onSaveInstanceState(bundle);
	}
	
	@Override
	protected void onAttachedToWindow() {
		super.onAttachedToWindow();
		if (mMapView != null && mMapView.getParent() != null) {
			removeAllViews();
		}
		addView(mMapView);
	}
	
	private void init(Context context) {
		mMapView = new KMapView(context);
		mMap = new Map(mMapView.getMap());
	}
}
