package com.cmmap.api.maps;

import android.graphics.Point;

import com.cmmap.api.maps.model.LatLng;
import com.cmmap.internal.maps.KProjection;
import com.cmmap.internal.maps.model.KLatLng;

/**
 * 地图坐标映射类。
 * <p>地图坐标映射类实现了屏幕位置和地理坐标（经纬度）之间的进行转换，
 * 屏幕位置是相对于地图视图的左上角的位置，并不一定是相对于屏幕左上角的位置。</p>
 */
public class Projection {
	KProjection mKProjection;
	
	/**
	 * 内部构造函数，不公开提供
	 */
	Projection() {
	}
	
	Projection(KProjection kProjection) {
		mKProjection = kProjection;
	}

	/**
	 * 根据输入的屏幕位置（屏幕像素）返回一个地图位置（经纬度）。
	 * @param point 屏幕位置（屏幕像素）
	 * @return 地图位置（经纬度）
	 */
	public LatLng fromScreenLocation(Point point) {
		KLatLng kLatLng = mKProjection.fromScreenLocation(point);
		return kLatLng.toLatLng();
	}

	/**
	 * 根据输入的地图位置（经纬度）返回一个屏幕位置（屏幕像素）。
	 * @param latLng 地图位置（经纬度）
	 * @return 屏幕位置（屏幕像素）
	 */
	public Point toScreenLocation(LatLng latLng) {
		KLatLng kLatLng = new KLatLng(latLng);
		return mKProjection.toScreenLocation(kLatLng);
	}
}
