package com.cmmap.api.maps;


import com.cmmap.internal.maps.KUiSettings;

/**
 * 地图用户界面设置类。
 * <p>该实例需要通过调用{@link Map#getUiSettings}获取。</p>
 */
public final class UiSettings {
	private KUiSettings mKUiSettings = new KUiSettings();
	
	/**
	 * 内部构造函数，不公开提供
	 */
	UiSettings() {
	}
	
	UiSettings(KUiSettings kUiSettings) {
		mKUiSettings = kUiSettings;
	}

	/**
	 * 获取地图自带Logo的显示位置。
	 * @return 地图自带Logo的显示位置，参见：
	 *         <br>{@link MapOptions#LOGO_POSITION_BOTTOM_LEFT}（默认值）
	 *         <br>{@link MapOptions#LOGO_POSITION_BOTTOM_CENTER}
	 *         <br>{@link MapOptions#LOGO_POSITION_BOTTOM_RIGHT}
	 */
	public int getLogoPosition() {
		return mKUiSettings.getLogoPosition();
	}

	/**
	 * 设置地图自带Logo的显示位置。
	 * @param position 地图自带Logo的显示位置，参见：
	 *        <br>{@link MapOptions#LOGO_POSITION_BOTTOM_LEFT}
	 *        <br>{@link MapOptions#LOGO_POSITION_BOTTOM_CENTER}
	 *        <br>{@link MapOptions#LOGO_POSITION_BOTTOM_RIGHT}
	 */
	public void setLogoPosition(int position) {
		mKUiSettings.setLogoPosition(position);
	}

	/**
	 * 获取地图自带缩放按钮的显示位置。
	 * @return 地图自带缩放按钮的显示位置：
	 *         <br>{@link MapOptions#ZOOM_POSITION_RIGHT_CENTER}
	 *         <br>{@link MapOptions#ZOOM_POSITION_RIGHT_BOTTOM}（默认值）
	 */
	public int getZoomPosition() {        
		return mKUiSettings.getZoomPosition();
	}

	/**
	 * 设置地图自带缩放按钮的显示位置。
	 * @param position 地图自带缩放按钮的显示位置：
	 *        <br>{@link MapOptions#ZOOM_POSITION_RIGHT_CENTER}
	 *        <br>{@link MapOptions#ZOOM_POSITION_RIGHT_BOTTOM}
	 */
	public void setZoomPosition(int position) {
		mKUiSettings.setZoomPosition(position);
	}

	/**
	 * 返回地图自带缩放按钮是否可用。
	 * @return 地图自带缩放按钮是否可用，默认为可用
	 */
	public boolean isZoomControlsEnabled() {
		return mKUiSettings.isZoomControlsEnabled();
	}

	/**
	 * 设置地图自带缩放按钮是否可用。
	 * @param enabled 地图自带缩放按钮是否可用
	 */
	public void setZoomControlsEnabled(boolean enabled) {
		mKUiSettings.setZoomControlsEnabled(enabled);
	}

	/**
	 * 返回地图自带指南针控件是否可用。
	 * @return 地图自带指南针控件是否可用，默认可用
	 */
	public boolean isCompassEnabled() {
		return mKUiSettings.isCompassEnabled();
	}

	/**
	 * 设置地图自带指南针控件是否可用。
	 * @param enabled 地图自带指南针控件是否可用
	 */
	public void setCompassEnabled(boolean enabled) {
		mKUiSettings.setCompassEnabled(enabled);
	}

	/**
	 * 返回地图自带比例尺控件是否可用。
	 * @return 地图自带比例尺是否可用，默认为可用
	 */
	public boolean isScaleControlsEnabled() {
		return mKUiSettings.isScaleControlsEnabled();
	}

	/**
	 * 设置地图自带比例尺控件是否可用。
	 * @param enabled 地图自带比例尺控件是否可用
	 */
	public void setScaleControlsEnabled(boolean enabled) {
		mKUiSettings.setScaleControlsEnabled(enabled);
	}

	/**
	 * 返回是否允许用户手势缩放地图。
	 * @return 是否允许用户手势缩放地图，默认：允许
	 */
	public boolean isZoomGesturesEnabled() {
		return mKUiSettings.isZoomGesturesEnabled();
	}

	/**
	 * 设置是否允许用户手势缩放地图。
	 * <br>
	 * 这些手势包括通过双指捏合和绽放地图、单指双击放大地图及双指单击缩小地图。
	 * <br>
	 * 这个设置不会影响缩放按钮的功能， 也不会影响程序对地图的操控。
	 * @param enabled 是否允许用户手势缩放地图
	 */
	public void setZoomGesturesEnabled(boolean enabled) {
		mKUiSettings.setZoomGesturesEnabled(enabled);
	}

	/**
	 * 返回是否允许用户手势旋转地图。
	 * @return 是否允许用户手势旋转地图，默认：允许
	 */
	public boolean isRotateGesturesEnabled() {
		return mKUiSettings.isRotateGesturesEnabled();
	}

	/**
	 * 设置是否允许用户手势旋转地图。
	 * <br>
	 * 这个设置不会影响用户在程序里对地图的旋转。
	 * @param enabled 是否允许用户手势旋转地图
	 */
	public void setRotateGesturesEnabled(boolean enabled) {
		mKUiSettings.setRotateGesturesEnabled(enabled);
	}

	/**
	 * 返回是否允许用户手势倾斜地图。
	 * @return 是否允许用户手势倾斜地图， 默认：允许
	 */
	public boolean isTiltGesturesEnabled() {
		return mKUiSettings.isTiltGesturesEnabled();
	}

	/**
	 * 设置是否允许用户手势倾斜地图。
	 * <br>
	 * 这个设置不会影响用户通过程序接口旋转地图。
	 * @param enabled 是否允许用户手势倾斜地图
	 */
	public void setTiltGesturesEnabled(boolean enabled) {
		mKUiSettings.setTiltGesturesEnabled(enabled);
	}

	/**
	 * 返回是否允许用户手势移动地图。
	 * @return 是否允许用户手势移动地图，默认：允许
	 */
	public boolean isScrollGesturesEnabled() {
		return mKUiSettings.isScrollGesturesEnabled();
	}

	/**
	 * 设置是否允许用户手势移动地图。
	 * <br>
	 * 这个设置不会影响用户通过程序接口移动地图。
	 * @param enabled 是否允许用户手势移动地图
	 */
	public void setScrollGesturesEnabled(boolean enabled) {
		mKUiSettings.setScrollGesturesEnabled(enabled);
	}
}
