package com.cmmap.api.maps.model;

import android.graphics.Color;

/**
 * 弧形覆盖物选项类。
 */
public class ArcOptions {
	LatLng m_startPoint = new LatLng(0, 0);
	LatLng m_passedPoint = new LatLng(0, 0);
	LatLng m_endPoint = new LatLng(0, 0);
	float m_strokeWidth = 10;
	int m_strokeColor = Color.BLACK;
	boolean m_draggable = false;
	int m_level = 0;
	float m_zIndex = 1.0f;
	boolean m_visible = true;
	
	/**
	 * 返回起点坐标。
	 * @return 起点坐标
	 */
	public LatLng getStart() {
		return m_startPoint;
	}

	/**
	 * 返回途经点坐标。
	 * @return 途经点坐标
	 */
	public LatLng getPassed() {
		return m_passedPoint;
	}

	/**
	 * 返回终点坐标。
	 * @return 终点坐标
	 */
	public LatLng getEnd() {
		return m_endPoint;
	}

	/**
	 * 设置起点、途径点和终点。
	 * @param startPoint 起点坐标
	 * @param passedPoint 途经点坐标
	 * @param endPoint 终点坐标
	 * @return 该对象本身
	 */
	public ArcOptions point(LatLng startPoint, LatLng passedPoint, LatLng endPoint) {
		m_startPoint = new LatLng(startPoint.latitude, startPoint.longitude);
		m_passedPoint = new LatLng(passedPoint.latitude, passedPoint.longitude);
		m_endPoint = new LatLng(endPoint.latitude, endPoint.longitude);
		return this;
	}

	/**
	 * 返回边框宽度。
	 * @return 边框宽度，单位：像素，默认：10像素
	 */
	public float getStrokeWidth() {
		return m_strokeWidth;
	}

	/**
	 * 设置边框宽度。
	 * @param width 边框宽度，单位：像素，默认：10像素
	 * @return 该对象本身
	 */
	public ArcOptions strokeWidth(float width) {
		if (width >= 0) {
			m_strokeWidth = width;
		}
		return this;
	}

	/**
	 * 返回边框颜色。
	 * @return 边框颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getStrokeColor() {
		return m_strokeColor;
	}

	/**
	 * 设置边框颜色。
	 * @param color ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 * @return 该对象本身
	 */
	public ArcOptions strokeColor(int color) {
		m_strokeColor = color;
		return this;
	}
	
	/**
	 * 返回是否可拖拽。
	 * @return 是否可拖拽，默认：否
	 */
	public boolean isDraggable() {
		return m_draggable;
	}

	/**
	 * 设置是否可拖拽。
	 * @param enable 是否可拖拽，默认：否
	 * @return 该对象本身
	 */
	public ArcOptions draggable(boolean enable) {
		m_draggable = enable;
		return this;
	}

	/**
	 * 返回level
	 * @return 
	 */
	public int getLevel() {
		return m_level;
	}
	
	
	/**
	 * 设置level
	 * @return 
	 */
	public ArcOptions level(int level) {
		m_level = level;
		return this;
	}
	
	/**
	 * 返回Z轴索引。
	 * @return Z轴索引，默认：1.0
	 */
	public float getZIndex() {
		return m_zIndex;
	}
	
	/**
	 * 设置Z轴索引。
	 * @param zIndex Z轴索引，默认：1.0
	 * @return 该对象本身
	 */
	public ArcOptions zIndex(float zIndex) {
		m_zIndex = zIndex;
		return this;
	}

	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */ 
	public boolean isVisible() {
		return m_visible;
	}

	/**
	 * 设置是否可见。
	 * @param visible 是否可见，默认：可见
	 * @return 该对象本身
	 */
	public ArcOptions visible(boolean visible) {
		m_visible = visible;
		return this;
	}
}
