package com.cmmap.api.maps.model;

import android.graphics.Bitmap;
import android.os.Parcel;
import android.os.Parcelable;

import com.cmmap.internal.mapcore.KBitmap;

/**
 * 图片描述符类。
 * <p>SDK中所有输入的图片都用该类描述，该对象的创建由{@link BitmapDescriptorFactory}支持。</p>
 */
public class BitmapDescriptor implements Cloneable, Parcelable {
	protected Bitmap m_bitmap;
	private int m_id;
	
	/**
	 * 内部构造函数，不公开提供
	 */
	BitmapDescriptor() {
		m_id = KBitmap.mallocId();
		m_bitmap = null;
	}

	/**
	 * 获取{@link android.graphics.Bitmap}对象。
	 * @return Bitmap对象
	 */
	public Bitmap getBitmap() {
		return this.m_bitmap;
	}
	
	void setBitmap(Bitmap _bitmap){
		this.m_bitmap = _bitmap;
	}

	/**
	 * 返回 {@link android.graphics.Bitmap}的宽度。
	 * @return Bitmap的宽度，单位：像素
	 */
	public int getWidth() {
		if (m_bitmap!=null) {
			return m_bitmap.getWidth();
		}
		return 0;
	}

	/**
	 * 返回 {@link android.graphics.Bitmap}的高度。
	 * @return Bitmap的高度，单位：像素
	 */
	public int getHeight() {
		if (m_bitmap!=null) {
			return m_bitmap.getHeight();
		}
		return 0;
	}

	/**
	 * 销毁该对象，回收内存
	 */
	public void recycle() {
		if (this.m_bitmap != null) {
			this.m_bitmap.recycle();
			this.m_bitmap = null;
		}
	}

	@Override
	public BitmapDescriptor clone() {
		BitmapDescriptor bitmapDescriptor= new BitmapDescriptor();
		bitmapDescriptor.m_bitmap = Bitmap.createBitmap(this.m_bitmap);
		return bitmapDescriptor;
	}

	@Override
	public boolean equals(Object object) {
		if (object == null || !(object instanceof BitmapDescriptor)) {
			return false;
		}
		
		BitmapDescriptor obj = (BitmapDescriptor) object;
		return obj.m_bitmap.equals(m_bitmap);
	}

	@Override
	public int describeContents() {
		return m_id;
	}

	@Override
	public void writeToParcel(Parcel dest, int flags) {
		dest.writeInt(m_id);
		dest.writeParcelable(m_bitmap, flags);
	}
	private void readFromParcel(Parcel source) {
		m_id = source.readInt();
		m_bitmap = source.readParcelable(Bitmap.class.getClassLoader());
	}

	/**
	 * 根据{@link android.os.Parcelable}规范实现。
	 */
	public static final Parcelable.Creator<BitmapDescriptor> CREATOR = new Parcelable.Creator<BitmapDescriptor>() {  //*************未完成*************//
		@Override
		public BitmapDescriptor createFromParcel(Parcel source) {
			BitmapDescriptor bitmapDescriptor = new BitmapDescriptor();
			bitmapDescriptor.readFromParcel(source);
			return bitmapDescriptor;
		}

		@Override
		public BitmapDescriptor[] newArray(int size) {
			return new BitmapDescriptor[size];
		}
	};
}
