package com.cmmap.api.maps.model;

import android.os.Parcel;
import android.os.Parcelable;

/**
 * 地图底图路况事件点类。 
 * <p>地图底图路况事件点包含如下属性：</p>
 * <ul>
 * <li>名称<br>路况事件点在底图上显示的名称。</li>
 * <li>名称<br>路况事件点在底图上显示的详细信息。</li>
 * <li>坐标<br>路况事件点在底图上的经纬度坐标。</li>
 * <li>ID<br>路况事件点唯一标识。</li>
 * </ul>
 */
public class Event implements Parcelable {
    /** 无图标*/
    public static final int ICON_TYPE_NONE =0;
    /** 图标类型-施工*/
    public static final int ICON_TYPE_CONSTRUCTION = 1;
    /** 图标类型-封闭、禁止通行*/
    public static final int ICON_TYPE_NO_PASS = 2;
    /** 图标类型-事故*/
    public static final int ICON_TYPE_TRAFFIC_ACCIDENT = 3;
    /** 图标类型-禁止左转*/
    public static final int ICON_TYPE_NO_LEFT = 4;
    /** 图标类型-禁止右转*/
    public static final int ICON_TYPE_NO_RIGHT = 5;
    /** 图标类型-禁止直行*/
    public static final int ICON_TYPE_NO_AHEAD = 6;
    /** 图标类型-禁止掉头*/
    public static final int ICON_TYPE_NO_UTURN = 7;
    /** 图标类型-禁止左右转弯*/
    public static final int ICON_TYPE_NO_LEFT_AND_RIGHT = 8;
    /** 图标类型-禁止直行和左转*/
    public static final int ICON_TYPE_NO_AHEAD_AND_LEFT = 9;
    /** 图标类型-雨雪*/
    public static final int ICON_TYPE_RAIN_AND_SNOW = 10;
    /** 图标类型-限速100km*/
    public static final int ICON_TYPE_SPEED_LIMIT_100KM = 11;
    /** 图标类型-限速110km*/
    public static final int ICON_TYPE_SPEED_LIMIT_110KM = 12;
    /** 图标类型-限速120km*/
    public static final int ICON_TYPE_SPEED_LIMIT_120KM = 13;
    /** 图标类型-单侧通行*/
    public static final int ICON_TYPE_ONE_WAY = 14;
    /** 图标类型-双侧通行*/
    public static final int ICON_TYPE_TWO_WAY = 15;
    /** 图标类型-限高*/
    public static final int ICON_TYPE_HEIGHT_LIMIT = 16;
    /** 图标类型-限重*/
    public static final int ICON_TYPE_WEIGHT_LIMIT = 17;
    /** 图标类型-禁止直行和右转*/
    public static final int ICON_TYPE_NO_AHEAD_RIGHT = 18;
    /** 图标类型-限速10km*/
    public static final int ICON_TYPE_SPEED_LIMIT_10KM = 19;
    /** 图标类型-限速20km*/
    public static final int ICON_TYPE_SPEED_LIMIT_20KM = 20;
    /** 图标类型-限速30km*/
    public static final int ICON_TYPE_SPEED_LIMIT_30KM = 21;
    /** 图标类型-限速40km*/
    public static final int ICON_TYPE_SPEED_LIMIT_40KM = 22;
    /** 图标类型-限速50km*/
    public static final int ICON_TYPE_SPEED_LIMIT_50KM = 23;
    /** 图标类型-限速60km*/
    public static final int ICON_TYPE_SPEED_LIMIT_60KM = 24;
    /** 图标类型-限速70km*/
    public static final int ICON_TYPE_SPEED_LIMIT_70KM = 25;
    /** 图标类型-限速80km*/
    public static final int ICON_TYPE_SPEED_LIMIT_80KM = 26;
    /** 图标类型-限速90km*/
    public static final int ICON_TYPE_SPEED_LIMIT_90KM = 27;
            
    
	/** 地图事件点的名称 */
	public final String name;
	/** 地图事件点的详细信息 */
	public final String info;
	/** 地图POI点的坐标 */
	public final LatLng coordinate;
	/** 地图POI点ID */
	public final String eventid;
	/** 地图事件图标类型*/
	public final int icon_type; 
	/**
	 * 构造函数
	 * @param name 地图路况事件点的名称
	 * @param name 地图路况事件点的详细名称
	 * @param coordinate 地图路况事件点的坐标
	 * @param eventid 地图路况事件点ID
	 */
	public Event(String name, String info, LatLng coordinate, String eventid, int type) {
		this.name = name;
		this.info = info;
		this.coordinate = coordinate;
		this.eventid = eventid;
		this.icon_type = type;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((coordinate == null) ? 0 : coordinate.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result + ((info == null) ? 0 : info.hashCode());
		result = prime * result + ((eventid == null) ? 0 : eventid.hashCode());
		result = prime * result + icon_type;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Event other = (Event) obj;
		if (coordinate == null) {
			if (other.coordinate != null)
				return false;
		} else if (!coordinate.equals(other.coordinate))
			return false;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		if (info == null) {
			if (other.info != null)
				return false;
		} else if (!info.equals(other.info))
			return false;
		if (eventid == null) {
			if (other.eventid != null)
				return false;
		} else if (!eventid.equals(other.eventid))
			return false;
		return icon_type == other.icon_type;
	}

	@Override
	public String toString() {
		return String.format("{name=%s,info=%s,coordinate=%s,eventid=%s,icontype:%d}", name, info, coordinate, eventid, icon_type);
	}

	@Override
	public int describeContents() {
		return 0;
	}

	@Override
	public void writeToParcel(Parcel dest, int flags) {
		dest.writeString(name);
		dest.writeString(info);
		dest.writeParcelable(coordinate, flags);
		dest.writeString(eventid);
		dest.writeInt(icon_type);
	}

	/**
	 * 根据{@link android.os.Parcelable}规范实现。
	 */
	public static final Parcelable.Creator<Event> CREATOR = new Parcelable.Creator<Event>() {
		@Override
		public Event createFromParcel(Parcel source) {
			String name = source.readString();
			String info = source.readString();
			LatLng coord = source.readParcelable(LatLng.class.getClassLoader());
			String id = source.readString();
			int type = source.readInt();
			return new Event(name, info, coord, id, type);
		}

		@Override
		public Event[] newArray(int size) {
			return new Event[size];
		}
	};
	
	
}
