package com.cmmap.api.maps.model;


/**
 * 图片覆盖物选项类。
 */
public class GroundOverlayOptions{
	LatLngBounds m_bounds = null;
	float m_transparency = 0;
	BitmapDescriptor m_bitmapdDescriptor = null;
	boolean m_draggable = false;
	float m_zIndex = 1.0f;
	int m_level = 0;
	boolean m_visible = true;

	/**
	 * 返回图片显示的的矩形区域 。
	 * @return 图片显示的的矩形区域 
	 */
	public LatLngBounds getBounds() {
		return m_bounds;
	}

	/**
	 * 根据矩形区域设置图片的显示位置。
	 * @param latLngBounds 图片的显示的矩形区域
	 * @return 该对象本身
	 */
	public GroundOverlayOptions bounds(LatLngBounds latLngBounds) {
		m_bounds = latLngBounds;
		return this;
	}

	/**
	 * 返回图片对象。
	 * @return 图片对象
	 */
	public BitmapDescriptor getImage() {
		return m_bitmapdDescriptor;
	}

	/**
	 * 设置图片对象。
	 * @param image 图片对象
	 * @return 该对象本身
	 */
	public GroundOverlayOptions image(BitmapDescriptor image) {
		m_bitmapdDescriptor = image;
		return this;
	}

	/**
	 * 返回绘制图片时的的透明度。
	 * @return 绘制图片时的的透明度，默认：0
	 */
	public float getTransparency() {
		return m_transparency;
	}

	/**
	 * 设置绘制图片时的的透明度。
	 * @param transparency 绘制图片时的的透明度，范围为[0,1]，0为不透明，1为全透明，默认：0
	 * @return 该对象本身
	 */
	public GroundOverlayOptions transparency(float transparency) {
		m_transparency = transparency;
		return this;
	}
	
	/**
	 * 返回是否可拖拽。
	 * @return 是否可拖拽，默认：否
	 */
	public boolean isDraggable() {
		return m_draggable;
	}

	/**
	 * 设置是否可拖拽。
	 * @param enable 是否可拖拽，默认：否
	 * @return 该对象本身
	 */
	public GroundOverlayOptions draggable(boolean enable) {
		m_draggable = enable;
		return this;
	}

	/**
	 * 返回level
	 * @return 
	 */
	public int getLevel() {
		return m_level;
	}
	
	
	/**
	 * 设置level
	 * @return 
	 */
	public GroundOverlayOptions level(int level) {
		m_level = level;
		return this;
	}
	
	/**
	 * 返回Z轴索引。
	 * @return Z轴索引，默认：1.0
	 */
	public float getZIndex() {
		return m_zIndex;
	}
	
	/**
	 * 设置Z轴索引。
	 * @param zIndex Z轴索引，默认：1.0
	 * @return 该对象本身
	 */
	public GroundOverlayOptions zIndex(float zIndex) {
		m_zIndex = zIndex;
		return this;
	}

	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */ 
	public boolean isVisible() {
		return m_visible;
	}

	/**
	 * 设置是否可见。
	 * @param visible 是否可见，默认：可见
	 * @return 该对象本身
	 */
	public GroundOverlayOptions visible(boolean visible) {
		m_visible = visible;
		return this;
	}
}
