package com.cmmap.api.maps.model;

import android.os.Parcel;
import android.os.Parcelable;

/**
 * 经纬度坐标类。
 */
public final class LatLng implements Cloneable, Parcelable {
	/** 纬度 (水平方向)，单位角度 */
	public final double latitude;
	/** 经度 (垂直方向)，单位角度 */
	public final double longitude;

	/**
	 * 使用传入的经纬度构造，一对经纬度值代表地球上一个地点。
	 * @param latitude 地点的纬度，在-90 与90 之间的double 型数值
	 * @param longitude 地点的经度，在-180 与180 之间的double 型数值
	 */
	public LatLng(double latitude, double longitude) {
		this.latitude = latitude;
		this.longitude = longitude;
	}

	@Override
	public LatLng clone() {
		return new LatLng(latitude, longitude);
	}

	

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		long temp;
		temp = Double.doubleToLongBits(latitude);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		temp = Double.doubleToLongBits(longitude);
		result = prime * result + (int) (temp ^ (temp >>> 32));
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		LatLng other = (LatLng) obj;
		if (Double.doubleToLongBits(latitude) != Double
				.doubleToLongBits(other.latitude))
			return false;
		if (Double.doubleToLongBits(longitude) != Double
				.doubleToLongBits(other.longitude))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return String.format("{latitude=%.6f,longitude=%.6f}", latitude, longitude);
	}

	@Override
	public int describeContents() {
		// TODO
		return 0;
	}

	@Override
	public void writeToParcel(Parcel out, int arg1) {
		out.writeDouble(latitude);
		out.writeDouble(longitude);
	}

	/**
	 * 根据{@link android.os.Parcelable}规范实现。
	 */
	public static final Parcelable.Creator<LatLng> CREATOR = new Parcelable.Creator<LatLng>() {
		@Override
		public LatLng createFromParcel(Parcel source) { 
			double latitude = source.readDouble();
			double longitude = source.readDouble();
			LatLng latLng =new LatLng(latitude, longitude);
			return latLng;
		}

		@Override
		public LatLng[] newArray(int size) {
			return new LatLng[size];
		}
	};
}
