package com.cmmap.api.maps.model;

import java.util.ArrayList;
import java.util.List;

import android.graphics.Color;

/**
 * 箭头覆盖物选项类。
 */
public class NavigateArrowOptions{
	ArrayList<LatLng> m_points = new ArrayList<LatLng>();
	float m_width = 10;
	int m_color = Color.BLACK;
	int m_level = 0;
	float m_zIndex = 1.0f;
	boolean m_visible = true;
	
	/**
	 * 返回 顶点坐标列表。
	 * @return 线段的顶点坐标列表
	 */
	public List<LatLng> getPoints() {
		return m_points;
	}

	/**
	 * 添加一个顶点到顶点坐标列表中。
	 * @param point 要添加的顶点
	 * @return 该对象本身
	 */
	public NavigateArrowOptions add(LatLng point) {
		m_points.add(point);
		return this;
	}
	
	/**
	 * 添加一批顶点到顶点坐标列表中。
	 * @param points 要添加的顶点集合
	 * @return 该对象本身
	 */
	public NavigateArrowOptions add(LatLng... points) {
		for (LatLng latLng : points) {
			m_points.add(latLng);
		}
		return this;
	}

	/**
	 * 添加一批顶点到顶点坐标列表中。
	 * @param points 要添加的顶点集合
	 * @return 该对象本身
	 */
	public NavigateArrowOptions addAll(Iterable<LatLng> points) {
		for (LatLng latLng : points) {
			m_points.add(latLng);
		}
		return this;
	}

	/**
	 * 返回线宽。
	 * @return 线宽，单位：像素，默认：10像素
	 */
	public float getWidth() {
		return m_width;
	}

	/**
	 * 设置线宽。
	 * @param width 线宽，单位：像素，默认：10像素
	 * @return 该对象本身
	 */
	public NavigateArrowOptions width(float width) {
		m_width = width;
		return this;
	}

	/**
	 * 返回箭头颜色。
	 * @return 箭头颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getColor() {
		return m_color;
	}

	/**
	 * 设置箭头颜色。
	 * @param color 箭头颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 * @return 该对象本身
	 */
	public NavigateArrowOptions color(int color) {
		m_color = color;
		return this;
	}

	
	/**
	 * 返回level
	 * @return 
	 */
	public int getLevel() {
		return m_level;
	}
	
	
	/**
	 * 设置level
	 * @return 
	 */
	public NavigateArrowOptions level(int level) {
		m_level = level;
		return this;
	}
	
	/**
	 * 返回Z轴索引。
	 * @return Z轴索引，默认：1.0
	 */
	public float getZIndex() {
		return m_zIndex;
	}
	
	/**
	 * 设置Z轴索引。
	 * @param zIndex Z轴索引，默认：1.0
	 * @return 该对象本身
	 */
	public NavigateArrowOptions zIndex(float zIndex) {
		m_zIndex = zIndex;
		return this;
	}

	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */ 
	public boolean isVisible() {
		return m_visible;
	}

	/**
	 * 设置是否可见。
	 * @param visible 是否可见，默认：可见
	 * @return 该对象本身
	 */
	public NavigateArrowOptions visible(boolean visible) {
		m_visible = visible;
		return this;
	}
}
