package com.cmmap.api.maps.model;

import android.os.Parcel;
import android.os.Parcelable;

/**
 * 地图底图POI点类。 
 * <p>地图底图POI点包含如下属性：</p>
 * <ul>
 * <li>名称<br>POI点在底图上显示的名称。</li>
 * <li>坐标<br>POI点在底图上的经纬度坐标。</li>
 * <li>ID<br>POI点唯一标识。</li>
 * </ul>
 */
public class Poi implements Parcelable {
	/** 地图POI点的名称 */
	public final String name;
	/** 地图POI点的坐标 */
	public final LatLng coordinate;
	/** 地图POI点ID */
	public final String poiid;
	
	/**
	 * 构造函数
	 * @param name 地图POI点的名称
	 * @param coordinate 地图POI点的坐标
	 * @param poiid 地图POI点ID
	 */
	public Poi(String name, LatLng coordinate, String poiid) {
		this.name = name;
		this.coordinate = coordinate;
		this.poiid = poiid;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((coordinate == null) ? 0 : coordinate.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result + ((poiid == null) ? 0 : poiid.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Poi other = (Poi) obj;
		if (coordinate == null) {
			if (other.coordinate != null)
				return false;
		} else if (!coordinate.equals(other.coordinate))
			return false;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		if (poiid == null) {
			if (other.poiid != null)
				return false;
		} else if (!poiid.equals(other.poiid))
			return false;
		return true;
	}

	@Override
	public String toString() {
		return String.format("{name=%s,coordinate=%s,poiid=%s}", name, coordinate, poiid);
	}

	@Override
	public int describeContents() {
		return 0;
	}

	@Override
	public void writeToParcel(Parcel dest, int flags) {
		dest.writeString(name);
		dest.writeParcelable(coordinate, flags);
		dest.writeString(poiid);
	}

	/**
	 * 根据{@link android.os.Parcelable}规范实现。
	 */
	public static final Parcelable.Creator<Poi> CREATOR = new Parcelable.Creator<Poi>() {
		@Override
		public Poi createFromParcel(Parcel source) {
			String name = source.readString();
			LatLng coord = source.readParcelable(LatLng.class.getClassLoader());
			String id = source.readString();
			return new Poi(name, coord, id);
		}

		@Override
		public Poi[] newArray(int size) {
			return new Poi[size];
		}
	};
	
	
}
