package com.cmmap.api.maps.model;

/**
 * 栅格瓦片提供者接口类。
 * <p>该接口的目的是为类{@link TileOverlay}提供栅格图源。</p>
 */
public interface TileProvider {
	/**
	 * 根据指定的栅格瓦片坐标和对应的地图缩放级别，获取栅格瓦片对象。
	 * @param x 栅格瓦片的横坐标，范围：[0, 2^zoom - 1]
	 * @param y 栅格瓦片的纵坐标，范围：[0, 2^zoom - 1]
	 * @param zoom 对应的地图缩放级别，其范围可以通过类
	 * {@link com.cmmap.api.maps.Map#getMinZoomLevel}
	 * 和{@link com.cmmap.api.maps.Map#getMaxZoomLevel}获取
	 * @return 栅格瓦片对象
	 */
	public Tile getTile(int x, int y, int zoom);

	/**
	 * 返回栅格瓦片像素宽度。
	 * @return 栅格瓦片的像素宽度
	 */
	public int getTileWidth();

	/**
	 * 返回栅格瓦片像素高度。
	 * @return 栅格瓦片的像素高度
	 */
	public int getTileHeight();
}
