package com.cmmap.api.maps.model;

import java.net.URL;

import com.cmmap.internal.driver.base.HttpTool;

/**
 * URL栅格瓦片提供者类。
 * <p>使用者使用该类，只需要实现{@link #getTileUrl}即可。</p>
 */
public abstract class UrlTileProvider implements TileProvider {
	private final int m_width;
	private final int m_height;
	/**
	 * 构造函数。
	 * @param width 栅格瓦片的像素宽度
	 * @param height 栅格瓦片的像素高度
	 */
	public UrlTileProvider(int width, int height) {
		m_width = width;
		m_height = height;
	}

	/**
	 * 返回指定的栅格瓦片对象需要的图片的URL地址。
	 * @param x 栅格瓦片的横坐标，范围：[0, 2^zoom - 1]
	 * @param y 栅格瓦片的纵坐标，范围：[0, 2^zoom - 1]
	 * @param zoom 对应的地图缩放级别，其范围可以通过类
	 * {@link com.cmmap.api.maps.Map#getMinZoomLevel}
	 * 和{@link com.cmmap.api.maps.Map#getMaxZoomLevel}获取
	 * @return 栅格瓦片对象需要的图片的URL地址
	 */
	public abstract URL getTileUrl(int x, int y, int zoom);

	@Override
	public Tile getTile(int x, int y, int zoom) {
		URL url = getTileUrl(x, y, zoom);
		Tile tile = null;
		// 使用url取网络数据
		byte[] b_body = new byte[0];
		HttpTool tool = new HttpTool();
		try {
			byte[] b_data = null;
			b_data = tool.executeHttpGet(url.toString(), m_width, m_height, b_body);
			if (b_data != null && b_data.length != 0) {
				tile = new Tile(m_width, m_height, b_data);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return tile;
	}

	@Override
	public int getTileWidth() {
		return m_width;
	}

	@Override
	public int getTileHeight() {
		return m_height;
	}
}
