package com.cmmap.api.maps.offlinemap;

import com.cmmap.internal.services.rtic.ByteHelper;



/**
 * 下载城市类。
 */
public class OfflineMapCity {

	String mName;
	int mCode = 0;
	int mAdcode = 0;
	String mPinyin;
	String mJianpin;
	String mUri;
	long mFilesize;
	int mState;
	int mVersion;
	int mComplete;

	public OfflineMapCity() {
	}

	/**
	 * 返回城市行政编码
	 * 
	 * @return 城市行政编码
	 */
	public String getAdcode() {
		if (mAdcode <= 0) {
			return new String("000000");
		}
		
		int adCode = mAdcode;
		while (adCode < 100000) {
			adCode *= 10;
		}
		return String.format("%d", adCode);
	}

	/**
	 * 返回城市名称。
	 * 
	 * @return 城市名称
	 */
	public String getName() {
		return mName;
	}

	/**
	 * 返回城市代码。
	 * 
	 * @return 城市代码
	 */
	public String getCode() {
		String strCode;
		if (mCode < 100) {
			strCode = String.format("%03d", mCode);
		} else {
			strCode = String.format("%04d", mCode);
		}
		return strCode;
	}

	/**
	 * 返回城市名称简拼拼音。
	 * 
	 * @return 城市名称简拼拼音
	 */
	public String getJianpin() {
		return mJianpin;
	}

	/**
	 * 返回城市名称拼音
	 * 
	 * @return 城市名称拼音
	 */
	public String getPinyin() {
		return mPinyin;
	}

	/**
	 * 返回城市下载状态。
	 * 
	 * @return 城市下载状态
	 */
	public int getState() {
		return mState;
	}

	/**
	 * 返回城市下载完成的百分比，100表示下载完成。
	 * 
	 * @return 下载完成的百分比
	 */
	public int getCompleteCode() {
		return mComplete;
	}

	/**
	 * 返回下载城市数据的大小，单位：字节。
	 * 
	 * @return 下载城市数据的大小
	 */
	public long getSize() {
		return mFilesize;
	}

	/**
	 * 返回所下载城市的数据地址。
	 * 
	 * @return 下载城市的数据地址
	 */
	public String getUrl() {
		return mUri;

	}

	/**
	 * 返回下载城市的数据版本。
	 * 
	 * @return 下载城市的数据版本
	 */
	public String getVersion() {
		return ByteHelper.getVersionByInt(mVersion);

	}
	
	@Override
	public String toString() {
		return String.format(
				"{adcode=%s,code=%s,name=%s,jianpin=%s,pinyin=%s}",
				getAdcode(), getCode(), getName(), getJianpin(), getPinyin());
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + mAdcode;
		result = prime * result + mCode;
		result = prime * result + mComplete;
		result = prime * result + (int) (mFilesize ^ (mFilesize >>> 32));
		result = prime * result
				+ ((mJianpin == null) ? 0 : mJianpin.hashCode());
		result = prime * result + ((mName == null) ? 0 : mName.hashCode());
		result = prime * result + ((mPinyin == null) ? 0 : mPinyin.hashCode());
		result = prime * result + mState;
		result = prime * result + ((mUri == null) ? 0 : mUri.hashCode());
		result = prime * result + mVersion;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		OfflineMapCity other = (OfflineMapCity) obj;
		if (mAdcode != other.mAdcode)
			return false;
		if (mCode != other.mCode)
			return false;
		if (mComplete != other.mComplete)
			return false;
		if (mFilesize != other.mFilesize)
			return false;
		if (mJianpin == null) {
			if (other.mJianpin != null)
				return false;
		} else if (!mJianpin.equals(other.mJianpin))
			return false;
		if (mName == null) {
			if (other.mName != null)
				return false;
		} else if (!mName.equals(other.mName))
			return false;
		if (mPinyin == null) {
			if (other.mPinyin != null)
				return false;
		} else if (!mPinyin.equals(other.mPinyin))
			return false;
		if (mState != other.mState)
			return false;
		if (mUri == null) {
			if (other.mUri != null)
				return false;
		} else if (!mUri.equals(other.mUri))
			return false;
		if (mVersion != other.mVersion)
			return false;
		return true;
	}
}
