package com.cmmap.api.maps.offlinemap;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

import com.cmmap.api.maps.MapException;
import com.cmmap.internal.mapcore.MapJni;
import com.cmmap.internal.mapcore.offlinemap.CommonDefine;
import com.cmmap.internal.mapcore.offlinemap.DownloadInstance;
import com.cmmap.internal.mapcore.offlinemap.DownloadInstance.OnDownloadStatusChangedListener;
import com.cmmap.internal.mapcore.offlinemap.WebServiceUtil;
import com.cmmap.internal.mapcore.offlinemap.entity.CityInfo;
import com.cmmap.internal.mapcore.offlinemap.entity.ObjectInfo;
import com.cmmap.internal.mapcore.offlinemap.entity.ProvinceInfo;

/**
 * 离线地图下载管理类。
 */
public class OfflineMapManager {
	private static final String TAG = OfflineMapManager.class.getName();
	private static List<OfflineMapProvince> mProvinces;
	private Context mContext;
	private OfflineMapDownloadListener mListener;
	private OmmHandler mHandler;
	private DownloadInstance dInstance;
	private static final int ON_DOWNLOAD = 11;
	private static final int ON_REMOVE = 12;
	private static final int ON_UPDATE = 13;
	private static final int ABROAD_ADCODE = 8400;//国外国家及城市adcode大于8400

	private long downTime = 0L;
	
	/**
	 * 构造函数。
	 * @param context 上下文
	 * @param listener 下载事件监听
	 */
	public OfflineMapManager(Context context, OfflineMapManager.OfflineMapDownloadListener listener) {
		init(context, listener);
		dInstance = DownloadInstance.getInstance(context.getApplicationContext(), mDownloadStatusChangedListener);
		mHandler = new OmmHandler();	
		getInternalProvinceList();
	}
	
	OnDownloadStatusChangedListener mDownloadStatusChangedListener = null;
	
	/**
	 * 
	 * @param context
	 * @param listener
	 */
	private void init(Context context, OfflineMapDownloadListener listener) {
		if (mListener == null) {
			mListener = listener;
		}
		if (mContext == null) {
			mContext = context.getApplicationContext();
		}
		mDownloadStatusChangedListener = new DownloadInstance.OnDownloadStatusChangedListener() {
			
			@Override
			public void onError(MapException exception) {
				
			}

			@Override
			public void onDownload(ObjectInfo info) {
				Message msg = Message.obtain();
				msg.what = ON_DOWNLOAD;
				msg.getData().putParcelable("info", info);
				if (info.getState() == OfflineMapStatus.DOWNLOADING) {
					// 避免回调过于频繁，1s回调一次
					if ((System.currentTimeMillis() - downTime) >= 1000) {
						mHandler.sendMessage(msg);
						downTime = System.currentTimeMillis();
					}
				} else {
					mHandler.sendMessage(msg);
				}
			}

			@Override
			public void onRemove(ObjectInfo info) {
				Message msg = Message.obtain();
				msg.what = ON_REMOVE;
				msg.getData().putParcelable("info", info);
				mHandler.sendMessage(msg);
			}

			@Override
			public void onUpdate(ObjectInfo info) {
				Message msg = Message.obtain();
				msg.what = ON_UPDATE;
				msg.getData().putParcelable("info", info);
				mHandler.sendMessage(msg);
			}

			@Override
			public void onVersionUpdate() {				
				Log.i(TAG, " onVersionUpdate:");
				// 版本更新，更新城市列表
				updateProvinceList();
			}
		};
	}

	private void updateInfoByAdCode(ObjectInfo info) {		
		for (int i = 0; i < mProvinces.size(); i++) {
			OfflineMapProvince province = mProvinces.get(i);
			if (info instanceof ProvinceInfo && province.mCode == info.getAdCode()) {
				province.mState = info.getState();
				int complete = WebServiceUtil.getProportion(info.getDownloadsize(), info.getFilesize());
				province.mComplete = complete;
				if (province.mCode != CommonDefine.ALLDATA_CODE) {
					break;
				}
			}
			
			List<OfflineMapCity> citys = mProvinces.get(i).getCityList();
			for (int j = 0; j < citys.size(); j++) {
				if (citys.get(j).mAdcode == info.getAdCode()) {
					citys.get(j).mState = info.getState();
					int complete = WebServiceUtil.getProportion(info.getDownloadsize(), info.getFilesize());
					citys.get(j).mComplete = complete;
					updateProvinceStatus(info, mProvinces.get(i));
					break;
				}
			}
		}
	}
	
	private void updateProvinceStatus(ObjectInfo info, OfflineMapProvince province) {
		HashMap<Integer, Integer> mapResult = new HashMap<Integer, Integer>();
		switch (info.getState()) {
		case OfflineMapStatus.FINISHED:
			if(province.getState() != OfflineMapStatus.FINISHED) {
				checkCityList(province, mapResult);
				if (mapResult != null) {
					if (mapResult.get(OfflineMapStatus.DOWNLOADING) > 0) {
						province.mState = OfflineMapStatus.DOWNLOADING;
					} else if (mapResult.get(OfflineMapStatus.PAUSE) > 0){
						province.mState = OfflineMapStatus.PAUSE;
					} else if (mapResult.get(OfflineMapStatus.WAITING) > 0) {
						province.mState = OfflineMapStatus.WAITING;
					} else if (mapResult.get(OfflineMapStatus.NONE) > 0) {
						province.mState = OfflineMapStatus.NONE;
					} else {
						province.mState = OfflineMapStatus.FINISHED;
					}
				}
			}
			break;
		case OfflineMapStatus.DOWNLOADING:
			province.mState = OfflineMapStatus.DOWNLOADING;
			break;
		case OfflineMapStatus.PAUSE:
			if (province.getState() != OfflineMapStatus.PAUSE) {
				checkCityList(province, mapResult);
				if (mapResult != null) {
					if (mapResult.get(OfflineMapStatus.DOWNLOADING) > 0) {
						province.mState = OfflineMapStatus.DOWNLOADING;
					} else {
						province.mState = OfflineMapStatus.PAUSE;
					}
				}
			}
			break;
		case OfflineMapStatus.WAITING:
			if (province.getState() != OfflineMapStatus.WAITING) {
				checkCityList(province, mapResult);
				if (mapResult != null) {
					if (mapResult.get(OfflineMapStatus.DOWNLOADING) > 0) {
						province.mState = OfflineMapStatus.DOWNLOADING;
					} else if (mapResult.get(OfflineMapStatus.PAUSE) > 0){
						province.mState = OfflineMapStatus.PAUSE;
					} else {
						province.mState = OfflineMapStatus.WAITING;
					}
				}
			}
			break;
		case OfflineMapStatus.NONE:
			if (province.getState() != OfflineMapStatus.NONE) {
				checkCityList(province, mapResult);
				if (mapResult != null) {
					if (mapResult.get(OfflineMapStatus.DOWNLOADING) > 0) {
						province.mState = OfflineMapStatus.DOWNLOADING;
					} else if (mapResult.get(OfflineMapStatus.PAUSE) > 0){
						province.mState = OfflineMapStatus.PAUSE;
					} else if (mapResult.get(OfflineMapStatus.WAITING) > 0) {
						province.mState = OfflineMapStatus.WAITING;
					} else {
						province.mState = OfflineMapStatus.NONE;
					}
				}
			}
			break;
		default:
			break;
		}
		ObjectInfo provinceInfo = dInstance.getInfoByName(province.mName);
		provinceInfo.setState(province.mState);
		MapJni.saveInfo(provinceInfo);
	}
	
	private void checkCityList(OfflineMapProvince province, HashMap<Integer, Integer> result) {
		if (result == null) {
			return;
		}
		
		int finish = 0;
		int downloading = 0;
		int pause = 0;
		int waiting = 0;
		int none = 0;
		List<OfflineMapCity> citys = province.getCityList();
		for (int i = 0; i < citys.size(); i++) {
			switch (citys.get(i).getState()) {
			case OfflineMapStatus.FINISHED:
				finish++;
				break;
			case OfflineMapStatus.DOWNLOADING:
				downloading++;
				break;
			case OfflineMapStatus.PAUSE:
				pause++;
				break;
			case OfflineMapStatus.WAITING:
				waiting++;
				break;
			case OfflineMapStatus.NONE:
				none++;
				break;
			default:
				break;
			}
		}
		result.put(OfflineMapStatus.FINISHED, finish);
		result.put(OfflineMapStatus.DOWNLOADING, downloading);
		result.put(OfflineMapStatus.PAUSE, pause);
		result.put(OfflineMapStatus.WAITING, waiting);
		result.put(OfflineMapStatus.NONE, none);
	}
	
	/**
	 * 根据城市名称获取城市对象。
	 * @param cityName 城市名称
	 * @return 城市对象
	 */
	public OfflineMapCity getCity(String cityName){	
		if(mProvinces == null || mProvinces.isEmpty()){
			getInternalProvinceList();
			
		}
		if(mProvinces == null || mProvinces.isEmpty())
			return null;
		for (int i = 0; i < mProvinces.size(); i++) {
			List<OfflineMapCity> tempList = mProvinces.get(i).getCityList();
			for (int j = 0; j < tempList.size(); j++) {
				if(tempList.get(j).getName().equals(cityName)){
					return tempList.get(j);
				}
			}
		}
		return null;
	}
	
	/**
	 * 根据海外城市名称获取城市对象。
	 * @param cityName 海外城市名称
	 * @return 海外城市对象
	 */
	public OfflineMapAbroadCity getAbroadCity(String cityName){		
		OfflineMapCity city = getCity(cityName);
		if(city instanceof OfflineMapAbroadCity){
			return (OfflineMapAbroadCity) city;
		}
		return null;
	}
	
	/**
	 * 根据省份名称获取省份对象。
	 * @param provinceName 省份名称
	 * @return 省份对象
	 */
	public OfflineMapProvince getProvince(String provinceName){		
		if(mProvinces == null || mProvinces.isEmpty()){
			getInternalProvinceList();
			
		}
		if(mProvinces == null || mProvinces.isEmpty())
			return null;
		for (int i = 0; i < mProvinces.size(); i++) {
			if(mProvinces.get(i).getName().equals(provinceName)){
				return mProvinces.get(i);
			}
		}
		return null;
	}
	
	/**
	 * 根据海外国家名称获取海湾国家对象。
	 * @param countryName 海外国家名称
	 * @return 海湾国家对象
	 */
	public OfflineMapAbroadCountry getAbroadCountry(String countryName){
		OfflineMapProvince province = getProvince(countryName);
		if(province instanceof OfflineMapAbroadCountry){
			
			return (OfflineMapAbroadCountry) province;
		}
		
		return null;
	}
	
	/**
	 * 根据给定的城市名称或省份名称，下载对应的城市或省份的离线地图包，异步方法，触发回调onDownload方法。
	 * 如果输入的是省份名称，等同于开始下载该省份下的所有城市。
	 * @param name 城市名称或省份名称
	 */
	public void download(String name) {
			dInstance.downloadByName(name);
	}
	

	/**
	 * 所有正在下载或等待下载离线地图的城市列表。
	 * @return 下载完成的城市列表
	 */
	public List<OfflineMapCity> getDownloadingCityList() {
		if(mProvinces == null || mProvinces.isEmpty()){
			getInternalProvinceList();
			
		}
		if(mProvinces == null || mProvinces.isEmpty())
			return null;
		List<OfflineMapCity> retList = new ArrayList<OfflineMapCity>();
		for (int i = 0; i < mProvinces.size(); i++) {
			List<OfflineMapCity> tempList = mProvinces.get(i).getCityList();
			for (int j = 0; j < tempList.size(); j++) {
				if(tempList.get(j).getState() == OfflineMapStatus.DOWNLOADING
						|| tempList.get(j).getState() == OfflineMapStatus.WAITING 
						|| tempList.get(j).getState() == OfflineMapStatus.UNZIPING
						|| tempList.get(j).getState() == OfflineMapStatus.PAUSE
						|| tempList.get(j).getState() == OfflineMapStatus.NEW_VERSION){
					OfflineMapCity offlineMapCity = tempList.get(j);
					if(!( offlineMapCity instanceof OfflineMapAbroadCity)){
						
						retList.add(tempList.get(j));
					}
				}
			}
		}
		return retList;
	}
	
	/**
	 * 所有正在下载或等待下载离线地图的海外城市列表。
	 * @return 下载完成的海外城市列表
	 */
	public List<OfflineMapAbroadCity> getDownloadingAbroadCityList() {	
		if(mProvinces == null || mProvinces.isEmpty()){
			getInternalProvinceList();
			
		}
		if(mProvinces == null || mProvinces.isEmpty())
			return null;
		List<OfflineMapAbroadCity> retList = new ArrayList<OfflineMapAbroadCity>();
		for (int i = 0; i < mProvinces.size(); i++) {
			List<OfflineMapCity> tempList = mProvinces.get(i).getCityList();
			for (int j = 0; j < tempList.size(); j++) {
				if(tempList.get(j).getState() == OfflineMapStatus.DOWNLOADING
						|| tempList.get(j).getState() == OfflineMapStatus.WAITING 
						|| tempList.get(j).getState() == OfflineMapStatus.UNZIPING
						|| tempList.get(j).getState() == OfflineMapStatus.PAUSE
						|| tempList.get(j).getState() == OfflineMapStatus.NEW_VERSION){
					OfflineMapCity offlineMapCity = tempList.get(j);
					if(offlineMapCity instanceof OfflineMapAbroadCity){
						
						retList.add((OfflineMapAbroadCity) offlineMapCity);
					}
				}
			}
		}
		return retList;
	}

	/**
	 * 所有正在下载或等待下载离线地图的省份列表。
	 * @return 正在下载的省份列表
	 */
	public List<OfflineMapProvince> getDownloadingProvinceList() {
		if(mProvinces == null || mProvinces.isEmpty()){
			getInternalProvinceList();
			
		}
		if(mProvinces == null || mProvinces.isEmpty())
			return null;
		List<OfflineMapProvince> retList = new ArrayList<OfflineMapProvince>();
		for (int i = 0; i < mProvinces.size(); i++) {
			if (mProvinces.get(i).getState() == OfflineMapStatus.DOWNLOADING
					|| mProvinces.get(i).getState() == OfflineMapStatus.WAITING
					|| mProvinces.get(i).getState() == OfflineMapStatus.UNZIPING
					|| mProvinces.get(i).getState() == OfflineMapStatus.PAUSE
					|| mProvinces.get(i).getState() == OfflineMapStatus.NEW_VERSION) {
				if(!(mProvinces.get(i) instanceof OfflineMapAbroadCountry)){
					
					retList.add(mProvinces.get(i));
				}
			}
		}
		return retList;
	}
	
	/**
	 * 所有正在下载或等待下载离线地图的海外国家列表。
	 * @return 正在下载的海外国家列表
	 */
	public List<OfflineMapAbroadCountry> getDownloadingAbroadCountryList() {
		if(mProvinces == null || mProvinces.isEmpty()){
			getInternalProvinceList();
			
		}
		if(mProvinces == null || mProvinces.isEmpty())
			return null;
		List<OfflineMapAbroadCountry> retList = new ArrayList<OfflineMapAbroadCountry>();
		for (int i = 0; i < mProvinces.size(); i++) {
			if (mProvinces.get(i).getState() == OfflineMapStatus.DOWNLOADING
					|| mProvinces.get(i).getState() == OfflineMapStatus.WAITING
					|| mProvinces.get(i).getState() == OfflineMapStatus.UNZIPING
					|| mProvinces.get(i).getState() == OfflineMapStatus.PAUSE
					|| mProvinces.get(i).getState() == OfflineMapStatus.NEW_VERSION) {
				if((mProvinces.get(i) instanceof OfflineMapAbroadCountry)){
					
					retList.add((OfflineMapAbroadCountry) mProvinces.get(i));
				}
			}
		}
		return retList;
	}

	/**
	 * 返回已经下载完成离线地图的城市列表。
	 * @return 下载完成的城市列表
	 */
	public List<OfflineMapCity> getDownloadedCityList() {
		if(mProvinces == null || mProvinces.isEmpty()){
			getInternalProvinceList();
			
		}
		if(mProvinces == null || mProvinces.isEmpty())
			return null;
		List<OfflineMapCity> retList = new ArrayList<OfflineMapCity>();
		for (int i = 0; i < mProvinces.size(); i++) {
			List<OfflineMapCity> tempList = mProvinces.get(i).getCityList();
			for (int j = 0; j < tempList.size(); j++) {
				OfflineMapCity offlineMapCity = tempList.get(j);
				if(offlineMapCity.getState() == OfflineMapStatus.FINISHED && !(offlineMapCity instanceof OfflineMapAbroadCity)){
					retList.add(tempList.get(j));
				}
			}
		}
		return retList;
	}
	
	/**
	 * 所有下载完成的离线地图的海外城市列表。
	 * @return 下载完成的海外城市列表
	 */
	public List<OfflineMapAbroadCity> getDownloadedAbroadCityList() {
		if(mProvinces == null || mProvinces.isEmpty()){
			getInternalProvinceList();
			
		}
		if(mProvinces == null || mProvinces.isEmpty())
			return null;
		List<OfflineMapAbroadCity> retList = new ArrayList<OfflineMapAbroadCity>();
		for (int i = 0; i < mProvinces.size(); i++) {
			List<OfflineMapCity> tempList = mProvinces.get(i).getCityList();
			for (int j = 0; j < tempList.size(); j++) {
				OfflineMapCity offlineMapCity = tempList.get(j);
				if(offlineMapCity.getState() == OfflineMapStatus.FINISHED && (offlineMapCity instanceof OfflineMapAbroadCity)){
					retList.add((OfflineMapAbroadCity) offlineMapCity);
				}
			}
		}
		return retList;
	}

	/**
	 * 返回已经下载完成离线地图的省份列表。
	 * @return 下载完成的省份列表
	 */
	public List<OfflineMapProvince> getDownloadedProvinceList() {
		if(mProvinces == null || mProvinces.isEmpty()){
			getInternalProvinceList();
			
		}
		if(mProvinces == null || mProvinces.isEmpty())
			return null;
		List<OfflineMapProvince> retList = new ArrayList<OfflineMapProvince>();
		for (int i = 0; i < mProvinces.size(); i++) {
			OfflineMapProvince offlineMapProvince = mProvinces.get(i);
			if (offlineMapProvince.getState() == OfflineMapStatus.FINISHED && !(offlineMapProvince instanceof OfflineMapAbroadCountry)) {
				retList.add(mProvinces.get(i));
			}
		}
		return retList;
	}
	
	/**
	 * 返回已经下载完成离线地图的海外国家列表。
	 * @return 下载完成的海外国家列表
	 */
	public List<OfflineMapAbroadCountry> getDownloadedAbroadCountyList() {
		if(mProvinces == null || mProvinces.isEmpty()){
			getInternalProvinceList();
			
		}
		if(mProvinces == null || mProvinces.isEmpty())
			return null;
		List<OfflineMapAbroadCountry> retList = new ArrayList<OfflineMapAbroadCountry>();
		for (int i = 0; i < mProvinces.size(); i++) {
			OfflineMapProvince offlineMapProvince = mProvinces.get(i);
			if (offlineMapProvince.getState() == OfflineMapStatus.FINISHED && (offlineMapProvince instanceof OfflineMapAbroadCountry)) {
				retList.add((OfflineMapAbroadCountry) offlineMapProvince);
			}
		}
		return retList;
	}

	/**
	 * 获取所有存在有离线地图的国内城市列表。
	 * @return 离线地图国内城市列表
	 */
	public List<OfflineMapCity> getCityList() {
		if(mProvinces == null || mProvinces.isEmpty()){
			getInternalProvinceList();
			
		}
		if(mProvinces == null || mProvinces.isEmpty())
			return null;
		List<OfflineMapCity> retList = new ArrayList<OfflineMapCity>();
		for (int i = 0; i < mProvinces.size(); i++) {
			List<OfflineMapCity> tempList = mProvinces.get(i).getCityList();
			for (int j = 0; j < tempList.size(); j++) {
				OfflineMapCity offlineMapCity = tempList.get(j);
				if(!(offlineMapCity instanceof OfflineMapAbroadCity)){
					
					retList.add(offlineMapCity);
				}
			}
		}
		return retList;
	}
	
	/**
	 * 获取所有存在有离线地图的海外城市列表。
	 * @return 离线地图海外城市列表
	 */
	public List<OfflineMapAbroadCity> getAbroadCityList() {
		if(mProvinces == null || mProvinces.isEmpty()){
			getInternalProvinceList();
			
		}
		if(mProvinces == null || mProvinces.isEmpty())
			return null;
		List<OfflineMapAbroadCity> retList = new ArrayList<OfflineMapAbroadCity>();
		for (int i = 0; i < mProvinces.size(); i++) {
			List<OfflineMapCity> tempList = mProvinces.get(i).getCityList();
			for (int j = 0; j < tempList.size(); j++) {
				OfflineMapCity offlineMapCity = tempList.get(j);
				if(offlineMapCity instanceof OfflineMapAbroadCity){
					
					retList.add((OfflineMapAbroadCity) offlineMapCity);
				}
			}
		}
		return retList;
	}

	public List<OfflineMapProvince> getInternalProvinceList() {
		//如果已经加载到内存就不再进行加载
		if(mProvinces != null && !mProvinces.isEmpty()){
			return mProvinces;
		} else {
			List<OfflineMapProvince> lstProvinces = getProvinceListFromJni();
			mProvinces = new ArrayList<OfflineMapProvince>();
			mProvinces.addAll(lstProvinces);
		}
		return mProvinces;
	}
	
	/**
	 * 获取所有存在有离线地图的省的列表。 同步方法
	 * @return 离线地图省份列表
	 */
	public List<OfflineMapProvince> getProvinceList(){
		if(mProvinces == null || mProvinces.isEmpty()){
			getInternalProvinceList();
			
		}
		if(mProvinces == null || mProvinces.isEmpty()){
			return null;
		} 
		List<OfflineMapProvince> retList = new ArrayList<OfflineMapProvince>();
		for (int i = 0; i < mProvinces.size(); i++) {
			OfflineMapProvince offlineMapProvince = mProvinces.get(i);
			if(!(offlineMapProvince instanceof OfflineMapAbroadCountry)){
				retList.add(offlineMapProvince);
			}
		}
		return retList;
	}
	
	/**
	 * 获取所有存在有离线地图的国外国家的列表
	 * @return 离线地图国外国家列表
	 */
	public List<OfflineMapAbroadCountry> getAbroadCountryList() {
		if(mProvinces == null || mProvinces.isEmpty()){
			getInternalProvinceList();
			
		}
		//如果已经加载到内存就不再进行加载
		if(mProvinces == null || mProvinces.isEmpty()){
			return null;
		} 
		
		List<OfflineMapAbroadCountry> retList = new ArrayList<OfflineMapAbroadCountry>();
		for (int i = 0; i < mProvinces.size(); i++) {
			OfflineMapProvince offlineMapProvince = mProvinces.get(i);
			if((offlineMapProvince instanceof OfflineMapAbroadCountry)){
				retList.add((OfflineMapAbroadCountry) offlineMapProvince);
			}
		}
		return retList;
	}
	
	/**
	 * 更新省份列表
	 * 版本更新后，会重新下载新的省份列表，需要同步更新mProvinces对象
	 */
	protected void updateProvinceList() {
		List<OfflineMapProvince> newProvinceList = getProvinceListFromJni();
		// 保留原对象，更新值
		for (int i = 0; i < mProvinces.size(); i++) {
			OfflineMapProvince province = mProvinces.get(i);
			for (int j = 0; j < newProvinceList.size(); j++) {
				OfflineMapProvince newProvince = newProvinceList.get(j);
				if (province.mCode == newProvince.mCode) {
					// 更新省份对象
					province.mState = newProvince.mState;
					province.mVersion = newProvince.mVersion;
					province.mComplete = newProvince.mComplete;
					province.mSize = newProvince.mSize;
					
					List<OfflineMapCity> citylist = province.getCityList();
					List<OfflineMapCity> newCitylist = newProvince.getCityList();
					for (int k = 0; k < citylist.size(); k++) {
						OfflineMapCity city = citylist.get(k);
						for (int l = 0; l < newCitylist.size(); l++) {
							OfflineMapCity newCity = newCitylist.get(l);
							if (city.mAdcode == newCity.mAdcode) {
								// 更新城市对象
								city.mState = newCity.mState;
								city.mComplete = newCity.mComplete;
								city.mVersion = newCity.mVersion;
								city.mFilesize = newCity.mFilesize;
								city.mUri = newCity.mUri;
								break;
							}
						}
					}
					break;
				}				
			}
		}
	}
	
	/**
	 * 从底层获取省份列表，只有在离线模块初始化和数据版本升级时才会调用
	 * @return 省份列表
	 */
	List<OfflineMapProvince> getProvinceListFromJni() {
		List<OfflineMapProvince> lstProvinces = new ArrayList<OfflineMapProvince>();
		ArrayList<ObjectInfo> pauseInfos = new ArrayList<ObjectInfo>();
		
		synchronized (DownloadInstance.OfflineDataLock) {			
			//获取allData
			ProvinceInfo alldata = MapJni.getNationdatainfo(CommonDefine.ALLDATA);
			
			//获取直辖市信息
			ArrayList<CityInfo> municipas = MapJni.GetAllMunicipality();
			ArrayList<CityInfo> xgamCitys = new ArrayList<CityInfo>();
			for (int i = 0; i < municipas.size(); i++) {
				if ((municipas.get(i).getName().contains("香港") || 
						municipas.get(i).getName().contains("澳门"))) {
					xgamCitys.add(municipas.get(i));
					municipas.remove(i);
					i--;
				}
			}
			//获取所有省的信息
			ArrayList<ProvinceInfo> provinces = MapJni.getProvincelist();
			
			//处理全国概要图
			OfflineMapProvince allDataProvince = infoToProvince(alldata);
			if (alldata.getAdCode() == CommonDefine.ALLDATA_CODE) {
				if (alldata.getState() == OfflineMapStatus.DOWNLOADING
						|| alldata.getState() == OfflineMapStatus.WAITING) {
					pauseInfos.add(alldata);
				}
				lstProvinces.add(allDataProvince);
			}
			
			//处理直辖市
			for (int i = 0; i < municipas.size(); i++) {
				if (municipas.get(i).getState() == OfflineMapStatus.DOWNLOADING
						|| municipas.get(i).getState() == OfflineMapStatus.WAITING) {
					pauseInfos.add(municipas.get(i));
				}
				lstProvinces.add(infoToProvince(cityInfoAddProvince(municipas.get(i))));
			}
			
			//添加所有省份列表
			for (int i = 0; i < provinces.size(); i++) {
				// 省份状态不是指的省份所有城市的状态，必须遍历城市分别处理
				if (provinces.get(i).getState() == OfflineMapStatus.DOWNLOADING 
						|| provinces.get(i).getState() == OfflineMapStatus.WAITING) {
					ArrayList<CityInfo> citys = provinces.get(i).getCitylist();
					for (int j = 0; j < citys.size(); j++) {
						CityInfo cityInfo = citys.get(j);
						if (cityInfo.getState() == OfflineMapStatus.DOWNLOADING
								|| cityInfo.getState() == OfflineMapStatus.WAITING) {
							pauseInfos.add(cityInfo);
						}
					}
				}
				lstProvinces.add(infoToProvince(provinces.get(i)));
			}
			
			//处理香港澳门
			for (int i = 0; i < xgamCitys.size(); i++) {
				if (xgamCitys.get(i).getState() == OfflineMapStatus.DOWNLOADING
						|| xgamCitys.get(i).getState() == OfflineMapStatus.WAITING) {
					pauseInfos.add(xgamCitys.get(i));
				}
				lstProvinces.add(infoToProvince(cityInfoAddProvince(xgamCitys.get(i))));
			}
			
			//将数据库中等待的增加至download中
			for (int i = 0; i < pauseInfos.size(); i++) {
				pause(pauseInfos.get(i).getName());
			}
		}
		return lstProvinces;
	}
	
	OfflineMapCity infoToCity(CityInfo cityInfo){
		OfflineMapCity retCity = null;
		if(cityInfo.getAdCode() >= ABROAD_ADCODE){
			retCity = new OfflineMapAbroadCity();
		}else{
			retCity = new OfflineMapCity();
		}
		retCity.mAdcode = cityInfo.getAdCode();
		retCity.mCode = cityInfo.getmCityCode();
		int complete = WebServiceUtil.getProportion(cityInfo.getDownloadsize(), cityInfo.getFilesize());
		retCity.mComplete = complete;
		retCity.mFilesize = cityInfo.getFilesize();
		retCity.mJianpin = cityInfo.getJianpin();
		retCity.mName = cityInfo.getName();
		retCity.mPinyin = cityInfo.getPinyin();
		retCity.mState = cityInfo.getState();
		retCity.mUri = cityInfo.getUrl();
		retCity.mVersion = cityInfo.getVersion();
		return retCity;
	}
	
	ProvinceInfo cityInfoAddProvince(CityInfo cityInfo) {
		ArrayList<CityInfo> citys = new ArrayList<CityInfo>();
		citys.add(cityInfo);
		ProvinceInfo info = new ProvinceInfo(cityInfo.getName(),
				cityInfo.getPinyin(), cityInfo.getJianpin(), cityInfo.getUrl(),
				citys, Integer.valueOf(cityInfo.getmCityCode()),
				Integer.valueOf(cityInfo.getVersion()), cityInfo.getState(),
				cityInfo.getDownloadsize(), cityInfo.getFilesize(),
				cityInfo.getType());
		return info;
	}

	OfflineMapProvince infoToProvince(ProvinceInfo provinceInfo) {
		OfflineMapProvince retProvince = null;
		int proAdCode = provinceInfo.getAdCode();
		if(proAdCode >= ABROAD_ADCODE){
			retProvince = new OfflineMapAbroadCountry();
		}else{
			retProvince = new OfflineMapProvince();
		}
		retProvince.mCode = provinceInfo.getAdCode();
		retProvince.mComplete = WebServiceUtil.getProportion(provinceInfo.getDownloadsize(), provinceInfo.getFilesize());
		retProvince.mJianpin = provinceInfo.getJianpin();
		retProvince.mName = provinceInfo.getName();
		retProvince.mPinyin = provinceInfo.getPinyin();
		retProvince.mSize = provinceInfo.getFilesize();
		retProvince.mState = provinceInfo.getState();
		retProvince.mUrl = provinceInfo.getUrl();
		retProvince.mVersion = provinceInfo.getVersion();
		ArrayList<OfflineMapCity> citys = new ArrayList<OfflineMapCity>();
		ArrayList<CityInfo> tempCitys = provinceInfo.getCitylist();
		for (int i = 0; i < provinceInfo.getCitylist().size(); i++) {
			citys.add(infoToCity(tempCitys.get(i)));
		}
		retProvince.mCitylist = citys;
		return retProvince;
	}

	/**
	 * 根据给定的城市名称或省份名称，暂停对应的城市或省份的离线地图包。
	 * 如果输入的是省份名称，等同于开始暂停该省份下的所有城市。
	 * @param name 城市或省份的名称
	 */
	public void pause(String name) {
		dInstance.pause(name);
	}

	/**
	 * 根据给定的城市名或省份名称删除对应的城市或省份的离线地图包，异步方法，触发回调onRemove方法。
	 * 如果输入的是省份名称，等同于删除该省份下的所有城市。
	 * @param name 城市或省份的名称
	 */
	public void remove(String name) {
		long startTime = System.currentTimeMillis();
		dInstance.remove(name);
		long endTime = System.currentTimeMillis();
		Log.i(TAG, " timeMillis:" + (endTime - startTime));
	}
	
	/**
	 * 判断传入的城市名称或省份名称是否有更新的离线数据包，异步方法，触发回调onCheckUpdate方法。
	 * 如果输入的是省份名称，等同于查询载该省份下的所有城市的更新状态。
	 * @param name 城市名称或省份名称
	 */
	public void update(String name) {
		dInstance.update(name);
	}

	/**
	 * 销毁offlineManager中的资源
	 */
	public void destroy() {
		dInstance.onDestroy();
	}

	/**
	 * 离线地图下载监听类。
	 */
	public static interface OfflineMapDownloadListener {
		/**
		 * 当调用{@link updateOfflineCityByCode}、{@link updateOfflineCityByCode}或{@link updateOfflineMapProvinceByName}时候会被调用
		 * @param hasNew true表示有更新，说明有新版或者本地未下载
		 * @param name 被检测更新的城市的名字
		 */
		void onCheckUpdate(boolean hasNew, String name);

		/**
		 * 下载状态回调，在调用downloadByCityName等下载方法的时候会启动
		 * @param status 下载状态，参见{@link OfflineMapStatus}
		 * @param completeCode 下载进度，下载完成之后为解压进度
		 * @param name 当前所下载的城市的名字
		 */
		void onDownload(int status, int completeCode, String name);

		/**
		 * 当调用OfflineMapManager.remove(String)方法时，会回调此方法，当删除省份时，该方法会被调用多次，返回省份内城市删除情况。
		 * @param success 是否成功删除
		 * @param name 所删除的城市的名字
		 * @param describe 删除描述，如 删除成功 "本地无数据"
		 */
		void onRemove(boolean success, String name,
				String describe);
	}
	class OmmHandler extends Handler {
		public void handleMessage(Message msg) {
			Bundle bundle = msg.getData();
			if (bundle != null) {
				ObjectInfo info = (ObjectInfo) bundle.get("info");
				switch (msg.what) {
				case ON_DOWNLOAD:
					updateInfoByAdCode(info);
					OnDownload(info);
					break;
				case ON_REMOVE:
					updateInfoByAdCode(info);
					onRemove(info);
					break;
				case ON_UPDATE:
					onUpdate(info);
					break;
				default:
					break;
				}
			}
		}

		private void onUpdate(ObjectInfo info) {
			if (mListener == null) {
				return;
			}
			int status = info.getState();
			String name = info.getName();
			if (status == OfflineMapStatus.NEW_VERSION) {
				mListener.onCheckUpdate(true, name);
			} else {
				mListener.onCheckUpdate(false, name);
			}
		}

		private void OnDownload(ObjectInfo info) {
			if (mListener == null) {
				return;
			}
			int status = info.getState();
			int completeCode = WebServiceUtil.getProportion(info.getDownloadsize(), info.getFilesize());
			String name = info.getName();
			mListener.onDownload(status, completeCode, name);
		}

		private void onRemove(ObjectInfo info) {
			if (mListener == null) {
				return;
			}
			boolean isRemove = false;
			if (info.getState() == OfflineMapStatus.NONE) {
				isRemove = true;
			}
			if (isRemove) {
//				Log.i(TAG, " remove name :"+info.getName()+" downloadSize:"+info.getDownloadsize());
				mListener.onRemove(isRemove, info.getName(), "删除成功");
			} else {
				mListener.onRemove(isRemove, info.getName(), "本地无数据无法执行删除");
			}
		}
	}
}
