package com.cmmap.api.maps.offlinemap;

import java.util.ArrayList;
import java.util.List;

import com.cmmap.internal.services.rtic.ByteHelper;

/**
 * 下载省属性的相关类
 */
public class OfflineMapProvince {

	String mName;
	int mCode;
	String mJianpin;
	String mPinyin;
	ArrayList<OfflineMapCity> mCitylist;
	String mUrl;
	long mSize;
	int mState;
	int mVersion;
	int mComplete;

	public OfflineMapProvince() {
	}

	/**
	 * 返回当前省下所有的城市。
	 * 
	 * @return 当前省下所有的城市
	 */
	public List<OfflineMapCity> getCityList() {
		return mCitylist;
	}

	/**
	 * 返回省下载状态。
	 * 
	 * @return 省下载状态
	 */
	public int getState() {
		return mState;
	}

	/**
	 * 返回省下载完成的百分比，100表示下载完成。
	 * 
	 * @return 省下载完成的百分比
	 */
	public int getCompleteCode() {
		return mComplete;
	}

	/**
	 * 返回下载省数据的大小，单位：字节。
	 * 
	 * @return 下载省数据的大小
	 */
	public long getSize() {
		return mSize;
	}

	/**
	 * 返回所下载省的数据地址。
	 * 
	 * @return 下载省的数据地址
	 */
	public String getUrl() {
		return mUrl;
	}

	/**
	 * 返回下载省的数据版本。
	 * 
	 * @return 下载省的数据版本
	 */
	public String getVersion() {
		return ByteHelper.getVersionByInt(mVersion);
	}

	/**
	 * 返回省名称简拼拼音。
	 * 
	 * @return 省名称简拼拼音
	 */
	public String getJianpin() {
		return mJianpin;
	}

	/**
	 * 返回省名称拼音。
	 * 
	 * @return 省名称拼音
	 */
	public String getPinyin() {
		return mPinyin;
	}

	/**
	 * 得到省的行政编码。
	 * 
	 * @return 省的行政编码
	 */
	public String getProvinceCode() {
		if (mCode <= 0) {
			return new String("000000");
		}
		
		int adCode = mCode;
		while (adCode < 100000) {
			adCode *= 10;
		}
		return String.format("%d", adCode);
	}

	/**
	 * 返回省名称。
	 * 
	 * @return 省名称
	 */
	public String getName() {
		return mName;
	}

	@Override
	public String toString() {
		return String.format("{name=%s,jianpin=%s,pinyin=%s}", getName(),
				getJianpin(), getPinyin());
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result
				+ ((mCitylist == null) ? 0 : mCitylist.hashCode());
		result = prime * result + mCode;
		result = prime * result + mComplete;
		result = prime * result
				+ ((mJianpin == null) ? 0 : mJianpin.hashCode());
		result = prime * result + ((mName == null) ? 0 : mName.hashCode());
		result = prime * result + ((mPinyin == null) ? 0 : mPinyin.hashCode());
		result = prime * result + (int) (mSize ^ (mSize >>> 32));
		result = prime * result + mState;
		result = prime * result + ((mUrl == null) ? 0 : mUrl.hashCode());
		result = prime * result + mVersion;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		OfflineMapProvince other = (OfflineMapProvince) obj;
		if (mCitylist == null) {
			if (other.mCitylist != null)
				return false;
		} else if (!mCitylist.equals(other.mCitylist))
			return false;
		if (mCode != other.mCode)
			return false;
		if (mComplete != other.mComplete)
			return false;
		if (mJianpin == null) {
			if (other.mJianpin != null)
				return false;
		} else if (!mJianpin.equals(other.mJianpin))
			return false;
		if (mName == null) {
			if (other.mName != null)
				return false;
		} else if (!mName.equals(other.mName))
			return false;
		if (mPinyin == null) {
			if (other.mPinyin != null)
				return false;
		} else if (!mPinyin.equals(other.mPinyin))
			return false;
		if (mSize != other.mSize)
			return false;
		if (mState != other.mState)
			return false;
		if (mUrl == null) {
			if (other.mUrl != null)
				return false;
		} else if (!mUrl.equals(other.mUrl))
			return false;
		if (mVersion != other.mVersion)
			return false;
		return true;
	}
}
