package com.cmmap.internal.driver.base;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import android.content.Context;
import android.content.res.AssetManager;
import android.os.Environment;
import android.util.Log;

/**
 * [internal] 初始化时操作数据
 * 
 * @author liangz
 * 
 */
public class DataOperator {
	// [internal] 初始化拷贝数据的信息比对；是发布配置数据时间
    // [internal]  如果Data目录下配置文件发生变化，修改此处
	static final byte[] VERSION = { 2, 0, 2, 0, 2, 2, 2, 9, 0 };
	static String s_file_dir = "";
	static String s_cache_dir = "";

	static String[] ASSET_CONFIG = {};
	static String[] ASSET_FONT = {};
	static String[] ASSET_HMI = {};
	static String[] ASSET_MD = {};
	static String[] ASSET_WINFONT = {};

	private static String m_s_SDKVersion = null;

	// 扩展数组长度
	private static String[] copyOf(String[] rets, int length) {
		String[] values = new String[length];
		for (int i = 0; i < rets.length; i++) {
			values[i] = rets[i];
		}
		return values;
	}

	private static String[] mergeData(String[]... datas) {
		String[] rets = new String[] {};
		for (int i = 0; i < datas.length; i++) {
			int length1 = rets.length;
			int length2 = datas[i].length;
			rets = copyOf(rets, length1 + length2);
			System.arraycopy(datas[i], 0, rets, length1, length2);
		}
		return rets;
	}

	private static String[] addPath(String[] datas, String path) {
		for (int i = 0; i < datas.length; i++) {
			datas[i] = path + datas[i];
		}
		return datas;
	}

	public static String getPath2() {
		String sdcard_path = null;
		String sd_default = Environment.getExternalStorageDirectory()
				.getAbsolutePath();
		if (sd_default.endsWith("/")) {
			sd_default = sd_default.substring(0, sd_default.length() - 1);
		}
		// 得到路径
		try {
			Runtime runtime = Runtime.getRuntime();
			Process proc = runtime.exec("mount");
			InputStream is = proc.getInputStream();
			InputStreamReader isr = new InputStreamReader(is);
			String line;
			BufferedReader br = new BufferedReader(isr);
			while ((line = br.readLine()) != null) {
				if (line.contains("secure"))
					continue;
				if (line.contains("asec"))
					continue;
				if (line.contains("fat") && line.contains("/mnt/")) {
					String columns[] = line.split(" ");
					if (columns != null && columns.length > 1) {
						if (sd_default.trim().equals(columns[1].trim())) {
							continue;
						}
						sdcard_path = columns[1];
					}
				} else if (line.contains("fuse") && line.contains("/mnt/")) {
					String columns[] = line.split(" ");
					if (columns != null && columns.length > 1) {
						if (sd_default.trim().equals(columns[1].trim())) {
							continue;
						}
						sdcard_path = columns[1];
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return sdcard_path;
	}

	public static List<String> getExtSDCardPath() {
		List<String> lResult = new ArrayList<String>();
		try {
			Runtime rt = Runtime.getRuntime();
			Process proc = rt.exec("mount");
			InputStream is = proc.getInputStream();
			InputStreamReader isr = new InputStreamReader(is);
			BufferedReader br = new BufferedReader(isr);
			String line;
			while ((line = br.readLine()) != null) {
				if (line.contains("extSdCard")) {
					String[] arr = line.split(" ");
					String path = arr[1];
					File file = new File(path);
					if (file.isDirectory()) {
						lResult.add(path);
					}
				}
			}
			isr.close();
		} catch (Exception e) {
		}
		return lResult;
	}

	public static void initialize(Context context) {
		s_file_dir = context.getFilesDir().getAbsolutePath()
				+ "/cmmap/";
		s_cache_dir = context.getCacheDir().getAbsolutePath()
				+ "/cmmap/";
		try {
			File file = new File(s_file_dir);
			if (!file.exists()) {
				file.mkdirs();
			}
		} catch (Exception e) {
			e.printStackTrace();
			return;
		}

		int i_init = 1;
		InputStream is;
		// [internal] 文件输出流
		FileOutputStream os_file;
		// [internal] 文件输入流
		FileInputStream is_file;
		// [internal] 数据流
		byte[] b_data;
		// [internal] 文件
		File file;
		try {
			file = new File(s_file_dir + "ver.dat");
			if (file.exists()) {
				// [internal] 比对数据版本文件
				is_file = new FileInputStream(file);
				b_data = new byte[is_file.available()];
				is_file.read(b_data);
				is_file.close();

				if (Arrays.equals(b_data, VERSION)) {			
					i_init = 0;
					return;
				}
			}

			if (i_init != 0) {
				// [internal] 创建数据版本文件
				if (file.exists()) {
					file.delete();
				}
				file.createNewFile();
				os_file = new FileOutputStream(file);
				os_file.write(VERSION);
				os_file.close();

				file = new File(s_file_dir + "config/sql");
				if (!file.exists()) {
					file.mkdirs();
				}
				// [internal] font文件夹
				file = new File(s_file_dir + "font");
				if (!file.exists()) {
					file.mkdirs();
				}
				// [internal] hmi文件夹
				file = new File(s_file_dir + "hmi");
				if (!file.exists()) {
					file.mkdirs();
				}
				// [internal] winfont文件夹
				file = new File(s_file_dir + "winfont");
				if (!file.exists()) {
					file.mkdirs();
				}
				// [internal] md/res文件夹
				file = new File(s_file_dir + "md/res");
				if (!file.exists()) {
					file.mkdirs();
				}
				// [internal] md/theme0文件夹
				file = new File(s_file_dir + "md/theme0");
				if (!file.exists()) {
					file.mkdirs();
				}
				// [internal] md/theme1文件夹
				file = new File(s_file_dir + "md/theme1");
				if (!file.exists()) {
					file.mkdirs();
				}
				// [internal] md/theme2文件夹
				file = new File(s_file_dir + "md/theme2");
				if (!file.exists()) {
					file.mkdirs();
				}
				// [internal] md/theme3文件夹
				file = new File(s_file_dir + "md/theme3");
				if (!file.exists()) {
					file.mkdirs();
				}
				// [internal] md/theme4文件夹
                file = new File(s_file_dir + "md/theme4");
                if (!file.exists()) {
                    file.mkdirs();
                }
                // [internal] md/theme5文件夹
                file = new File(s_file_dir + "md/theme5");
                if (!file.exists()) {
                    file.mkdirs();
                }
                // [internal] md/theme6文件夹
				file = new File(s_file_dir + "md/theme6");
                if (!file.exists()) {
                    file.mkdirs();
                }
				// [internal] md/landmark文件夹
				file = new File(s_file_dir + "md/landmark");
				if (!file.exists()) {
					file.mkdirs();
				}
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		if (i_init != 0) {
			AssetManager assetManager = context.getAssets();
			try {
				
				String[] configs = addPath(assetManager.list("cmmap/config"), "config/");
				String[] config_sqls = addPath(assetManager.list("cmmap/config/sql"),
						"config/sql/");
				ASSET_CONFIG = mergeData(configs, config_sqls);

				ASSET_FONT = addPath(assetManager.list("cmmap/font"), "font/");

				ASSET_HMI = addPath(assetManager.list("cmmap/hmi"), "hmi/");
				ASSET_WINFONT = addPath(assetManager.list("cmmap/winfont"), "winfont/");;

				String[] md = addPath(assetManager.list("cmmap/md"), "md/");
				String[] md_landmark = addPath(assetManager.list("cmmap/md/landmark"),
						"md/landmark/");
				String[] md_res = addPath(assetManager.list("cmmap/md/res"), "md/res/");
				String[] md_theme0 = addPath(assetManager.list("cmmap/md/theme0"),
						"md/theme0/");
				String[] md_theme1 = addPath(assetManager.list("cmmap/md/theme1"),
						"md/theme1/");
				String[] md_theme2 = addPath(assetManager.list("cmmap/md/theme2"),
						"md/theme2/");
				String[] md_theme3 = addPath(assetManager.list("cmmap/md/theme3"),
						"md/theme3/");
				String[] md_theme4 = addPath(assetManager.list("cmmap/md/theme4"),
                        "md/theme4/");
				String[] md_theme5 = addPath(assetManager.list("cmmap/md/theme5"),
                        "md/theme5/");
				String[] md_theme6 = addPath(assetManager.list("cmmap/md/theme6"), 
                        "md/theme6/");
				ASSET_MD = mergeData(md, md_landmark, md_res, md_theme0, md_theme1,
				md_theme2, md_theme3, md_theme4, md_theme5, md_theme6);
				
				// [internal]拷贝config数据到file文件夹中
				for (int i = 0; i < ASSET_CONFIG.length; i++) {

					file = new File(s_file_dir + ASSET_CONFIG[i]);

					if (file.exists()) {
						if (file.isDirectory())
							continue;
						file.delete();
					}

					is = assetManager.open("cmmap/"+ASSET_CONFIG[i]);
					b_data = new byte[is.available()];
					is.read(b_data);
					is.close();

					file.createNewFile();
					os_file = new FileOutputStream(file);
					os_file.write(b_data);
					os_file.close();
				}

				// [internal]拷贝font数据到file文件夹中
				for (int i = 0; i < ASSET_FONT.length; i++) {
					file = new File(s_file_dir + ASSET_FONT[i]);
					if (file.exists()) {
						if (file.isDirectory())
							continue;
						file.delete();
					}
					is = assetManager.open("cmmap/"+ASSET_FONT[i]);
					b_data = new byte[is.available()];
					is.read(b_data);
					is.close();

					file.createNewFile();
					os_file = new FileOutputStream(file);
					os_file.write(b_data);
					os_file.close();
				}
				// [internal]拷贝hmi数据到file文件夹中
				for (int i = 0; i < ASSET_HMI.length; i++) {
					file = new File(s_file_dir + ASSET_HMI[i]);
					if (file.exists()) {
						if (file.isDirectory())
							continue;
						file.delete();
					}
					is = assetManager.open("cmmap/"+ASSET_HMI[i]);
					b_data = new byte[is.available()];
					is.read(b_data);
					is.close();

					file.createNewFile();
					os_file = new FileOutputStream(file);
					os_file.write(b_data);
					os_file.close();
				}
				// [internal]拷贝winfont数据到file文件夹中
				for (int i = 0; i < ASSET_WINFONT.length; i++) {
					file = new File(s_file_dir + ASSET_WINFONT[i]);
					if (file.exists()) {
						if (file.isDirectory())
							continue;
						file.delete();
					}
					is = assetManager.open("cmmap/"+ASSET_WINFONT[i]);
					b_data = new byte[is.available()];
					is.read(b_data);
					is.close();

					file.createNewFile();
					os_file = new FileOutputStream(file);
					os_file.write(b_data);
					os_file.close();
				}
				// [internal]拷贝md数据到file文件夹中
				for (int i = 0; i < ASSET_MD.length; i++) {
					file = new File(s_file_dir + ASSET_MD[i]);
					if (file.exists()) {
						if (file.isDirectory())
							continue;
						file.delete();
					}
					is = assetManager.open("cmmap/"+ASSET_MD[i]);
					b_data = new byte[is.available()];
					is.read(b_data);
					is.close();

					file.createNewFile();
					os_file = new FileOutputStream(file);
					Log.i("dataOperator", file.getName());
					os_file.write(b_data);
					os_file.close();
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	public static String getFileDir() {
		return s_file_dir;
	}

	/**
	 * 获取SDK版本号
	 * @param context 应用的上下文
	 * @return SDK版本号
	 */
	public static String getSDKVersion(Context context) {
		if (m_s_SDKVersion  == null) {
			// 文件中读取SDK版本号
			if (context != null) {
				AssetManager assetManager = context.getAssets();
				InputStream is = null;
				try {
					is = assetManager.open("cmmap/config/SdkVersion.txt");
					if (is != null) {						
						byte[] b_data = new byte[is.available()];
						is.read(b_data);
						is.close();	
						m_s_SDKVersion = new String(b_data, "UTF8");
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		}
		return m_s_SDKVersion;
	}
}
