package com.cmmap.internal.driver.base;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import android.content.Context;
import android.content.res.Configuration;
import android.os.Handler;
import android.os.Message;
import android.util.DisplayMetrics;
import android.util.Log;

import com.cmmap.internal.driver.gesture.MapGestureManager;
import com.cmmap.internal.mapcore.KHttpData;
import com.cmmap.internal.maps.KMapInitializer;

public class GlobalSettings {
	private static final String TAG = "GlobalSettings";
	private static final GlobalSettings INSTANCE = new GlobalSettings();
	private Context context;
	private Context applicationContext;
	private Handler mainThreadHandler;
	private float density;
	private float densityDpi;
	private int screenWidth;
	private int screenHeight;
	// [internal]网络数据回调消息
	public static final int MESSAGE_REFRESH = 30001;
	public static final int MESSAGE_NETWORK = 30002;
	public static final int MESSAGE_NETWORK_CLOSE = 30003;
	private ExecutorService threadPool;
	public static GlobalSettings getInstance() {
		return INSTANCE;
	}
	
	public void init(Context context) {
		if (context == null) {
			return;
		}
		this.context = context;
		this.applicationContext = context.getApplicationContext();
		
		DisplayMetrics metric = context.getResources().getDisplayMetrics();
		density = metric.density;
		densityDpi = (metric.xdpi + metric.ydpi) / 2;
		mainThreadHandler = new Handler() {
			@Override
			public void handleMessage(Message msg) {
				super.handleMessage(msg);
				switch (msg.what) {
				case MESSAGE_REFRESH:
					if(MapGestureManager.s_handler != null){
						MapGestureManager.s_handler.sendEmptyMessage(MapGestureManager.GL_REFRESH_MAP);
					}
					break;
				case MESSAGE_NETWORK:
					if (msg.obj != null && msg.obj instanceof KHttpData) {
						KHttpData data = (KHttpData) msg.obj;
						Log.i("download", "type=" + data.m_data_type +  "GlobalSettings .MESSAGE_NETWORK=" + data.m_str_url);						
						KMapInitializer.getInstance().post((KHttpData) msg.obj);
						//Lg.e("zhuliang", "GlobalSettings Handler end");
					}
					break;
				case MESSAGE_NETWORK_CLOSE:
					if (msg.obj != null && msg.obj instanceof KHttpData) {
						KHttpData data = (KHttpData) msg.obj;
						Log.e("download", "GlobalSettings mainThreadHandler.MESSAGE_NETWORK_CLOSE=" + data.m_str_url);
						KMapInitializer.getInstance().stop((KHttpData) msg.obj);
					}
					break;
				}
			}
		};
		threadPool = Executors.newFixedThreadPool(20);
		screenWidth = context.getResources().getDisplayMetrics().widthPixels;
		screenHeight = context.getResources().getDisplayMetrics().heightPixels;
	}

	// [internal] 获取屏幕的方向
	public int getScreenOrientation() {
		if (this.context.getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
			return Configuration.ORIENTATION_LANDSCAPE;
		} else if (this.context.getResources().getConfiguration().orientation == Configuration.ORIENTATION_PORTRAIT) {
			return Configuration.ORIENTATION_PORTRAIT;
		} else {
			return Configuration.ORIENTATION_PORTRAIT;
		}
	}

	public Handler getMainThreadHandler() {
		return this.mainThreadHandler;
	}

	public ExecutorService getWorkThreadPool(){
		return this.threadPool;
	}
	public Context getContext() {
		if (context == null) {
			return null;
		}
		return this.context;
	}
	
	public Context getApplicationContext() {
		return this.applicationContext;
	}

	public float getDensity() {
		return this.density;
	}

	public float getDensityDpi() {
		return this.densityDpi;
	}

	public int getScreenWidth() {
		return screenWidth;
	}

	public int getScreenHeight() {
		return screenHeight;
	}

}
