package com.cmmap.internal.driver.base;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.SocketTimeoutException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.ByteBuffer;

import org.apache.commons.httpclient.HttpClient;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.HttpVersion;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.conn.tsccm.ThreadSafeClientConnManager;
import org.apache.http.params.BasicHttpParams;
import org.apache.http.params.HttpConnectionParams;
import org.apache.http.params.HttpProtocolParams;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;

import android.annotation.SuppressLint;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.Log;

import com.cmmap.internal.mapcore.ByteUlit;
import com.cmmap.internal.mapcore.KGEOCOORD;
import com.cmmap.internal.mapcore.KHttpData;
import com.cmmap.internal.mapcore.KInt;
import com.cmmap.internal.mapcore.MapCore;
import com.cmmap.internal.mapcore.MapJni;

/**
 * [internal] Http协议网络数据获取工具类 
 * 
 * @author yil
 */
public class HttpTool {
	/**
	 * [internal] 创建HTTP连接 
	 * 
	 * @return
	 */
	private HttpClient createHttpClient() {
		return new HttpClient();
	}

	/**
	 * [internal] 获取HTTP连接
	 * 
	 * @return
	 */
	public HttpClient getHttpClient() {
		return createHttpClient();
	}
	
	public org.apache.http.client.HttpClient getAndroidHttpClient()
	{
		org.apache.http.params.HttpParams params = new BasicHttpParams();
		HttpProtocolParams.setVersion(params, HttpVersion.HTTP_1_1);
		HttpProtocolParams.setContentCharset(params, HTTP.UTF_8);
		HttpProtocolParams.setUseExpectContinue(params, true);
		// [internal] Set connection timeout
		HttpConnectionParams.setSoTimeout(params, 1 * 10 * 1000);
		HttpConnectionParams.setConnectionTimeout(params, 1 * 10 * 1000);
		// [internal] Creates the scheme for manager
		SchemeRegistry schReg = new SchemeRegistry();
		schReg.register(new Scheme("http", PlainSocketFactory.getSocketFactory(), 80));
		schReg.register(new Scheme("https", SSLSocketFactory.getSocketFactory(), 443));
		// [internal] Creates a management for client connections.
		ClientConnectionManager conMgr = new ThreadSafeClientConnManager(params, schReg);
		// [internal] Creates a new HTTP client from parameters and a connection
		// manager.
		return new DefaultHttpClient(conMgr, params);
	}

	/**
	 * [internal] 执行Get请求 Get
	 * 
	 * @param url [internal] 数据请求URL地址
	 * @return [internal] 返回数据字节数组
	 * @throws URISyntaxException
	 * @throws ClientProtocolException
	 * @throws IOException
	 */
	@SuppressLint("NewApi")
	public byte[] executeHttpGet(String url, int width, int height, byte[] b_body) throws Exception {

		byte[] byBuf = null;
		// [internal] 创建GET请求
		HttpGet request = new HttpGet();
		org.apache.http.client.HttpClient httpClient = getAndroidHttpClient();
		try {
			request.setURI(new URI(url));
			// [internal] 发送GET请求获取响应内容
			HttpResponse response = httpClient.execute(request);

			Log.i("HttpTool", "response status code : " + response.getStatusLine().getStatusCode());
			if (response.getStatusLine().getStatusCode() == 200) {

				HttpEntity entity = response.getEntity();

				// [internal] 判断请求是否成功
				if (entity != null) {

					byBuf = EntityUtils.toByteArray(entity);
					Log.i("byBuf", "byBuf:"+byBuf.length);
					Bitmap bitmap1 = decodeSampledBitmapFromByte(byBuf,0,width,height);
					if (bitmap1 == null) {
						httpClient.getConnectionManager().shutdown();
						return null;
					}
					Bitmap bitmap = bitmap1.copy(Bitmap.Config.ARGB_8888, true);
					
					ByteBuffer buffer = ByteBuffer.allocate(bitmap.getByteCount());
					bitmap.copyPixelsToBuffer(buffer);
					
					byte [] pic_buf = buffer.array();
					int len = pic_buf.length;
					
					byBuf = new byte [len + b_body.length];
					System.arraycopy(b_body, 0, byBuf, 0, b_body.length);
					System.arraycopy(pic_buf, 0, byBuf, b_body.length, len);
					bitmap1.recycle();
					bitmap.recycle();
				}
			}
		} catch (ClientProtocolException e) {
			throw e;
		} catch (SocketTimeoutException e) {
			throw e;
		} catch (IOException e) {
			throw e;
		} finally {
			if(httpClient != null && httpClient.getConnectionManager() != null)
			{
				httpClient.getConnectionManager().shutdown();
			}
		}

		return byBuf;
	}

	public static Bitmap decodeSampledBitmapFromByte(byte[] imgBytes,int offset,int reqWidth,int reqHeight) {
		final BitmapFactory.Options options = new BitmapFactory.Options();
		options.inJustDecodeBounds = true;
		BitmapFactory.decodeByteArray(imgBytes, offset, imgBytes.length,
				options);
		options.inSampleSize = calculateInSampleSize(options, reqWidth, reqHeight);
		options.inJustDecodeBounds = false;
		return BitmapFactory.decodeByteArray(imgBytes, offset, imgBytes.length,options);
	}

	private static int calculateInSampleSize(BitmapFactory.Options options,int reqWidth,int reqHeight) {
		final int height = options.outHeight;
		final int width = options.outWidth;
		Log.i("byBuf", "height:"+height);
		Log.i("byBuf", "width:"+width);
		int inSampleSize = 1;
		if(height > reqHeight || width >reqWidth){
			final int halfHeight = height / 2;
			final int halfWidth = width / 2;
			while((halfHeight / inSampleSize) >= reqHeight && (halfWidth / inSampleSize) >= reqWidth){
				inSampleSize *= 2;
			}
		}
		return inSampleSize;
	}

	/**
	 * [internal] 执行post请求 
	 * 
	 * @param url
	 * @param b_body [internal] body字节码
	 * @return
	 * @throws Exception
	 */
	public byte[] executeHttpPost(String url, byte[] b_body) throws Exception {
		byte[] bReturn = new byte[0];
		if (MapCore.getNetworkStatus() != 0) {
			bReturn = GetMapData(url, false);
		}
		
		if (bReturn.length != 0) {
			byte[] b_content = new byte[bReturn.length + b_body.length];
			System.arraycopy(b_body, 0, b_content, 0, b_body.length);
			System.arraycopy(bReturn, 0, b_content, b_body.length, bReturn.length);
			
			bReturn = b_content;
		}
		return bReturn;
	}

	/**
	 * [internal] 关闭数据下载客户端连接 
	 */
	public void shutDownClient() {
		// if(download_httpClient.getConnectionManager() == null){
		// System.out.println("========================download_httpClient.getConnectionManager() null");
		// }
		// download_httpClient.
		//download_httpClient.getHttpConnectionManager().releaseConnection(download_httpClient.getHttpConnectionManager().getConnection(download_httpClient.getHostConfiguration()));
	}

	public static KHttpData encodeHttpData(byte[] buf) {
		KHttpData httpdata = new KHttpData();
		KInt iIndex = new KInt(0);
		httpdata.m_data_type = ((int)ByteUlit.getLong4(buf, iIndex));
		httpdata.m_Rect.iTop = ByteUlit.getLong4(buf, iIndex);
		httpdata.m_Rect.iLeft = ByteUlit.getLong4(buf, iIndex);
		httpdata.m_Rect.iBottom = ByteUlit.getLong4(buf, iIndex);
		httpdata.m_Rect.iRight = ByteUlit.getLong4(buf, iIndex);
		httpdata.m_iLevel = (int)ByteUlit.getLong4(buf, iIndex);
		httpdata.m_iDivIndex = (short) ByteUlit.getShort2(buf, iIndex);
		
		httpdata.m_str_url = ByteUlit.DerializeString(buf, iIndex);		
		httpdata.m_body = buf;
		
		return httpdata;
	}
	
	public static byte[] GetMapData(String strURL, boolean bGet)
	{
		Log.i("download", String.format("GetMapData URL=%s", strURL));
		KGEOCOORD crd = new KGEOCOORD();
		MapCore.GetMapCenter(crd);
		String strCity = String.format("%d,%d", crd.lLongitude, crd.lLatitude);
		String[] strDataLst = strURL.split("&");
		if (strURL.startsWith("https")) {
			return postHttpsData(strDataLst[0], strDataLst[1], strCity);
		}
		else {
			return getHttpData(strDataLst[0], strDataLst[1], strCity, bGet);
		}
	}

	public static byte[] getHttpData(String strURL, String strPara, String strCity, boolean bGet)
	{
		byte[] byBuf = new byte[0];
		try {
			URL url = new URL(strURL);	
			HttpURLConnection urlConn = (HttpURLConnection) url.openConnection();
			urlConn.setRequestMethod(bGet ? "GET" : "POST");
			urlConn.setConnectTimeout(3000);
			urlConn.setReadTimeout(10000);
			urlConn.setDoInput(true);
			urlConn.setDoOutput(true);
			urlConn.addRequestProperty("Content-type", "text/plain;charset=utf-8");
			urlConn.addRequestProperty("User-NetType", DeviceTool.GetNetworkType());
			urlConn.addRequestProperty("User-Version", MapCore.getSDKVersion());
			urlConn.addRequestProperty("User-Imei", DeviceTool.getIMEI());
			urlConn.addRequestProperty("User-Imsi", DeviceTool.getIMSI());
			urlConn.addRequestProperty("DeviceId", DeviceTool.getAndroidId());
            if (strCity != null)
            {
            	urlConn.addRequestProperty("city", strCity);
            }
			urlConn.getOutputStream().write(strPara.getBytes());
			
			int iCode = urlConn.getResponseCode();			
			if(iCode == HttpURLConnection.HTTP_OK)
			{
				byBuf = readFromInputStream(urlConn.getInputStream(), urlConn.getContentEncoding());
			}	
			else {
				byte[] bufError = HttpTool.readFromInputStream(urlConn.getErrorStream(), urlConn.getContentEncoding());
				String strError = new String(bufError, "UTF-8");
				Log.e("Network", "Http Code="+iCode + "!Error="+strError);
			}
			
			urlConn.disconnect();
		}
		catch (Exception e) {
			Log.e("download", String.format("GetMapData.url=%s.data=%s.error=%s", strURL, strPara,e.toString()));
		}
		Log.i("download", String.format("GetMapData.buf len=%d.url=%s.data=%s", byBuf.length, strURL, strPara));
		return byBuf;
	}

	//static boolean mIsHttpsInitialized = false;
	public static synchronized  boolean isHttpsInitialized() {
		return true;
//		if (!mIsHttpsInitialized) {
//			try {
//				HttpsPost.initHttpsURLConnection();
//				mIsHttpsInitialized = true;
//			} catch (Exception e) {
//				e.printStackTrace();
//			}
//		}
//		return mIsHttpsInitialized;
	}

	public static byte[] postHttpsData(String strURL, String strPara, String strCity)
	{
		//if (isHttpsInitialized()) {
			return HttpsPost.post(strURL, strPara, strCity);
		//}
		//return new byte[0];
	}

	public static byte[] readFromInputStream(InputStream in, String enc) throws IOException {
		byte[] byBuf = null;		

		if(enc != null && (enc.indexOf("gzip") != -1)) {
			in = new java.util.zip.GZIPInputStream(in);
		}
		ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        byte[] buffer = new byte[1024];
        int len = 0;
        while( (len = in.read(buffer)) !=-1 ){
            outStream.write(buffer, 0, len);
        }
        byBuf = outStream.toByteArray();
        outStream.close();
        in.close();
        return byBuf;
	}
}
