package com.cmmap.internal.driver.base;

import java.io.IOException;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.GeneralSecurityException;
import java.security.SecureRandom;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManager;

import android.util.Log;

import com.cmmap.internal.mapcore.MapCore;

public class HttpsPost {

    
    /** 
     * 获得SSLSocketFactory. 
     * @param password 
     *            密码 
     * @param keyStorePath 
     *            密钥库路径 
     * @param trustStorePath 
     *            信任库路径 
     * @return SSLSocketFactory 
     * @throws Exception 
     */  
    public static SSLContext getSSLContext() throws Exception {  
 
        // 实例化SSL上下文  
        SSLContext ctx = SSLContext.getInstance("TLS");  
        // 初始化SSL上下文  
        ctx.init( null,  new TrustManager[]{new KTrustManager()}, new SecureRandom());  
        // 获得SSLSocketFactory  
        return ctx;  
    }  
  
    /** 
     * 初始化HttpsURLConnection. 
     * @param password 
     *            密码 
     * @param keyStorePath 
     *            密钥库路径 
     * @param trustStorePath 
     *            信任库路径 
     * @throws Exception 
     */  
    public static void initHttpsURLConnection() throws Exception {  
        // 声明SSL上下文  
        SSLContext sslContext = null;  
        // 实例化主机名验证接口  
        HostnameVerifier hnv = new KHostnameVerifier();  
        try {  
            sslContext = getSSLContext();
        } catch (GeneralSecurityException e) {  
            e.printStackTrace();  
        }  
        if (sslContext != null) {  
            HttpsURLConnection.setDefaultSSLSocketFactory(sslContext  
                    .getSocketFactory());  
        }  
        HttpsURLConnection.setDefaultHostnameVerifier(hnv);  
    }  
  
    /** 
     * 发送请求. 
     * @param httpsUrl 
     *            请求的地址 
     * @param xmlStr 
     *            请求的数据 
     * @return 
     */  
    public static byte[] post(String httpsUrl, String xmlStr, String strCity) {  
        HttpsURLConnection urlCon = null;
		byte[] byBuf = new byte[0]; 
        try {
        	initHttpsURLConnection();
            urlCon = (HttpsURLConnection) (new URL(httpsUrl)).openConnection();
            urlCon.setRequestMethod("POST");
            //urlCon.setUseCaches(false);
            urlCon.setConnectTimeout(3000);
            urlCon.setReadTimeout(10000);
            urlCon.setDoInput(true);
            urlCon.setDoOutput(true);
            urlCon.addRequestProperty("Content-type", "text/plain;charset=utf-8");
            urlCon.addRequestProperty("User-NetType", DeviceTool.GetNetworkType());
            urlCon.addRequestProperty("User-Version", MapCore.getSDKVersion());
            urlCon.addRequestProperty("User-Imei", DeviceTool.getIMEI());
            urlCon.addRequestProperty("User-Imsi", DeviceTool.getIMSI());
            urlCon.addRequestProperty("DeviceId", DeviceTool.getAndroidId());
            if (strCity != null)
            {
            	urlCon.addRequestProperty("city", strCity);
            }
            //urlCon.getOutputStream().write(xmlStr.getBytes());
        	OutputStream os = urlCon.getOutputStream();
        	if (os != null) {
            	Log.i("download", "https post write");
            	os.write(xmlStr.getBytes());
            	os.flush();
            	os.close();				
			}
			
			int iCode = urlCon.getResponseCode();			
			if(iCode == HttpURLConnection.HTTP_OK)
			{
				byBuf = HttpTool.readFromInputStream(urlCon.getInputStream(), urlCon.getContentEncoding());
			}	
			else {
				byte[] bufError = HttpTool.readFromInputStream(urlCon.getErrorStream(), urlCon.getContentEncoding());
				String strError = new String(bufError, "UTF-8");
				Log.e("download", "Https Post Code="+iCode + "!Error="+strError);
			}
			urlCon.disconnect(); 
        } catch (MalformedURLException e) { 
        	Log.e("download", "https post MalformedURLException:" + Log.getStackTraceString(e));
        } catch (IOException e) { 
        	Log.e("download", "https post IOException:" + Log.getStackTraceString(e));
        } catch (Exception e) { 
        	Log.e("download", "https post Exception:" + Log.getStackTraceString(e));
        }
        return byBuf;
    }    
}
