package com.cmmap.internal.driver.base;

import android.graphics.Point;
import android.text.TextUtils;

import com.cmmap.internal.mapcore.KGEOCOORD;
import com.cmmap.internal.mapcore.KPoint;
import com.cmmap.internal.mapcore.MapCore;
import com.cmmap.internal.maps.model.KLatLng;

public final class MapTool {
	public static final int Map2DMode=0;

	public static final int Map3DMode=1;

	//[internal] 屏幕点转换到坐标
	public static KLatLng pointToLatlon(Point p_screen) {
		if (p_screen == null) {
			return null;
		}

		KPoint k_screen = new KPoint();
		k_screen.iX = (int) p_screen.x;
		k_screen.iY = (int) p_screen.y;
		KGEOCOORD k_coord = new KGEOCOORD();
		MapCore.ScreenPointToCoord(k_screen, k_coord);
		return new KLatLng(MapTool.longToDouble(k_coord.lLatitude),MapTool.longToDouble(k_coord.lLongitude));
	}

	//[internal] 坐标到屏幕点
	public static Point latlonToPoint(KLatLng location) {
		if (location == null) {
			return null;
		}
		KGEOCOORD k_coord = new KGEOCOORD();
		k_coord.lLatitude = MapTool.doubleToLong(location.latitude) ;
		k_coord.lLongitude = MapTool.doubleToLong(location.longitude);
		KPoint k_screen = new KPoint();
		MapCore.CoordToScreenPoint(k_coord, k_screen);
		return new Point(k_screen.iX, k_screen.iY);
	}

	//[internal] 地图主题String到int
	public static int[] styleStringToInt(String s_styleid) {
		int[] b_style = null;
		if (!TextUtils.isEmpty(s_styleid)) {
			b_style = new int[2];
			if (s_styleid.equals("DayBig")) {
				b_style[0] = 0;
				b_style[1] = 2;
			} else if (s_styleid.equals("DayMid")) {
				b_style[0] = 0;
				b_style[1] = 1;
			} else if (s_styleid.equals("DaySmall")) {
				b_style[0] = 0;
				b_style[1] = 0;
			} else if (s_styleid.equals("NightBig")) {
				b_style[0] = 1;
				b_style[1] = 2;
			} else if (s_styleid.equals("NightMid")) {
				b_style[0] = 1;
				b_style[1] = 1;
			} else if (s_styleid.equals("NightSmall")) {
				b_style[0] = 1;
				b_style[1] = 0;
			} else {
				b_style = null;
			}
		}
		return b_style;
	}

	//[internal] 引擎样式转换为tmi样式
	public static String styleIntToString(int type, int value) {
		String s_style = null;
		if (type == 0) {
			//[internal] 白天
			if (value == 0) {
				s_style = "DaySmall";
			} else if (value == 1) {
				s_style = "DayMid";
			} else if (value == 2) {
				s_style = "DayBig";
			}
		} else if (type == 1) {
			//[internal] 黑夜
			if (value == 0) {
				s_style = "NightSmall";
			} else if (value == 1) {
				s_style = "NightMid";
			} else if (value == 2) {
				s_style = "NightBig";
			}
		}
		return s_style;
	}

	//[internal]引擎long到现实double
	public static Double longToDouble(long l_value) {
		return (double) (l_value / (3600.0 * 1024));
	}
	//[internal]现实double到引擎long
	public static long doubleToLong(Double d_value) {
		return (long) (d_value * (3600 * 1024));
	}
}
