package com.cmmap.internal.driver.base;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.zip.GZIPInputStream;

import android.content.Context;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.os.Environment;
import android.util.Log;

import com.cmmap.internal.mapcore.Lg;

public class Tools {

	private Context context;
	private String TAG = "tools";
	
	//[internal] 获取所有的挂载点
	public ArrayList<String> getMountPoints() {

		ArrayList<String> arrList = new ArrayList<String>();

		try {
			java.lang.Runtime runtime = java.lang.Runtime.getRuntime();
			Process proc = runtime.exec("mount");
			InputStream is = proc.getInputStream();
			InputStreamReader isr = new InputStreamReader(is);
			String line;
			String mount = new String();
			BufferedReader br = new BufferedReader(isr);
			while ((line = br.readLine()) != null) {
				if (line.contains("secure"))
					continue;
				if (line.contains("asec"))
					continue;

				if (line.contains("fat")) {
					String columns[] = line.split(" ");
					if (columns != null && columns.length > 1) {
						mount = mount.concat("*" + columns[1] + "\n");

						arrList.add(columns[1]);
					}
				} else if (line.contains("fuse")) {
					String columns[] = line.split(" ");
					if (columns != null && columns.length > 1) {
						mount = mount.concat(columns[1] + "\n");
						arrList.add(columns[1]);
					}
				}
			}

			Log.d(TAG, "mount = " + mount);

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		return arrList;
	}

	public boolean fileIsExists(String strFileName) {
		try {

			File f = new File(strFileName);
			if (!f.exists()) {
				return false;
			}

		} catch (Exception e) {
			return false;
		}
		return true;
	}

	//[internal] 测试该目录是否OK
	public boolean checkDirAvaliable(String strDir,String path) {
		String strFile = strDir;
		strFile += path;

		return fileIsExists(strFile);

	}
	
	public Tools(Context context){
		this.context  = context;
	}
	public String getAppDataPath() {
		String strDataPath = "";
		File filedle =context.getExternalFilesDir(Environment.DIRECTORY_DOCUMENTS);
		String strDefaultSDCard = filedle.getAbsolutePath();
		String strAlternativePath = "";
		try {
			strAlternativePath = context.getExternalFilesDir("").getAbsolutePath();
		}catch(Exception e) {
			strAlternativePath = context.getFilesDir().getAbsolutePath();
		}
		
		String strMapConfigPath = getMapConfigPath();
		strDataPath = strDefaultSDCard;
		Lg.e(TAG, "getExternalStorageDirectory = " + strDefaultSDCard);
		
		if (checkDirAvaliable(strDefaultSDCard,strMapConfigPath)) {
			strDataPath = strDefaultSDCard;
		} else {

			ArrayList<String> arrMountList = getMountPoints();

			for (int i = 0; i < arrMountList.size(); i++) {
				if (checkDirAvaliable(arrMountList.get(i),strMapConfigPath)) {
					strDataPath = arrMountList.get(i);
					break;
				}
			}
		}
		
		strDataPath += strMapConfigPath;
		File file = new File(strDataPath);
		// 缓存路径异常，无法创建、写入，使用替代路径
		if(!file.exists() && !file.mkdirs()){
			strDataPath = strAlternativePath + strMapConfigPath;
			file = new File(strDataPath);
			if (!file.exists() && !file.mkdirs()) {
				strDataPath = null;
			}
		}
		file = null;
		return strDataPath;
	}

	private String getMapConfigPath() {
		ApplicationInfo info;
		try {
			info = context.getPackageManager().getApplicationInfo(
					context.getPackageName(), PackageManager.GET_META_DATA);
			String strPath = null;
			if (info != null && info.metaData != null) {
				strPath = info.metaData.getString("com.cmmap.api.mapcachepath");
			}
			if (strPath == null || strPath.isEmpty()) {
				// 没有配置使用默认路径
				return "/cmmap/mapcache/";
				
			}
			/**
			 * 检测两边是否带斜杠 ' / '
			 */
			String firstc =strPath.substring(0, 1);
			String lastc = strPath.substring(strPath.length() - 1, strPath.length());
			if (firstc.equals("/")) {
				strPath = strPath.substring(1, strPath.length());
			}
			if (lastc.equals("/")) {
				strPath = strPath.substring(0, strPath.length() - 1);
			}
			return "/" + strPath + "/";
		} catch (NameNotFoundException e) {
			e.printStackTrace();
		}
		return "/cmmap/mapcache/";
	}

	/**
	 * 判断是否存在SD卡
	 * @return
	 */
	public static boolean ExistSDCard() {
		if (android.os.Environment.getExternalStorageState().equals(android.os.Environment.MEDIA_MOUNTED)) {
			return true;
		}
		return false;
	}

	/**
	 * [internal] 根据path设置外置存储路径
	 * @param path
	 * @return
	 */
	public static String getExternalStorageDirectory(String path) {
		if (ExistSDCard()) {
			String s_path = android.os.Environment.getExternalStorageDirectory().getPath() + path;
			File file = new File(s_path);
			if(!file.exists()){
				file.mkdirs();
			}
			return s_path;
		}
		return null;
	}

	/**
	 * [internal] 根据path设置内置存储路径
	 * @param context
	 * @param path
	 * @return
	 */
	public static String getInternalStorageDirectory(Context context, String path) {
		if(context != null){
			String s_path = context.getApplicationContext().getFilesDir().getPath() + path;
			File file = new File(s_path);
			if(!file.exists()){
				file.mkdirs();
			}
			return s_path;
		}
		return null;
	}

	public synchronized static void ungzipFile(byte[] content,String name) throws IOException{
		ByteArrayInputStream is_byte = new ByteArrayInputStream(content);
		GZIPInputStream is_gzip = new GZIPInputStream(is_byte);
		File file = new File(getExternalStorageDirectory("/"+name));
		if(!file.exists()){
			file.createNewFile();
		}
		byte[] buffer = new byte[1024];
		FileOutputStream os_file = new FileOutputStream(file);
		int n = 0;
		while((n = is_gzip.read(buffer))>=0){
			os_file.write(buffer,0,n);
		}
		is_byte.close();
		is_gzip.close();
		os_file.flush();
		os_file.close();
	}
	/**
	 * [internal] gzip解压转换
	 * @param content [internal] gzip字节流
	 * @return
	 * @throws IOException
	 */
	public synchronized static byte[] ungzip(byte[] content) throws IOException{
		byte[] b_content = null;
		ByteArrayInputStream is_byte = new ByteArrayInputStream(content);
		GZIPInputStream is_gzip = new GZIPInputStream(is_byte);
		byte[] buffer = new byte[1024];
		ByteArrayOutputStream os_byte = new ByteArrayOutputStream();
		int n = 0;
		while((n = is_gzip.read(buffer))>=0){
			os_byte.write(buffer,0,n);
			os_byte.flush();
		}
		b_content = os_byte.toByteArray();
		is_byte.close();
		is_gzip.close();
		os_byte.flush();
		os_byte.close();
		return b_content;
	}
	
	public synchronized static void writeFile(byte[] content,String name) throws IOException{
		ByteArrayInputStream is_byte = new ByteArrayInputStream(content);
		File file = new File(getExternalStorageDirectory("/"+name));
		if(!file.exists()){
			file.createNewFile();
		}
		FileOutputStream os_file = new FileOutputStream(file);
		byte[] buffer = new byte[1024];
		int n = 0;
		while((n = is_byte.read(buffer))>=0){
			os_file.write(buffer,0,n);
		}
		is_byte.close();
		os_file.flush();
		os_file.close();
	}
}
