package com.cmmap.internal.driver.gesture;

import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import android.annotation.SuppressLint;
import android.graphics.Point;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.MotionEvent;

import com.cmmap.internal.driver.base.GlobalSettings;
import com.cmmap.internal.driver.base.MapTool;
import com.cmmap.internal.driver.view.MapGLSurfaceView;
import com.cmmap.internal.driver.view.MapRenderer;
import com.cmmap.internal.driver.view.ViewContext;
import com.cmmap.internal.mapcore.KFloat;
import com.cmmap.internal.mapcore.KGEOCOORD;
import com.cmmap.internal.mapcore.KInt;
import com.cmmap.internal.mapcore.KPoint;
import com.cmmap.internal.mapcore.MapCore;
import com.cmmap.internal.maps.KCameraUpdate;
import com.cmmap.internal.maps.KCameraUpdateFactory;
import com.cmmap.internal.maps.KMap;
import com.cmmap.internal.maps.KMap.CancelableCallback;
import com.cmmap.internal.maps.KMapWidgetView;
import com.cmmap.internal.maps.model.KCameraPosition;
import com.cmmap.internal.maps.model.KLatLng;

public class MapGestureManager {

	private static String Tag = "MapGestureManager";

	// 动画时间
	private final static int ZOOM_DELAY = 200;
	private final static int RESTORE_ZERO_DELAY = 300;
	private final static int FLING_DELAY = 600;
	private final static int FLING_BEYOND_DELAY = 300;

	// 消息定义
	private static final int MSG_TIMER_REFRESH_FEED = 3001;
	private static final int MSG_TIMER_REFRESH_RESET = 3002;
	public static final int GL_REFRESH_MAP = 3003;

	// 仅支持两点触摸
	private static final int TOUCH_POINT_COUNT = 2;

	// 两指操作类型
	private static final int TWO_FINGERS_MUL_ZOOM_IN = 1;
	private static final int TWO_FINGERS_MUL_ZOOM_OUT = 2;
	private static final int TWO_FINGERS_DRAG_MOVE = 3;
	private static final int TWO_FINGERS_MUL_ROTATE = 4;

	// 变量
	private KMap m_map;
	private MapRenderer m_render;
	private ViewContext m_context;
	private MapGLSurfaceView m_surface;

	// 两点触摸起点之间连线的中点
	private int m_iOX = 0;
	private int m_iOY = 0;

	// 多点手势列表
	private ArrayList<TouchPoint> m_alMulTouchList = new ArrayList<TouchPoint>();

	// 每个MapView用自己的Activity，避免消息发给进入后台的Activity
	protected Handler m_handler = null;

	// 当前显示的activity 的handler
	public static Handler s_handler = null;

	// 是否定时刷新模式
	private boolean m_b_timer_fresh_map = false;

	// 是否正在缩小地图
	private boolean m_b_is_zoom_out = false;

	// 是否清除地图文字
	private boolean m_b_clear_text = false;

	// 记录上次手指按下时间
	private long m_two_point_down_time_bak = 0;

	// 记录上次手指双击时间
	private long m_last_dbl_click_time_bak = 0;

	// 刷新定时器计数
	private int m_timer_refresh_count = 0;

	// 多点缩放的距离
	private int m_MulZoomLastDistance = 0;

	// 地图高度
	private int m_ViewHeight = 0;

	// 地图宽度
	private int m_ViewWidth = 0;

	// zhic 记录移动的偏移，用于移动地图坐标计算
	private float m_mov_x_bak;
	private float m_mov_y_bak;
	private KGEOCOORD m_center_bak;

	// 是否两指连续移动模式
	private boolean m_bTrackMove = false;
	// 是否已经判断两指连续移动模式
	private boolean m_bProccessFlag = false;
	// zhic 手势消息处理

	// 当前处于动画状态
	private boolean m_b_is_animating = false;

	// 长按消息处理
	public void handleLongPress(MotionEvent event) {
		Log.i(Tag, "MGM: handleLongPress()");
		processLongPress((int) event.getX(), (int) event.getY());
	}

	// 单次点击消息
	public void handleClick(MotionEvent event) {
		Log.i(Tag, "MGM: handleClick()");
		processClick((int) event.getX(), (int) event.getY());
	}

	// 单指双击(以单指为中心,放大一级地图比例尺)
	public void handleDoubleClick(MotionEvent e) {
		Log.i(Tag, "MGM: handleDoubleClick()");

		if (e.getEventTime() - m_last_dbl_click_time_bak < 1000) {
			return;
		}

		KLatLng crd = new KLatLng();
		int r;

		KCameraUpdate update = KCameraUpdateFactory.zoomIn();
		r = MapCore.CalcPixelZoom((int) e.getX(), (int) e.getY(),
				update.targetPosition.zoom, crd);
		if (r == 1) {
			// 发生错误
			return;
		}

		update.targetPosition.target.longitude = crd.longitude;
		update.targetPosition.target.latitude = crd.latitude;
		update.m_byUpdateType = KCameraUpdate.Camera_Update_LatLng_Only
				| KCameraUpdate.Camera_Update_Zoom_Only;

		animate(update, ZOOM_DELAY);

		m_last_dbl_click_time_bak = e.getEventTime();
	}

	// 单指移动地图
	public void handleMove(MotionEvent e1, MotionEvent e2, float distanceX,
			float distanceY) {
		if (m_b_is_zoom_out) {
			return;
		}

		Log.i(Tag, "MGM: handleMove()");
		// 如果关闭了全体手势 直接返回不move
		if (!this.m_context.getMapOptions().getAllGesturesEnabled())
			return;
		if (this.m_context.getMapOptions().getScrollGesturesEnabled()) {
			float iMoveX = (int) (e2.getX() - m_mov_x_bak);
			float iMoveY = (int) (e2.getY() - m_mov_y_bak);
			m_render.SetMovePoint(
					(int) (m_render.m_center_point.x - iMoveX),
					(int) (m_context.getViewport().y - (m_render.m_center_point.y - iMoveY)));
			m_render.SimplifyMovePoint(
					(int) (m_render.m_center_point.x),
					(int) (m_context.getViewport().y - m_render.m_center_point.y));
			m_surface.requestRender();
		}
		m_mov_x_bak = e2.getX();
		m_mov_y_bak = e2.getY();

		// 启动刷新定时器
		if (!m_b_timer_fresh_map) {
			startTimerRefresh();
		}
	}

	// 双指移动消息
	public void handleMovePoint2(MotionEvent e1, MotionEvent e2,
			float distanceX, float distanceY) {
		Log.i(Tag, "MGM: handleMovePoint2()");

		m_render.mDoubleFingleMidX = (int) (e2.getX(0) + e2.getX(1)) / 2;
		m_render.mDoubleFingleMidY = (int) (e2.getY(0) + e2.getY(1)) / 2;
		CaptureTouchList(e2);

		// 判断手势的意图
		Vector<Integer> iOptrate = AnalyseTouchOperate();
		boolean changed = false;
		for (int i = 0; iOptrate != null && i < iOptrate.size(); i++) {
			if (operateMap2(iOptrate.get(i), e2)) {
				changed = true;
			}
		}
		if (changed) {
			m_surface.requestRender();
		}

		startTimerRefresh();
	}

	// 单指按下消息
	public void handleDown(MotionEvent e) {
		Log.i(Tag, "MGM: handleDown() " + e.getPointerCount());

		m_mov_x_bak = e.getX();
		m_mov_y_bak = e.getY();

		MapCore.GetMapCenter(m_center_bak);
		m_mov_x_bak = e.getX();
		m_mov_y_bak = e.getY();
		m_alMulTouchList.clear();
		// 使用m_twoPointDis 判断是否有多点触控 m_twoPointDis > 0有多点
		m_twoPointDis = 0;
		m_render.setFastRefresh(true);
	}

	// 双指按下消息
	public void handle2PointerDown(MotionEvent e) {
		Log.i(Tag, "MGM: handle2PointerDown()");

		// 记录时间，用于双指tap判断
		m_two_point_down_time_bak = e.getEventTime();

		int x0 = (int) e.getX(0);
		int y0 = (int) e.getY(0);
		int x1 = (int) e.getX(1);
		int y1 = (int) e.getY(1);

		int dx = x1 - x0;
		int dy = y1 - y0;

		m_twoPointDis = Math.sqrt(dx * dx + dy * dy);
		m_twoPointAngle = Math.toDegrees(Math.atan2(dy, dx));
		m_twoPointAngle_scale = m_twoPointAngle;
		m_b_rotate_start = false;
		m_b_scale_start = false;
		m_trace_move_starty_bak = 0;

		KFloat kvalue = new KFloat(0);
		MapCore.GetConfiguration(
				MapCore.NE_MapParam_Object.NE_Map_RotateAngle.value(), kvalue);
		m_mapRotateAngle = (int)(kvalue.FloatVal());

		m_iOX = ((int) e.getX(0) + (int) e.getX(1)) / 2;
		m_iOY = ((int) e.getY(0) + (int) e.getY(1)) / 2;
		m_bProccessFlag = false;
		m_MulZoomLastDistance = GetTwoPointsDistance((int) e.getX(0),
				(int) e.getY(0), (int) e.getX(1), (int) e.getY(1));
		m_alMulTouchList.clear();
		CaptureTouchList(e);
	}

	// 双指抬起消息
	public void handle2PointerUp(MotionEvent e) {
		if (e.getPointerCount() < 2) {
			return;
		}

		Log.i(Tag, "MGM: handle2PointerUp()");
		stopTimerRefresh();

		if (m_context.getMapOptions().getZoomGesturesEnabled()
				&& (e.getEventTime() - m_two_point_down_time_bak < 100)) {
			// 当手势有放大趋势时，不判断为双指单击
			float dx = e.getX(0) - e.getX(1);
			float dy = e.getY(0) - e.getY(1);
			double fEndDist = Math.sqrt(dx * dx + dy * dy);
			Log.i("xxx", "hhhhhhhh " + (fEndDist - m_twoPointDis));
			if (fEndDist - m_twoPointDis < 5) {
				// 双指单击,以地图中心为中心,缩小地图
				Log.i(Tag, "MGM: zoomOut()");
				KCameraUpdate update = KCameraUpdateFactory.zoomOut();
				animate(update, ZOOM_DELAY);
				m_b_is_zoom_out = true;
				return;
			}
		}

		// Log.i("CZD", "2 point up index=" + e.getActionIndex() );
		// 校准单指移动坐标
		// getActionIndex表示up动作是哪个手指产生的，要取另外down的坐标
		int index = 0;
		if (e.getActionIndex() == 0) {
			index = 1;
		}
		m_mov_x_bak = e.getX(index);
		m_mov_y_bak = e.getY(index);

		Log.i(Tag, "MGM: handle2PointerUp() refresh");
		if (m_render.isFastRefresh()) {
			m_render.setFastRefresh(false);
			m_surface.requestRender();
		}
	}

	// 单指抬起消息
	public void handle1PointerUp(MotionEvent e) {
		Log.i(Tag, "MGM: handel1PointerUp()");
		stopTimerRefresh();
		KFloat kvalue = new KFloat(0);
		MapCore.GetConfiguration(
				MapCore.NE_MapParam_Object.NE_Map_RotateAngle.value(), kvalue);
		if ((kvalue.FloatVal() != 0)
				&& (Math.abs(kvalue.FloatVal()) < 20 || Math.abs(360 - kvalue
						.FloatVal()) < 20)) {
			// 角度接近正北 自动还原
			animate(KCameraUpdateFactory.changeBearing(0), RESTORE_ZERO_DELAY);
		} 
		
		if (m_render.isFastRefresh()) {
			m_render.setFastRefresh(false);
			m_surface.requestRender();
		}
		m_b_is_zoom_out = false;
	}

	// 点击消息抬起
	public void handleTapUp(MotionEvent e) {
		Log.i(Tag, "MGM: handleTapUp()");
		m_alMulTouchList.clear();
	}

	// 甩动作
	public boolean handleFling(MotionEvent e1, MotionEvent e2, float velocityX,
			float velocityY) {
		if (m_b_is_animating) {
			return false;
		}

		// 参数解释：
		// e1：第1个ACTION_DOWN MotionEvent
		// e2：最后一个ACTION_MOVE MotionEvent
		// velocityX：X轴上的移动速度，像素/秒
		// velocityY：Y轴上的移动速度，像素/秒
		if (m_twoPointDis > 0) {
			return false; // 双指 不响应甩动
		}

		if (this.m_context.getMapOptions().getAllGesturesEnabled()
				&& this.m_context.getMapOptions().getScrollGesturesEnabled()) {
			int w = (int) m_render.m_center_point.x;
			int h = (int) m_context.getViewport().y
					- (int) m_render.m_center_point.y;
			int max_wh = Math.max(w, h);
			if (max_wh == 0)
				return false;
			// Log.i("CZD","handleFling velocityX=" +velocityX
			// +" velocityY="+velocityY +" w:"+w+"h:"+h );
			m_map.stopAnimation();
			double max_len = Math.max(Math.abs(velocityX), Math.abs(velocityY));
			if (max_len > 500) {
				Log.i(Tag, "MGM: onFling()");
				KPoint cPointScreen = new KPoint();
				cPointScreen.iX = (int) (w - velocityX * w / (max_wh * 4));
				cPointScreen.iY = (int) (h + velocityY * h / (max_wh * 4)); // 长宽移动考虑长宽比系数

				// 取地图模式
				KFloat kvalue = new KFloat(0);
				float PitchAngle;
				MapCore.GetConfiguration(
						MapCore.NE_MapParam_Object.NE_Map_PitchAngle.value(),
						kvalue);
				PitchAngle = kvalue.FloatVal();
				if (PitchAngle > 0) {// 2.5D 显示时 上下移动不均匀，矫正一下
					if (velocityY > 0) {
						cPointScreen.iY -= velocityY * h / (max_wh * 6);
					} else {
						cPointScreen.iY += velocityY * h / (max_wh * 4);
					}
				}

				// 取中心点
				KGEOCOORD cPointCoord = new KGEOCOORD();
				int r = MapCore.ScreenPointToCoord(cPointScreen, cPointCoord);
				KLatLng coord = new KLatLng();
				coord.latitude = cPointCoord.lLatitude / (3600 * 1024.0);
				coord.longitude = cPointCoord.lLongitude / (3600 * 1024.0);

				m_render.ClearMovePoint();
				if (r == 2) {
					// 表示到达坐标范围边界
					animate(KCameraUpdateFactory.changeLatLng(coord),
							FLING_BEYOND_DELAY);
				} else {
					animate(KCameraUpdateFactory.changeLatLng(coord),
							FLING_DELAY);
				}
			}
		}
		return true;
	}

	// zhic --- end

	private class RefreshMapTask extends TimerTask {

		@Override
		public void run() {
			Message msg = new Message();
			msg.what = MSG_TIMER_REFRESH_FEED;
			m_handler.sendMessage(msg);
		}
	}
	Timer mRefreshMapTimer = null;
	RefreshMapTask mRefreshMapTimerTask = null;

	public MapGestureManager(MapGLSurfaceView m_surface, ViewContext m_context,
			KMap m_map, MapRenderer render) {
		this.m_surface = m_surface;
		this.m_context = m_context;
		this.m_map = m_map;
		this.m_render = render;
		m_ViewHeight = m_surface.getHeight();
		DoMessageHandler();
		m_center_bak = new KGEOCOORD();
	}

	// 设置当前 activity的 handler
	public void setCurrentHandler() {

		s_handler = m_handler;
	}

	// 重置刷新定时器
	void resetTimerRefreshCount() {
		Message msg = new Message();
		msg.what = MSG_TIMER_REFRESH_RESET;
		m_handler.sendMessage(msg);
	}

	// 停止刷新定时器
	public void stopTimerRefresh() {
		m_b_timer_fresh_map = false;
		resetTimerRefreshCount();
		if (mRefreshMapTimer != null) {
			mRefreshMapTimer.cancel();
			mRefreshMapTimer = null;
		}
	}

	// 启动刷新定时器
	void startTimerRefresh() {
		resetTimerRefreshCount();
		m_b_timer_fresh_map = true;
		m_render.setFastRefresh(true);

		if (mRefreshMapTimer != null) {
			mRefreshMapTimer.cancel();
		}
		mRefreshMapTimer = new Timer();
		mRefreshMapTimerTask = new RefreshMapTask();
		mRefreshMapTimer.schedule(mRefreshMapTimerTask, 0, 250);
	}

	private void resetClearTextFlag() {
		m_b_clear_text = false;
	}

	// 消息处理
	@SuppressLint("HandlerLeak")
	void DoMessageHandler() {

		Log.d("MapSysJNI", "DoMessageHandler");

		if (m_handler == null) {
			m_handler = new Handler() {
				public void handleMessage(Message msg) {
					switch (msg.what) {
					case MSG_TIMER_REFRESH_FEED: // 累计刷新定时器计数
						if (m_b_timer_fresh_map) {
							m_timer_refresh_count++;
							if (m_timer_refresh_count >= 4) {
								// 每隔一段时间刷新1次地图
								Log.i(Tag, "MGM: timeRefresh()");
								m_timer_refresh_count = 0;
								resetClearTextFlag();
								m_render.setTimerRefresh(true);
								m_surface.requestRender();
							}
						}
						break;
					case MSG_TIMER_REFRESH_RESET: // 刷新定时器计数复位
						m_timer_refresh_count = 0;
						break;
					case GL_REFRESH_MAP:
						if (!m_b_is_animating) {
							// zhic 动画执行时减少外部刷新，减少闪烁感
							m_surface.requestRender();
							m_map.onRefreshInfoLayer();
						}
						break;
					}
				}
			};
		}
		setCurrentHandler();
	}

	// 动画处理
	void animate(KCameraUpdate update, long delay) {
		m_b_is_animating = true;
		m_surface.requestRender();
		m_map.animateCamera(update, delay, new CancelableCallback() {
			@Override
			public void onCancel() {
				m_b_is_animating = false;
				m_surface.requestRender();
			};

			@Override
			public void onFinish() {
				m_b_is_animating = false;
				m_surface.requestRender();
			};
		});
	}

	// 两手指之间的移动距离
	private double m_twoPointDis;
	// 双指动作初次的两点矢量角
	private double m_twoPointAngle;
	// 缩放时会重新矫正的角度，用于控制缩放时的初次旋转锁定
	private double m_twoPointAngle_scale;
	// 双指旋转是否开始的标记
	private boolean m_b_rotate_start = false;
	// 双指缩放是否开始的标记
	private boolean m_b_scale_start = false;

	// 地图旋转角
	private int m_mapRotateAngle;

	private int m_operateRotateAngle;

	private double m_saveAngle;

	private Point m_operatePoint = new Point();

	private int m_trace_move_starty_bak = 0;

	// 处理双指上下滑动
	private void processTraceMove() {
		Log.i(Tag, "MGM: processTraceMove()");

		if (!this.m_context.getMapOptions().getTiltGesturesEnabled()
				|| !this.m_context.getMapOptions().getAllGesturesEnabled()) {
			// 如果用户规定了不可上下滑动就直接return
			return;
		}
		KFloat kvalue = new KFloat(0);
		MapCore.GetConfiguration(
				MapCore.NE_MapParam_Object.NE_Map_PitchAngle.value(), kvalue);
		float LAST_PITCHING_ANGLE = kvalue.FloatVal();
		if (m_alMulTouchList.size() < 4) {
			return;
		}

		// 起点两指中心坐标
		int iLastIndex = m_alMulTouchList.size() - 2;

		if (m_trace_move_starty_bak == 0) {
			int iStartCenterY = (m_alMulTouchList.get(iLastIndex - 2).y + m_alMulTouchList
					.get(iLastIndex - 1).y) / 2;
			m_trace_move_starty_bak = iStartCenterY;
		}

		// 终点点两指中心坐标
		int iLastCenterY = (m_alMulTouchList.get(iLastIndex).y + m_alMulTouchList
				.get(iLastIndex + 1).y) / 2;

		// 滑动像素范围是 1/4屏
		float fMovePixsRange = (float) m_ViewHeight / 5f;

		// 每个角度对应的像素长度
		float min_pitch = 90;//KMapWidgetView.MAP_3D_TITL_DEFA;
		float fAnglePerPix = min_pitch / fMovePixsRange;

		float iAngle = (int) ((iLastCenterY - m_trace_move_starty_bak) * fAnglePerPix)
				+ LAST_PITCHING_ANGLE;

		if (iAngle < 0) {
			iAngle = 0;
		} else if (iAngle > min_pitch) {
			iAngle = (int) min_pitch;
		}

		KCameraPosition c = m_map.getCameraPosition();
		c.tilt = iAngle;
		m_surface.setMapTilt(c);
		m_surface.requestRender();

		m_trace_move_starty_bak = iLastCenterY;
		// Log.d(Tag, "pitchingAngleEnque iAngle = " + iAngle);
	}

	// return false mean not changed
	private boolean operateMap2(int iOperate, MotionEvent event) {
		if (!this.m_context.getMapOptions().getAllGesturesEnabled()) {
			return false;
		}
		if (iOperate == TWO_FINGERS_MUL_ZOOM_IN
				|| iOperate == TWO_FINGERS_MUL_ZOOM_OUT) {
			KInt scale = new KInt(0);
			MapCore.GetMapScaleValue(scale);
			KCameraPosition position = m_map.getCameraPosition();
			if (this.m_context.getMapOptions().getZoomGesturesEnabled()) {
				int iMulZoomDistance = GetTwoPointsDistance(
						(int) event.getX(0), (int) event.getY(0),
						(int) event.getX(1), (int) event.getY(1));

				m_render.mDoubleFingleScalingRate = (float) iMulZoomDistance
						/ m_MulZoomLastDistance;

				if (((iOperate == TWO_FINGERS_MUL_ZOOM_IN)
						&& (Math.abs(position.zoom
								- (float) MapCore.GetMaxScale()) < 0.001) && (1 < m_render.mDoubleFingleScalingRate))) {
					Log.i(Tag, "iOperate == OPERATE_MUL_ZOOM_IN 到底了");
					// 到底了不准再放大
					return false;
				}
				if (((iOperate == TWO_FINGERS_MUL_ZOOM_OUT)
						&& (Math.abs(position.zoom
								- (float) MapCore.GetMinScale()) < 0.001) && (1 > m_render.mDoubleFingleScalingRate))) {
					Log.i(Tag, "iOperate == OPERATE_MUL_ZOOM_OUT 到顶了");
					// 到顶了不能再缩小
					return false;
				}

				if (iOperate == TWO_FINGERS_MUL_ZOOM_OUT) {
					if (!m_b_clear_text) {
						// 缩小地图清除文字
						m_render.clearMapText(event.getEventTime(), false);
						m_b_clear_text = true;
					}
				}

				m_MulZoomLastDistance = iMulZoomDistance;
				m_render.SetDoubleFingerScaling(true);
				return true;
			}
		}// 双指旋转
		else if (iOperate == TWO_FINGERS_MUL_ROTATE) {
			Log.i(Tag, "double rotate:" + m_operateRotateAngle);
			if (this.m_context.getMapOptions().getRotateGesturesEnabled()) {
				m_render.SetRotateOprate(m_operateRotateAngle);
				return true;
			}
		} else if (iOperate == TWO_FINGERS_DRAG_MOVE) {
			if (this.m_context.getMapOptions().getScrollGesturesEnabled()) {
				Log.i(Tag, "MoveToScreenPt x:" + m_operatePoint.x + ",y:"
						+ m_operatePoint.y);
				m_render.SetMovePoint(m_operatePoint.x, m_operatePoint.y);
				m_render.SimplifyMovePoint(
						m_render.m_center_point.x,
						(int) (m_context.getViewport().y - m_render.m_center_point.y));
				return true;
			}
		}
		return false;
	}

	// 触摸点
	class TouchPoint {
		public int x;
		public int y;

		// 圆心角
		public int iCenterAngle;
		// 圆心距的平方
		public int iCenterDistance;
		// 触摸点距的平方
		public int iPointDistance;

		public TouchPoint(int _x, int _y) {
			x = _x;
			y = _y;
		}

		// 计算圆心角和圆心距
		public void CalAngleDis(int iOX, int iOY) {
			int dx = x - iOX;
			int dy = y - iOY;
			double sita = Math.atan2(dx, dy);
			iCenterAngle = (int) (sita * 180.0 / 3.14159);
			iCenterDistance = dx * dx + dy * dy;
		}

		// 调整角度差在180度以内
		void AdjustAngle(int iLastAngle) {
			if (Math.abs(iCenterAngle - iLastAngle) < 180) {
				// 不用调整
				return;
			}

			int iAngle = iCenterAngle;

			if (iAngle > iLastAngle) {
				iCenterAngle -= 360;
			} else {
				iCenterAngle += 360;
			}

		}

		// 计算两个触摸点距
		void CalPointDistance(TouchPoint tp) {
			int dx = this.x - tp.x;
			int dy = this.y - tp.y;

			this.iPointDistance = dx * dx + dy + dy;
			tp.iPointDistance = this.iPointDistance;
		}
	}

	// 捕获触摸点列表
	void CaptureTouchList(MotionEvent event) {
		for (int i = 0; i < TOUCH_POINT_COUNT; i++) {
			TouchPoint touchPoint = new TouchPoint((int) event.getX(i),
					(int) event.getY(i));
			touchPoint.CalAngleDis(m_iOX, m_iOY);

			int iPrevIndex = m_alMulTouchList.size() - TOUCH_POINT_COUNT;
			if (iPrevIndex >= 0) {
				// 调整角度
				int iLastAngle = m_alMulTouchList.get(iPrevIndex).iCenterAngle;
				touchPoint.AdjustAngle(iLastAngle);

			}

			// 计算间距
			if (i == TOUCH_POINT_COUNT - 1) {
				m_alMulTouchList.get(m_alMulTouchList.size() - 1)
						.CalPointDistance(touchPoint);
			}

			m_alMulTouchList.add(touchPoint);
		}
	}

	Vector<Integer> AnalyseTouchOperate() {
		// 如果第一次手势 就不进入 必须大于两次onTouch才进行计算
		if (m_alMulTouchList.size() < 6) {
			return null;
		}

		Vector<Integer> iOperate = new Vector<Integer>(5);

		// List 0 1 2 3 4 5
		// last1=0 last2=1 last3=4 last4=5
		TouchPoint last1 = m_alMulTouchList.get(m_alMulTouchList.size() - 2);
		TouchPoint last2 = m_alMulTouchList.get(m_alMulTouchList.size() - 1);
		TouchPoint last3 = m_alMulTouchList.get(m_alMulTouchList.size() - 4);
		TouchPoint last4 = m_alMulTouchList.get(m_alMulTouchList.size() - 3);
		TouchPoint last5 = m_alMulTouchList.get(m_alMulTouchList.size() - 6);
		TouchPoint last6 = m_alMulTouchList.get(m_alMulTouchList.size() - 5);

		int dx = last2.x - last1.x;
		int dy = last2.y - last1.y;
		double tempDis = Math.sqrt(dx * dx + dy * dy);
		double tempAngle = Math.toDegrees(Math.atan2(dy, dx));

		// 计算旋转角度
		dx = last4.x - last3.x;
		dy = last4.y - last3.y;
		double tempAngle2 = Math.toDegrees(Math.atan2(dy, dx));

		// 进入先判定是否俯仰角模式
		if (!m_bProccessFlag) {
			// 进入一次就不让进入
			if (Math.abs(last2.y - last6.y) > 0
					&& Math.abs(last1.y - last5.y) > 0) {
				m_bProccessFlag = true;
				// 判断俯仰角模式还是其它双指操作模式
				boolean isDoubleMove = ((last2.y - last6.y) * (last1.y - last5.y)) > 0;
				Log.i(Tag,
						"one: 1-3: " + Math.abs(last1.y - last5.y) + ","
								+ Math.abs(last1.x - last5.x) + " 2-4: "
								+ Math.abs(last2.y - last6.y) + ","
								+ Math.abs(last2.x - last6.x));
				if (isDoubleMove
						&& Math.abs(last1.y - last5.y) > Math.abs(last1.x
								- last5.x) * 0.6
						&& Math.abs(last2.y - last6.y) > Math.abs(last2.x
								- last6.x) * 0.6) {
					m_bTrackMove = true;// 可以改变俯仰角
				} else {
					m_bTrackMove = false;// 可以做缩放、旋转、平移操作
				}
			}
		}

		if (m_bTrackMove) {
			processTraceMove();
			return iOperate;
		}
		// 旋转
		int angle_step = 5; // 初次旋转的角度阈值
		if (m_mapRotateAngle == 0) {
			angle_step = 10;
		}
		if (m_b_scale_start) {
			angle_step = 4;
		}
		if ((Math.abs(tempAngle - m_twoPointAngle_scale) > angle_step || m_b_rotate_start)) // Math.abs(tempDis
																							// -
																							// m_twoPointDis)
																							// <
																							// 30
																							// ||
		{
			this.m_operateRotateAngle = 360
					- (int) (tempAngle - m_twoPointAngle_scale)
					+ m_mapRotateAngle;
			m_b_rotate_start = true;
			if (m_operateRotateAngle == 0) {
				iOperate.add(TWO_FINGERS_MUL_ROTATE);
				this.m_saveAngle = this.m_operateRotateAngle;
			} else {
				Log.i(Tag,
						"saveAngle2:"
								+ Math.abs(this.m_saveAngle
										- this.m_operateRotateAngle));
				if (Math.abs(this.m_saveAngle - this.m_operateRotateAngle) > 0) {
					iOperate.add(TWO_FINGERS_MUL_ROTATE);
					this.m_saveAngle = this.m_operateRotateAngle;
				}
			}
		}

		int dx1 = last2.x - last4.x;
		int dy1 = last2.y - last4.y;
		int dx2 = last1.x - last3.x;
		int dy2 = last1.y - last3.y;
		boolean bIsMove = false;
		boolean bIsZoom = false;

		if (Math.abs(dx1 * dx2) > Math.abs(dy1 * dy2)) {
			// 横
			if (dx1 * dx2 > 0) {
				bIsMove = true;
			} else {
				bIsZoom = true;
			}
		} else {
			if (dy1 * dy2 > 0) {
				bIsMove = true;
			} else {
				bIsZoom = true;
			}
		}

		// Log.i(Tag, "dx1:"+dx1+",\r dx2:"+dx2+",\r dy1:"+dy1+",\r dy2:"+dy2 +
		// " bIsZoom:"+bIsZoom);
		// 缩放
		GlobalSettings globalSetting = GlobalSettings.getInstance();
		float fDpi = globalSetting.getDensityDpi();
		if (bIsZoom && Math.abs(tempDis - m_twoPointDis) > fDpi / 20) {
			m_b_scale_start = true;
			if (m_b_rotate_start == false
					&& Math.abs(m_twoPointAngle_scale - tempAngle) > 2) {
				m_twoPointAngle_scale = tempAngle;
			}
			if (tempDis > m_twoPointDis) {
				iOperate.add(TWO_FINGERS_MUL_ZOOM_IN);
			} else if (tempDis < m_twoPointDis) {
				iOperate.add(TWO_FINGERS_MUL_ZOOM_OUT);
			}
		}

		// 平移
		// Log.i(Tag, "operateRotateAngle absx:"+ Math.abs(tempAngle -
		// tempAngle2));
		if (bIsMove && Math.abs(tempAngle - tempAngle2) < 1.5) {
			this.m_operatePoint.x = (int) (m_render.m_center_point.x - (last1.x - last3.x));
			this.m_operatePoint.y = (int) (m_context.getViewport().y - (m_render.m_center_point.y
					- last1.y + last3.y));
			iOperate.add(TWO_FINGERS_DRAG_MOVE);
		}

		return iOperate;
	}

	// 求两点距离
	int GetTwoPointsDistance(int x1, int y1, int x2, int y2) {
		return (int) Math.sqrt(Square(x1 - x2) + Square(y1 - y2));
	}

	// 求平方
	int Square(int x) {
		return x * x;
	}

	// 单击事件
	private void processClick(int x, int y) {
		Point cp = new Point(x, y);
		this.m_map.onMapClick(cp);
	}

	// 长按事件
	private void processLongPress(int x, int y) {
		m_render.ClearMovePoint();
		KPoint cp = new KPoint();
		cp.iX = x;
		cp.iY = (int) (m_context.getViewport().y - y);

		KGEOCOORD k_coord = new KGEOCOORD();
		MapCore.ScreenPointToCoord(cp, k_coord);
		KLatLng location = new KLatLng(MapTool.longToDouble(k_coord.lLatitude),
				MapTool.longToDouble(k_coord.lLongitude));
		m_map.onMapLongClick(location);

		m_surface.requestRender();
	}

	public int getViewHeight() {
		return m_ViewHeight;
	}

	public void setViewHeight(int m_ViewHeight) {
		this.m_ViewHeight = m_ViewHeight;
	}

	public int getViewWidth() {
		return m_ViewWidth;
	}

	public void setViewWidth(int m_ViewWidth) {
		this.m_ViewWidth = m_ViewWidth;
	}
}
