package com.cmmap.internal.driver.service;

import java.nio.ByteBuffer;

import android.annotation.SuppressLint;
import android.graphics.Bitmap;
import android.util.Log;

import com.cmmap.api.maps.model.Tile;
import com.cmmap.api.maps.model.TileProvider;
import com.cmmap.api.maps.model.UrlTileProvider;
import com.cmmap.internal.driver.base.HttpTool;
import com.cmmap.internal.mapcore.KHttpData;
import com.cmmap.internal.mapcore.MapCore;
import com.cmmap.internal.maps.model.KOverlayManager;

/**
 * [internal] 网络数据下载 
 * 
 * @author liangz
 * 
 */

public class DownloadTask implements Runnable {

	private KHttpData data;
	private HttpTool tool;
	public DownloadTask(KHttpData data) {
		this.data = data;
		tool = new HttpTool();
	}

	@SuppressLint("NewApi")
	@Override
	public void run() {
		if (data == null) {
			removeTask(true);
			return;
		}
		Log.i("download", "0 Type="+data.m_data_type + ",Lv="+data.m_iLevel + ",Rct=(" +
				data.m_Rect.iLeft + "," + data.m_Rect.iBottom + "," + data.m_Rect.iRight + "," + data.m_Rect.iTop + ")" + "URL=" + data.m_str_url);
		byte[] b_data = null;
		try {
			if (data.m_body != null) {
				if (data.m_data_type == KHttpData.REQ_TILE) {
					b_data = tool.executeHttpGet(data.m_str_url, 256, 256, data.m_body);
				} else if (data.m_data_type == KHttpData.REQ_OVERLAY_TILE) {
					int id = (int) data.m_Rect.iLeft;
					TileProvider tileProvider = KOverlayManager.getInstance().getTileProvider(String.format("%d", id));
					Tile tile = tileProvider.getTile((int)data.m_Rect.iRight, (int)data.m_Rect.iTop, data.m_iLevel);
					if (tile == null || tile.data == null || tile.data.length == 0) {
						b_data = null;
					} else if (!(tileProvider instanceof UrlTileProvider)) {
						Bitmap bitmap1 = HttpTool.decodeSampledBitmapFromByte(tile.data,0,tileProvider.getTileWidth(),tileProvider.getTileHeight());
						Bitmap bitmap = bitmap1.copy(Bitmap.Config.ARGB_8888, true);
						ByteBuffer buffer = ByteBuffer.allocate(bitmap.getByteCount());
						bitmap.copyPixelsToBuffer(buffer);
						byte [] pic_buf = buffer.array();
						int len = pic_buf.length;
						
						b_data = new byte [len + data.m_body.length];
						System.arraycopy(data.m_body, 0, b_data, 0, data.m_body.length);
						System.arraycopy(pic_buf, 0, b_data, data.m_body.length, len);
						bitmap1.recycle();
						bitmap.recycle();
					}else {
						b_data = new byte[tile.data.length + data.m_body.length];
						System.arraycopy(data.m_body, 0, b_data, 0, data.m_body.length);
						System.arraycopy(tile.data, 0, b_data, data.m_body.length, tile.data.length);
					}
				}else {
					b_data = tool.executeHttpPost(data.m_str_url, data.m_body);
				}
			}
			
			
		} catch (Exception e) {
			Log.w("download", "data download error-- type = " + data.m_data_type + "--" + Log.getStackTraceString(e));
			// 网络异常设置空
			{
				 if (data.m_data_type != KHttpData.REQ_OVERLAY_TILE)
				 {
					data.m_idata_size = data.m_body.length;
					data.m_data = new byte [data.m_idata_size];
					System.arraycopy(data.m_body, 0, data.m_data, 0, data.m_idata_size);
					Log.i("download","Exception Type=" + data.m_data_type  + " Rst Len=" + data.m_data.length);
					MapCore.SetNetworkData(data);
					data.m_data = null;
				 }
			}
			removeTask(false);
			if (!MapCore.requestData(data)) {
				data.release();
			}
			b_data = null;
			return;
		}
		
		if(b_data == null || b_data.length == 0){
			// 获取失败还是要设置下去，清理缓存，否则网络恢复或其他情况正常后，无法再去重新获取
			Log.w("download","data download error 2-- Type=" + data.m_data_type);
			//i_return = MapCore.SetNetworkData(data);
			removeTask(false);
			if (!MapCore.requestData(data)) {
				data.release();
			}
			b_data = null;
			return;
		}
		if (data.m_data_type == KHttpData.REQ_LANDMARK || data.m_data_type == KHttpData.REQ_POLYGON) {
			byte[] b_content = null;
			try {
				//b_content = SimpleScrambler.decode(b_data, password);
			} catch (Exception e) {
				e.printStackTrace();
				removeTask(true);
				return;
			}
			data.m_idata_size = 0;//b_content.length;
			data.m_data = b_content;
			Log.i("download","3 Type=" + data.m_data_type + " Rst Len=" + data.m_data.length);
			MapCore.SetNetworkData(data);
			//释放占用内存
			b_content = null;
		} else {
			data.m_idata_size = b_data.length;
			data.m_data = b_data;
			Log.i("download","4 Type=" + data.m_data_type  + " Rst Len=" + data.m_data.length + "URL=" + data.m_str_url);
			MapCore.SetNetworkData(data);
			MapCore.deleteRequest(data);
			//释放占用内存
		}
		b_data = null;
		removeTask(true);
	}

	private void removeTask(boolean bRelease) {
		if (data != null) {
			DownloadInvoke.removeTask(data);
			if (bRelease) {
				data.release();
			}			
		}
	}

	public void shutDownClient() {
		if (tool != null) {
			try {
				tool.shutDownClient();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
}
