package com.cmmap.internal.driver.view;

import javax.microedition.khronos.egl.EGL10;
import javax.microedition.khronos.opengles.GL10;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.PointF;
import android.graphics.Rect;
import android.location.Location;
import android.opengl.GLSurfaceView;
import android.os.Message;
import android.util.AttributeSet;
import android.util.Log;
import android.view.GestureDetector;
import android.view.GestureDetector.OnDoubleTapListener;
import android.view.GestureDetector.OnGestureListener;
import android.view.MotionEvent;
import android.view.View;

import com.cmmap.api.maps.Map.OnMapScreenShotListener;
import com.cmmap.internal.driver.base.MapTool;
import com.cmmap.internal.driver.gesture.MapGestureManager;
import com.cmmap.internal.mapcore.KFloat;
import com.cmmap.internal.mapcore.KGEOCOORD;
import com.cmmap.internal.mapcore.KInt;
import com.cmmap.internal.mapcore.MapCore;
import com.cmmap.internal.mapcore.MapCore.NE_MapParam_Object;
import com.cmmap.internal.mapcore.MapJni;
import com.cmmap.internal.maps.KCameraUpdate;
import com.cmmap.internal.maps.KInfoOverlayView;
import com.cmmap.internal.maps.KMap;
import com.cmmap.internal.maps.KMap.InfoView;
import com.cmmap.internal.maps.KProjection;
import com.cmmap.internal.maps.model.KCameraPosition;
import com.cmmap.internal.maps.model.KLatLng;
import com.cmmap.internal.maps.model.KMarker;
import com.cmmap.internal.maps.model.KShapeOverlay;

class ContextFactory implements GLSurfaceView.EGLContextFactory {
    private static int EGL_CONTEXT_CLIENT_VERSION = 0x3098;
    
	@Override 
	public javax.microedition.khronos.egl.EGLContext createContext(EGL10 egl,
			javax.microedition.khronos.egl.EGLDisplay display,
			javax.microedition.khronos.egl.EGLConfig eglConfig) {
        int[] attrib_list = {
        		EGL_CONTEXT_CLIENT_VERSION, 2,
        		//EGL10.EGL_ALPHA_SIZE, 0,
        		//EGL10.EGL_RED_SIZE, 5,
        		//EGL10.EGL_GREEN_SIZE, 6,
        		//EGL10.EGL_BLUE_SIZE, 5, 
        		//EGL10.EGL_DEPTH_SIZE, 16,
        		//EGL10.EGL_SAMPLE_BUFFERS, 1,
        		//EGL10.EGL_SAMPLES, 8,                
                //EGL10.EGL_SURFACE_TYPE, EGL_WINDOW_BIT,
        		EGL10.EGL_NONE };
        javax.microedition.khronos.egl.EGLContext context = egl.eglCreateContext(display, 
        		eglConfig, EGL10.EGL_NO_CONTEXT, attrib_list);
        //checkEglError("After eglCreateContext", egl);
        return context;
	}

	@Override
	public void destroyContext(EGL10 egl,
			javax.microedition.khronos.egl.EGLDisplay display,
			javax.microedition.khronos.egl.EGLContext context) {
		egl.eglDestroyContext(display, context);
	}
}


class ConfigChooser implements GLSurfaceView.EGLConfigChooser {

	
    public ConfigChooser(int r, int g, int b, int a, int depth, int stencil) {
        mRedSize = r;
        mGreenSize = g;
        mBlueSize = b;
        mAlphaSize = a;
        mDepthSize = depth;
        mStencilSize = stencil;
    }

    /* This EGL config specification is used to specify 2.0 rendering.
     * We use a minimum size of 4 bits for red/green/blue, but will
     * perform actual matching in chooseConfig() below.
     */
    private static int EGL_OPENGL_ES2_BIT = 4;
    private static int[] s_configAttribs2 =
    {
        EGL10.EGL_RED_SIZE, 5,
        EGL10.EGL_GREEN_SIZE, 6,
        EGL10.EGL_BLUE_SIZE, 5,
        EGL10.EGL_DEPTH_SIZE, 8,
        EGL10.EGL_SAMPLE_BUFFERS, 1,
        EGL10.EGL_SAMPLES, 4,
        EGL10.EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
        EGL10.EGL_NONE
    };
    
    public javax.microedition.khronos.egl.EGLConfig chooseConfig(EGL10 egl,
			javax.microedition.khronos.egl.EGLDisplay display) {

        /* Get the number of minimally matching EGL configurations
         */
        int[] num_config = new int[1];
        egl.eglChooseConfig(display, s_configAttribs2, null, 0, num_config);

        int numConfigs = num_config[0];

        if (numConfigs <= 0) {
            throw new IllegalArgumentException("No configs match configSpec");
        }

        /* Allocate then read the array of minimally matching EGL configs
         */
        javax.microedition.khronos.egl.EGLConfig[] configs = new javax.microedition.khronos.egl.EGLConfig[numConfigs];        
        egl.eglChooseConfig(display, s_configAttribs2, configs, numConfigs, num_config);
        
        /* Now return the "best" one
         */
        return chooseConfig(egl, display, configs);
    }

    public javax.microedition.khronos.egl.EGLConfig chooseConfig(EGL10 egl, 
    		javax.microedition.khronos.egl.EGLDisplay display,
    		javax.microedition.khronos.egl.EGLConfig[] configs) {
    	
        for(javax.microedition.khronos.egl.EGLConfig config : configs) {
            int d = findConfigAttrib(egl, display, config,
                    EGL10.EGL_DEPTH_SIZE, 0);
            
            int s = findConfigAttrib(egl, display, config,
                    EGL10.EGL_STENCIL_SIZE, 0);

            // We need at least mDepthSize and mStencilSize bits
            if (d < mDepthSize || s < mStencilSize)
                continue;

            // We want an *exact* match for red/green/blue/alpha
            int r = findConfigAttrib(egl, display, config,
                    EGL10.EGL_RED_SIZE, 0);
            int g = findConfigAttrib(egl, display, config,
                        EGL10.EGL_GREEN_SIZE, 0);
            int b = findConfigAttrib(egl, display, config,
                        EGL10.EGL_BLUE_SIZE, 0);
            int a = findConfigAttrib(egl, display, config,
                    EGL10.EGL_ALPHA_SIZE, 0);
            
            //int s1 = findConfigAttrib(egl, display, config,
              //      EGL10.EGL_SAMPLE_BUFFERS, 1);
            
            
            if (r == mRedSize && g == mGreenSize && b == mBlueSize && a == mAlphaSize)
                return config;
        }
        return null;
    }

    private int findConfigAttrib(EGL10 egl, javax.microedition.khronos.egl.EGLDisplay display,
    		javax.microedition.khronos.egl.EGLConfig config, int attribute, int defaultValue) {
    	
        if (egl.eglGetConfigAttrib(display, config, attribute, mValue)) {
            return mValue[0];
        }
        return defaultValue;
    }

    // Subclasses can adjust these values:
    protected int mRedSize;
    protected int mGreenSize;
    protected int mBlueSize;
    protected int mAlphaSize;
    protected int mDepthSize;
    protected int mStencilSize;
    private int[] mValue = new int[1];
}

/**
 * [internal]用于提供地图描绘环境
 * 
 * @author liangz
 * 
 */
public class MapGLSurfaceView extends GLSurfaceView implements OnGestureListener {
	private static final String TAG = "MapGLSurfaceView";
	private KMap map;
	private MapGestureManager m_gestureManager;
	private KInfoOverlayView m_infoOverlayView = null;
	private GestureDetector m_detector;
	
	private MapRenderer m_render;
	private ViewContext viewContext;
	private boolean isRealtimeMode = false;
	//软绘制上屏
	Bitmap m_SkiaBitmap = null;
	// [internal]存储手势操作前的参数值 
    
	private static final MapParams mp_preOperate = new MapParams();
	public MapGLSurfaceView(Context context, AttributeSet attrs) {
		super(context, attrs);
		setEGLContextClientVersion(2);	
	}
	
	public void setFastRefresh(boolean b)
	{
		m_render.setFastRefresh(b);
	}
	
	public void clearMapText(long timer, boolean force)
	{
		m_render.clearMapText(timer, force);
	}
	
	public void setRefreshDebugMask(int debugMask)
	{
		m_render.setRefreshDebugMask(debugMask);
	}
	
	public int getRefreshDebugMask() {
		return m_render.getRefreshDebugMask();
	}

	public MapGLSurfaceView(Context context) {
		super(context);
		setEGLContextClientVersion(2);
	}

	@SuppressWarnings("deprecation")
	public MapGLSurfaceView(Context context, final ViewContext viewContext, final KMap map) {
		super(context);
		
		setEGLContextClientVersion(2);
		
		setEGLContextFactory(new ContextFactory());
		
		setEGLConfigChooser(
                 new ConfigChooser(5, 6, 5, 0, 0, 0) );
		 
		this.map = map;
		this.m_render = new MapRenderer(viewContext, mDrawTemplet);
		m_render.map = map;
		
		setRenderer(m_render);
		setRenderMode(RENDERMODE_WHEN_DIRTY);
		this.viewContext = viewContext;
		m_gestureManager = new MapGestureManager(this, viewContext, map, m_render);
		
		//zhic 软绘制上屏开关
		//setWillNotDraw(false);
		
		m_detector = new GestureDetector(this);  
		OnDoubleTapListener onMapDoubleTapListener = new OnDoubleTapListener() 
		{
			@Override
			public boolean onSingleTapConfirmed(MotionEvent e) {
				Log.i("CZD", "onSingleTapConfirmed!");
				map.mHandler.removeMessages(KMap.MAP_GESTURE_END);
				m_gestureManager.handleClick(e);
				map.processInfowindowInQuickChange(View.VISIBLE);
			return true;
			}
			@Override
			public boolean onDoubleTapEvent(MotionEvent e) {
				Log.i("CZD", "onDoubleTapEvent!");
			return false;
			}
			@Override
			public boolean onDoubleTap(MotionEvent e) {
				Log.i("CZD", "onDoubleTap!");
				if(viewContext.getMapOptions().getZoomGesturesEnabled()){
					map.mHandler.removeMessages(KMap.MAP_GESTURE_END);
					m_gestureManager.handleDoubleClick(e);
				}
			return false;
			}
		};
		m_detector.setIsLongpressEnabled(true);
		m_detector.setOnDoubleTapListener(onMapDoubleTapListener);
	}

	public PointF getSize() {
		return new PointF(getWidth(), getHeight());
	}
	
	public int getViewWidth(){
		return getWidth();
	}
	
	public int getViewHeight(){
		return getHeight();
	}
	
	public void setMapRotateCenter(PointF pointF) {
		m_render.setMapRotateCenter(pointF);
		requestRender();
	}
	
	public void setOverlayInfoView(KInfoOverlayView infoView){
		m_infoOverlayView = infoView;
	}
	@Override
	public boolean onTouchEvent(MotionEvent event) {

			// [internal]手势操作前更新当前update参数，为了手势操作后参数变化比较
			if ((event.getAction() & MotionEvent.ACTION_MASK) == MotionEvent.ACTION_DOWN) {
				map.stopAnimation();
				map.mHandler.removeMessages(KMap.MAP_GESTURE_END);
				map.processInfowindowInQuickChange(View.GONE);
				saveMapParams();
			}
			
		
			if ( event.getPointerCount() == 2 && (event.getAction() & MotionEvent.ACTION_MASK) == MotionEvent.ACTION_POINTER_DOWN ) 
			{
				Log.i("CZD", "2 point down, action=" + event.getAction());
				this.m_gestureManager.handle2PointerDown(event);
			}
			else if ( event.getPointerCount() == 2 && (event.getAction() & MotionEvent.ACTION_MASK) == MotionEvent.ACTION_POINTER_UP ) 
			{
				Log.i("CZD", "2 point up, action=" + event.getAction());
				this.m_gestureManager.handle2PointerUp(event);
			}			
			
			else if ( event.getPointerCount() == 1 && event.getAction() == MotionEvent.ACTION_UP  ) 
			{
				Log.i("CZD", "1 point up, action=" + event.getAction());
				this.m_gestureManager.handle1PointerUp(event);
				Message message = new Message();
				message.what = KMap.MAP_GESTURE_END;
				map.mHandler.sendMessageDelayed(message, 500);
			}
			this.m_detector.onTouchEvent(event);

		map.onMapTouch(event);
		return true;
	}

	@Override
	protected void onLayout(boolean changed, int left, int top, int right,
			int bottom) {
		super.onLayout(changed, left, top, right, bottom);
		this.viewContext.setViewport(getWidth(), getHeight());
		KProjection.getInstance().setSize(getSize());

		m_gestureManager.setViewHeight(getHeight());
		m_gestureManager.setViewWidth(getWidth());
	}

	// [internal] 添加图形图层
	public void addShapOverlay(KShapeOverlay overlay) {
		m_render.shapeOverlayEnque(overlay);
		requestRender();
	}

	// [internal] 移除图形图层
	public void removeShapeOverlay(KShapeOverlay overlay) {
		m_render.removeShapeEnque(overlay);
		requestRender();
	}

	// [internal] 更新图形图层
	public void updateShapeOverlay(KShapeOverlay overlay, int updateMask) {
		m_render.updateShapeEnque(overlay, updateMask);
		requestRender();
	}
	
	public void updateShapeOverlayBitmap(KShapeOverlay overlay) {
		m_render.updateShapeBitmapEnque(overlay);
	}
	
	public void removeShapeOverlayBitmap(KShapeOverlay overlay) {
		m_render.removeShapeBitmapEnque(overlay);
	}
	
	public void setLocation(Location loc) {
		m_render.setLocation(loc);
		requestRender();
	}

	public void setRealtimeRenderMode() {
		setRenderMode(RENDERMODE_CONTINUOUSLY);
		isRealtimeMode = true;
	}

	public void setRequestRenderMode() {
		setRenderMode(RENDERMODE_WHEN_DIRTY);
		isRealtimeMode = false;
	}

	public boolean isRealtime() {
		return this.isRealtimeMode;
	}

	// [internal] 自定义主地图点击事件
	private RendererDrawTemplet mDrawTemplet = new RendererDrawTemplet() {

		@Override
		public void beforeMapRefresh(int operator) {
			
		}

		@Override
		public void afterMapRefresh(int operator) {
			map.mHandler.removeMessages(KMap.MAP_REFRESH);
			Message message = new Message();
			message.what = KMap.MAP_REFRESH;
			map.mHandler.sendMessage(message);
		}

		@Override
		public void refreshShaderView(GL10 gl) {
		}
	};

	/**
	 * 在地图有变化之前保存地图参数
	 */
	private void saveMapParams() {
		// [internal] 清空MapParam中的数值
		//mp_preOperate.clear();
		// [internal] 位置
		KGEOCOORD k_coord = new KGEOCOORD();
		MapCore.GetMapCenter(k_coord);
		mp_preOperate.setLatlon(new KLatLng(MapTool
				.longToDouble(k_coord.lLatitude), MapTool
				.longToDouble(k_coord.lLongitude)));
		// [internal] 描画的平行移动量
		mp_preOperate.setTranslation(this.viewContext.getMapParams().getTranslation());
		// [internal] 缩放比例
		KFloat ki_level = new KFloat(0);
		MapCore.GetMapScaleLevel(ki_level);
		mp_preOperate.setLevel(ki_level.FloatVal());
		// [internal] 旋转角
		KFloat ki_rotate = new KFloat(0);
		MapCore.GetConfiguration(NE_MapParam_Object.NE_Map_RotateAngle.value(),
				ki_rotate);
		mp_preOperate.setRotation(ki_rotate.FloatVal());
		// [internal] 样式
		KFloat ki_theme = new KFloat(0);
		MapCore.GetConfiguration(NE_MapParam_Object.NE_Map_Theme.value(),
				ki_theme);
		KFloat ki_textSize = new KFloat(0);
		MapCore.GetConfiguration(NE_MapParam_Object.NE_Map_TextSize.value(),
				ki_textSize);
		mp_preOperate.setStyleId(MapTool.styleIntToString((int)(ki_theme.FloatVal()),
				(int)(ki_textSize.FloatVal())));
		// [internal] 俯仰角
		KFloat ki_pitch = new KFloat(0);
		MapCore.GetConfiguration(NE_MapParam_Object.NE_Map_PitchAngle.value(),
				ki_pitch);
		mp_preOperate.setPitch( ki_pitch.FloatVal());
		this.viewContext.setMapParams(mp_preOperate);
	}

	/**
	 * [internal]获取手势操作前的参数
	 * 
	 * @return [internal]手势操作前的MapParams参数
	 */
	public static MapParams getPreMapParam() {
		return mp_preOperate;
	}

	public void updateCamera(KCameraUpdate update) {
		m_render.cameraEnque(update);
		requestRender();
	}
	
	public void setMapBearing(KCameraPosition target) {
		m_render.SetCameraRotateQueue(target);
	}
	
	public void setMapCameraMove(KCameraPosition target) {
		m_render.SetCameraMoveQueue(target);
	}
	
	public void setMapTilt(KCameraPosition target) {
		m_render.SetCameraTiltQueue(target);
	}
	
	public void setMapZoom(KCameraPosition target) {
		m_render.SetCameraZoomQueue(target);
	}
	
	// 设置当前手势的handler, 在activity onResume时调用
	public void setCurrentGestureHandler(){
		m_gestureManager.setCurrentHandler();
	}
	
	public void setMapType(int type) {
		m_render.mapTypeQueue(type);
		requestRender();
	}
	

	public void clearOverlayReq() {
		m_render.clearOverlayReq();
	}

	//--------------zhic 2d support begin---------------
    int m_width = 0;
    int m_height = 0;	
    @Override
    protected void onSizeChanged(int w, int h, int oldw, int oldh)
    {
    	//m_SkiaBitmap = Bitmap.createBitmap(w, h, Bitmap.Config.RGB_565);
//    	if(w>h)
//    	{
//    		m_SkiaBitmap = Bitmap.createBitmap(1280, 720, Bitmap.Config.RGB_565);
//    	}
//    	else
//    	{
//        	m_SkiaBitmap = Bitmap.createBitmap(720, 1280, Bitmap.Config.RGB_565);
//    	}
    	m_width = w;
    	m_height = h;
    	
    }
	  @Override
      protected void onDraw(Canvas canvas) {
          // Call into our C++ code that renders to the bitmap using Skia
//      	 Log.d("TAG", "onDraw");
//		  Paint p = new Paint(); //创建画笔
//		  p.setColor(Color.WHITE);
//		  Rect r = new Rect(100, 50, 300, 250);
//		  canvas.drawRect(r, p);

		  if(m_SkiaBitmap == null)
        	{
        		return;
        	}
	      MapJni.drawIntoBitmap(m_SkiaBitmap, m_width, m_height);

       	Paint paint = new Paint();
       	paint.setFilterBitmap(true);
       	
          Rect src = new Rect (0, 0, 720, 1080);
          Rect dst = new Rect (0, 0, m_width, m_height);
          canvas.drawBitmap(m_SkiaBitmap, src,dst, paint);
          
		  //canvas.drawBitmap(m_SkiaBitmap, 0, 0, null);

      }	
	//--------------end-------------

	public void clearCameraReq() {
		m_render.clearCameraReq();
	}
	
	// 手势消息 zhic ---------------------

	
	   @Override  
	   public boolean onDown(MotionEvent e) {  
		   Log.i("CZD", "onDown!");  
		   m_gestureManager.handleDown(e);
	       return true;  
	   }  
	     
	   /** 
	    * 监听滑动 
	    */  
	   @Override  
	   public boolean onFling(MotionEvent e1, MotionEvent e2, float velocityX, float velocityY) {  
	    //   Log.i("CZD", "Fling Happened!");  
			map.mHandler.removeMessages(KMap.MAP_GESTURE_END);
	       m_gestureManager.handleFling(e1, e2, velocityX, velocityY);
	       
	       Message message = new Message();
	       message.what = KMap.MAP_GESTURE_END;
	       map.mHandler.sendMessage(message);
	       return true;  
	   }  
	   @Override  
	   public void onLongPress(MotionEvent e) 
	   {   
		   Log.i("CZD", "onLongPress!");
		   m_gestureManager.handleLongPress(e);
	   }  
	   @Override  
	   public boolean onScroll(MotionEvent e1, MotionEvent e2, float distanceX, float distanceY) 
	   {  
		//   Log.i("CZD", "onScroll!");
		   
		   if( e2.getPointerCount() == 1 )
		   {
			   m_gestureManager.handleMove(e1,e2, distanceX, distanceY);
		   }
		   if( e2.getPointerCount() == 2 )
		   {
			   m_gestureManager.handleMovePoint2(e1,e2, distanceX, distanceY);
		   }		   
		   
	       return false;  
	   }  
	   @Override  
	   public void onShowPress(MotionEvent e) {  
		   Log.i("CZD", "onShowPress!");
	//	   m_gestureManager.handleDown(e);
	   }  
	   @Override  
	   public boolean onSingleTapUp(MotionEvent e) {  
		   Log.i("CZD", "onSingleTapUp!");
		   if (m_infoOverlayView == null || !m_infoOverlayView.singleTapUpOverlayClick(e)){
			   m_gestureManager.handleTapUp(e);
		   }
	       return true;  
	   }  

	public void showBuildings(boolean enabled) {
		m_render.setMapSwitchEnque(MapCore.MP_ENUM_SWITCH.MP_SHOW_BUILDING, enabled);
		requestRender();
	}

	public void showMapText(boolean enabled) {
		m_render.setMapSwitchEnque(MapCore.MP_ENUM_SWITCH.MP_SHOW_TEXT, enabled);
		requestRender();
	}

	public void clearTileCache(int _id) {
		m_render.clearTileCache(_id);
		requestRender();
	}  

	public void setMapTextZIndex(float index) {
		m_render.setMapTextZIndex(index);
		requestRender();
	}  

	public void showSkyBackground(boolean enabled) {
		m_render.setMapSwitchEnque(MapCore.MP_ENUM_SWITCH.MP_SHOW_SKY, enabled);
		requestRender();
	}

	public void setInfowindow(InfoView infoView, KMarker infowindowSubstitute) {
		m_render.setInfowindow(infoView, infowindowSubstitute);
	}  

	public void onDestroy() {
		m_gestureManager.stopTimerRefresh();
		m_render.onDestroy();
	}
    /**
     * 发起截屏
     * @param listener 截屏监听
     */
    public void getMapScreenShot(OnMapScreenShotListener listener) {
        m_render.getMapScreenShot(listener);
        requestRender();
    }

    public void setTrafficEnabled(boolean bEnable) {
        m_render.setTrafficEnabled(bEnable);
        requestRender();
    }

    public void setTrafficEventEnabled(boolean enabled) {
        m_render.setTrafficEventEnabled(enabled);
        requestRender();
    }
}
