package com.cmmap.internal.driver.view;

import android.graphics.PointF;
import android.text.TextUtils;

import com.cmmap.internal.mapcore.Lg;
import com.cmmap.internal.maps.model.KCameraPosition;
import com.cmmap.internal.maps.model.KLatLng;

/**
 * [internal]更新参数
 * 
 * @author liangz
 * 
 */
public class MapParams {
	/**
	 * [internal]位置
	 */
	// private LatLon latlon;
	/**
	 * [internal]描画的平行移动量
	 */
	private PointF translation;
	/**
	 * [internal]比例尺
	 */
	// private float level;
	/**
	 * [internal]旋转角
	 */
	// private float rotation;
	/**
	 * [internal]样式
	 */
	private String styleId;
	/**
	 * [internal]俯仰角
	 */
	// private short pitch;
	/**
	 * [internal]地图的当前模式
	 */
	// private Mode mode;
	private PointF _viewPortSize;

	private KCameraPosition cameraPosition;

	public MapParams(KLatLng latlon, PointF translation, float level,
			float rotation, String styleId, float pitch, PointF size) {

		this.translation = new PointF(translation.x, translation.y);

		this.styleId = styleId;

		// this.mode = mode;
		this._viewPortSize = new PointF(size.x, size.y);

		this.cameraPosition = new KCameraPosition(latlon, level, pitch, rotation);
	}

	public MapParams(PointF translation, String styleId, PointF size,
			KCameraPosition position) {
		this.translation = new PointF(translation.x, translation.y);

		this.styleId = styleId;

		// this.mode = mode;
		this._viewPortSize = new PointF(size.x, size.y);

		this.cameraPosition = new KCameraPosition(position.target, position.zoom, position.tilt, position.bearing);
	}

	public MapParams() {
		// this.latlon = new LatLon(-1, -1);
		// this.level = -1;
		this.translation = new PointF();
		// this.rotation = Integer.MAX_VALUE;
		this.styleId = "";
		// this.pitch = Short.MAX_VALUE;
		// this.mode = Mode.TWO_DIMENSION;
		this._viewPortSize = new PointF();
		Lg.e("cameraposition", "mapparaminitial");
		this.cameraPosition = new KCameraPosition();
	}

	public KLatLng getLatlon() {
		return cameraPosition.target;
	}

	public void setLatlon(KLatLng latlon) {
		// Lg.e("cameraposition", this.cameraPosition.toString());
		// Lg.e("latlon", latlon.toString());
		this.cameraPosition.target = latlon;
	}

	public PointF getTranslation() {
		return translation;
	}

	public void setTranslation(PointF translation) {
		this.translation = translation;
	}

	public float getLevel() {
		return cameraPosition.zoom;
	}

	public void setLevel(float level) {
		this.cameraPosition.zoom = level;
	}

	public float getRotation() {
		return this.cameraPosition.bearing;
	}

	public void setRotation(float rotation) {
		this.cameraPosition.bearing = rotation;
	}

	public String getStyleId() {
		return styleId;
	}

	public void setStyleId(String styleId) {
		this.styleId = styleId;
	}

	public float getPitch() {
		return this.cameraPosition.tilt;
	}

	public void setPitch(float pitch) {
		this.cameraPosition.tilt = pitch;
	}

	// public Mode getMode() {
	// return mode;
	// }
	//
	// public void setMode(Mode mode) {
	// this.mode = mode;
	// }

	public PointF getViewPortSize() {
		return this._viewPortSize;
	}

	public KCameraPosition getCameraPosition() {
		return cameraPosition;
	}

	public void setCameraPosition(KCameraPosition cameraPosition) {
		this.cameraPosition = cameraPosition;
	}

	@Override
	public boolean equals(Object o) {
		if (o == null) {
			return false;
		}

		if (!(o instanceof MapParams)) {
			return false;
		}

		MapParams param = (MapParams) o;

		if (this.cameraPosition != null && param.getCameraPosition() != null) {
			if (!this.cameraPosition.equals(param.getCameraPosition())) {
				return false;
			}
		} else {
			return false;
		}
		// if ((this.latlon == null && param.getLatlon() != null) ||
		// !this.latlon.equals(param.getLatlon())) {
		// return false;
		// }
		//
		// if (Float.compare(this.level, param.getLevel()) != 0) {
		// return false;
		// }

		// if ((this.mode == null && param.getMode() != null) ||
		// this.mode.getValue() != param.getMode().getValue()) {
		// return false;
		// }

		// if (this.rotation != param.getRotation()) {
		// return false;
		// }
		//
		// if (this.pitch != param.getPitch()) {
		// return false;
		// }

		if ((TextUtils.isEmpty(this.styleId) && !TextUtils.isEmpty(param
				.getStyleId())) || !this.styleId.equals(param.getStyleId())) {
			return false;
		}

		if (!pointEquals(this.translation, param.getTranslation())) {
			return false;
		}

		if (!pointEquals(this._viewPortSize, param.getViewPortSize())) {
			return false;
		}
		return true;
	}

	private static boolean pointEquals(PointF point1, PointF point2) {
		return (Float.compare(point1.x, point2.x) == 0)
				&& (Float.compare(point1.y, point2.y) == 0);
	}

	public void clear() {
		// this.latlon = null;
		// this.level = -1;
		this.translation = null;
		// this.rotation = Integer.MAX_VALUE;
		this.styleId = "";
		// this.pitch = Short.MAX_VALUE;
		// this.mode = Mode.TWO_DIMENSION;
		this._viewPortSize = null;
		this.cameraPosition = null;
	}
}
