package com.cmmap.internal.driver.view;

import java.nio.IntBuffer;
import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.graphics.Bitmap;
import android.graphics.Point;
import android.graphics.PointF;
import android.location.Location;
import android.opengl.GLES20;
import android.opengl.GLSurfaceView.Renderer;
import android.os.Bundle;
import android.os.Debug;
import android.os.Debug.MemoryInfo;
import android.os.Message;
import android.util.Log;
import android.util.Pair;
import android.view.MotionEvent;

import com.cmmap.api.maps.Map.OnMapScreenShotListener;
import com.cmmap.api.maps.model.BitmapDescriptor;
import com.cmmap.internal.driver.base.MapTool;
import com.cmmap.internal.driver.base.PubHead;
import com.cmmap.internal.mapcore.KBitmap;
import com.cmmap.internal.mapcore.KFloat;
import com.cmmap.internal.mapcore.KGEOCOORD;
import com.cmmap.internal.mapcore.KInt;
import com.cmmap.internal.mapcore.KRect;
import com.cmmap.internal.mapcore.Lg;
import com.cmmap.internal.mapcore.MapCore;
import com.cmmap.internal.mapcore.MapCore.NE_MapParam_Object;
import com.cmmap.internal.mapcore.MapCore.NE_UserShape_DelRule;
import com.cmmap.internal.mapcore.MapOverLayShape;
import com.cmmap.internal.mapcore.MapOverLayShapeArc;
import com.cmmap.internal.mapcore.MapOverLayShapeCircle;
import com.cmmap.internal.mapcore.MapOverLayShapeGround;
import com.cmmap.internal.mapcore.MapOverLayShapeMarker;
import com.cmmap.internal.mapcore.MapOverLayShapeNavigateArrow;
import com.cmmap.internal.mapcore.MapOverLayShapePoints;
import com.cmmap.internal.mapcore.MapOverLayShapePolyLine;
import com.cmmap.internal.mapcore.MapOverLayShapePolygon;
import com.cmmap.internal.mapcore.MapOverlayBitmap;
import com.cmmap.internal.mapcore.MapOverlayShapeTile;
import com.cmmap.internal.maps.KCameraUpdate;
import com.cmmap.internal.maps.KMap;
import com.cmmap.internal.maps.KMap.InfoView;
import com.cmmap.internal.maps.KMapInitializer;
import com.cmmap.internal.maps.KMapView;
import com.cmmap.internal.maps.KProjection;
import com.cmmap.internal.maps.model.KCameraPosition;
import com.cmmap.internal.maps.model.KGroundOverlay;
import com.cmmap.internal.maps.model.KLatLng;
import com.cmmap.internal.maps.model.KMarker;
import com.cmmap.internal.maps.model.KOverlay;
import com.cmmap.internal.maps.model.KShapeOverlay;
import com.cmmap.internal.maps.model.KShapeOverlayFactory;

public class MapRenderer implements Renderer {
	private String TAG = "MapRenderer";

	public KMap map;
	private ViewContext viewContext;
	private static final MapParams curMapParams = new MapParams();
	
	private BlockingQueue<Location> m_qLocation = new ArrayBlockingQueue<Location>(20);
	
	//[internal] 地图移动操作列表
	private BlockingQueue<Point> m_qMovePoints = new ArrayBlockingQueue<Point>(100);
	private BlockingQueue<Float> pitchingQueue = new ArrayBlockingQueue<Float>(10);

	//[internal] Zoom操作列表
	private BlockingQueue<ZoomOperate> m_qZoomOperates = new ArrayBlockingQueue<ZoomOperate>(50);

	//[internal] 3D旋转列表
	private BlockingQueue<Integer> m_qRotateOperates = new ArrayBlockingQueue<Integer>(50);
		
	private boolean m_rotate_center_changed = false;
	
	private PointF m_rotate_center = new PointF(0.5f, 0.5f);
	
	//[internal] 是否是单指连续点击
	private boolean m_singleDoubleClick = false;
	//[internal] 是否是双指点击
	private boolean m_doubleFingerClick = false;
	
	//[internal] 是否双指缩放
	private boolean m_doubleFingerScaling = false;

	//[internal] 物理减速移动效果
	private boolean m_bPhysicMove = false;

	//[internal] 旋转操作的类型, 默认为旋转地图
	private int m_iRotateType = PubHead.MP_ROTATE_MAP;

	private int m_iZoomLongPressHandler = 0;

	//[internal] 地图缩放，移动，标志
	private int mOperatorType = PubHead.OPERATOR_OTHER;
	
	public Point m_center_point = new Point();

	//[internal] UI交互接口
	RendererDrawTemplet mDrawTemplet;
	
	public MotionEvent event;
	public int mDoubleFingleMidX;
	public int mDoubleFingleMidY;
	public float mDoubleFingleScalingRate;
	
	public int v_height = 1080;
	public int v_width = 720;
	
	private KCameraUpdate updateRequst = null;
	private boolean bNeedUpdatePosition = false;
	
	private BlockingQueue<Integer> m_qMapTypeOperates = new ArrayBlockingQueue<Integer>(1);
	private BlockingQueue<Integer> m_qLocTypeOperates = new ArrayBlockingQueue<Integer>(1);
	private BlockingQueue<MapOverLayShape> m_qAddOverlay = new LinkedBlockingQueue<MapOverLayShape>();
	private BlockingQueue<MapOverLayShape> m_qUpdateOverlay = new LinkedBlockingQueue<MapOverLayShape>();
	private BlockingQueue<KShapeOverlay> m_qRemoveOverlay = new LinkedBlockingQueue<KShapeOverlay>();
	private BlockingQueue<Integer> m_qClearTileCache = new LinkedBlockingQueue<Integer>();
	
	private BlockingQueue<KCameraPosition> m_qCameraBearingQueue = new ArrayBlockingQueue<KCameraPosition>(1);
	private BlockingQueue<KCameraPosition> m_qCameraMoveQueue = new ArrayBlockingQueue<KCameraPosition>(1);
	private BlockingQueue<KCameraPosition> m_qCameraTiltQueue = new ArrayBlockingQueue<KCameraPosition>(1);
	private BlockingQueue<KCameraPosition> m_qCameraZoomQueue = new ArrayBlockingQueue<KCameraPosition>(1);
	
	private BlockingQueue<Integer> m_qRemoveBitmapQueue = new LinkedBlockingQueue<Integer>();
	private BlockingQueue<MapOverlayBitmap> m_qAddBitmapQueue = new LinkedBlockingQueue<MapOverlayBitmap>();
	private BlockingQueue<MapOverlayBitmap> m_qUpdateBitmapQueue = new LinkedBlockingQueue<MapOverlayBitmap>();
	
	private BlockingQueue<Pair<MapCore.MP_ENUM_SWITCH, Boolean>> m_qMapSwitch = new LinkedBlockingQueue<Pair<MapCore.MP_ENUM_SWITCH,Boolean>>();
	private BlockingQueue<Float> m_qMapTextZIndex = new ArrayBlockingQueue<Float>(1);
	private BlockingQueue<Boolean> m_qTrafficStatus = new ArrayBlockingQueue<Boolean>(1);
	private BlockingQueue<Boolean> m_qTrafficEventStatus = new ArrayBlockingQueue<Boolean>(1);
	
	InfoView m_infoview = null;
	private MapOverLayShapeMarker m_infoViewSubtitude = null;
	private Integer m_infowindowSync = new Integer(0);
	
	public boolean m_set_low_memory = false;
	public boolean m_enable_low_memory = false;
	private Timer m_checkMemoryTimer = null;
	
	public void setLowMemory(boolean enable) {
		m_set_low_memory = true;
		m_enable_low_memory = enable;
	}
	
	public MapRenderer(ViewContext viewContext, RendererDrawTemplet tem) {
		this.viewContext = viewContext;
		this.mDrawTemplet = tem;
		startCheckMemoryTimer();
	}
	
	// 定时检查内存
	private void startCheckMemoryTimer() {
		if (m_checkMemoryTimer != null) {
			m_checkMemoryTimer.cancel();
			m_checkMemoryTimer = null;
		}
		m_checkMemoryTimer = new Timer();
		m_checkMemoryTimer.schedule(new TimerTask() {
			public void run() {
				boolean low_memory = false;
				long FreeMemSize = map.mapView.getFreeMemorySize();
				if (FreeMemSize < 40000) {
					low_memory = true;
				}
				setLowMemory(low_memory);
            }
        }, 
        30000,
        30000);
    }

	/**
	 * [internal]更新位置
	 * @param location
	 */
	public void locationEnque(KLatLng location) {
//		Lg.e(TAG, "locationEnque--->lat:" + location.latitude + "/lon:" + location.longitude);
		if (location != null) {
			MapCore.SetMapCenter(MapTool.doubleToLong(location.longitude), MapTool.doubleToLong(location.latitude));
		}
	}

	/**
	 * [internal]更新level
	 * @param level
	 */
	public void levelEnque(float level) {
		MapCore.SetMapScaleLevel(level);
//		Lg.i(TAG, "update level--->level:" + level);
	}

	/**
	 * [internal]更新旋转角
	 * @param rotation
	 */
	public void rotationEnque(int rotation) {
		KFloat k_rotate = new KFloat(0);
		MapCore.GetConfiguration(NE_MapParam_Object.NE_Map_RotateAngle.value(), k_rotate);
		float l_rotate = k_rotate.FloatVal() + rotation;
		if (l_rotate > 360) {
			l_rotate -= 360;
		}
		if (l_rotate < 0) {
			l_rotate += 360;
		}
		if(l_rotate == 360){
			l_rotate = 0;
		}
		MapCore.SetConfiguration(NE_MapParam_Object.NE_Map_RotateAngle, Float.valueOf(String.valueOf(l_rotate)));
//		Lg.i(TAG, "update rotation--->rotate:" + l_rotate);
	}

	/**
	 *[internal] 更新俯仰角
	 * @param pitch
	 */
	public void pitchEnque(float pitch) {
		MapCore.SetConfiguration(MapCore.NE_MapParam_Object.NE_Map_PitchAngle, pitch);
	}

	public void pitchingAngleEnque(Float pitch) {
		pitchingQueue.offer(pitch);
	}

	/**
	 * 改变地图的camera
	 * @param position
	 */
	public void cameraEnque(KCameraUpdate update){
		if (update != null && update.targetPosition != null) {
			if (updateRequst == null) {
				updateRequst = new KCameraUpdate();
				updateRequst.m_byUpdateType = 0;
			}
			if(update.isUpdateBearing()) {
				updateRequst.targetPosition.bearing = update.targetPosition.bearing;
			}
			if(update.isUpdateLatLng()) {
				updateRequst.targetPosition.target = update.targetPosition.target;
			}
			if(update.isUpdateTilt()) {
				updateRequst.targetPosition.tilt = update.targetPosition.tilt;
			}
			if(update.isUpdateZoom()) {
				updateRequst.targetPosition.zoom = update.targetPosition.zoom;
			}
			updateRequst.m_byUpdateType |= update.m_byUpdateType;
			bNeedUpdatePosition = true;			
		}
	}
	
	public void updateCamera()
	{
//		Log.w("CZD", "cnv updateCamera");
		if(bNeedUpdatePosition && updateRequst != null 
				&& updateRequst.targetPosition != null)
		{
			MapCore.updateCamera(updateRequst);
			updateRequst.m_byUpdateType = 0;
			bNeedUpdatePosition = false;
		//	Log.w("CZD", "cnv updatePosition");
		}

		if(!m_qCameraMoveQueue.isEmpty()) {
			KCameraPosition target = m_qCameraMoveQueue.poll();
			if(target == null) {
				return;
			}
			MapCore.SetMapCenter(MapTool.doubleToLong(target.target.longitude), MapTool.doubleToLong(target.target.latitude));
		//	Log.w("CZD", "cnv update m_qCameraMoveQueue");
		}
		
		if(!m_qCameraBearingQueue.isEmpty()) {
			KCameraPosition target = m_qCameraBearingQueue.poll();
			if(target == null) {
				return;
			}
			MapCore.SetConfiguration(NE_MapParam_Object.NE_Map_RotateAngle, target.bearing);
		}
		
		if(!m_qCameraTiltQueue.isEmpty()) {
			KCameraPosition target = m_qCameraTiltQueue.poll();
			if(target == null) {
				return;
			}
			MapCore.SetConfiguration(NE_MapParam_Object.NE_Map_PitchAngle,  target.tilt);
		}
		
		if(!m_qCameraZoomQueue.isEmpty()) {
			KCameraPosition target = m_qCameraZoomQueue.poll();
			if(target == null) {
				return;
			}
			MapCore.SetMapScaleLevel(target.zoom);
		}
		
	}
	//[internal] 添加图形消息入列
	void shapeOverlayEnque(KShapeOverlay overlay) {
		KShapeOverlayFactory.createShapeBitmap(overlay, m_qAddBitmapQueue);
		MapOverLayShape m_Shape = KShapeOverlayFactory.createShapeOverlay(overlay);
		m_qAddOverlay.offer(m_Shape);
		
	}
	
	// 处理overlay请求
	private void processOverlay() {
		processAddOverlay();
		processUpdateOverlay();
		processRemoveOverlay();
		processInfoWindow();
	}
	
	private void processInfoWindow() {
		synchronized (m_infowindowSync) {
			if (m_infoview != null && m_infoViewSubtitude != null) {
				KLatLng pos = m_infoview.calcSubtitudePos();
				if (pos != null && !pos.equals(m_infoViewSubtitude.m_opt.getPosition())) {
					m_infoViewSubtitude.m_opt.position(pos);
					MapCore.UpdateUserShape(m_infoViewSubtitude);
				}
			}			
		}
	}

	private void processAddOverlay() {
		// 添加覆盖物的纹理信息
		while(!m_qAddBitmapQueue.isEmpty()) {
			MapOverlayBitmap overlayBitmap = m_qAddBitmapQueue.poll();
			if (overlayBitmap != null) {
				switch (overlayBitmap.getType()) {
				case MapOverLay_Shape_Ground:
				case MapOverLay_Shape_PolyLine:
					KBitmap.add(overlayBitmap.getId(), overlayBitmap.getBitmap(), 1);
					break;
				case MapOverLay_Shape_Marker:
					KBitmap.add(overlayBitmap.getId(), overlayBitmap.getBitmap(), 0);
				default:
					break;
				}
				overlayBitmap.recycle();
			}
		}
		
		// 添加覆盖物的属性信息
		while(!m_qAddOverlay.isEmpty())
		{
			MapOverLayShape m_Shape = m_qAddOverlay.poll();
			if(m_Shape == null){
				return;
			}
			if (m_Shape instanceof MapOverLayShapeCircle) {
				int i = MapCore.AddUserShape((MapOverLayShapeCircle) m_Shape);
				Lg.e(TAG, "add circle---->return:" + i);
			} else if (m_Shape instanceof MapOverLayShapePolyLine) {
				int i = MapCore.AddUserShape((MapOverLayShapePolyLine) m_Shape);
				Lg.e(TAG, "add polyline---->return:" + i);
			} else if (m_Shape instanceof MapOverLayShapePolygon) {
				int i = MapCore.AddUserShape((MapOverLayShapePolygon) m_Shape);
				Lg.e(TAG, "add polygon---->return:" + i);
			} else if (m_Shape instanceof MapOverLayShapeGround) {
 				int i = MapCore.AddUserShape((MapOverLayShapeGround) m_Shape);
				Lg.e(TAG, "add point---->return:" + i);
			} else if (m_Shape instanceof MapOverLayShapeArc) {
				int i = MapCore.AddUserShape((MapOverLayShapeArc) m_Shape);
				Lg.e(TAG, "add arc---->return:" + i);
			} else if (m_Shape instanceof MapOverLayShapeNavigateArrow) {
				int i = MapCore.AddUserShape((MapOverLayShapeNavigateArrow) m_Shape);
				Lg.e(TAG, "add arc---->return:" + i);
			} else if (m_Shape instanceof MapOverLayShapeMarker) {
				MapOverLayShapeMarker marker = (MapOverLayShapeMarker) m_Shape;
				// 如果需要执行落下动画，避免在地图上闪现marker，现将位置设置为动画初始值
				if (marker.m_opt.isAnimatesDrop()) {
					marker.m_opt.setX(0);
					marker.m_opt.setY(0);
				}
				int i = MapCore.AddUserShape(marker);
			} else if (m_Shape instanceof MapOverlayShapeTile) {
				int i = MapCore.AddUserShape((MapOverlayShapeTile)m_Shape);
			} else if (m_Shape instanceof MapOverLayShapePoints) {
				int i = MapCore.AddUserShape((MapOverLayShapePoints) m_Shape);
			}
		}
	}

	//[internal] 移除图形消息入列
	void removeShapeEnque(KShapeOverlay overlay) {
		m_qRemoveOverlay.offer(overlay);
	}
	
	private void processRemoveOverlay() {
		while(!m_qRemoveOverlay.isEmpty())
		{
			KShapeOverlay overlay = m_qRemoveOverlay.poll();
			if (overlay != null) {
				int ids [] = {overlay.getOverlayId()};
				int returnValue = MapCore.DelUserShape(NE_UserShape_DelRule.NE_UserShape_IDs, ids);
				Lg.e(TAG, String.format("remove shape id:%d----return value:%d", overlay.getOverlayId(), returnValue));
			}
		}
	}

	//[internal] 更新图形消息入列
	void updateShapeEnque(KShapeOverlay overlay, int updateMask) {
		MapOverLayShape m_Shape = KShapeOverlayFactory.createShapeOverlay(overlay);
		m_Shape.setUpdateMask(updateMask);
		m_qUpdateOverlay.offer(m_Shape);
	}
	
	private void processUpdateOverlay() {
		// 更新覆盖物的属性信息
		while(!m_qUpdateOverlay.isEmpty())
		{
			MapOverLayShape m_Shape = m_qUpdateOverlay.poll();
			if (m_Shape.m_updateMask != KOverlay.UPDATE_ALL) {
				MapCore.UpdateUserShapeParam(m_Shape);
				continue;
			}
			
			if (m_Shape instanceof MapOverLayShapeCircle) {
				int i = MapCore.UpdateUserShape((MapOverLayShapeCircle) m_Shape);
				Lg.e(TAG, "update circle" + m_Shape.m_i_shapid + "---->return:" + i);
			} else if (m_Shape instanceof MapOverLayShapePolyLine) {
				int i = MapCore.UpdateUserShape((MapOverLayShapePolyLine) m_Shape);
				Lg.e(TAG, "update polyline" + m_Shape.m_i_shapid + "---->return:" + i);
			} else if (m_Shape instanceof MapOverLayShapePolygon) {
				int i = MapCore.UpdateUserShape((MapOverLayShapePolygon) m_Shape);
				Lg.e(TAG, "update polygon" + m_Shape.m_i_shapid + "---->return:" + i);
			} else if (m_Shape instanceof MapOverLayShapeGround) {
				int i = MapCore.UpdateUserShape((MapOverLayShapeGround) m_Shape);
				Lg.e(TAG, "update point" + m_Shape.m_i_shapid + "---->return:" + i);
			} else if (m_Shape instanceof MapOverLayShapeArc) {
				int i = MapCore.UpdateUserShape((MapOverLayShapeArc) m_Shape);
				Lg.e(TAG, "update arc" + m_Shape.m_i_shapid + "---->return:" + i);
			} else if (m_Shape instanceof MapOverLayShapeNavigateArrow) {
				int i = MapCore.UpdateUserShape((MapOverLayShapeNavigateArrow) m_Shape);
			} else if (m_Shape instanceof MapOverLayShapeMarker) {
				int i = MapCore.UpdateUserShape((MapOverLayShapeMarker) m_Shape);
			} else if (m_Shape instanceof MapOverlayShapeTile) {
				int i = MapCore.UpdateUserShape((MapOverlayShapeTile)m_Shape);
			} else if (m_Shape instanceof MapOverLayShapePoints) {
				int i = MapCore.UpdateUserShape((MapOverLayShapePoints)m_Shape);
			} 
		}
		
		// 更新覆盖物的纹理信息
		while(!m_qUpdateBitmapQueue.isEmpty()) {
			MapOverlayBitmap overlayBitmap = m_qUpdateBitmapQueue.poll();
			if (overlayBitmap != null) {
				switch (overlayBitmap.getType()) {
				case MapOverLay_Shape_Ground:
				case MapOverLay_Shape_PolyLine:
					KBitmap.add(overlayBitmap.getId(), overlayBitmap.getBitmap(), 1);
					break;
				case MapOverLay_Shape_Marker:
					KBitmap.add(overlayBitmap.getId(), overlayBitmap.getBitmap(), 0);
				default:
					break;
				}
				// 临时bitmap释放
				overlayBitmap.recycle();
			}
		}
		
		// 释放无效的纹理信息
		while(!m_qRemoveBitmapQueue.isEmpty()) {
			Integer id = m_qRemoveBitmapQueue.poll();
			if(id != null)
			{
				KBitmap.remove(id);
			}
		}
		
		//　清空栅格覆盖物的缓存
		while(!m_qClearTileCache.isEmpty()) {
			Integer id = m_qClearTileCache.poll();
			if (id != null) {
				MapCore.clearTileCache(id);
			}
		}
	}
	
	void updateShapeBitmapEnque(KShapeOverlay overlay) {
		KShapeOverlayFactory.createShapeBitmap(overlay, m_qUpdateBitmapQueue);
	}
	
	void removeShapeBitmapEnque(KShapeOverlay overlay) {
		if(overlay instanceof KGroundOverlay) {
			KGroundOverlay groundOverlay = (KGroundOverlay)overlay;
			m_qRemoveBitmapQueue.offer(groundOverlay.getImage().describeContents());
		} else if(overlay instanceof KMarker) {
			KMarker marker = (KMarker)overlay;
			ArrayList<BitmapDescriptor> icons = marker.getIcons();
			for(int i = 0 ; i < icons.size(); ++i) {
				m_qRemoveBitmapQueue.offer(icons.get(i).describeContents());
			}
		}
	}

	//[internal] 设置为单指连续两次点击
	public void SetSingleDoubleClick(boolean iSingleDoubleClick) {
		m_singleDoubleClick = iSingleDoubleClick;
	}

	//[internal] 设置为单指连续两次点击
	public void SetDoubleFingerClick(boolean iDoubleFingerClick) {
		m_doubleFingerClick = iDoubleFingerClick;
	}
	
	public void SetDoubleFingerScaling(boolean bMoving) {
		m_doubleFingerScaling = bMoving;
	}

	//[internal] 设置zoom button long press 操作
	public void SetZoomLongPressHandler(int iZoom) {
		m_iZoomLongPressHandler = iZoom;
	}

	public void SetRotateType(int iType) {
		m_iRotateType = iType;
	}

	public int GetRotateType() {
		return m_iRotateType;
	}

	//[internal] 确认是否有地图操作
	public boolean CheckMapOperate() {
		if (!m_bPhysicMove && m_qMovePoints.isEmpty() && m_qZoomOperates.isEmpty() && m_qRotateOperates.isEmpty() && m_qMapTypeOperates.isEmpty()) {
			return false;
		}
		return true;
	}
	
	public void setLocation(Location obj) {
		m_qLocation.clear();
		m_qLocation.offer(obj);
	}
	
	//[internal] 设置移动地图参数
	public void SetMovePoint(int x, int y) {
		m_qMovePoints.offer(new Point(x, y));
	}

	//[internal] 设置缩放地图参数
	public void SetZoomOprate(int iOperate, float fParam) {
		ZoomOperate zo = new ZoomOperate(iOperate, fParam);
		m_qZoomOperates.offer(zo);
	}

	//[internal] 设置缩放地图参数
	public void SetZoomOprate(int iOperate) {
		ZoomOperate zo = new ZoomOperate(iOperate, 0);
		m_qZoomOperates.offer(zo);

	}

	//[internal] 设置旋转地图的角度
	public void SetRotateOprate(int iDegree) {
		m_qRotateOperates.offer(iDegree);

	}
	
	public void SetCameraRotateQueue(KCameraPosition target) {
		m_qCameraBearingQueue.clear();
		m_qCameraBearingQueue.offer(target);
	}
	
	public void SetCameraMoveQueue(KCameraPosition target) {
		m_qCameraMoveQueue.clear();
		m_qCameraMoveQueue.offer(target);
	}
	
	public void SetCameraTiltQueue(KCameraPosition target) {
		m_qCameraTiltQueue.clear();
		m_qCameraTiltQueue.offer(target);
	}
	
	public void SetCameraZoomQueue(KCameraPosition target) {
		m_qCameraZoomQueue.clear();
		m_qCameraZoomQueue.offer(target);
	}

	//[internal] 删除移动地图参数
	public void ClearMovePoint() {

		m_qMovePoints.clear();

	}

	//[internal] 删除移动地图参数
	public void ClearZoomList() {

		m_qZoomOperates.clear();

	}

	//[internal] 删除旋转地图参数
	public void ClearRotateList() {
		m_qRotateOperates.clear();
	}

	//[internal] 简化过多的坐标, 保留1-2个点
	public void SimplifyMovePoint(int iOX, int iOY) {

		if (m_qMovePoints.isEmpty()) {
			return;
		}

		int dx = 0;
		int dy = 0;

		while (!m_qMovePoints.isEmpty()) {
			Point point = m_qMovePoints.poll();

			if (point == null) {
				break;
			}
			dx += point.x;
			dx -= iOX;

			dy += point.y;
			dy -= iOY;
		}
		m_qMovePoints.offer(new Point(iOX + dx, iOY + dy));

	}
	
	private void CheckLocation() {
		if (!m_qLocation.isEmpty()) {
			Location loc = m_qLocation.poll();
			if (loc == null) 
			{
				return;
			}
			MapCore.SetLocation(loc);
		}
	}

	//[internal] move map in GL Thread
	private void MoveMap() {
		if (!m_qMovePoints.isEmpty()) 
		{
			//Lg.e("MapRender", "MoveMap");
			Point point = m_qMovePoints.poll();
			//[internal] HMI清空了队列
			if (point == null) 
			{
				return;
			}

			MapCore.MoveToScreenPt(point.x , point.y, false);
			mOperatorType = PubHead.OPERATOR_MOVE;
		}

	}

	//[internal] 缩放操作
	private void ZoomMap() {
		//[internal] 处理长按zoom按钮的操作
		if (m_iZoomLongPressHandler == PubHead.MP_ZOOM_STEP_IN_DEFAULT || m_iZoomLongPressHandler == PubHead.MP_ZOOM_STEP_OUT_DEFAULT) {
			ZoomOperate zo = new ZoomOperate(m_iZoomLongPressHandler, 0);
			m_qZoomOperates.offer(zo);

		}

		if (!m_qZoomOperates.isEmpty()) {
			//[internal] 合并操作
			while (!m_qZoomOperates.isEmpty()) {
				ZoomOperate zo = m_qZoomOperates.poll();
				//Lg.e(TAG, "zo.fParam " + zo.fParam);

				KInt kj = new KInt(0);
				MapCore.GetMapScaleValue(kj);
				int currentScaleValue = kj.IntVal();
				MapCore.SetMapScaleValue((int) (zo.fParam * currentScaleValue),false);
				mOperatorType = PubHead.OPERATOR_ZOOM;
			}
		}
	}

	//[internal] 地图俯仰角操作
	void SetMapPitchingAngle() {
		if (!pitchingQueue.isEmpty()) {
			float iAngle = 0;

			//[internal] 合并操作
			while (!pitchingQueue.isEmpty()) {
				iAngle = pitchingQueue.poll();
			}

			MapCore.SetConfiguration(MapCore.NE_MapParam_Object.NE_Map_PitchAngle, iAngle);
		}
	}

	//[internal] 旋转操作
	private void RotateMap() {
		if (!m_qRotateOperates.isEmpty()) {
			int iAngle = 0;
			
//			Log.i(TAG, "iAngle:"+iAngle);
			//[internal] 合并操作
			while (!m_qRotateOperates.isEmpty()) {
				iAngle = m_qRotateOperates.poll();
				m_qRotateOperates.clear();
			}
			MapCore.SetConfiguration(MapCore.NE_MapParam_Object.NE_Map_RotateAngle, (float)iAngle);
		}
	}

	public void setMapRotateCenter(PointF pointF) {
		m_rotate_center_changed = true;
		m_rotate_center = pointF;
	}
	
	public void checkSetMapRotateCenter() {
		if (m_rotate_center_changed)
		{
			if (m_rotate_center != null)
			{
				m_center_point.x = (int)(m_rotate_center.x * viewContext.getViewport().x);
				m_center_point.y = (int)(m_rotate_center.y * viewContext.getViewport().y);
				MapCore.setMapRotateCenter(m_center_point.x, m_center_point.y, 0);
			}
			m_rotate_center_changed = false;
		}
	}
	
	//[internal] 单指连续两次点击处理
	private void SignelDoubleClick() {
		if (m_singleDoubleClick) {
			MapCore.ZoomInAndKeepingScreenRelativePosition((int)event.getX(), (int)event.getY(), true);
			m_singleDoubleClick = false;
		}

	}

	//[internal] 双指点击处理
	private void DoubleFingerClick() {
		if (m_doubleFingerClick) {
						
			KRect rect = new KRect();
			MapCore.GetMapViewport(rect);

			float rate = 2.0f;
			int width = (int)rect.iRight - (int)rect.iLeft;
			int height = (int)rect.iTop - (int)rect.iBottom;
			float cx = (float)mDoubleFingleMidX - (float)width/2;
			float cy = (float)mDoubleFingleMidY - (float)height/2;
			float delta_x = cx * rate - cx;
			float delta_y = cy * rate - cy;
			
			int screen_x = (int)(delta_x + width/2);
			int screen_y = (int)(delta_y + height/2);
			MapCore.ZoomInAndKeepingScreenRelativePosition(screen_x, screen_y, false);
			
			m_doubleFingerClick = false;
		}
	}
	
	private void DoubleFingerScaling() {
		if (m_doubleFingerScaling) {
			// 以地图中心为缩放中心
			KInt scale = new KInt(0);
			MapCore.GetMapScaleValue(scale);
			int new_scale = (int) ((float) (scale.IntVal()) / mDoubleFingleScalingRate);
			Log.i("scale", "setScale" + new_scale);
			MapCore.SetMapScaleValue(new_scale, false);

			// 以手势操作中心为缩放中心
/*
			KRect rect = new KRect();
			MapCore.GetMapViewport(rect);
			
			float cx = (float) mDoubleFingleMidX - (float) m_center_point.x;
			float cy = (float) mDoubleFingleMidY - (float) m_center_point.y;

			float delta_x = cx * mDoubleFingleScalingRate - cx;
			float delta_y = cy * mDoubleFingleScalingRate - cy;
//			Log.i(TAG, "delta_x:"+delta_x);
//			Log.i(TAG, "delta_y:"+delta_y);
//			Log.i(TAG, "mDoubleFingleScalingRate:"+mDoubleFingleScalingRate);
			MapCore.mapMoveScale(delta_x + m_center_point.x, delta_y + m_center_point.y, mDoubleFingleScalingRate, -1);
*/
			m_doubleFingerScaling = false;
		}
	}
	
	private void setMapType()
	{
		if(!m_qMapTypeOperates.isEmpty())
		{
			int type = m_qMapTypeOperates.poll();
			MapCore.SetConfiguration(NE_MapParam_Object.NE_Map_Theme, (float)type);
		}
		
		if(!m_qLocTypeOperates.isEmpty())
		{
			int locType = m_qLocTypeOperates.poll();
			MapCore.SetConfiguration(NE_MapParam_Object.NE_Map_LocType, locType);
		}
	}
	
	public void mapTypeQueue(int type) {
		m_qMapTypeOperates.clear();
		m_qMapTypeOperates.offer(type);
	}
	
	public void locTypeQueue(int type) {
		m_qLocTypeOperates.clear();
		m_qLocTypeOperates.offer(type);
	}
	
	int m_refresh_debug_mask = 0xFFFF;
	
	public void setFastRefresh(boolean b) {
		if(m_b_fast_refresh && !b)
		{
			m_b_fast_refresh = b;
		}
		else
		{
			m_b_fast_refresh = b;
		}
		
	}
	
	public boolean isFastRefresh() {
		return m_b_fast_refresh;
	}
	
	boolean m_b_clear_map_text = false;
	long m_l_clear_map_text_timer = 0;
	
	public void clearMapText(long timer, boolean force) {
		Log.i("","clearMapText() 1 timer:" + timer);
		if (force) {
			//Log.i("","clearMapText() 2 force");
			m_l_clear_map_text_timer = timer;
			m_b_clear_map_text = true;
		}else if (timer - m_l_clear_map_text_timer > 10000) {
			//Log.i("","clearMapText() 3 timer:" + timer);
			m_l_clear_map_text_timer = timer;
			m_b_clear_map_text = true;
		}
	}	
	
	boolean m_b_timer_refresh = false;
	public void setTimerRefresh(boolean b) {
		m_b_timer_refresh = b;
	}
	
	private boolean m_b_fast_refresh = false;
	
	public void setRefreshDebugMask(int debug_mask) {
		m_refresh_debug_mask = debug_mask;
	}
	
	public int getRefreshDebugMask() {
		return m_refresh_debug_mask;
	}
	
	
	public void onDrawFrame(GL10 gl) {
		synchronized(viewContext){
//			Log.i("Map", "onCameraChangeFinish onDrawFrame");
		// 设置地图旋转中心
		checkSetMapRotateCenter();
		//[internal] 拖动地图
		MoveMap();
		//[internal] 伸缩地图
		ZoomMap();
		//[internal] 旋转地图
		RotateMap();
		//地図俯仰角操作
		SetMapPitchingAngle();
		//[internal] 单指连续点击
		SignelDoubleClick();
		//[internal] 双指点击
		DoubleFingerClick();
		
		DoubleFingerScaling();
		// camera动画刷新
		updateCamera();
		
		// 卫星定位
		CheckLocation();

		// 设置地图类型（白天、黑夜、卫星）
		setMapType();
		synchronized (map.m_OverlayProcSync) {
			// 处理overlay
			processOverlay();			
		}
		//设置地图元素
		proSetMapParam();
		// 更新实时交通状态
		updateTrafficStatus();
		//[internal] 刷新前的UI操作
		mDrawTemplet.beforeMapRefresh(mOperatorType);
		
		if (m_b_clear_map_text) {
			MapCore.clearMapText();
			m_b_clear_map_text = false;
		}
		
		if (m_b_timer_refresh)
		{
			MapCore.SetFastRefreshMap(false);
			m_b_timer_refresh = false;
		}else
		{
			MapCore.SetFastRefreshMap(m_b_fast_refresh);
		}
		
		if (m_set_low_memory)
		{
			MapCore.setLowMemory(m_enable_low_memory);
			m_set_low_memory = false;
		}
		
		// 刷新地图
		synchronized (mMapScreenShotLock) {            
		    boolean refreshStatusValid = (mMapScreenShotListener != null);
		    int refreshStatus = MapCore.refresh(refreshStatusValid, m_refresh_debug_mask);
		    updateMapParams();
		    processMapScreenShot(refreshStatus);
        }
}
		//[internal] 刷新后的UI操作
		mDrawTemplet.afterMapRefresh(mOperatorType);

		mDrawTemplet.refreshShaderView(gl);
	}

	public void onSurfaceChanged(GL10 gl, int width, int height) {
		
		Log.i("onConfiguration", "onSurfaceChanged:width"+width+",height:"+height);
		PointF viewport = new PointF(width, height);
		viewContext.getViewport().x = viewport.x;
		viewContext.getViewport().y = viewport.y;
		KProjection.getInstance().setSize(new PointF(viewport.x, viewport.y));

		v_width = width;
		v_height = height;
		// calculate new map rotate center

		// set new viewport
		KRect viewPort = new KRect();
		viewPort.iTop = height;
		viewPort.iLeft = 0;
		viewPort.iRight = width;
		viewPort.iBottom = 0;
//		Lg.e("err", String.format("onSurfaceChanged===========top:%d,bottom:%d,left:%d,right:%d", viewPort.iTop,viewPort.iBottom,viewPort.iLeft,viewPort.iRight));
		// 重新设置视口信息
		int m = MapCore.setMapViewport(viewPort, KMapView.getDeviceWidth(), KMapView.getDeviceHeight(),
				KMapView.getDPIx(), KMapView.getDPIy(), (width > height)? true:false);
		
//		Lg.e(TAG, "MapCore.setMapViewport() return: " + m);
		
		// set new map rotate center
		if (m_rotate_center != null)
		{
//			Log.i("xxx", "setMapRotateCenter onSurfaceChanged ---" + viewContext.getViewport().x + ", " + viewContext.getViewport().y);
			// 重新设置地图旋转中心
			m_center_point.x = (int)(m_rotate_center.x * width);
			m_center_point.y = (int)(m_rotate_center.y * height);
			MapCore.setMapRotateCenter(m_center_point.x, m_center_point.y, 0);
		}
		else
		{
			m_center_point.x = (int)(0.5 * width);
			m_center_point.y = (int)(0.5 * height);
		}
		
		Message message = new Message();
		message.what = KMap.MAP_SURFACE_CHANGE;
		map.mHandler.sendMessage(message);
	}

	public void onSurfaceCreated(GL10 gl, EGLConfig config) {
		if (KMapInitializer.isIntialized()) {
		}
		MapCore.InitializeRender();
	}

	private long m_lLastNativeMemory = 0;
	private long m_lLastDalvikMemory = 0;

    private OnMapScreenShotListener mMapScreenShotListener = null;
    private Integer mMapScreenShotLock = new Integer(1000);

	void PrintMemInfo() {
		MemoryInfo mem = new MemoryInfo();
		Debug.getMemoryInfo(mem);

		long lCurrentNativeMemory = Debug.getNativeHeapAllocatedSize();
		//[internal] 进程已用的内存大小
		long lCurrentDalvikMemory = Runtime.getRuntime().totalMemory(); 

		//[internal] 进程可用总的内存大小
		long lMaxDalvikMemory = Runtime.getRuntime().maxMemory(); 

		long deltaNative = lCurrentNativeMemory - m_lLastNativeMemory;
		long deltaDalvik = lCurrentDalvikMemory - m_lLastDalvikMemory;

		m_lLastNativeMemory = lCurrentNativeMemory;
		m_lLastDalvikMemory = lCurrentDalvikMemory;

		String strDelta = String.format("Memory Native (%dK), Dalvik (%dK / %dK), DELTA Native / Dalvik  = %d / %d, ", lCurrentNativeMemory >> 10, lCurrentDalvikMemory >> 10, lMaxDalvikMemory >> 10,
				deltaNative, deltaDalvik);
		Log.d("mem", strDelta);

		KGEOCOORD coord = new KGEOCOORD();
		Log.d("mem", String.format("Large Memory (%d, %d)", coord.lLatitude, coord.lLongitude));

	}


	public static class ZoomOperate {
		public int iOperate;
		public float fParam;

		public ZoomOperate(int _iOperate, float _fParam) {
			iOperate = _iOperate;
			fParam = _fParam;

		}

	}

	//[internal] 刷新后数值的改变，数值改变会引起info图层的变化，会引起map中onChanged的变化
	private void updateMapParams() {
		//[internal] 清空MapParam中的数值
		//curMapParams.clear();
		//[internal] 位置
		KGEOCOORD k_coord = new KGEOCOORD();
		MapCore.GetMapCenter(k_coord);
		curMapParams.setLatlon(new KLatLng(MapTool.longToDouble(k_coord.lLatitude).doubleValue(), MapTool.longToDouble(k_coord.lLongitude).doubleValue()));
		//[internal] 描画的平行移动量
		curMapParams.setTranslation(this.viewContext.getMapParams().getTranslation());
		//[internal] 缩放比例
		KFloat ki_level = new KFloat(0);
		int i = MapCore.GetMapScaleLevel(ki_level);
		curMapParams.setLevel(ki_level.FloatVal());
		//[internal] 旋转角
		KFloat ki_rotate = new KFloat(0);
		MapCore.GetConfiguration(NE_MapParam_Object.NE_Map_RotateAngle.value(), ki_rotate);
		//Lg.e(TAG, String.format("旋转角为：%d", ki_rotate.IntVal()));
		curMapParams.setRotation(ki_rotate.FloatVal());
		//[internal] 样式
		KFloat ki_theme = new KFloat(0);
		MapCore.GetConfiguration(NE_MapParam_Object.NE_Map_Theme.value(), ki_theme);
		KFloat ki_textSize = new KFloat(0);
		MapCore.GetConfiguration(NE_MapParam_Object.NE_Map_TextSize.value(), ki_textSize);
		curMapParams.setStyleId(MapTool.styleIntToString((int)(ki_theme.FloatVal()), (int)(ki_textSize.FloatVal())));
		//[internal] 俯仰角
		KFloat ki_pitch = new KFloat(0);
		MapCore.GetConfiguration(NE_MapParam_Object.NE_Map_PitchAngle.value(), ki_pitch);
		curMapParams.setPitch(ki_pitch.FloatVal());
		this.viewContext.setMapParams(curMapParams);
	}

	public void clearOverlayReq() {
		processOverlay();
	}

	public void clearCameraReq() {
		bNeedUpdatePosition = false;
		m_qCameraBearingQueue.clear();
		m_qCameraMoveQueue.clear();
		m_qCameraTiltQueue.clear();
		m_qCameraZoomQueue.clear();
		m_qMovePoints.clear();
		pitchingQueue.clear();
		m_qZoomOperates.clear();
		m_qRotateOperates.clear();
	}
	
	public void setMapSwitchEnque(MapCore.MP_ENUM_SWITCH eSwitch, boolean enabled) {
		m_qMapSwitch.offer(new Pair<MapCore.MP_ENUM_SWITCH, Boolean>(eSwitch, enabled));
	}
	
	public void proSetMapParam() {
		while (!m_qMapSwitch.isEmpty()) {
			Pair<MapCore.MP_ENUM_SWITCH, Boolean> req = m_qMapSwitch.poll();
			MapCore.SetMapSwitch(req.first, req.second);
		}
		
		if(!m_qMapTextZIndex.isEmpty()) {
			Float zIndex = m_qMapTextZIndex.poll();
			if(zIndex != null) {
				MapCore.SetMapTextZIndex(zIndex);
			}
		}
	}

	public void clearTileCache(int _id) {
		m_qClearTileCache.offer(_id);
	}

	public void setMapTextZIndex(float index) {
		m_qMapTextZIndex.clear();
		m_qMapTextZIndex.offer(index);
	}

	public void setInfowindow(InfoView infoView, KMarker infowindowSubstitute) {
		synchronized (m_infowindowSync) {
			m_infoview = infoView;
			m_infoViewSubtitude = (MapOverLayShapeMarker) KShapeOverlayFactory.createShapeOverlay(infowindowSubstitute);			
		}
	}

	public void onDestroy() {
		if (m_checkMemoryTimer != null) {
			m_checkMemoryTimer.cancel();
			m_checkMemoryTimer = null;
		}
	}

    public void getMapScreenShot(OnMapScreenShotListener listener) {
        synchronized (mMapScreenShotLock) {            
            mMapScreenShotListener = listener;
        }
    }
    
    private void processMapScreenShot(int status) {
        if (mMapScreenShotListener == null) {
            return;
        }
        
        // mapGLSurface截屏
        int b[] = new int[v_width * v_height];
        int bt[] = new int[v_width * v_height];
        IntBuffer buffer = IntBuffer.wrap(b);
        buffer.position(0);
        GLES20.glReadPixels(0, 0, v_width, v_height, GLES20.GL_RGBA, GLES20.GL_UNSIGNED_BYTE, buffer);
        for (int i = 0; i < v_height; ++i) {
            // OpenGL bitmap is incompatible with Android bitmap
            // and so, some correction need.
            for (int j = 0; j < v_width; ++j) {
                int pix = b[i * v_width + j];
                int pb = (pix >> 16) & 0xff;
                int pr = (pix << 16) & 0x00ff0000;
                int pix1 = (pix & 0xff00ff00) | pr | pb;
                bt[(v_height - i - 1) * v_width + j] = pix1;
            }
        }
        Bitmap mapBitmap = Bitmap.createBitmap(bt, v_width, v_height, Bitmap.Config.ARGB_8888);
        
        // 发消息到主线程，对外回调
        Message msg = map.mHandler.obtainMessage();
        msg.what = KMap.MAP_SCREEN_SHOT;
        Bundle dataBundle = new Bundle();
        dataBundle.putParcelable("bitmap", mapBitmap);
        msg.setData(dataBundle);
        msg.obj = mMapScreenShotListener;
        msg.arg1 = status;
        map.mHandler.sendMessage(msg);
        
        mMapScreenShotListener = null;
    }

    public void setTrafficEnabled(boolean bEnable) {
        m_qTrafficStatus.poll();
        m_qTrafficStatus.offer(bEnable);
    }

    public void setTrafficEventEnabled(boolean enabled) {
        m_qTrafficEventStatus.poll();
        m_qTrafficEventStatus.offer(enabled);
    }
    
    private void updateTrafficStatus()
    {
        if (!m_qTrafficStatus.isEmpty()) {
            boolean enable = m_qTrafficStatus.poll();
            MapCore.SetRTICUse(enable);
        }
        
        if (!m_qTrafficEventStatus.isEmpty()) {
            boolean enable = m_qTrafficEventStatus.poll();
            MapCore.SetEventUse(enable);
        }
    }
}
