package com.cmmap.internal.mapcore;

import java.io.UnsupportedEncodingException;
import java.util.List;

import com.cmmap.api.maps.model.BitmapDescriptor;
import com.cmmap.internal.maps.model.KLatLng;
import com.cmmap.internal.maps.model.KPolylineOptions;

public class MapOverLayShapePolyLine extends MapOverLayShape{
	
	public KPolylineOptions m_options;
	
	public MapOverLayShapePolyLine()
	{
		m_shape_type = NE_MapOverLayShape_Type.MapOverLay_Shape_PolyLine;
	}

	/**
	* 取序列化的字节数
	*/
	int getSize() {
		int iStrBufLength = 0;
		byte [] bytes= null;
		try 
		{
			iStrBufLength = 4 + m_options.getlineName().getBytes("UTF-8").length;
			for (int i = 0; i < m_options.getLineNameList().size(); ++ i)
			{
				iStrBufLength += (4 + m_options.getLineNameList().get(i).getBytes("UTF-8").length);
			}
		} 
		catch (UnsupportedEncodingException e) 
		{
			e.printStackTrace();
		} 
		
		return 77 + 8 * m_options.getPoints().size() + iStrBufLength +  4 * (m_options.getColorValues().size() + m_options.getCustomTextureList().size() + m_options.getCustomTextureIndex().size() + m_options.getLineNameIndex().size());
	}
	
	byte[] Serialize(byte[] pBuf, KInt iIndex) {
		ByteUlit.putboolean(pBuf, m_b_visible, iIndex);		// 1 byte	
		ByteUlit.putLong4(pBuf, m_i_level, iIndex);
		ByteUlit.putFloat(pBuf, m_i_zindex, iIndex);		// 8 bytes
		ByteUlit.putLong4(pBuf, m_i_shapid, iIndex);		// 4 bytes
		ByteUlit.putLong4(pBuf, m_shape_type.value(), iIndex);		// 4 bytes
		
		ByteUlit.putFloat(pBuf, m_options.getWidth(), iIndex);		// 8 bytes				
		ByteUlit.putLong4(pBuf, m_options.getColor(), iIndex);		// 4 bytes
		
		int tex_id = -1;
		if (m_options.getCustomTexture() != null)
		{
			tex_id = m_options.getCustomTexture().describeContents();
		}		
		ByteUlit.putLong4(pBuf, tex_id, iIndex);		// 4 bytes
		
		List<KLatLng> points = m_options.getPoints();
		int point_size = points.size();
		ByteUlit.putLong4(pBuf, point_size, iIndex);					// 4 bytes		
		for (int i = 0; i < point_size; i++){
			KLatLng a = points.get(i);
			double x = a.longitude*3600.0*1024.0;
			double y = a.latitude*3600.0*1024.0;
			ByteUlit.putLong4(pBuf, (int)x, iIndex);					// 4 bytes
			ByteUlit.putLong4(pBuf, (int)y, iIndex);					// 4 bytes
		}
		
		ByteUlit.putboolean(pBuf, m_options.isDottedLine(), iIndex);	// 1 byte
		ByteUlit.putboolean(pBuf, m_options.isGeodesic(), iIndex);		// 1 byte
		ByteUlit.putboolean(pBuf, m_options.isUseTexture(), iIndex);	// 1 byte
		ByteUlit.putboolean(pBuf, m_options.getLineNameVisible(), iIndex); // 1 byte
		ByteUlit.SerializeString(pBuf, m_options.getlineName(), iIndex);
		
		ByteUlit.putLong4(pBuf, (int)m_options.getNameColor(), iIndex);		// 4 bytes
		ByteUlit.putLong4(pBuf, (int)m_options.getNameEdgeColor(), iIndex);	// 4 bytes
		ByteUlit.putLong4(pBuf, (int)m_options.getNameFontSize(), iIndex);	// 4 bytes
		
		List<Integer> lstColor = m_options.getColorValues();	
		int iColorSize = lstColor.size();	// 4 byte
		ByteUlit.putLong4(pBuf, iColorSize, iIndex);
		for (int i = 0; i < iColorSize; ++i)
		{
			ByteUlit.putLong4(pBuf, lstColor.get(i).intValue(), iIndex);	// 4 byte
		}

		List<BitmapDescriptor> lstBitmapDescriptor = m_options.getCustomTextureList();			
		int iBitmapSize = lstBitmapDescriptor.size();
		ByteUlit.putLong4(pBuf, iBitmapSize, iIndex);	// 4 byte
		for (int i = 0; i < iBitmapSize; ++i)
		{
			int bitmap_id = -1;
			if (lstBitmapDescriptor.get(i) != null)
			{
				bitmap_id = lstBitmapDescriptor.get(i).describeContents();
			}	
			ByteUlit.putLong4(pBuf, bitmap_id, iIndex);	// 4 byte
		}
		
		List<Integer> lstTextureIndex = m_options.getCustomTextureIndex();	
		int iTextureIndexSize = lstTextureIndex.size();
		ByteUlit.putLong4(pBuf, iTextureIndexSize, iIndex);	// 4 byte
		for (int i = 0; i < iTextureIndexSize; ++i)
		{
			ByteUlit.putLong4(pBuf, lstTextureIndex.get(i).intValue(), iIndex);	// 4 byte
		}
		
		List<String> lstLineName = m_options.getLineNameList();			
		int iNameSize = lstLineName.size();
		ByteUlit.putLong4(pBuf, iNameSize, iIndex);	// 4 byte
		for (int i = 0; i < iNameSize; ++i)
		{
			ByteUlit.SerializeString(pBuf, lstLineName.get(i), iIndex);
		}
		
		List<Integer> lstNameIndex = m_options.getLineNameIndex();	
		int iNameIndexSize = lstNameIndex.size();
		ByteUlit.putLong4(pBuf, iNameIndexSize, iIndex);	// 4 byte
		for (int i = 0; i < iNameIndexSize; ++i)
		{
			ByteUlit.putLong4(pBuf, lstNameIndex.get(i).intValue(), iIndex);	// 4 byte
		}
		return pBuf;
	}

	MapOverLayShapePolyLine Derialize(byte[] pBuf, KInt iIndex) {	
		m_shape_type = NE_MapOverLayShape_Type.MapOverLay_Shape_PolyLine;		
		return this;
	}
}
