package com.cmmap.internal.mapcore.offlinemap;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.SocketTimeoutException;
import java.net.URL;
import java.util.ArrayList;

import android.content.Context;
import android.os.Handler;
import android.os.Handler.Callback;
import android.os.Message;
import android.util.Log;

import com.cmmap.api.maps.MapException;
import com.cmmap.api.maps.offlinemap.OfflineMapStatus;
import com.cmmap.internal.driver.base.HttpTool;
import com.cmmap.internal.driver.base.HttpsPost;
import com.cmmap.internal.driver.broadcastReceiver.NetWorkMonitorReceiver;
import com.cmmap.internal.driver.broadcastReceiver.NetWorkMonitorReceiver.NET_TYPE;
import com.cmmap.internal.mapcore.MapCore;
import com.cmmap.internal.mapcore.MapJni;
import com.cmmap.internal.mapcore.offlinemap.WebServiceUtil.SERVICE_URL_TYPE;
import com.cmmap.internal.mapcore.offlinemap.entity.CityInfo;
import com.cmmap.internal.mapcore.offlinemap.entity.MapdataInfo;
import com.cmmap.internal.mapcore.offlinemap.entity.ObjectInfo;
import com.cmmap.internal.mapcore.offlinemap.entity.ProvinceInfo;
import com.cmmap.internal.maps.KMapInitializer;
import com.cmmap.internal.services.rtic.ByteHelper;

public class DownloadInstance {

	public static DownloadInstance instance;
	public static final String TAG = DownloadInstance.class.getName();
	public ArrayList<ObjectInfo> mDownloadQueue = new ArrayList<ObjectInfo>();
	private DownloadThread mDownloadThread = new DownloadThread();
	public static final int READ_BUFFER = 1024 * 1024 * 5;

	private HttpURLConnection mConn = null;
	private InputStream mInStream = null;
	
	public static Integer OfflineDataLock = Integer.valueOf(0);

	private static final int mCheckUpdate = 1;
	private Handler mHandler = new Handler(new Callback() {
		
		@Override
		public boolean handleMessage(Message msg) {
			String name = msg.getData().getString("name");
			if(msg.what == mCheckUpdate){
				ObjectInfo info = getInfoByName(name);
				if (info instanceof CityInfo || info.getAdCode() == CommonDefine.ALLDATA_CODE) {
					mDownloadStatusChangedListener.onUpdate(info);
				} else if (info instanceof ProvinceInfo) {
					ArrayList<CityInfo> citys = ((ProvinceInfo) info).getCitylist();
					for (int i = 0; i < citys.size(); i++) {
						ObjectInfo objInfo = citys.get(i);
						mDownloadStatusChangedListener.onUpdate(objInfo);
					}
				}
			}
			return false;
		}
	});
	public static VERSION_STATE VERSION_STATE = DownloadInstance.VERSION_STATE.OK_DOWNLOAD;

	public enum VERSION_STATE {
		OK_DOWNLOAD, OK_UPDATE, UPDATE_APP
	}

	public enum DOWNLOAD_REQ {
		REQ_NULL, REQ_PAUSE, REQ_REMOVE
	}

	private static OnDownloadStatusChangedListener mDownloadStatusChangedListener;

	public static interface OnDownloadStatusChangedListener {
		/**
		 * 发生错误触发
		 * 
		 * @param exception
		 */
		void onError(MapException exception);

		/**
		 * 下载中触发
		 * 
		 * @param info
		 *            下载中的对象
		 */
		void onDownload(ObjectInfo info);

		/**
		 * 删除的时候触发
		 * 
		 * @param info
		 *            删除的对象信息
		 */
		void onRemove(ObjectInfo info);

		/**
		 * 更新的时候触发
		 * 
		 * @param info
		 *            更新的对象信息
		 */
		void onUpdate(ObjectInfo info);
		
		/**
		 * 版本更新触发
		 */
		void onVersionUpdate();
	}

	private DownloadInstance() {
		mDownloadThread.start();
	}

	private static Context mContext;

	public static DownloadInstance getInstance(Context context, OnDownloadStatusChangedListener listener) {
		if (instance == null) {
			instance = new DownloadInstance();
		}
		mDownloadStatusChangedListener = listener;
		mContext = context;
		return instance;
	}
	
	public static DownloadInstance getInstance() {
		return instance;
	}
	
	public void onVersionUpdate() {
		mDownloadStatusChangedListener.onVersionUpdate();
	}

	public ObjectInfo getInfoByName(String name) {
		ObjectInfo info = MapJni.getitemBycityname(name);
		// 支持全国概要图
		if (info == null || info.getName().isEmpty()) {
			info = MapJni.getitemByprovincename(name);
		}
		return info;
	}

	private int getIndexofQueue(ObjectInfo info) {
		int iIndex = -1;
		for (int i = 0; i < mDownloadQueue.size(); ++i) {
			if (mDownloadQueue.get(i).getAdCode() == info.getAdCode()) {
				iIndex = i;
				break;
			}
		}
		return iIndex;
	}

	private void removeInQueue(ObjectInfo info) {
		int index = getIndexofQueue(info);
		if (index != -1) {
			mDownloadQueue.remove(index);
		}
	}
	
	private boolean isInfoDownloading(ObjectInfo info) {
//		if (mDownloadThread.mCurInfo != null) {
//			Log.i(TAG, "mDownloadThread.mCurInfo:" + " name:"
//					+ mDownloadThread.mCurInfo.getName() + " State:"
//					+ mDownloadThread.mCurInfo.getState());
//		}
		if (mDownloadThread.mCurInfo != null && mDownloadThread.mCurInfo.getAdCode() == info.getAdCode()) {
			return true;
		}
		return false;
	}

	public void downloadByName(String name) {
		ObjectInfo info = getInfoByName(name);

		int iIndex = -1;
		synchronized (mDownloadQueue) {
			iIndex = getIndexofQueue(info);
		}
		if (iIndex != -1) {
			return;
		}

		if (info.getState() == OfflineMapStatus.FINISHED || isInfoDownloading(info)) {
			return;
		}
		
		// 本地存在过期数据，先删除过期数据，再下载
		if (info.getState() == OfflineMapStatus.NEW_VERSION) {
			remove(name);
			info.setDownloadsize(0);
		}

		info.setState(OfflineMapStatus.WAITING);
		// 修改db
		MapJni.saveInfo(info);
		//回调对外onDownLoad函数
		mDownloadStatusChangedListener.onDownload(info);

		// 全国概要图单独处理
		if (info instanceof CityInfo  || info.getAdCode() == CommonDefine.ALLDATA_CODE) {
			synchronized (mDownloadQueue) {
				mDownloadQueue.add(info);
			}
			synchronized (mDownloadThread) {
				mDownloadThread.notifyAll();
			}
		} else if (info instanceof ProvinceInfo) {
			ProvinceInfo provinceInfo = (ProvinceInfo) info;
			ArrayList<CityInfo> citys = provinceInfo.getCitylist();
			for (int i = 0; i < citys.size(); i++) {
				CityInfo cityInfo = citys.get(i);
				boolean bCityInfoInQueue = false;
				synchronized (mDownloadQueue) {
					bCityInfoInQueue = (getIndexofQueue(cityInfo) != -1);
				}
				if (!bCityInfoInQueue
						&& !isInfoDownloading(cityInfo)
						&& cityInfo.getState() != OfflineMapStatus.FINISHED) {
					cityInfo.setState(OfflineMapStatus.WAITING);
					MapJni.saveInfo(cityInfo);
					mDownloadStatusChangedListener.onDownload(cityInfo);
					synchronized (mDownloadQueue) {
						mDownloadQueue.add(cityInfo);
					}
					synchronized (mDownloadThread) {
						mDownloadThread.notifyAll();
					}
				}
			}
		}

		if (NetWorkMonitorReceiver.NetType == NET_TYPE.no_net) {
			mDownloadStatusChangedListener.onError(new MapException(MapException.ERROR_CONNECTION));
		}
	}

	class DownloadThread extends Thread {
		private String strURL = MapJni.getOfflineURL(new byte[10]);
		private String version = MapCore.getSDKVersion();
		private DOWNLOAD_REQ mCurReq = DOWNLOAD_REQ.REQ_NULL;
		ObjectInfo mCurInfo = null;
		File saveFile = null;

		public void request(DOWNLOAD_REQ req) {
			synchronized (mCurReq) {
				mCurReq = req;
			}
		}

		@Override
		public void run() {
			while (true) {
				boolean bQueueEmpty = false;
				synchronized (mDownloadQueue) {
					bQueueEmpty = mDownloadQueue.isEmpty();
				}
				if (bQueueEmpty) {
					try {
						synchronized (this) {
							wait();
						}
					} catch (InterruptedException e) {
						e.printStackTrace();
					}
				} else {
					ObjectInfo info;
					synchronized (mDownloadQueue) {
						info = mDownloadQueue.remove(0);
					}
					info.setState(OfflineMapStatus.DOWNLOADING);
					mCurInfo = info;
					MapJni.saveInfo(info);
					mDownloadStatusChangedListener.onDownload(info);

					request(DOWNLOAD_REQ.REQ_NULL);
					downloadFile(info);
					mCurInfo = null;
				}
			}
		}

		private boolean isCityShareFile(MapdataInfo info)
		{
			int code = info.getCode();
			return !( (code < 10000) && (code > 0) );
		}
				
		private void downloadFile(ObjectInfo info) {
			ArrayList<MapdataInfo> filelist = null;
			if (info instanceof ProvinceInfo) {
				// 通过省份名字获得数据
				filelist = MapJni.getDownloadinfoByprovincename(info.getName());
			}
			if (info instanceof CityInfo) {
				// 通过城市名字获得数据
				filelist = MapJni.getDownloadinfoByname(info.getName());
			}
			Log.i(TAG, "fileListSize:" + filelist.size());
			if (filelist == null || filelist.size() == 0) {
				pauseObjectInfo(info);
				return;
			}
			ArrayList<String> urList = parseUrl(filelist);
			if (urList == null || urList.size() != filelist.size()) {
				pauseObjectInfo(info);
				return;
			}

			int i = 0;
			boolean bReq = true;
			boolean bExceptionInDownload = false;
			for (MapdataInfo mapdataInfo : filelist) {
				Log.i(TAG, "request Remove5");
				if (judgeReq(info) || !bReq) {
					return;
				}
				String realUrl = urList.get(i);
				bReq = download(info, mapdataInfo, realUrl, !isCityShareFile(mapdataInfo));
				if (bReq) {
					if (i == (urList.size() - 1)) {
						info.setState(OfflineMapStatus.DOWNLOADED);
						MapJni.saveInfo(info);
						mDownloadStatusChangedListener.onDownload(info);
						info.setState(OfflineMapStatus.UNZIPING);
						MapJni.saveInfo(info);
						mDownloadStatusChangedListener.onDownload(info);
					}
					MapJni.parserMapfile(saveFile.getAbsolutePath(), mapdataInfo.getLevel());
					//如果解析结果不对就remove 
				}
				else {
					bExceptionInDownload = true;
					break;
				}
				Log.i(TAG, "infoAdCode:" + info.getAdCode() + " i:" + i + "/" + filelist.size() + "mapdataInfo:" + mapdataInfo.getSize());
				i++;
			}
			Log.i(TAG, "request Remove6"+mCurReq);
			if (!bExceptionInDownload && !judgeReq(info) && bReq) {
				//由于城市共享文件的存在所以fileList下载完毕则设置fileSize与downloadSize相同 视为下载成功
				info.setDownloadsize(info.getFilesize());
				info.setState(OfflineMapStatus.FINISHED);
				MapJni.saveInfo(info);
				mDownloadStatusChangedListener.onDownload(info);
			}
		}

		private ArrayList<String> parseUrl(ArrayList<MapdataInfo> fileList) {
			MapdataInfo minfo = fileList.get(0);
			String strKey = KMapInitializer.STR_KEY;
			String strPackageName = KMapInitializer.STR_PACKAGE_NAME;
			String strSha1 = KMapInitializer.STR_SHA1;
			String str_body;
			if (minfo.getCode() == CommonDefine.ALLDATA_CODE) {
				str_body = "{\"auth\":{\"key\":\"%s\",\"service\":\"1\",\"user\":{\"type\":\"android\",\"android\":{\"package_name\":\"%s\",\"signature_sha1\":\"%s\"}},\"flow\":0},\"data\":{\"ver\":\"%s\",\"type\":\"offline\",\"func\":\"getOfflineFileUrl\",\"reqinfo\":{\"functype\":1}}}";
				str_body = String.format(str_body, strKey, strPackageName, strSha1, version);
			} else {
				str_body = getRealUrlList(fileList);
			}
			String URL = new StringBuilder(strURL).append("&").append(str_body).toString();
			byte[] rev = HttpTool.GetMapData(URL, false);
			ArrayList<String> urlList = new ArrayList<String>();
			if (rev == null || rev.length == 0) {
				return null;
			}
			int version = ByteHelper.byteArray2Int(rev, 0);
			// 高16位存大版本号，低16位存小版本号
			int v1 = ((version & 0x000000ff) << 8)
					| ((version & 0x0000ff00) >> 8);
			int v2 = ((version & 0x00ff0000) >> 8)
					| ((version & 0xff000000) >> 24);
			String versionString = v2 + "." + v1;
			if (!versionString.equals(UpdateThread.SERVICE_NUMBER)) {
				UpdateThread.checkVersion(versionString,
						UpdateThread.SERVICE_NUMBER);
			}

			int num = (int) rev[4];
			int pos = 5;
			if (minfo.getCode() == CommonDefine.ALLDATA_CODE) {
				String allUrl = new String(rev, 4, rev.length-4);
				urlList.add(allUrl);
			} else {
				for (int j = 0; j < num; j++) {
					byte[] bytelen = new byte[] { 0, 0, rev[pos + 1], rev[pos] };
					int len = WebServiceUtil.byteArrayToInt(bytelen);
					pos += 2;
					String tmp = new String(rev, pos, len);
					pos += len;
					urlList.add(tmp);
				}
			}

			Log.i(TAG, "urlListSize:"+urlList.size());
			for (int i = 0; i < urlList.size(); i++) {
				Log.i(TAG, "urlList :"+urlList.get(i));
			}
			return urlList;
		}

		private String getRealUrlList(ArrayList<MapdataInfo> list) {
			boolean haslevel2 = false;
			StringBuilder lv2codes = new StringBuilder("");
			StringBuilder lv3codes = new StringBuilder("");
			for (MapdataInfo info : list) {
				Log.v(TAG, "info.getCode() = " + info.getCode());
				if (info.getLevel() == CommonDefine.FILETYPE_LEVEL2) {
					int code_int = info.getCode(); // In world map there is using long variables. modified by yujun
					long code =  MapJni.UInt32ToInt64(code_int);
					
					lv2codes.append(code);
					haslevel2 = true;
				} else if (info.getLevel() == CommonDefine.FILETYPE_LEVEL3) {
//					if (!haslevel2) {
//						haslevel2 = true;
//					}
					int code_int = info.getCode(); // In world map there is using long variables. modified by yujun
					long code =  MapJni.UInt32ToInt64(code_int);
					
					if (lv3codes == null || lv3codes.length() == 0) {
						lv3codes.append(code);
					} else {
						lv3codes.append("," + code);
					}
				}
			}
			String str_body = "";
			String strKey = KMapInitializer.STR_KEY;
			String strPackageName = KMapInitializer.STR_PACKAGE_NAME;
			String strSha1 = KMapInitializer.STR_SHA1;
//			if(lv2codes.length() == 0){
//				lv2codes.append(0);
//			}
//			if(lv3codes.length() == 0){
//				lv3codes.append(0);
//			}
			if (haslevel2) {
				str_body = "{\"auth\":{\"key\":\"%s\",\"service\":\"1\",\"user\":{\"type\":\"android\",\"android\":{\"package_name\":\"%s\",\"signature_sha1\":\"%s\"}},\"flow\":0},\"data\":{\"ver\":\""
						+ version
						+ "\",\"type\":\"offline\",\"func\":\"getOfflineFileUrl\",\"reqinfo\":{\"functype\":2,\"citycode\":"
						+ lv2codes + ",\"meshcode\":[" + lv3codes + "]}}}";
				str_body = String.format(str_body, strKey, strPackageName,
						strSha1);
			} else {
				str_body = "{\"auth\":{\"key\":\"%s\",\"service\":\"1\",\"user\":{\"type\":\"android\",\"android\":{\"package_name\":\"%s\",\"signature_sha1\":\"%s\"}},\"flow\":0},\"data\":{\"ver\":\""
						+ version
						+ "\",\"type\":\"offline\",\"func\":\"getOfflineFileUrl\",\"reqinfo\":{\"functype\":3,\"citycode\":0,\"meshcode\":["
						+ lv3codes + "]}}}";
				str_body = String.format(str_body, strKey, strPackageName,
						strSha1);
			}
			return str_body;
		}

		private boolean download(ObjectInfo objInfo, MapdataInfo info, String url, boolean saveProcess) {

			InputStream inStream = null;
			FileOutputStream out = null;
			HttpURLConnection conn = null;

			long dlSize = 0; // file size may be more than 2^31, modified by yujun
			int iReadSize = 0;
//			long startTick = 0;
//			long jniTick = 0;
			try {
				Log.i("download", "offline url: "+ url);
				URL realUrl = new URL(url);
				if (url.startsWith("https")) {					
					try {
						HttpsPost.initHttpsURLConnection();
					} catch (Exception e) {
						Log.e("download", "offline " + e.toString());
					}
				}
				conn = (HttpURLConnection) realUrl.openConnection();
				conn.setConnectTimeout(5000);
				conn.setReadTimeout(10000);
				conn.setRequestMethod("POST");
				conn.setRequestProperty("Accept-Language", "zh-CN");
				conn.setRequestProperty("Referer", realUrl.toString());
				conn.setRequestProperty("Charset", "UTF-8");
//				conn.set
				// 带上当前已经下载的进度去服务器请求
				Log.i("downLoadInfo", " download name:" + objInfo.getName()+ " Range:" + info.getDownloadsize());
				conn.setRequestProperty("Range", "bytes=" + info.getDownloadsize() + "-");

				int filesize = conn.getContentLength();
				dlSize = MapJni.UInt32ToInt64(info.getDownloadsize()); // modified by yujun
				long totalsize = filesize + dlSize;
				info.setSize((int)totalsize);
				Log.v(TAG, " totalsize ==== " + totalsize);
				conn.connect();

				// 获取文件path
				File filesaveDir = DownloadInstance.getDefaultSavePath();
				int pos = url.lastIndexOf("/");
				String filename = url.substring(pos + 1, url.length());
				info.setPath(filename);
				saveFile = new File(filesaveDir, filename);
				try {
					out = new FileOutputStream(saveFile, true);
				} catch (Exception e) {
					e.printStackTrace();
					mDownloadStatusChangedListener.onError(new MapException(MapException.ERROR_NOT_AVAILABLE));
				}

				// 获取服务器来的流进行读取
				inStream =conn.getInputStream();
				byte[] buffer = new byte[READ_BUFFER];
				int offset = 0;
				if (!saveFile.exists()) {
					saveFile.createNewFile();
				}
//				startTick = System.currentTimeMillis();
				long lastSaveDBTick = 0;
				while ((offset = inStream.read(buffer, 0, buffer.length)) != -1) {
					if (judgeReq(objInfo)) {
						mCurInfo = null;
						return false;
					}
					try {
						out.write(buffer, 0, offset);
						dlSize += offset;
						iReadSize += offset;
						Log.i(TAG, "objInfo: offset:" + offset + " dlSize:" + dlSize + " buffer.length:"+buffer.length);
						// 2s写一次数据库，避免过于频繁写数据库影响下载速度
						if ((System.currentTimeMillis() - lastSaveDBTick > 2000) && (saveProcess)) {
//							long jnistartTick = System.currentTimeMillis();
							objInfo.setDownloadsize(objInfo.getDownloadsize() + iReadSize);
							objInfo.setState(OfflineMapStatus.DOWNLOADING);
							MapJni.saveInfo(objInfo);		// set province info
							info.setDownloadsize((int)dlSize);
							MapJni.setMapdatainfo(info);	// set city info
//							Log.i(TAG, " objInfo: name:" + objInfo.getName()
//									+ " status:" + objInfo.getState()
//									+ " downloadSize:" + objInfo.getDownloadsize()
//									+ " fileSize:" + objInfo.getFilesize());
							mDownloadStatusChangedListener.onDownload(objInfo);
//							jniTick += System.currentTimeMillis() - jnistartTick;
							lastSaveDBTick = System.currentTimeMillis();
							iReadSize = 0;
						}
//						Thread.sleep(10);
					} catch (Exception e) {
						e.printStackTrace();
						// 写文件异常，写数据库，保证文件与数据库信息一致
						if ((iReadSize > 0) && (saveProcess)) {
							objInfo.setDownloadsize(objInfo.getDownloadsize() + iReadSize);
							MapJni.saveInfo(objInfo);
							info.setDownloadsize((int)dlSize);
							MapJni.setMapdatainfo(info);							
						}
//						Log.i(TAG, " objInfo: name:" + objInfo.getName()
//								+ " status:" + objInfo.getState()
//								+ " downloadSize:" + objInfo.getDownloadsize()
//								+ " fileSize:" + objInfo.getFilesize());
						mDownloadStatusChangedListener.onDownload(objInfo);
						mDownloadStatusChangedListener.onError(new MapException(MapException.ERROR_IO));
						pauseObjectInfo(objInfo);
						getNetError();
						judgeReq(objInfo);
//						Log.i("xxx", "objInfo: download exception 1 tick: " + (System.currentTimeMillis() - startTick) + " jni tick: " + jniTick);
						return false;
					}
				}
				// 流读完后，更新数据库
				if ((iReadSize > 0) && (saveProcess)) {
					objInfo.setDownloadsize(objInfo.getDownloadsize() + iReadSize);
					MapJni.saveInfo(objInfo);
					info.setDownloadsize((int)dlSize);
					MapJni.setMapdatainfo(info);					
				}
//				Log.i("xxx", "objInfo: download tick: " + (System.currentTimeMillis() - startTick) + " jni tick: " + jniTick);
			} catch (IOException e) {
				e.printStackTrace();
				mDownloadStatusChangedListener.onError(new MapException(MapException.ERROR_CONNECTION));
				// 最新状态写入数据库
				if ((iReadSize > 0) && (saveProcess)) {
					objInfo.setDownloadsize(objInfo.getDownloadsize() + iReadSize);
					MapJni.saveInfo(objInfo);
					info.setDownloadsize((int)dlSize);
					MapJni.setMapdatainfo(info);							
				}
				if (e instanceof SocketTimeoutException) {					
					// 异常项插入请求队列第一项，下次循环再次请求下载
					synchronized (mDownloadQueue) {
						ArrayList<ObjectInfo> tempQueue = new ArrayList<ObjectInfo>(mDownloadQueue);
						mDownloadQueue.clear();
						mDownloadQueue.add(objInfo);
						mDownloadQueue.addAll(tempQueue);
					}
				} else {
					pauseObjectInfo(objInfo);
					getNetError();
				}
				judgeReq(objInfo);
//				Log.i("xxx", "objInfo: download exception 2 tick: " + (System.currentTimeMillis() - startTick) + " jni tick: " + jniTick);
				return false;   
			} finally {
				try {
					if (out != null) {
						out.flush();
						out.close();
					}
					if ((mCurReq != DOWNLOAD_REQ.REQ_REMOVE) && (saveProcess)) {
						info.setDownloadsize((int)dlSize);
						MapJni.setMapdatainfo(info);
					}
					mConn = conn;
					mInStream = inStream;
					// disconnect 时间过长，放到线程中执行
					Runnable r = new Runnable() {
						@Override
						public void run() {
							HttpURLConnection mThreadConn = mConn;
							InputStream mThreadInStream = mInStream;

							if (mThreadConn != null) {
								mThreadConn.disconnect();
								mThreadConn = null;
							}

							if (mThreadInStream != null) {
								try {
									mThreadInStream.close();
									Log.v(TAG, "mThreadInStream.close");
									mThreadInStream = null;
								} catch (IOException e) {
									e.printStackTrace();
								}
								mThreadInStream = null;
							}
						}
					};
					new Thread(r).start();
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
			return true;
		}

		private boolean judgeReq(ObjectInfo info) {
			boolean bRet = false;
			synchronized (mCurReq) {
				if (mCurReq == DOWNLOAD_REQ.REQ_REMOVE) {
					Log.i(TAG, "request Remove3");
					if (info.getState() != OfflineMapStatus.NONE) {
						// 删除文件
						removeObjectInfo(info);
					}
					bRet = true;
				} else if (mCurReq == DOWNLOAD_REQ.REQ_PAUSE) {
					pauseObjectInfo(info);
					bRet = true;
				}
			}
			return bRet;
		}
	}

	public static File getDefaultSavePath() {
		String savepath = MapJni.getSavepath();
		Log.v(TAG, " savepath = " + savepath);
		File savedir = new File(savepath);
		if (!savedir.exists()) {
			savedir.mkdirs(); // 创建文件
		}
		return savedir;
	}

	public void pause(String name) {
		ObjectInfo info = getInfoByName(name);
		Log.i(TAG, "pause:" + info.getName());
		if (info.getState() == OfflineMapStatus.FINISHED 
				|| info.getState() == OfflineMapStatus.PAUSE
				|| info.getState() == OfflineMapStatus.NONE) {
			return;
		}
		if (info instanceof CityInfo || info.getAdCode() == CommonDefine.ALLDATA_CODE) {
			pauseCityDownload(info);
		} else if (info instanceof ProvinceInfo) {
			ProvinceInfo pInfo = (ProvinceInfo) info;
			ArrayList<CityInfo> citys = pInfo.getCitylist();
			CityInfo downloadingInfo = null;
			synchronized (mDownloadQueue) {
				for (int i = 0; i < citys.size(); i++) {
					CityInfo cityInfo = citys.get(i);
					if (isInfoDownloading(cityInfo)) {
						// 先保存，再暂停，否则等待列表中的城市可能变成正在下载
						downloadingInfo = cityInfo;
					} else {
						if (cityInfo.getState() == OfflineMapStatus.WAITING) {
							removeInQueue(cityInfo);
						}
						pauseObjectInfo(cityInfo);
					}
				}
			}
			if (downloadingInfo != null) {
				mDownloadThread.request(DOWNLOAD_REQ.REQ_PAUSE);
			}
			pauseObjectInfo(info);
		}
	}
	
	private void pauseCityDownload(ObjectInfo info) {
		if (isInfoDownloading(info)) {
			mDownloadThread.request(DOWNLOAD_REQ.REQ_PAUSE);
		} else {
			if (info.getState() == OfflineMapStatus.WAITING) {
				removeInQueue(info);
			}
			pauseObjectInfo(info);
		}
	}

	private void pauseObjectInfo(ObjectInfo info) {
		if (info.getState() != OfflineMapStatus.PAUSE) {
			info.setState(OfflineMapStatus.PAUSE);
			MapJni.saveInfo(info);
			mDownloadStatusChangedListener.onDownload(info);
		}
	}
	
	private void getNetError(){
		pauseMQueue();
	}

	private void pauseMQueue() {
		synchronized (mDownloadQueue) {
			for (int i = 0; i < mDownloadQueue.size(); i++) {
				ObjectInfo objInfo = mDownloadQueue.remove(i);
				i--;
				objInfo.setState(OfflineMapStatus.PAUSE);
				mDownloadStatusChangedListener.onDownload(objInfo);
				MapJni.saveInfo(objInfo);
			}
		}
	}
	
	public void remove(String name) {
		ObjectInfo info = getInfoByName(name);
		if (info instanceof CityInfo || info.getAdCode() == CommonDefine.ALLDATA_CODE) {
			removeCityDownload(info);
		} else if (info instanceof ProvinceInfo) {
			ProvinceInfo pInfo = (ProvinceInfo) info;
			ArrayList<CityInfo> citys = pInfo.getCitylist();
			CityInfo downloadingInfo = null;
			// 先删除队列中的cityInfo，再请求删除正在下载的cityInfo
			synchronized (mDownloadQueue) {
				for (int i = 0; i < citys.size(); i++) {
					CityInfo cityInfo = citys.get(i);
					if (cityInfo.getState() == OfflineMapStatus.WAITING) {
						removeInQueue(cityInfo);
						removeObjectInfo(cityInfo);
					} else if (isInfoDownloading(cityInfo)) {
						downloadingInfo = cityInfo;
					} else if (cityInfo.getState() != OfflineMapStatus.NONE) {
						// 完成状态就从本地文件中删除 并且保存新状态到数据库
						removeObjectInfo(cityInfo);
					}
				}
				if (downloadingInfo != null) {
					Log.i(TAG, "request Remove1");
					mDownloadThread.request(DOWNLOAD_REQ.REQ_REMOVE);
				}
			}
			removeObjectInfo(info);
		}
	}

	private void removeCityDownload(ObjectInfo info) {
		if (info.getState() == OfflineMapStatus.DOWNLOADING) {
			// 如果是下载中的文件 就发出暂停命令
			mDownloadThread.request(DOWNLOAD_REQ.REQ_REMOVE);
		} else if (info.getState() == OfflineMapStatus.WAITING) {
			// 等待中的状态 就从队列中删除掉
			synchronized (mDownloadQueue) {
				removeInQueue(info);
			}
			removeObjectInfo(info);
		} else if (info.getState() != OfflineMapStatus.NONE) {
			// 完成状态就从本地文件中删除 并且保存新状态到数据库
			removeObjectInfo(info);
		}
	}

	private void removeObjectInfo(ObjectInfo info) {
		if (info.getState() != OfflineMapStatus.NONE) {
			if (info instanceof CityInfo
					|| info.getAdCode() == CommonDefine.ALLDATA_CODE) {
				removeFile(info.getName());
			}
			info.setState(OfflineMapStatus.NONE);
			info.setDownloadsize(0);
			MapJni.saveInfo(info);
			mDownloadStatusChangedListener.onRemove(info);
		}
	}

	public boolean removeFile(String name) {
		long startTime = System.currentTimeMillis();
		ArrayList<MapdataInfo> filelist = MapJni.getRemoveinfo(name);
		long endTime = System.currentTimeMillis();
		Log.i(TAG, "name :" + name + " delTime:" + (endTime - startTime));
		String savepath = MapJni.getSavepath();
		boolean bRet = true;
		for (int i = 0; i < filelist.size(); i++) {
			String filePath = savepath + "/" + filelist.get(i).getPath() + CommonDefine.FILE_KWI;
			File file = new File(filePath);
			
			if (file != null && file.exists() && file.isFile()) {
				if (!file.delete()) {
					bRet = false;
					break;
				}
			}
		}
		return bRet;
	}

	class updateThread extends Thread{
		public String name = "";
		@Override
		public void run() {
			//获取本地版本号
			int version = MapJni.getCitylistVersion();
			String localVersion = ByteHelper.getVersionByInt(version);
			String serverVersion = "";
			
			String str_body = WebServiceUtil.findServiceUrl(SERVICE_URL_TYPE.getOfflineDataVersion, 0, 0);
			byte[] respond = HttpTool.GetMapData(str_body, false);
			if (respond == null || respond.length == 0) {
			} else {
				String s = new String(respond);
				int j = Integer.valueOf(s);
				serverVersion = ByteHelper.getVersionByInt(j);
				
				if(!localVersion.equals(serverVersion)){
					str_body = WebServiceUtil.findServiceUrl(SERVICE_URL_TYPE.getCityList, 0, 0);
					String strURL = MapJni.getOfflineURL(new byte[10]);
					UpdateThread t = new UpdateThread(null,null);
					t.updateCityList(strURL, str_body, 100);
					onVersionUpdate();
				}
			}
			Message msg = Message.obtain();
			msg.what = mCheckUpdate;
			msg.getData().putString("name", name);
			mHandler.sendMessage(msg);
		}
	}
	public void update(final String name) {
		updateThread uThread = new updateThread();
		uThread.name = name;
		uThread.start();
	}

	public void processDownloadingInfo(ObjectInfo downloadingInfo) {
		if (downloadingInfo == null) {
			return;
		}
		if (mDownloadThread.mCurInfo == null) {
			downloadByName(downloadingInfo.getName());
		} else if (mDownloadThread.mCurInfo.getAdCode() != downloadingInfo.getAdCode()) {
			pauseObjectInfo(downloadingInfo);
		}
	}
	
	public void onDestroy(){
		pauseMQueue();
		pauseLoadingCity();
	}

	private void pauseLoadingCity() {
		mDownloadThread.request(DOWNLOAD_REQ.REQ_PAUSE);
	}
}
