package com.cmmap.internal.mapcore.offlinemap;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.Timer;
import java.util.TimerTask;

import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

import com.cmmap.api.maps.offlinemap.OfflineMapStatus;
import com.cmmap.internal.driver.base.HttpTool;
import com.cmmap.internal.driver.broadcastReceiver.NetWorkMonitorReceiver;
import com.cmmap.internal.driver.broadcastReceiver.NetWorkMonitorReceiver.NET_TYPE;
import com.cmmap.internal.mapcore.MapCore;
import com.cmmap.internal.mapcore.MapJni;
import com.cmmap.internal.mapcore.offlinemap.DownloadInstance.VERSION_STATE;
import com.cmmap.internal.mapcore.offlinemap.WebServiceUtil.SERVICE_URL_TYPE;
import com.cmmap.internal.mapcore.offlinemap.entity.CityInfo;
import com.cmmap.internal.mapcore.offlinemap.entity.MapdataInfo;
import com.cmmap.internal.mapcore.offlinemap.entity.ObjectInfo;
import com.cmmap.internal.mapcore.offlinemap.entity.ProvinceInfo;
import com.cmmap.internal.maps.KMapView;
import com.cmmap.internal.services.rtic.ByteHelper;

public class UpdateThread extends Thread {
    public static final String TAG = "UpdateThread";
    public static final int OK = 0;
    public static final int NETERROR = 1;
    public static final int SDCARDERROR = 2;
    public static final int DOWNLOAD_BUFFER = 1024*1024;
    public static final int CITYLISTDAT = 100;
    public static final int LV3LISTDAT = 101;
    private boolean mFinished = false;
    private int mError = OK;
    private Handler mHandler;
    private ObjectInfo mInfo;
    public static boolean isInit = false; 	
    private HttpURLConnection mConn = null;
	private InputStream mInStream = null;
	String strURL = MapJni.getOfflineURL(new byte[10]);
	public static String SERVICE_NUMBER = "0.0";
	public boolean firstTime = true; //每次更新完cityList置为true
    
    
    public UpdateThread(Handler handler,ObjectInfo info) {
        mHandler = handler;
        mInfo = info;
    }
    
	public void updateCityList(String url, String body, int type) {
		InputStream inStream = null;
		FileOutputStream out = null;
		HttpURLConnection conn = null;

		try {
			URL realUrl = new URL(url);
			conn = (HttpURLConnection) realUrl.openConnection();
			conn.setConnectTimeout(5000);
			conn.setRequestMethod("POST");
			conn.setRequestProperty("Accept-Language", "zh-CN");
			conn.setRequestProperty("Referer", realUrl.toString());
			conn.setRequestProperty("Charset", "UTF-8");
			OutputStreamWriter writer = new OutputStreamWriter(
					conn.getOutputStream());
			writer.write(body);
			writer.flush();
			writer.close();
			int fileSize = conn.getContentLength();
			Log.v(TAG, " fileSize = " + fileSize);
			conn.connect();

			File filesaveDir = DownloadInstance.getDefaultSavePath();
			Log.v(TAG, "downloadUrl = " + filesaveDir);

			int pos = url.lastIndexOf("/");
			String filename = url.substring(pos + 1, url.length());
			Log.v(TAG, " update filename = " + filename);
			File saveFile = new File(filesaveDir, filename);
			if (saveFile.isFile() && saveFile.exists()) {
				saveFile.delete();
			}

			int code = conn.getResponseCode();
			if (code == 200){
				inStream = conn.getInputStream();
				byte[] buffer = new byte[DOWNLOAD_BUFFER];
				int offset = 0;
				out = new FileOutputStream(saveFile, true);
				while ((offset = inStream.read(buffer, 0, 1024 * 100)) != -1) {
					try {
						out.write(buffer, 0, offset);
					} catch (Exception e) {
						e.printStackTrace();
						this.mError = OfflineMapStatus.EXCEPTION_SDCARD;
					}
				}

				synchronized (DownloadInstance.OfflineDataLock) {				
					MapJni.parserMapfile(saveFile.getAbsolutePath(), type);
				}
				Log.v(TAG, "Thread update finish");
			}		
		} catch (IOException e) {
			e.printStackTrace();
			this.mError = OfflineMapStatus.EXCEPTION_NETWORK;
		} finally {
			try {
				if (out != null) {
					out.close();
				}

				mConn = conn;
				mInStream = inStream;

				// disconnect 时间过长，放到线程中执行
				Runnable disconnectRunable = new Runnable() {

					@Override
					public void run() {

						HttpURLConnection mThreadConn = mConn;
						InputStream mThreadInStream = mInStream;

						if (mThreadConn != null) {
							mThreadConn.disconnect();
							mThreadConn = null;
						}

						if (mThreadInStream != null) {
							try {
								mThreadInStream.close();
							} catch (IOException e) {
								e.printStackTrace();
							}
							mThreadInStream = null;
						}

					}
				};
				new Thread(disconnectRunable).start();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
    public boolean isFinished() {
        return mFinished;
    }

    @Override
    public void run() {
		if (mInfo == null) {
			// 更新 citylist
			updateCitylist();
		} else {
			updateInfo();
		}
		isInit = true;
		mFinished = true;
    }

    Timer mUpdateCityListTimer = null;
	void updateCitylist() {
		mUpdateCityListTimer = new Timer();
		TimerTask task =new TimerTask() {
			
			@Override
			public void run() {
				//从本地获取cityList 版本
				int version = MapJni.getCitylistVersion();
				String localVersion = ByteHelper.getVersionByInt(version);
				
				if (NetWorkMonitorReceiver.NetType == NET_TYPE.no_net) {
					SERVICE_NUMBER = localVersion;
					if (mHandler != null) {
						Message msg = Message.obtain();
						msg.what = KMapView.WHAT_SERVICE_NUMBER;
						msg.obj = SERVICE_NUMBER;
						mHandler.sendMessage(msg);
						return;
					}
				}
				String str_body = WebServiceUtil.findServiceUrl(SERVICE_URL_TYPE.getOfflineDataVersion, 0, 0);
				byte[] respond = HttpTool.GetMapData(str_body, false);
				if (respond == null || respond.length == 0) {
					Log.v(TAG, "respond == null");
					SERVICE_NUMBER = localVersion;
				} else {
					String s = new String(respond);
					int j = Integer.valueOf(s);
					SERVICE_NUMBER = ByteHelper.getVersionByInt(j);
					Log.i(TAG, "serviceVersion:" + SERVICE_NUMBER);
				}

				Log.i(TAG, "stringLocalVersion:" + localVersion);
		        //将 localversion 分解为头和尾  然后比较  再做升级策略
		        //疑问  现在的版本号是 1.1  1.2 或者 2.1  为什么返回值是int
				if (!String.valueOf(localVersion).equals(SERVICE_NUMBER)) {
					str_body = WebServiceUtil.findServiceUrl(SERVICE_URL_TYPE.getCityList, 0, 0);
					Log.v(TAG, " strURL = " + strURL);
					updateCityList(strURL, str_body, CITYLISTDAT);
					// 离线下载模块处理版本更新
					DownloadInstance downloadInstance = DownloadInstance.getInstance();
					if (downloadInstance != null) {
						downloadInstance.onVersionUpdate();
					}
					firstTime = true;
				} else {
					checkVersion(MapCore.getSDKVersion(), SERVICE_NUMBER);
				}
				// 发送消息让sdk回调给app
				if (firstTime && mHandler != null) {
					Message msg = Message.obtain();
					msg.what = KMapView.WHAT_SERVICE_NUMBER;
					msg.obj = SERVICE_NUMBER;
					mHandler.sendMessage(msg);
					firstTime = false;
				}
			}
		};
		mUpdateCityListTimer.schedule(task, 0, 1000 * 60 * 10);//10分钟检测一次服务器是否有最新版本
    }
	
	public void Release() {
		if (mUpdateCityListTimer != null) {
			mUpdateCityListTimer.cancel();
			mUpdateCityListTimer = null;
		}
	}
    
	public static void checkVersion(String SDKVer, String ServerDataVer) {
		String sdkMaxVer = SDKVer.substring(0, SDKVer.indexOf("."));
		String serverMaxVer = ServerDataVer.substring(0, ServerDataVer.indexOf("."));
		if (serverMaxVer.equals(sdkMaxVer)) {
			DownloadInstance.VERSION_STATE = VERSION_STATE.OK_DOWNLOAD;
		} else {
			DownloadInstance.VERSION_STATE = VERSION_STATE.UPDATE_APP;
		}
	}
    
    void updateLv3list(){
        int localversion = MapJni.getLvl3Version();
        byte[] send =  new byte[]{1,1};
        byte[] respond = WebServiceUtil.getResponse(send);
		if (respond == null || respond.length == 0) {
			Log.v(TAG, "respond == null");
			return;
		}
        int remoteversion = Integer.valueOf(new String(respond));
        Log.v(TAG, " updateLv3list localversion = "+ localversion + " remoteversion = "+ remoteversion);
        if(localversion != remoteversion){
			byte[] citylist = new byte[] { 3 };
			byte[] clrespond = WebServiceUtil.getResponse(citylist);
			if (clrespond == null) {
				Log.v(TAG, "clrespond == null");
				return;
			}
            String url = new String(clrespond);
            Log.v(TAG, " url = " + url );
            updateCityList(url,"",LV3LISTDAT);
        }
    }
    
    void setmInfoToJni(ObjectInfo mInfo){
    	if (mInfo.getType() == CommonDefine.TYPE_PROVINCE) {
			MapJni.setProvinceinfo((ProvinceInfo) mInfo);
		} else if (mInfo.getType() == CommonDefine.TYPE_CITY) {
			MapJni.setCityinfo((CityInfo) mInfo);
		}
    }
    
    ArrayList<MapdataInfo> getMapdataInfo(ObjectInfo mInfo){
    	ArrayList<MapdataInfo> filelist = null;
		if (mInfo.getType() == CommonDefine.TYPE_PROVINCE) {
			// 通过省份获得数据
			filelist = MapJni.getDownloadinfoByprovincename(mInfo.getName());
		} else if (mInfo.getType() == CommonDefine.TYPE_CITY) {
			// 通过城市名字获得数据
			filelist = MapJni.getDownloadinfoByname(mInfo.getName());
		}
		return filelist;
    }
    
	void updateInfo() {
		
		if (mInfo == null) {
			return;
		}
		
		if (DownloadInstance.VERSION_STATE != VERSION_STATE.OK_DOWNLOAD) {
			try {
				throw new Exception("SDK NEW_VERSION !");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		mInfo.setState(OfflineMapStatus.DOWNLOADING);
		//先写入loading进入底层
		setmInfoToJni(mInfo);

		//获取升级mInfo的MapdataInfo信息
//		ArrayList<MapdataInfo> dataFiles = getMapdataInfo(mInfo);
//		String str_body = downloadMapData(dataFiles);
//		String URL = new StringBuilder(strURL).append("&").append(str_body).toString();
		
		//先获取服务器版本号进行比较
		String URL = WebServiceUtil.findServiceUrl(SERVICE_URL_TYPE.getOfflineDataVersion, 0, 0);
		byte[] respond = HttpTool.GetMapData(URL, false);
		if (respond == null) {
			Log.v(TAG, "respond == null");
			mInfo.setState(OfflineMapStatus.EXCEPTION_NETWORK);
			setmInfoToJni(mInfo);
			return;
		}
		
		int remoteversion = WebServiceUtil.byteArrayToInt(respond);
		int localversion = MapJni.getCitylistVersion();
		
		if (localversion == remoteversion) {
			Message message = new Message();
			message.what = CommonDefine.LOADING_UPDATE;
			Bundle bundle = new Bundle();
			bundle.putBoolean("status", false);
			bundle.putString("name", mInfo.getName());
			message.setData(bundle);
			mHandler.sendMessage(message);
			mInfo.setState(OfflineMapStatus.FINISHED);
			setmInfoToJni(mInfo);
		} else {
			String cityListStr = WebServiceUtil.findServiceUrl(SERVICE_URL_TYPE.getCityList, 0, 0);
			Log.v(TAG, " strURL = " + strURL);
			updateCityList(strURL, cityListStr, CITYLISTDAT);
			if (mError == OK) {
				//如果没有错误的情况
				ObjectInfo newinfo = null;
				if (mInfo.getType() == CommonDefine.TYPE_PROVINCE) {
					newinfo = MapJni.getitemByprovincename(mInfo.getName());
				} else if (mInfo.getType() == CommonDefine.TYPE_CITY) {
					newinfo = MapJni.getitemBycityname(mInfo.getName());
				} else {
					return;
				}
				if (newinfo.getVersion() == mInfo.getVersion()) {
					//如果版本号相同
					Message message = new Message();
					message.what = CommonDefine.LOADING_UPDATE;
					Bundle bundle = new Bundle();
					bundle.putBoolean("status", false);
					bundle.putString("name", newinfo.getName());
					message.setData(bundle);
					mInfo.setState(OfflineMapStatus.FINISHED);
					mHandler.sendMessage(message);
				} else {
					//如果版本号不同
					Message message = new Message();
					message.what = CommonDefine.LOADING_UPDATE;
					Bundle bundle = new Bundle();
					bundle.putBoolean("status", true);
					bundle.putString("name", newinfo.getName());
					message.setData(bundle);
					mHandler.sendMessage(message);
					mInfo.setState(OfflineMapStatus.NEW_VERSION);
				}
			} else {
				mInfo.setState(mError);
			}
			setmInfoToJni(mInfo);
		}
	}

	public int getErrorInfo() {
        return mError;
    }
}
