package com.cmmap.internal.maps;





import com.cmmap.api.maps.CameraUpdateFactory;
import com.cmmap.api.maps.Map;
import com.cmmap.api.maps.model.CameraPosition;
import com.cmmap.api.maps.model.LatLngBounds;
import com.cmmap.internal.driver.base.MapTool;
import com.cmmap.internal.mapcore.KFloat;
import com.cmmap.internal.mapcore.KGEOCOORD;
import com.cmmap.internal.mapcore.KInt;
import com.cmmap.internal.mapcore.MapCore;
import com.cmmap.internal.mapcore.MapCore.NE_MapParam_Object;
import com.cmmap.internal.maps.model.KCameraPosition;
import com.cmmap.internal.maps.model.KLatLng;

/**
 * 地图可视范围更新类。
 * <p>在用户改变地图可视范围的时候被调用，如{@link KMap#moveCamera}，{@link KMap#animateCamera}。
 * 使用{@link CameraUpdateFactory}可以构造该对象。</p>
 */
public class KCameraUpdate {


	public KCameraPosition targetPosition;
	
	public static final byte Camera_Update_All = 0x0F;
	public static final byte Camera_Update_Bearing_Only = 0x08;
	public static final byte Camera_Update_Tilt_Only = 0x04;
	public static final byte Camera_Update_Zoom_Only = 0x02;
	public static final byte Camera_Update_LatLng_Only = 0x01;
	// 更新类型
	public byte m_byUpdateType = 0x0F;
	
	public boolean isUpdateAll() {
		if((m_byUpdateType & Camera_Update_All) == Camera_Update_All) {
			return true;
		}
		return false;
	}
	
	public boolean isUpdateBearing() {
		if((m_byUpdateType & Camera_Update_Bearing_Only) == Camera_Update_Bearing_Only) {
			return true;
		}
		return false;
	}
	
	public boolean isUpdateTilt() {
		if((m_byUpdateType & Camera_Update_Tilt_Only) == Camera_Update_Tilt_Only) {
			return true;
		}
		return false;
	}
	
	public boolean isUpdateZoom() {
		if((m_byUpdateType & Camera_Update_Zoom_Only) == Camera_Update_Zoom_Only) {
			return true;
		}
		return false;
	}
	
	public boolean isUpdateLatLng() {
		if((m_byUpdateType & Camera_Update_LatLng_Only) == Camera_Update_LatLng_Only) {
			return true;
		}
		return false;
	}
	
	/**
	 * 构造出一个包含当前地图的可视范围的CamearPosition
	 */
	public KCameraUpdate() {
		targetPosition = getCurrentCameraPosition();
	}
	public KCameraUpdate(KCameraPosition position) {
		targetPosition = 
				new KCameraPosition(position.target, position.zoom, position.tilt, position.bearing);
	}
	public KCameraUpdate(CameraPosition position) {
		targetPosition = 
				new KCameraPosition(new KLatLng(position.target), position.zoom, position.tilt, position.bearing);
	}
	
	public KCameraUpdate(LatLngBounds bounds, int width, int height, int padding){
		targetPosition = getCameraPosition(bounds, width, height, padding);
	}
	
	public KCameraUpdate(LatLngBounds bounds, int leftPadding,
			int rightPadding, int topPadding, int bottomPadding) {
		if (bounds == null || bounds.northeast == null || bounds.southwest == null) {
			targetPosition = getCurrentCameraPosition();
		} else {
			targetPosition = getCameraPosition(bounds, leftPadding, rightPadding, topPadding, bottomPadding);
		}
	}

	private KCameraPosition getCameraPosition(LatLngBounds bounds,
			int leftPadding, int rightPadding, int topPadding, int bottomPadding) {
		return MapCore.OverviewMapEx(bounds, leftPadding, rightPadding, topPadding, bottomPadding);
	}

	private KCameraPosition getCameraPosition(LatLngBounds bounds, int width, int height, int padding)
	{
		return MapCore.OverviewMapEx(bounds, width, height, padding);
	}
	
	private KCameraPosition getCurrentCameraPosition()
	{
		KGEOCOORD k_coord = new KGEOCOORD();
		MapCore.GetMapCenter(k_coord);
		KLatLng location = new KLatLng(MapTool.longToDouble(k_coord.lLatitude), MapTool.longToDouble(k_coord.lLongitude));
		KFloat ki_level = new KFloat(0);
		MapCore.GetMapScaleLevel(ki_level);
		KFloat ki_rotate = new KFloat(0);
		MapCore.GetConfiguration(NE_MapParam_Object.NE_Map_RotateAngle.value(), ki_rotate);
		KFloat ki_pitch = new KFloat(0);
		MapCore.GetConfiguration(NE_MapParam_Object.NE_Map_PitchAngle.value(), ki_pitch);
		return new KCameraPosition(location,  ki_level.FloatVal(), ki_pitch.FloatVal(), ki_rotate.FloatVal());
	}
}
