package com.cmmap.internal.maps;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.CopyOnWriteArrayList;

import android.animation.Animator;
import android.animation.Animator.AnimatorListener;
import android.animation.TypeEvaluator;
import android.animation.ValueAnimator;
import android.animation.ValueAnimator.AnimatorUpdateListener;
import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Matrix;
import android.graphics.Point;
import android.graphics.PointF;
import android.graphics.Rect;
import android.graphics.RectF;
import android.location.Location;
import android.os.Handler;
import android.os.Handler.Callback;
import android.os.Message;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.view.animation.DecelerateInterpolator;
import android.widget.FrameLayout;

import com.cmmap.api.maps.Map.OnEventClickListener;
import com.cmmap.api.maps.Map.OnMapScreenShotListener;
import com.cmmap.api.maps.model.Arc;
import com.cmmap.api.maps.model.BitmapDescriptor;
import com.cmmap.api.maps.model.BitmapDescriptorFactory;
import com.cmmap.api.maps.model.Circle;
import com.cmmap.api.maps.model.GroundOverlay;
import com.cmmap.api.maps.model.Marker;
import com.cmmap.api.maps.model.MyTrafficStyle;
import com.cmmap.api.maps.model.Polygon;
import com.cmmap.internal.driver.base.DataOperator;
import com.cmmap.internal.driver.base.GlobalSettings;
import com.cmmap.internal.driver.gesture.MapGestureListener;
import com.cmmap.internal.driver.view.MapGLSurfaceView;
import com.cmmap.internal.driver.view.MapParams;
import com.cmmap.internal.driver.view.ViewContext;
import com.cmmap.internal.mapcore.MapCore;
import com.cmmap.internal.maps.model.KArc;
import com.cmmap.internal.maps.model.KArcOptions;
import com.cmmap.internal.maps.model.KCameraPosition;
import com.cmmap.internal.maps.model.KCircle;
import com.cmmap.internal.maps.model.KCircleOptions;
import com.cmmap.internal.maps.model.KDefaultInfoWindowView;
import com.cmmap.internal.maps.model.KGroundOverlay;
import com.cmmap.internal.maps.model.KGroundOverlayOptions;
import com.cmmap.internal.maps.model.KInfoOverlay;
import com.cmmap.internal.maps.model.KLatLng;
import com.cmmap.internal.maps.model.KMarker;
import com.cmmap.internal.maps.model.KMarkerListener;
import com.cmmap.internal.maps.model.KMarkerOptions;
import com.cmmap.internal.maps.model.KNavigateArrow;
import com.cmmap.internal.maps.model.KNavigateArrowOptions;
import com.cmmap.internal.maps.model.KOverlay;
import com.cmmap.internal.maps.model.KOverlayListener;
import com.cmmap.internal.maps.model.KOverlayPoints;
import com.cmmap.internal.maps.model.KOverlayPointsOptions;
import com.cmmap.internal.maps.model.KPoi;
import com.cmmap.internal.maps.model.KPolygon;
import com.cmmap.internal.maps.model.KPolygonOptions;
import com.cmmap.internal.maps.model.KPolyline;
import com.cmmap.internal.maps.model.KPolylineOptions;
import com.cmmap.internal.maps.model.KShapeOverlay;
import com.cmmap.internal.maps.model.KTileOverlay;
import com.cmmap.internal.maps.model.KTileOverlayOptions;
import com.cmmap.internal.maps.model.MyLocationStyle;
import com.cmmap.internal.services.rtic.RTICThread;

/**
 * 地图对象。
 * <p>
 * 定义地图对象的操作方法与接口。
 * </p>
 * 
 */
public class KMap implements MapGestureListener,
		KLocationSource.OnLocationChangedListener {
	private static final String TAG = "Map";
	// [internal] 坐标转换
	private KProjection projection;

	public KMapView mapView;

	private KCameraPosition cameraPosition;

	// 地图类型为标准类型
	public static final int MAP_TYPE_NORMAL = 0;
	// 地图类型为夜间类型
	public static final int MAP_TYPE_NIGHT = 1;
	// 地图类型为卫星地图
	public static final int MAP_TYPE_SATELLITE = 2;
	// 地图模式：实时路况底图
	public static final int MAP_TYPE_TRAFFIC = 3;
	// 地图模式：导航模式
	public static final int MAP_TYPE_NAVI = 4;
	private OnMapClickListener m_mapClickListener;
	private OnMapLoadedListener m_mapLoadedListener;
	private OnMapPropUpdateListener m_mapPropUpdateListener;
	private OnMapLongClickListener m_mapLongClickListener;
	private OnPOIClickListener m_mapPOIClickListener;
	private OnEventClickListener m_mapEventClickListener;
	OnArcClickListener m_mapArcClickListener;
	OnArcDragListener m_mapArcDragListener;
	OnCircleClickListener m_mapCircleClickListener;
	OnCircleDragListener m_mapCircleDragListener;
	OnPolylineClickListener m_mapPolylineClickListener;
	OnPolylineDragListener m_mapPolylineDragListener;
	OnPolygonClickListener m_mapPolygonClickListener;
	OnPolygonDragListener m_mapPolygonDragListener;
	OnGroundOverlayClickListener m_mapGroundOverlayClickListener;
	OnGroundOverlayDragListener m_mapGroundOverlayDragListener;
	OnMarkerClickListener m_markerClickListener;
	OnMarkerDragListener m_markerDragListener;
	OnInfoWindowClickListener m_infoWindowClickListener;
	private OnCameraChangeListener m_cameraChangeListener;
	private OnMapTouchListener m_mapTouchListener;
	private InfoWindowAdapter m_infoWindowAdapter;
	private OnMyLocationChangeListener myLocationChangeListener = null;

	private KLocationSource m_locationSource = null;
	private Location m_location = null; // 记录定位信息
	private boolean m_bLocationSourceEnable = false;

	public final static int LOCATION_TYPE_LOCATE = 1; // 表示只在第一次定位移动到地图中心点，值为1；
	public final static int LOCATION_TYPE_MAP_FOLLOW = 2; // 表示保持移动到地图中心点并跟随，值为2；
	public final static int LOCATION_TYPE_MAP_ROTATE = 3; // 表示保持移动到地图中心点并跟随，并根据方向旋转地图，值为3。

	/**
	 * 图形overlay
	 */
	private CopyOnWriteArrayList<KShapeOverlay> shapeOverlays = new CopyOnWriteArrayList<KShapeOverlay>();

	private KMarker m_infowindowSubstitute = null;
	private InfoView m_infoViewWithwindow = null;
	private BitmapDescriptor m_infowindowDescriptor = null;
	private boolean m_bWaitingAddInfoWindow = false;

	/**
	 * infoview列表，代表了所有的marker和对应的infowindow
	 */
	private CopyOnWriteArrayList<InfoView> infoViewList = new CopyOnWriteArrayList<KMap.InfoView>();

	/**
	 * 地图所有状态
	 */
	private ViewContext viewContext;

	/**
	 * 地图surfaceView
	 */
	private MapGLSurfaceView mapGLSurfaceView;

	/**
	 * 地图动画
	 */
	private ValueAnimator m_animator;
	/**
	 * 动画回调
	 */
	private CancelableCallback cancelableCallback;

	/*
	 * 比例尺：   zoom    	size
	 * 			3		1000KM
	 * 			4		500KM
	 *			5	 	200KM	
	 *			6		100KM
	 *			7		50KM
	 */
	// 审图要求，启动默认比例尺现改为1000KM
	private static final KCameraUpdate DEFAULT_CAMERA = new KCameraUpdate(
			new KCameraPosition(new KLatLng(39.90403, 116.407525), 3, 0, 0));

	private class MapDispStatus {
		KCameraUpdate m_Camera = DEFAULT_CAMERA;
		int m_MapType = MAP_TYPE_NORMAL;
		boolean m_bRTIC = false;
		boolean m_bRTICEvent = false;
		boolean m_buildingShow = true;
		boolean m_textShow = true;
		boolean m_skyShow = true;
		float m_textZIndex = 0;
	}

	private MapDispStatus mCurMapStatus = new MapDispStatus();
	private boolean m_bBackground = true;
	private boolean m_bMapRefreshFirstTime = true;

	public static final int MAP_REFRESH = 1000;
	public static final int MAP_GESTURE_END = 1001;
	public static final int MAP_SURFACE_CHANGE = 1002;
	public static final int MAP_ADD_INFOWINDOW = 1003;
	public static final int MAP_CAMERA_ANIMATE_END = 1004;
	public static final int INFOWINDOW_MOVE = 1005;
	public static final int MAP_SCREEN_SHOT = 1006;
	private boolean bLoaded = false;

	private BlockingQueue<KCameraUpdate> m_qCameraMoveReq = new ArrayBlockingQueue<KCameraUpdate>(20);
	
	public Integer m_OverlayProcSync = new Integer(0);
	

	public Handler mHandler = new Handler(new Callback() {

		@SuppressLint("NewApi")
		@Override
		public boolean handleMessage(Message msg) {
			switch (msg.what) {
			case MAP_REFRESH:
				// 地图刷新
				onMapChanged(viewContext.getMapParams());
				// 做到refresh之前
				mapView.widgetView.updateWidget();

				if (m_bMapRefreshFirstTime) {
					m_bMapRefreshFirstTime = false;
					mapView.onMapRefreshFirstTime();
				}
				break;
			case MAP_GESTURE_END:
				// 手势结束
				KCameraPosition position = getCameraPosition();
				// Log.i("CZD","Map handler message -> MAP_GESTURE_END");
				if (m_animator == null || !m_animator.isRunning()) {
					if (m_cameraChangeListener != null) {
						m_cameraChangeListener.onCameraChange(position);
						m_cameraChangeListener.onCameraChangeFinish(position);
					}
					processInfowindowInQuickChange(View.VISIBLE);
				}
				break;
			case MAP_SURFACE_CHANGE:
				// 地图视图变化
				if (!bLoaded) {
					bLoaded = true;
					onMapLoaded();
				}
				break;
			case MAP_ADD_INFOWINDOW:
				// marker的信息窗口显示
				// 由于地图渲染不在ui线程，为了避免地图移动时，信息窗口无法与地图同步移动，所以生成图片，在地图camera变化时隐藏信息窗口view的显示，而只是信息窗口view的图片。
			  synchronized (infoViewList) {
				if (m_infoViewWithwindow != null
						&& m_infoViewWithwindow.infoView != null) {
					if (m_infowindowDescriptor != null) {
						m_infowindowDescriptor.recycle();
					}
					m_infowindowDescriptor = BitmapDescriptorFactory.fromView(m_infoViewWithwindow.infoView);
					if (m_infowindowDescriptor != null) {
						if (m_infowindowSubstitute == null) {
							m_infowindowSubstitute = addMarker(new KMarkerOptions().icon(m_infowindowDescriptor)
									.position(m_infoViewWithwindow.calcSubtitudePos())
									.zIndex(100)
									.visible(false));
						} else {
							m_infowindowSubstitute.setIcon(m_infowindowDescriptor);
							m_infowindowSubstitute.setPosition(m_infoViewWithwindow.calcSubtitudePos());
						}
						mapView.refreshInfowindowSubtitude(m_infoViewWithwindow, m_infowindowSubstitute);
					}
				}
				}
				m_bWaitingAddInfoWindow = false;
				break;
			case MAP_CAMERA_ANIMATE_END:
				// 地图camera动画结束
				if (m_animator == null || !m_animator.isRunning()) {
					if (!m_qCameraMoveReq.isEmpty()) {
						KCameraUpdate cameraMoveReq = new KCameraUpdate();
						cameraMoveReq.m_byUpdateType = 0;
						while (!m_qCameraMoveReq.isEmpty()) {
							KCameraUpdate req = m_qCameraMoveReq.poll();
							if (req.isUpdateBearing()) {
								cameraMoveReq.targetPosition.bearing = req.targetPosition.bearing;
							}
							if (req.isUpdateLatLng()) {
								cameraMoveReq.targetPosition.target = req.targetPosition.target;
							}
							if (req.isUpdateTilt()) {
								cameraMoveReq.targetPosition.tilt = req.targetPosition.tilt;
							}
							if (req.isUpdateZoom()) {
								cameraMoveReq.targetPosition.zoom = req.targetPosition.zoom;
							}
							cameraMoveReq.m_byUpdateType |= req.m_byUpdateType;
						}
						moveCamera(cameraMoveReq);
					} else {
						if (m_cameraChangeListener != null) {
							KCameraPosition pos = getCameraPosition();
							m_cameraChangeListener.onCameraChange(pos);
							m_cameraChangeListener.onCameraChangeFinish(pos);
						}
						processInfowindowInQuickChange(View.VISIBLE);
					}
				}
				break;
			case INFOWINDOW_MOVE:
			  synchronized (infoViewList) {
			    if (m_infoViewWithwindow != null) {
    				int visible = msg.arg1;
    				if (visible == View.VISIBLE) {					
    					if (m_infowindowSubstitute != null 
    							&& m_infoViewWithwindow.infoView != null && m_infoViewWithwindow.infoView.getVisibility() == 0) {
    						m_infowindowSubstitute.setVisible(false);
    						mapView.refreshInfowindowSubtitude(null, null);
    						mapGLSurfaceView.requestRender();
    					}
    				} else {
    					if (m_infowindowSubstitute != null && m_infowindowSubstitute.isVisible()) {						
    						m_infoViewWithwindow.setInfoWindowVisible(visible);
    					}
    				}
			    }
			  }
				break;
			case MAP_SCREEN_SHOT:
			    OnMapScreenShotListener listener = (OnMapScreenShotListener) msg.obj;
			    Bitmap mapBitmap = msg.getData().getParcelable("bitmap");
			    int status = msg.arg1;
			    
			    Bitmap finalBitmap = mapView.processMapScreenShot(mapBitmap);
			    if (status == 1) {
			    	listener.onMapScreenShot(finalBitmap);
			    }
			    listener.onMapScreenShot(finalBitmap, status);
		        break;
			}
			return false;
		}
	});
	private static final Comparator<KInfoOverlay> COMPARATOR = new Comparator<KInfoOverlay>() {
		private static final int GRATER_THAN = 1;
		private static final int EQUAL_TO = 0;
		private static final int LESS_THAN = -1;

		@Override
		public int compare(KInfoOverlay overlay1, KInfoOverlay overlay2) {
			if (overlay1.getZIndex() < overlay2.getZIndex())
				return GRATER_THAN;
			if (overlay1.getZIndex() > overlay2.getZIndex()) {
				return LESS_THAN;
			}
			return EQUAL_TO;
		}

	};

	public KMap(ViewContext viewContext, KMapView view) {
		this.mapView = view;
		this.viewContext = viewContext;
		this.mCurMapStatus.m_bRTIC      = true;
		this.mCurMapStatus.m_bRTICEvent = false;

		cameraPosition = new KCameraPosition();
		projection = KProjection.getInstance();
	}

	/**
	 * 地图点击事件
	 */
	public static interface OnMapClickListener {
		/**
		 * 用户点击了地图事件回调
		 * @param point 点击位置的经纬度坐标
		 */
		void onMapClick(KLatLng point);
	}

	/**
	 * 地图载入成功事件
	 * 
	 */
	public static interface OnMapLoadedListener {
		/**
		 * 地图载入成功后回调
		 */
		void onMapLoaded();
		
		/**
		 * 地图加载失败后回调此方法。
		 */
		void onMapLoadFailure();
	}

	/**
	 * 地图属性更新回调接口。当用户设置该回调接口时，所有的接口函数立刻回调一次。
	 */
	public static interface OnMapPropUpdateListener {
		/**
		 * 地图版本更新后回调此方法。
		 * 
		 * @param mapVersion
		 *            地图版本号。版本号由一个主版本号和一个副版本号组成，例如"1.2"、"2.3"等。
		 */
		void onMapVersionUpdate(String mapVersion);
	}

	/**
	 * 地图长按事件
	 * 
	 */
	public static interface OnMapLongClickListener {
		/**
		 * 地图长按回调
		 * @param point 长按位置的经纬度坐标
		 */
		void onMapLongClick(KLatLng point);
	}

	/**
	 * 用户触摸地图回调
	 * 
	 */
	public static interface OnMapTouchListener {

		/**
		 * 用户触摸地图的回调
		 * @param event
		 */
		void onTouch(MotionEvent event);
	}

	/**
	 * 地图底图POI点被选中时的回调接口。
	 * 
	 */
	public static interface OnPOIClickListener {
		/**
		 * 地图底图POI点被选中时回调此方法 。
		 * @param poi 点击的地图POI对象
		 */
		void onPOIClick(KPoi poi);
	}
	
	/**
	 * 地图底图路况事件点被选中时的回调接口。
	 * 
	 */
	public static interface OnEventClickListener {
		/**
		 * 地图底图路况事件点被选中时回调此方法 。
		 * @param event 点击的地图路况事件对象
		 */
		void onEventClick(KPoi event);
	}

	/**
	* 弧形覆盖物对象被点击时的回调接口。
	 */
	public static interface OnArcClickListener {
		/**
		 * 弧形覆盖物对象被点击时调用此方法。
		 * @param arc 被点击的弧形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onArcClick(KArc arc, KLatLng clickPoint);
	}

	/**
	 * 弧形覆盖物对象被点击时的回调接口。
	 */
	public static interface OnArcDragListener {
		/**
		 * 当弧形覆盖物对象开始被拖动时回调此方法。
		 * <br>弧形覆盖物对象的位置可以通过{@link Arc#getStartPoint @link Arc#getPassedPoint @link Arc#getEndPoint}方法返回。
		 * @param arc 被拖动的弧形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onArcDragStart(KArc arc, KLatLng clickPoint);

		/**
		 * 在弧形覆盖物对象拖动过程当中回调此方法。
		 * <br>弧形覆盖物对象的位置可以通过{@link Arc#getStartPoint @link Arc#getPassedPoint @link Arc#getEndPoint}方法返回。
		 * @param arc 被拖动的弧形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onArcDrag(KArc arc, KLatLng clickPoint); 

		/**
		 * 当弧形覆盖物对象拖动完成后回调此方法。 
		 * <br>弧形覆盖物对象的位置可以通过{@link Arc#getStartPoint @link Arc#getPassedPoint @link Arc#getEndPoint}方法返回。
		 * @param arc 被拖动的弧形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onArcDragEnd(KArc arc, KLatLng clickPoint); 
	}

	/**
	 * 圆形覆盖物对象被点击时的回调接口。
	 */
	public static interface OnCircleClickListener {
		/**
		 * 圆形覆盖物对象被点击时调用此方法。
		 * @param circle 被点击的圆形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onCircleClick(KCircle circle, KLatLng clickPoint);
	}

	/**
	 * 圆形覆盖物对象被点击时的回调接口。
	 */
	public static interface OnCircleDragListener {
		/**
		 * 当圆形覆盖物对象开始被拖动时回调此方法。
		 * <br>圆形覆盖物对象的位置可以通过{@link Circle#getCenter}方法返回。
		 * @param circle 被拖动的圆形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onCircleDragStart(KCircle circle, KLatLng clickPoint);

		/**
		 * 在圆形覆盖物对象拖动过程当中回调此方法。
		 * <br>圆形覆盖物对象的位置可以通过{@link Circle#getCenter}方法返回。
		 * @param circle 被拖动的圆形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onCircleDrag(KCircle circle, KLatLng clickPoint); 

		/**
		 * 当圆形覆盖物对象拖动完成后回调此方法。 
		 * <br>圆形覆盖物对象的位置可以通过{@link Circle#getCenter}方法返回。
		 * @param circle 被拖动的圆形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onCircleDragEnd(KCircle circle, KLatLng clickPoint); 
	}

	/**
	 * 线段覆盖物对象被点击时的回调接口。
	 */
	public static interface OnPolylineClickListener {
		/**
		 * 线段覆盖物对象被点击时调用此方法。
		 * @param polyline 被点击的线段覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onPolylineClick(KPolyline polyline, KLatLng clickPoint);
	}

	/**
	 * 线段覆盖物对象被点击时的回调接口。
	 */
	public static interface OnPolylineDragListener {
		/**
		 * 当线段覆盖物对象开始被拖动时回调此方法。
		 * <br>线段覆盖物对象的位置可以通过{@link KPolyline#getPoints}方法返回。
		 * @param polyline 被拖动的线段覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onPolylineDragStart(KPolyline polyline, KLatLng clickPoint);

		/**
		 * 在线段覆盖物对象拖动过程当中回调此方法。
		 * <br>线段覆盖物对象的位置可以通过{@link KPolyline#getPoints}方法返回。
		 * @param polyline 被拖动的线段覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onPolylineDrag(KPolyline polyline, KLatLng clickPoint); 

		/**
		 * 当线段覆盖物对象拖动完成后回调此方法。 
		 * <br>线段覆盖物对象的位置可以通过{@link KPolyline#getPoints}方法返回。
		 * @param polyline 被拖动的线段覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onPolylineDragEnd(KPolyline polyline, KLatLng clickPoint); 
	}

	/**
	 * 多边形覆盖物对象被点击时的回调接口。
	 */
	public static interface OnPolygonClickListener {
		/**
		 * 多边形覆盖物对象被点击时调用此方法。
		 * @param polygon 被点击的多边形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onPolygonClick(KPolygon polygon, KLatLng clickPoint);
	}

	/**
	 * 多边形覆盖物对象被点击时的回调接口。
	 */
	public static interface OnPolygonDragListener {
		/**
		 * 当多边形覆盖物对象开始被拖动时回调此方法。
		 * <br>多边形覆盖物对象的位置可以通过{@link Polygon#getPoints}方法返回。
		 * @param polygon 被拖动的多边形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onPolygonDragStart(KPolygon polygon, KLatLng clickPoint);

		/**
		 * 在多边形覆盖物对象拖动过程当中回调此方法。
		 * <br>多边形覆盖物对象的位置可以通过{@link Polygon#getPoints}方法返回。
		 * @param polygon 被拖动的多边形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onPolygonDrag(KPolygon polygon, KLatLng clickPoint); 

		/**
		 * 当多边形覆盖物对象拖动完成后回调此方法。 
		 * <br>多边形覆盖物对象的位置可以通过{@link Polygon#getPoints}方法返回。
		 * @param polygon 被拖动的多边形覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onPolygonDragEnd(KPolygon polygon, KLatLng clickPoint); 
	}

	/**
	 * 图片覆盖物对象被点击时的回调接口。
	 */
	public static interface OnGroundOverlayClickListener {
		/**
		 * 图片覆盖物对象被点击时调用此方法。
		 * @param groundOverlay 被点击的图片覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onGroundOverlayClick(KGroundOverlay groundOverlay, KLatLng clickPoint);
	}

	/**
	 * 图片覆盖物对象被点击时的回调接口。
	 */
	public static interface OnGroundOverlayDragListener {
		/**
		 * 当图片覆盖物对象开始被拖动时回调此方法。
		 * <br>图片覆盖物对象的位置可以通过{@link GroundOverlay#getBounds}方法返回。
		 * @param groundOverlay 被拖动的图片覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onGroundOverlayDragStart(KGroundOverlay groundOverlay, KLatLng clickPoint);

		/**
		 * 在图片覆盖物对象拖动过程当中回调此方法。
		 * <br>图片覆盖物对象的位置可以通过{@link GroundOverlay#getBounds}方法返回。
		 * @param groundOverlay 被拖动的图片覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onGroundOverlayDrag(KGroundOverlay groundOverlay, KLatLng clickPoint); 

		/**
		 * 当图片覆盖物对象拖动完成后回调此方法。 
		 * <br>图片覆盖物对象的位置可以通过{@link GroundOverlay#getBounds}方法返回。
		 * @param groundOverlay 被拖动的图片覆盖物对象
		 * @param clickPoint 点击坐标
		 */
		void onGroundOverlayDragEnd(KGroundOverlay groundOverlay, KLatLng clickPoint); 
	}
	/**
	 * 地图可视范围改变事件 平移/缩放/旋转/倾角变化
	 */
	public static interface OnCameraChangeListener {
		/**
		 * 地图可视范围改变回调
		 */
		void onCameraChange(KCameraPosition pos);

		/**
		 * 地图可视范围改变完成回调
		 * @param pos 可视范围
		 * 
		 */
		void onCameraChangeFinish(KCameraPosition pos);
	}

	/**
	 * 定位改变事件
	 * 
	 */
	public static interface OnMyLocationChangeListener {
		void onMyLocationChange(Location location);
	}

	/**
	 * marker点击事件
	 */
	public static interface OnMarkerClickListener {

		/**
		 * marker被点击后事件回调
		 * @param marker 点击的marker覆盖物对象
		 */
		boolean onMarkerClick(KMarker marker);
	}

	/**
	 * marker拖动事件
	 * 
	 */
	public static interface OnMarkerDragListener {

		/**
		 * marker拖动过程中事件回调
		 * @param marker 拖动的marker覆盖物对象
		 */
		void onMarkerDrag(KMarker marker);

		/**
		 * marker即将开始拖动
		 * @param marker 拖动的marker覆盖物对象
		 */
		void onMarkerDragStart(KMarker marker);

		/**
		 * marker拖动结束
		 * @param marker 拖动的marker覆盖物对象
		 */
		void onMarkerDragEnd(KMarker marker);
	}

	/**
	 * 信息窗口点击事件
	 */
	public static interface OnInfoWindowClickListener {

		/**
		 * 信息窗口被点击后事件回调
		 * @param marker 点击的信息窗口对应的marker覆盖物对象
		 */
		void onInfoWindowClick(KMarker marker);
	}

	/**
	 * 针对具体的标记覆盖物的个性化定制的信息窗口的回调接口。
	 */
	public static interface InfoWindowAdapter {
		View getInfoContents(KMarker marker);
		
		/**
		 * 返回针对具体的标记覆盖物的个性化定制的信息窗口的视图。
		 * @param marker 与该信息窗口对应的标记覆盖物对象
		 * @return 针对该标记覆盖物的个性化定制的信息窗口的视图，
		 * 如果返回null，SDK将以默认的信息窗口样式显示，信息窗口内容为{@link Marker#getTitle}和{@link Marker#getSnippet}
		 */
		View getInfoWindow(KMarker marker);
	}

	/**
	 * 一个任务完成或取消时的回调接口。
	 * <br>
	 * 这是一个通用的接口，典型的用于{@link #animateCamera}调用。
	 */
	public static interface CancelableCallback {
		/**
		 * 当一个任务被终止时回调此方法
		 */
		void onCancel();

		/**
		 * 当一个任务完成时回调此方法
		 */
		void onFinish();
	}

	/**
	 * 设置地图点击回调接口。
	 * @param listener  地图点击回调接口
	 */

	public void setOnMapClickListener(OnMapClickListener listener) {
		this.m_mapClickListener = listener;
	}

	/**
	 * 设置地图加载完成回调接口。
	 * @param listener  地图加载完成回调接口
	 */
	public void setOnMapLoadedListener(OnMapLoadedListener listener) {
		this.m_mapLoadedListener = listener;
	}

	/**
	 * 设置地图属性更新回调接口。
	 * @param listener 地图属性更新回调接口
	 */
	public void setOnMapPropUpdateListener(OnMapPropUpdateListener listener) {
		this.m_mapPropUpdateListener = listener;
	}

	/**
	 * 设置地图长按回调接口。
	 * @param listener 地图长按回调接口
	 */
	public void setOnMapLongClickListener(OnMapLongClickListener listener) {
		this.m_mapLongClickListener = listener;
	}

	/**
	 * 设置地图底图POI点击选中回调接口。
	 * @param listener 地图底图POI点击选中回调接口
	 */
	public void setOnPOIClickListener(OnPOIClickListener listener) {
		this.m_mapPOIClickListener = listener;
	}
	
	/**
	 * 设置地图底图路况事件点击选中回调接口。
	 * @param listener 地图底图路况事件点击选中回调接口
	 */
	public void setOnEventClickListener(OnEventClickListener listener) {
		this.m_mapEventClickListener = listener;
	}

	/**
	 * 设置弧形覆盖物点击回调接口。
	 * @param listener 弧形覆盖物点击回调接口
	 */
	public void setOnArcClickListener(OnArcClickListener listener) {
		m_mapArcClickListener = listener;
	}

	/**
	 * 设置弧形覆盖物拖动回调接口。
	 * @param listener 弧形覆盖物拖动回调接口
	 */
	public void setOnArcDragListener(OnArcDragListener listener) {
		m_mapArcDragListener = listener;
	}

	/**
	 * 设置圆形覆盖物点击回调接口。
	 * @param listener 圆形覆盖物点击回调接口
	 */
	public void setOnCircleClickListener(OnCircleClickListener listener) {
		m_mapCircleClickListener = listener;
	}

	/**
	 * 设置圆形覆盖物拖动回调接口。
	 * @param listener 圆形覆盖物拖动回调接口
	 */
	public void setOnCircleDragListener(OnCircleDragListener listener) {
		m_mapCircleDragListener = listener;
	}
	
	/**
	 * 设置线段覆盖物点击回调接口。
	 * @param listener 线段覆盖物点击回调接口
	 */
	public void setOnPolylineClickListener(OnPolylineClickListener listener) {
		this.m_mapPolylineClickListener = listener;
	}

	/**
	 * 设置线段覆盖物拖动回调接口。
	 * @param listener 线段覆盖物拖动回调接口
	 */
	public void setOnPolylineDragListener(OnPolylineDragListener listener) {
		m_mapPolylineDragListener = listener;
	}

	/**
	 * 设置多边形覆盖物点击回调接口。
	 * @param listener 多边形覆盖物点击回调接口
	 */
	public void setOnPolygonClickListener(OnPolygonClickListener listener) {
		m_mapPolygonClickListener = listener;
	}

	/**
	 * 设置多边形覆盖物拖动回调接口。
	 * @param listener 多边形覆盖物拖动回调接口
	 */
	public void setOnPolygonDragListener(OnPolygonDragListener listener) {
		m_mapPolygonDragListener = listener;
	}

	/**
	 * 设置图片覆盖物点击回调接口。
	 * @param listener 图片覆盖物点击回调接口
	 */
	public void setOnGroundOverlayClickListener(OnGroundOverlayClickListener listener) {
		m_mapGroundOverlayClickListener = listener;
	}

	/**
	 * 设置图片覆盖物拖动回调接口。
	 * @param listener 图片覆盖物拖动回调接口
	 */
	public void setOnGroundOverlayDragListener(OnGroundOverlayDragListener listener) {
		m_mapGroundOverlayDragListener = listener;
	}

	/**
	 * 设置地图可视范围变化回调接口。
	 * @param listener 地图可视范围变化回调接口
	 */
	public void setOnCameraChangeListener(OnCameraChangeListener listener) {
		this.m_cameraChangeListener = listener;
	}

	/**
	 * 设置标记覆盖物点击回调接口。
	 * @param listener  标记覆盖物点击回调接口
	 */
	public void setOnMarkerClickListener(OnMarkerClickListener listener) {
		this.m_markerClickListener = listener;
	}

	/**
	 * 设置标记覆盖物拖动回调接口。
	 * @param listener  标记覆盖物拖动回调接口
	 */
	public void setOnMapMarkerDragListener(OnMarkerDragListener listener) {
		this.m_markerDragListener = listener;
	}

	/**
	 * 设置信息窗口点击回调接口。
	 * @param listener 信息窗口点击回调接口
	 */
	public void setOnInfoWindowClickListener(OnInfoWindowClickListener listener) {
		this.m_infoWindowClickListener = listener;
	}

	/**
	 * 设置地图触摸回调接口。
	 * @param listener 地图触摸回调接口
	 */
	public void setOnMapTouchListener(OnMapTouchListener listener) {
		this.m_mapTouchListener = listener;
	}
	/**
     * 发起地图截屏请求
     * @param listener 地图截屏监听接口
     */
    public void getMapScreenShot(OnMapScreenShotListener listener) {
        mapGLSurfaceView.getMapScreenShot(listener);
    }

	// 对外接口---监听器----end-----------------------

	void setMapOperatorView(MapGLSurfaceView view) {
		mapGLSurfaceView = view;
	}

	@Override
	public void onGestureStart() {

	}

	// [internal] 地图操作结束后回调
	@Override
	public void onGestureEnd() {
		if (isMapChanged(this.viewContext.getMapParams())) {
			if (this.m_cameraChangeListener != null) {
				this.m_cameraChangeListener.onCameraChange(getCameraPosition());
				this.m_cameraChangeListener
						.onCameraChangeFinish(new KCameraPosition(
								cameraPosition.target, cameraPosition.zoom,
								cameraPosition.tilt, cameraPosition.bearing));
				processInfowindowInQuickChange(View.VISIBLE);
			}
			mapView.updateCompass();
		}
		this.mapView.updateWidget();
	}

	// [internal] 点击地图回调
	@Override
	public void onMapClick(Point point) {
		// 判断POI和shape是否被选中
		KPoi poi = new KPoi();
		// 没有设置监听，则不做判断
		boolean bSelectPOI = (m_mapPOIClickListener != null || m_mapEventClickListener != null) 
				? MapCore.SelectPOI(poi, projection.fromScreenLocation(point)) : false;
		if (bSelectPOI) {
			if(poi.getIsEvent()){
				if (this.m_mapEventClickListener != null){
					this.m_mapEventClickListener.onEventClick(poi);
				}
			}
			else{
				if (this.m_mapPOIClickListener != null) {
					this.m_mapPOIClickListener.onPOIClick(poi);
				}
			}
		}else if (this.m_mapClickListener != null) {
			this.m_mapClickListener.onMapClick(projection.fromScreenLocation(point));
		}
	}

	// [internal] 长按地图回调
	@Override
	public void onMapLongClick(KLatLng location) {
		if (this.m_mapLongClickListener != null) {
			this.m_mapLongClickListener.onMapLongClick(location);
		}
	}

	@Override
	public void onMapTouch(MotionEvent event) {
		if (this.m_mapTouchListener != null) {
			this.m_mapTouchListener.onTouch(event);
		}
	};

	// [internal] 地图刷新后回调更新
	@Override
	public void onMapChanged(MapParams params) {
		if (isMapChanged(params)) {
			if (this.m_cameraChangeListener != null) {
				this.m_cameraChangeListener.onCameraChange(new KCameraPosition(
						cameraPosition.target, cameraPosition.zoom,
						cameraPosition.tilt, cameraPosition.bearing));
			}
			mapView.updateCompass();
		}

		this.mapView.updateWidget();
		mapView.reFreshInfoOverlay();
	}

	/**
	 * 判断地图camera是否变化
	 * @param params 地图参数
	 * @return 是否变化
	 */
	private boolean isMapChanged(MapParams params) {
		MapParams mp_preMapParam = MapGLSurfaceView.getPreMapParam();
		// [internal] 位置是否变化
		KLatLng curLocation = params.getLatlon();
		KLatLng preLocation = mp_preMapParam.getLatlon();
		boolean hasLocationChanged = !curLocation.equals(preLocation);
		this.cameraPosition.target = curLocation;
		// [internal] 比例尺是否变化
		float curLevel = params.getLevel();
		float preLevel = mp_preMapParam.getLevel();
		boolean hasLevelChanged = !(curLevel == preLevel);
		this.cameraPosition.zoom = curLevel;
		// [internal] 旋转角度是否变化
		float curRotation = params.getRotation();
		float preRotation = mp_preMapParam.getRotation();
		boolean hasRotationChanged = !(curRotation == preRotation);
		this.cameraPosition.bearing = curRotation;
		// [internal] 俯仰角度是否变化
		float curPitch = params.getPitch();
		float prePitch = mp_preMapParam.getPitch();
		boolean hasPitchChanged = !(curPitch == prePitch);
		this.cameraPosition.tilt = curPitch;

		return (hasLocationChanged || hasLevelChanged || hasRotationChanged || hasPitchChanged);
	}

	// [internal] 手势执行操作完后，会回调此方法，刷新Info图层
	@Override
	public void onRefreshInfoLayer() {
		this.mapView.reFreshInfoOverlay();
	}

	// [internal] 在绘画地图的同时刷新Info图层
	public void postRefreshInfoOverlay() {
		this.mapView.postRefreshInfoOverlay();
	}

	PointF getSize() {
		return mapView.getSize();
	}

	int getViewWidth() {
		return mapView.getViewWidth();
	}

	int getViewHeight() {
		return mapView.getViewHeight();
	}

	/**
	 * camera自定义动画
	 */
	@SuppressLint("NewApi")
	class CameraEvaluator implements TypeEvaluator<KCameraPosition> {

		@Override
		public KCameraPosition evaluate(float fraction,
				KCameraPosition startValue, KCameraPosition endValue) {
			KCameraPosition position = new KCameraPosition();
			KCameraPosition start = (KCameraPosition) startValue;
			KCameraPosition end = (KCameraPosition) endValue;

			float endBearing = end.bearing;
			float startBearing = start.bearing;
			if (start.bearing > end.bearing) {
				if (start.bearing - end.bearing > 180) {
					// 逆时针
					endBearing = end.bearing + 360;
				}
			} else {
				if (end.bearing - start.bearing > 180) {
					// 顺时针
					startBearing = start.bearing + 360;
				}
			}
			position.bearing = (start.bearing + fraction
					* (endBearing - startBearing));
			position.tilt = (start.tilt + fraction * (end.tilt - start.tilt));
			position.zoom = (start.zoom + fraction * (end.zoom - start.zoom));

			KLatLng pos = new KLatLng(start.target);
			pos.setLatitude(start.target.latitude + fraction
					* (end.target.latitude - start.target.latitude));
			pos.setLongitude(start.target.longitude + fraction
					* (end.target.longitude - start.target.longitude));
			position.target = pos;
			return position;
		}

	}

	/**
	 * marker 和 infowindow状态变化监听器
	 */
	private KMarkerListener infoOverlayListener = new KMarkerListener() {

		@Override
		public void updateZIndex(int _id) {
			InfoView infoView = findInfoViewById(_id);
			if (infoView != null) {
				// 排序
				resortInfoView();
				KMap.this.mapView.updateShapeOverlay(infoView.marker, KOverlay.UPDATE_ZINDEX);
			}
		}
		
		@Override
		public void updateLevel(int _id) {
			InfoView infoView = findInfoViewById(_id);
			if (infoView != null) {
				// 排序
				resortInfoView();
				KMap.this.mapView.updateShapeOverlay(infoView.marker, KOverlay.UPDATE_LEVEL);
			}
		}

		@Override
		public void remove(int _id) {
			synchronized (infoViewList) {
				InfoView infoView = findInfoViewById(_id);
				if (infoView != null) {
					if (m_infoViewWithwindow != null
							&& m_infoViewWithwindow.marker.equals(infoView.marker)) {
						m_infoViewWithwindow = null;
						if (m_infowindowSubstitute != null) {
							m_infowindowSubstitute.setVisible(false);
							mapView.refreshInfowindowSubtitude(null, null);
						}
					}
					KMap.this.mapView.removeShapeOverlay(infoView.marker);
					KMap.this.infoViewList.remove(infoView);
					infoView.releaseResource();
					infoView = null;
				}				
			}
		}
		
		@Override
		public void firePropertyChange(int _id, int updateMask) {
			synchronized (infoViewList) {
				InfoView infoView = findInfoViewById(_id);
				if (infoView != null) {
					KMarker marker = infoView.marker;
					KMap.this.mapView.updateShapeOverlay(marker, updateMask);
				}				
			}
		}

		@Override
		public void InfoWindowShowStatusChanged(int _id) {
			synchronized (infoViewList) {
				InfoView infoView = findInfoViewById(_id);
				if (infoView != null && infoView.marker != null) {
					mapView.refreshInfowindowSubtitude(null, null);
					if (infoView.marker.isInfoWindowShown()) {
						// 新的marker的信息窗口需要显示，创建信息窗口view
						View infowindow = constructInfoWindow(infoView.marker);
						if (infowindow != null) {
							infoView.showInfoWindow(infowindow);
							// marker位置是经纬度，需要生成图片
							if (infoView.marker.getMarkerOptions().isGps()) {
								m_bWaitingAddInfoWindow = true;
								mHandler.removeMessages(KMap.MAP_ADD_INFOWINDOW);
								Message message = new Message();
								message.what = KMap.MAP_ADD_INFOWINDOW;
								mHandler.sendMessageDelayed(message, 300);
								
							} else {
								if (m_infowindowSubstitute != null) {
									m_infowindowSubstitute.setVisible(false);
								}
							}
						} else {
							if (m_infowindowSubstitute != null) {
								m_infowindowSubstitute.setVisible(false);
							}
						}
						KMap.this.mapView.refreshInfowindow(infoView);
						m_infoViewWithwindow = infoView;
					} else {
						if (infoView.infoView != null) {
							if (m_infowindowSubstitute != null) {
								m_infowindowSubstitute.setVisible(false);
							}
							infoView.hideInfoWindow();
							m_infoViewWithwindow = null;
						}
					}
				}
				KMap.this.mapView.reSortInfoView();
			}
		}

		@Override
		public void RotateAngleChanged(int _id) {
			// 图片旋转角度
			firePropertyChange(_id, KOverlay.UPDATE_ALL);
		}

		@Override
		public void InfoWindowContentChanged(int _id) {
			synchronized (infoViewList) {
				InfoView infoView = findInfoViewById(_id);
				if (infoView.marker.isInfoWindowShown()) {
					InfoWindowShowStatusChanged(_id);
				}				
			}
		}

		@Override
		public void PositionChanged(int _id) {
			synchronized (infoViewList) {				
				InfoView infoView = findInfoViewById(_id);
				if (infoView != null) {
					KMap.this.mapView.updateShapeOverlay(infoView.marker, KOverlay.UPDATE_ALL);
					infoView.recalcPos();
				}
			}
		}

		@Override
		public void overlayImageRemove(int _id) {
			synchronized (infoViewList) {
				InfoView infoView = findInfoViewById(_id);
				if (infoView != null) {
					mapView.removeShapeOverlayBitmap(infoView.marker);
				}				
			}
		}

		@Override
		public void overlayImageChange(int _id) {
			synchronized (infoViewList) {
				InfoView infoView = findInfoViewById(_id);
				if (infoView != null) {
					infoView.changeMarkerIcon();
					mapView.updateShapeOverlayBitmap(infoView.marker);
					mapView.updateShapeOverlay(infoView.marker, KOverlay.UPDATE_ALL);
				}				
			}
		}

		@Override
		public void clearCache(int _id) {
		}

		@Override
		public void stopMarkerAnimates(int _id) {
			// 停止下落动画
			synchronized (infoViewList) {
				InfoView infoView = findInfoViewById(_id);
				if (infoView != null) {
					infoView.stopAnimateDrop();
				}				
			}
		}
	};

	/**
	 * ShapeOverlay状态变化监听器。
	 */
	private KOverlayListener shapeoverlayListener = new KOverlayListener() {

		@Override
		public void firePropertyChange(int _id, int updateMask) {
			synchronized (shapeOverlays) {
				KShapeOverlay overlay = findShapeOverlayById(_id);
				if (overlay != null) {
					KMap.this.mapView.updateShapeOverlay(overlay, updateMask);
				}				
			}
		}

		@Override
		public void remove(int _id) {
			synchronized (shapeOverlays) {				
				KShapeOverlay overlay = findShapeOverlayById(_id);
				if (overlay != null) {
					KMap.this.mapView.removeShapeOverlay(overlay);
					KMap.this.shapeOverlays.remove(overlay);
					overlay = null;
				}
			}
		}

		@Override
		public void updateZIndex(int _id) {
			synchronized (shapeOverlays) {
				KShapeOverlay overlay = findShapeOverlayById(_id);
				if (overlay != null) {
					KMap.this.mapView.updateShapeOverlay(overlay, KOverlay.UPDATE_ZINDEX);
				}				
			}
		}
		
		@Override
		public void updateLevel(int _id) {
			synchronized (shapeOverlays) {
				KShapeOverlay overlay = findShapeOverlayById(_id);
				if (overlay != null) {
					KMap.this.mapView.updateShapeOverlay(overlay, KOverlay.UPDATE_LEVEL);
				}				
			}
		}

		@Override
		public void overlayImageRemove(int _id) {
			synchronized (shapeOverlays) {
				KShapeOverlay overlay = findShapeOverlayById(_id);
				if (overlay != null) {
					KMap.this.mapView.removeShapeOverlayBitmap(overlay);
				}				
			}
		}

		@Override
		public void overlayImageChange(int _id) {
			synchronized (shapeOverlays) {				
				KShapeOverlay overlay = findShapeOverlayById(_id);
				if (overlay != null) {
					KMap.this.mapView.updateShapeOverlayBitmap(overlay);
					KMap.this.mapView.updateShapeOverlay(overlay, KOverlay.UPDATE_ALL);
				}
			}
		}

		@Override
		public void clearCache(int _id) {
			synchronized (shapeOverlays) {
				KShapeOverlay overlay = findShapeOverlayById(_id);
				if (overlay != null && overlay instanceof KTileOverlay) {
					mapView.clearTileCache(_id);
				}				
			}
		}

	};

	/**
	 * 根据shape的id找到对应的shapeOverlay
	 * @param id shapeOverlay的id
	 * @return shapeOverlay对象
	 */
	private KShapeOverlay findShapeOverlayById(int id) {
		for (int i = 0; i < shapeOverlays.size(); ++i) {
			KShapeOverlay overlay = shapeOverlays.get(i);
			if (overlay != null && overlay.getOverlayId() == id) {
				return overlay;
			}
		}
		return null;
	}

	/**
	 * 根据id找到对应infoView
	 * @param id infoView的id
	 * @return infoView的对象
	 */
	private InfoView findInfoViewById(int id) {
		for (int i = 0; i < infoViewList.size(); ++i) {
			InfoView overlay = infoViewList.get(i);
			if (overlay != null && overlay.marker.getOverlayId() == id) {
				return overlay;
			}
		}		
		return null;
	}

	/**
	 * 添加弧形覆盖物到地图上。
	 * @param options 弧形覆盖物选项对象
	 * @return 弧形覆盖物对象
	 */
	public KArc addArc(KArcOptions options) {
		if (options == null) {
			return null;
		}
		KArc arc = new KArc();
		arc.setArcOptions(options);
		arc.setPropertyChangeListener(shapeoverlayListener);
		addShapeOverlays(arc);

		KMap.this.mapView.addShapeOverlay(arc);

		return arc;
	}

	/**
	 * 添加圆形覆盖物到地图上。
	 * @param options 圆形覆盖物选项对象
	 * @return 圆形覆盖物对象
	 */
	public KCircle addCircle(KCircleOptions options) {
		if (options == null) {
			return null;
		}
		KCircle circle = new KCircle();
		circle.setCircleOptions(options);
		circle.setPropertyChangeListener(shapeoverlayListener);
		addShapeOverlays(circle);

		KMap.this.mapView.addShapeOverlay(circle);

		return circle;
	}

	/**
	 * 添加图片覆盖物到到地图上。
	 * @param options 图片覆盖物选项对象
	 * @return 图片覆盖物对象
	 */
	public KGroundOverlay addGroundOverlay(KGroundOverlayOptions options) {
		if (options == null) {
			return null;
		}
		KGroundOverlay overlay = new KGroundOverlay();
		overlay.setGroundOverlayOptions(options);

		overlay.setPropertyChangeListener(shapeoverlayListener);
		addShapeOverlays(overlay);
		KMap.this.mapView.addShapeOverlay(overlay);

		return overlay;
	}

	/**
	 * 添加标记覆盖物到地图上。
	 * @param options 标记覆盖物选项对象
	 * @return 标记覆盖物对象
	 */
	public KMarker addMarker(KMarkerOptions options) {
		if (options == null) {
			return null;
		}
		KMarker marker = new KMarker();
		marker.setMarkerOptions(options);

		marker.setPropertyChangeListener(infoOverlayListener);
		KMap.this.mapView.addShapeOverlay(marker);

		viewForMarker(marker);

		resortInfoView();

		return marker;
	}

	/**
	 * 添加多个标记覆盖物到地图上。
	 * @param options 多个标记覆盖物选项对象
	 * @return 多个标记覆盖物对象
	 */
	public java.util.ArrayList<KMarker> addMarkers(
			java.util.ArrayList<KMarkerOptions> list, boolean moveToCenter) {
		if (list == null || list.isEmpty()) {
			return null;
		}
		ArrayList<KMarker> markers = new ArrayList<KMarker>();
		int size = list.size();
		for (int i = 0; i < size; i++) {
			KMarkerOptions options = list.get(i);

			KMarker marker = new KMarker();
			marker.setMarkerOptions(options);

			viewForMarker(marker);

			markers.add(marker);
		}

		resortInfoView();
		return markers;

	}

	/**
	 * 添加多边形覆盖物到地图上。
	 * @param options 多边形覆盖物选项对象
	 * @return 多边形覆盖物对象
	 */
	public KPolygon addPolygon(KPolygonOptions options) {
		if (options == null) {
			return null;
		}
		KPolygon polygon = new KPolygon();
		polygon.setPolygonOptions(options);
		polygon.setPropertyChangeListener(shapeoverlayListener);
		addShapeOverlays(polygon);

		KMap.this.mapView.addShapeOverlay(polygon);

		return polygon;
	}

	/**
	 * 添加段线覆盖物到地图上。
	 * @param options 段线覆盖物选项对象
	 * @return 段线覆盖物对象
	 */
	public KPolyline addPolyline(KPolylineOptions options) {
		if (options == null) {
			return null;
		}
		KPolyline polyline = new KPolyline();
		polyline.setPolyLineOptions(options);
		polyline.setPropertyChangeListener(shapeoverlayListener);
		addShapeOverlays(polyline);

		KMap.this.mapView.addShapeOverlay(polyline);

		return polyline;
	}

	/**
	 * 添加箭头覆盖物对象到地图上。
	 * @param options 箭头覆盖物选项对象
	 * @return 箭头覆盖物对象
	 */
	public KNavigateArrow addNavigateArrow(KNavigateArrowOptions options) {
		if (options == null) {
			return null;
		}

		KNavigateArrow arrow = new KNavigateArrow();
		arrow.setNavigateArrowOptions(options);
		arrow.setPropertyChangeListener(shapeoverlayListener);
		synchronized (shapeOverlays) {
			shapeOverlays.add(arrow);			
		}

		KMap.this.mapView.addShapeOverlay(arrow);

		return arrow;
	}

	/**
	 * 添加栅格图层到地图上。
	 * @param options 栅格图层选项对象
	 * @return 栅格图层对象
	 */
	public final KTileOverlay addTileOverlay(KTileOverlayOptions options) {
		if (options == null || options.getTileProvider() == null) {
			return null;
		}

		KTileOverlay tileOverlay = new KTileOverlay();
		tileOverlay.setTileOverlayOptions(options);
		tileOverlay.setPropertyChangeListener(shapeoverlayListener);
		synchronized (shapeOverlays) {			
			shapeOverlays.add(tileOverlay);
		}

		mapView.addShapeOverlay(tileOverlay);

		if (tileOverlay.m_Options.isClearOldCache()) {
			mapView.clearTileCache(tileOverlay.getOverlayId());
		}

		return tileOverlay;
	}
	
	/**
	 * 添加点集覆盖物到地图上
	 * @param options 点集覆盖物选项对象
	 * @return 点击覆盖物对象
	 */
	public final KOverlayPoints addOverlayPoints(KOverlayPointsOptions options) {
		if (options == null) {
			return null;
		}
		
		KOverlayPoints overlayPoints = new KOverlayPoints(options);
		overlayPoints.setPropertyChangeListener(shapeoverlayListener);
		synchronized (shapeOverlays) {
			shapeOverlays.add(overlayPoints);
		}
		mapView.addShapeOverlay(overlayPoints);
		
		return overlayPoints;
	}

	/**
	 * 以动画方式改变地图可视范围。
	 * @param update 地图可视范围更新对象
	 */
	public void animateCamera(KCameraUpdate update) {
		animateCamera(update, null);
	}

	/**
	 * 以动画方式改变地图可视范围。
	 * @param update 地图可视范围更新对象
	 * @param cancelableCallback
	 *            如果动画正常完成，则会调用{@link CancelableCallback#onFinish()}方法，否则调用
	 *            {@link CancelableCallback#onCancel()}
	 */
	public void animateCamera(KCameraUpdate update,
			KMap.CancelableCallback paramCancelableCallback) {
		// 客户建议动画时间调整为300ms
		animateCamera(update, 300, paramCancelableCallback);
	}

	/**
	 * 在指定的持续时间内，以动画方式改变地图可视范围。
	 * @param update 地图可视范围更新对象
	 * @param durationMs 动画持续时间，单位毫秒
	 * @param cancelableCallback
	 *            如果动画正常完成，则会调用onFinish()方法，否则调用onCancel()
	 */
	@SuppressLint("NewApi")
	public void animateCamera(KCameraUpdate update, long durationMs,
			KMap.CancelableCallback paramCancelableCallback) {
	    if (update == null
                || update.targetPosition == null) {
            return;
        }
	    
		// mapView在后台，只保存请求，不直接设置
		if (m_bBackground) {
			moveCameraInBackground(update);
			if (paramCancelableCallback != null) {
				paramCancelableCallback.onFinish();
			}
			return;
		}

		// 先刷新cameraPosition
		synchronized (viewContext) {
			mapGLSurfaceView.clearCameraReq();
			isMapChanged(viewContext.getMapParams());
		}

		final KCameraUpdate targetUpdate = new KCameraUpdate(
				update.targetPosition);
		if (cameraPosition.target.equals(update.targetPosition.target)) {
			targetUpdate.m_byUpdateType ^= KCameraUpdate.Camera_Update_LatLng_Only;
		}
		if (Math.abs(cameraPosition.tilt - update.targetPosition.tilt) < 1.0) {
			targetUpdate.m_byUpdateType ^= KCameraUpdate.Camera_Update_Tilt_Only;
		}
		int bearing = (int)Math.abs(cameraPosition.bearing - update.targetPosition.bearing) % 360;
		if (bearing < 1) {
			targetUpdate.m_byUpdateType ^= KCameraUpdate.Camera_Update_Bearing_Only;
		}
		if (Math.abs(cameraPosition.zoom - update.targetPosition.zoom) < 0.01) {
			targetUpdate.m_byUpdateType ^= KCameraUpdate.Camera_Update_Zoom_Only;
			if (cameraPosition.zoom < update.targetPosition.zoom) {
				// 缩小时,删除文字
				mapGLSurfaceView.clearMapText(System.currentTimeMillis(), true);
			}
		}
		targetUpdate.m_byUpdateType &= update.m_byUpdateType;

		if ((targetUpdate.m_byUpdateType & KCameraUpdate.Camera_Update_All) == 0) {
			if (paramCancelableCallback != null) {
				paramCancelableCallback.onFinish();
			}
			return;
		}

		mapGLSurfaceView.setFastRefresh(true);

		processInfowindowInQuickChange(View.GONE);
		if (m_animator == null) {
			m_animator = ValueAnimator.ofObject(new CameraEvaluator(),
					new KCameraPosition(cameraPosition.target,
							cameraPosition.zoom, cameraPosition.tilt,
							cameraPosition.bearing), new KCameraPosition(
							update.targetPosition.target,
							update.targetPosition.zoom,
							update.targetPosition.tilt,
							update.targetPosition.bearing));
		} else {
			if (m_animator.isRunning()) {
				// 如果当前有动画，则不执行本次动画请求，而是在当前动画结束时，直接设置
				m_qCameraMoveReq.offer(targetUpdate);
				if (paramCancelableCallback != null) {
					paramCancelableCallback.onFinish();
				}
				return;
			}
			m_animator.removeAllListeners();
			// m_animator.cancel();
			m_animator = null;
			m_animator = ValueAnimator.ofObject(new CameraEvaluator(),
					new KCameraPosition(cameraPosition.target,
							cameraPosition.zoom, cameraPosition.tilt,
							cameraPosition.bearing), new KCameraPosition(
							update.targetPosition.target,
							update.targetPosition.zoom,
							update.targetPosition.tilt,
							update.targetPosition.bearing));
		}
		cancelableCallback = paramCancelableCallback;

		m_animator.addUpdateListener(new ValueAnimator.AnimatorUpdateListener() {
					@Override
					public void onAnimationUpdate(ValueAnimator animation) {
						KCameraPosition cur = (KCameraPosition) animation.getAnimatedValue();
						KCameraUpdate curUpdate = new KCameraUpdate(cur);
						curUpdate.m_byUpdateType = targetUpdate.m_byUpdateType;
						mapGLSurfaceView.updateCamera(curUpdate);
						if (m_cameraChangeListener != null) {
							m_cameraChangeListener.onCameraChange(cur);
						}
					}
				});

		m_animator.addListener(new AnimatorListener() {

			@Override
			public void onAnimationStart(Animator animation) {

			}

			@Override
			public void onAnimationRepeat(Animator animation) {

			}

			@Override
			public void onAnimationEnd(Animator animation) {
				if (cancelableCallback != null) {
					cancelableCallback.onFinish();
					cancelableCallback = null;
				}
				mapGLSurfaceView.setFastRefresh(false);
				mapGLSurfaceView.requestRender();
				Message message = new Message();
				message.what = KMap.MAP_CAMERA_ANIMATE_END;
				mHandler.sendMessageDelayed(message, 100);
			}

			@Override
			public void onAnimationCancel(Animator animation) {
				if (cancelableCallback != null) {
					cancelableCallback.onCancel();
					cancelableCallback = null;
				}
				mapGLSurfaceView.setFastRefresh(false);
				mapGLSurfaceView.requestRender();
			}
		});
		m_animator.setDuration(durationMs);
		m_animator.setInterpolator(new DecelerateInterpolator()); // LinearInterpolator
		m_animator.start();

	}

	/**
	 * 从地图上删除所有的Marker，Overlay，Polyline 等覆盖物
	 */
	public void clear() {
		synchronized (shapeOverlays) {
			for (int i = 0; i < shapeOverlays.size(); i++) {
				KShapeOverlay overlay = shapeOverlays.get(i);
				if (overlay != null) {
					if (overlay instanceof KOverlayPoints) {
						((KOverlayPoints)overlay).remove();
					}
				}
			}
			shapeOverlays.clear();
		}
		synchronized (infoViewList) {
			infoViewList.clear();			
		}
	}

	/**
	 * 底层地图中删除所有overlay
	 */
	protected void DelOverlayInMap() {
		synchronized (infoViewList) {
			for (int i = 0; i < infoViewList.size(); ++i) {
				InfoView infoView = infoViewList.get(i);
				if (infoView != null
						&& !infoView.marker.equals(m_infowindowSubstitute)) {
					KMap.this.mapView.removeShapeOverlay(infoView.marker);
				}
			}			
		}

		if (m_infowindowSubstitute != null) {
			m_infowindowSubstitute.remove();
			m_infowindowSubstitute = null;
		}
		
		if (m_infowindowDescriptor != null) {
			m_infowindowDescriptor.recycle();
			m_infowindowDescriptor = null;
		}

		synchronized (shapeOverlays) {
			for (int i = 0; i < shapeOverlays.size(); ++i) {
				KShapeOverlay shape = shapeOverlays.get(i);
				KMap.this.mapView.removeShapeOverlay(shape);
			}			
		}
	}

	/**
	 * 在底层地图中添加所有overlay
	 */
	protected void AddOverlayInMap() {
		synchronized (infoViewList) {
			for (int i = 0; i < infoViewList.size(); ++i) {
				InfoView infoView = infoViewList.get(i);
				KMap.this.mapView.addShapeOverlay(infoView.marker);
				// 需要显示infowindow的marker，重新显示infowindow
				if (infoView.marker.isInfoWindowShown()) {
					infoView.marker.hideInfoWindow();
					infoView.marker.showInfoWindow();
				}
			}			
		}

		synchronized (shapeOverlays) {
			for (int i = 0; i < shapeOverlays.size(); ++i) {
				KShapeOverlay shape = shapeOverlays.get(i);
				KMap.this.mapView.addShapeOverlay(shape);
			}			
		}
	}

	/**
	 * 获取地图的当前的可视区域
	 * @return 地图的当前的可视区域
	 */
	public KCameraPosition getCameraPosition() {
		return new KCameraUpdate().targetPosition;
	}

	public boolean is2DMode() {
		KCameraPosition c = getCameraPosition();
		if (c.tilt == 0) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * 返回地图底图文字的z轴索引。
	 * @return 地图底图文字的z轴索引，默认：0
	 */
	public float getMapTextZIndex() {
		// 由于是异步设置到c++层，从c++层获取的值不可靠
		return mCurMapStatus.m_textZIndex;
	}

	/**
	 * 获取当前的地图显示类型。0：白天。1：黑夜。2：卫星 * @return 地图显示类型，参见： <br>
	 * {@link #MAP_TYPE_NORMAL} <br>
	 * {@link #MAP_TYPE_NIGHT} <br>
	 * {@link #MAP_TYPE_SATELLITE} <br>
	 * {@link #MAP_TYPE_NAVI}
	 */
	public int getMapType() {
		// 由于是异步设置到c++层，从c++层获取的值不可靠
		return mCurMapStatus.m_MapType;
	}

	/**
	 * 获取可视区域的最大缩放级别
	 * 
	 * @return 地图最大缩放级别
	 */
	public float getMaxZoomLevel() {
		return (float) MapCore.GetMaxScale();
	}

	/**
	 * 获取可视区域的最小缩放级别
	 * 
	 * @return 地图最小缩放级别
	 */
	public float getMinZoomLevel() {
		return MapCore.GetMinScale();
	}

	/**
	 * 返回当前的定位信息
	 * 
	 * @return
	 */
	public Location getMyLocation() {
		if (m_locationSource == null) {
			return null;
		}
		return m_location;
	}

	/**
	 * 获取设置的实时交通的式样对象
	 * @return 实时交通路况拥堵对应的颜色属性，参见{@link MyTrafficStyle}
	 */
	public MyTrafficStyle getMyTrafficStyle() {
		return MapCore.GetTrafficStyle();
	}

	/**
	 * 返回地图坐标映射对象。 <br>
	 * 用户通过这个对象可以在屏幕坐标与经纬度坐标之间进行转换。
	 * 
	 * @return 地图坐标映射对象。
	 */
	public KProjection getProjection() {
		return this.projection;
	}

	/**
	 * 获取当前地图上一个像素代表的比例尺长度(米)
	 * @return 当前缩放级别下，地图上单位像素点对应的长度，单位：米
	 */
	public float getScalePerPixel() {
		return MapCore.GetScalePerPixel();
	}

	/**
	 * 返回地图用户界面设置对象。 <br>
	 * 地图用户界面设置对象，可以允许用户控制地图自带控件的显示以及手势操作使能。
	 * 
	 * @return 地图用户界面设置对象
	 */
	public KUiSettings getUiSettings() {
		KUiSettings settings = mapView.widgetView.getSettings();
		return settings;
	}

	/**
	 * 返回地图用户界面设置对象。 <br>
	 * 地图用户界面设置对象，可以允许用户控制地图自带控件的显示以及手势操作使能。
	 * 
	 * @return 地图用户界面设置对象
	 */
	public String getSDKVersion() {
		return MapCore.getSDKVersion();
	}

	/**
	 * 返回location层的显示状态
	 * 
	 * @return
	 */
	public boolean isMyLocationEnabled() {

		return mapView.widgetView.isMyLocationEnabled();
	}

	/**
	 * 返回当前地图是否有实时交通数据
	 * 
	 * @return 是否显示实时交通，默认：否
	 */
	public boolean isTrafficEnabled() {

		return RTICThread.GetInstance().GetRTICUse();
	}
	
	/**
     * 返回是否显示路况事件。
     * @return 是否显示路况事件，默认：否
     */
	public boolean isTrafficEventEnabled() {
	    return RTICThread.GetInstance().GetRTICEventUse();
	}

	/**
	 * 改变地图可视范围。 <br>
	 * 这个方法为瞬间移动，没有移动过程。
	 * 
	 * @param update 地图可视范围更新对象
	 */
	@SuppressLint("NewApi")
	public void moveCamera(KCameraUpdate update) {
	    if (update == null
	            || update.targetPosition == null) {
	        return;
	    }
	    
		if (m_animator != null && m_animator.isRunning()) {
			// 当前有动画，则保存，等动画结束后设置
			m_qCameraMoveReq.offer(update);
			return;
		}
		
		// 当前mapview在后台运行，保存camera请求
		if (m_bBackground) {
			moveCameraInBackground(update);
			return;
		}

		if (mapGLSurfaceView != null) {
			mapGLSurfaceView.updateCamera(update);
		}
		Message message = new Message();
		message.what = KMap.MAP_GESTURE_END;
		mHandler.sendMessageDelayed(message, 500);
	}

	private void moveCameraInBackground(KCameraUpdate update) {
		// mapview在后台状态，设置camera，只保存，不设置
		if (update.isUpdateBearing()) {
			mCurMapStatus.m_Camera.targetPosition.bearing = update.targetPosition.bearing;
		}
		if (update.isUpdateLatLng()) {
			mCurMapStatus.m_Camera.targetPosition.target = update.targetPosition.target;
		}
		if (update.isUpdateTilt()) {
			mCurMapStatus.m_Camera.targetPosition.tilt = update.targetPosition.tilt;
		}
		if (update.isUpdateZoom()) {
			mCurMapStatus.m_Camera.targetPosition.zoom = update.targetPosition.zoom;
		}
	}

	/**
	 * 删除地图存储目录下的手机地图缓存
	 */
	public void removeCache() {
		MapCore.ClearCache();
	}
	
	/**
	 * 设置文本放大比例,参数必须大于零。
	 */	
	public void SetTextSizeZoomRate(float rate) {
		MapCore.SetTextSizeZoomRate(rate);
	}	

	/**
	 * 设置信息窗口创建回调接口。
	 * 
	 * @param adapter 信息窗口创建回调接口
	 */
	public void setInfoWindowAdapter(InfoWindowAdapter adapter) {
		m_infoWindowAdapter = adapter;
	}

	/**
	 * 设置地图底图文字的z轴索引。
	 * 
	 * @param zIndex 地图底图文字的z轴索引
	 */
	public void setMapTextZIndex(float index) {
		if (mCurMapStatus.m_textZIndex != index) {
			mCurMapStatus.m_textZIndex = index;
			mapView.setMapTextZIndex(index);
		}
	}

	/**
	 * 设置地图显示的类型(建议在onMapLoaded之后调用)。
	 * 
	 * @param type
	 *            地图显示类型，参见： <br>
	 *            {@link #MAP_TYPE_NORMAL} <br>
	 *            {@link #MAP_TYPE_NIGHT} <br>
	 *            {@link #MAP_TYPE_SATELLITE} <br>
	 *            {@link #MAP_TYPE_NAVI}
	 */
	public void setMapType(int type) {
		if (mCurMapStatus.m_MapType != type) {
			mCurMapStatus.m_MapType = type;
			mapGLSurfaceView.setMapType(type);
		}
	}

	/**
	 * 设置定位层是否显示
	 * 
	 * @param bEnable
	 */
	public void setMyLocationEnabled(boolean bEnable) {
		if (m_locationSource != null && bEnable != m_bLocationSourceEnable) {
			m_bLocationSourceEnable = bEnable;
			if (bEnable) {
				m_locationSource.activate(this);
			} else {
				m_locationSource.deactivate();
			}
		}

		Message msg = new Message();
		msg.what = KMapWidgetView.SET_LOCATION_ENABLE_MSG;
		msg.arg1 = bEnable ? 1 : 0;
		mapView.widgetView.mHandler.sendMessage(msg);
	}

	public final void setMyLocationRotateAngle(float rotate) {
		if (m_location == null) {
			m_location = new Location("");
			m_location.setAccuracy(100000); // 设置定位不可信
		}
		m_location.setBearing(rotate);

		if (m_locationSource != null) {
			Message msg = new Message();
			msg.what = KMapWidgetView.SET_LOCATION_CHANGED_MSG;
			mapView.widgetView.mHandler.sendMessage(msg);
		}
	}

	public void onLocationChanged(Location location) {
		if (location == null)
			return;

		if (myLocationChangeListener != null) {
			myLocationChangeListener.onMyLocationChange(location); // 向监听者发消息
		}

		if (m_location == null) {
			m_location = location;
		}

		m_location.setLatitude(location.getLatitude());
		m_location.setLongitude(location.getLongitude());

		if (this.m_locationSource != null) {
			Message msg = new Message();
			msg.what = KMapWidgetView.SET_LOCATION_CHANGED_MSG;
			mapView.widgetView.mHandler.sendMessage(msg);
		}
	}

	/**
	 * 设置定位（当前位置）的绘制样式
	 * 
	 * @param style
	 */
	public void setMyLocationStyle(MyLocationStyle style) {
		mapView.widgetView.m_location_style = style;
		Message msg = new Message();
		msg.what = KMapWidgetView.SET_LOCATION_STYLE_MSG;
		mapView.widgetView.mHandler.sendMessage(msg);
	}

	/**
	 * 设置定位的类型
	 * 
	 * @param type
	 *            提供3种类型： LOCATION_TYPE_LOCATE 表示只在第一次定位移动到地图中心点，值为1；
	 *            LOCATION_TYPE_MAP_FOLLOW 表示定位、移动到地图中心点并跟随，值为2；
	 *            LOCATION_TYPE_MAP_ROTATE 表示定位、移动到地图中心点，跟踪并根据方向旋转地图，值为3。
	 */
	public void setMyLocationType(int type) {
		Message msg = new Message();
		msg.what = KMapWidgetView.SET_LOCATION_TYPE_MSG;
		msg.arg1 = type;
		mapView.widgetView.mHandler.sendMessage(msg);
	}

	// 设置定位信息监听者
	public final void setOnMyLocationChangeListener(
			OnMyLocationChangeListener listener) {
		if (m_locationSource != null) {
			myLocationChangeListener = listener;
		}
	}

	/**
	 * 设置实时交通自定义的路况拥堵颜色属性。
	 * 
	 * @param style 实时交通自定义的路况拥堵颜色属性，参见{@link MyTrafficStyle}
	 */
	public void setMyTrafficStyle(MyTrafficStyle style) {
		MapCore.SetTrafficStyle(style);
	}

	/**
	 * 设置屏幕上的某个点为地图视图旋转中心点，默认值为(0.5,0.5)，标识以地图视图中心点为地图视图旋转中心点。
	 * 
	 * @param x 地图视图旋转中心点横向比例，范围为 (0,1)，默认：0.5
	 * @param y 地图视图旋转中心点横向比例，范围为 (0,1)，默认：0.5
	 */
	public void setPointToCenter(float x, float y) {
		mapView.setMapRotateCenter(new PointF(x, y));
	}

	/**
	 * 设置是否显示地图3D建筑物，默认：显示。
	 * 
	 * @param enabled 是否显示地图3D建筑物
	 */
	public void showBuildings(boolean enabled) {
		if (mCurMapStatus.m_buildingShow != enabled) {
			mapView.showBuildings(enabled);
			mCurMapStatus.m_buildingShow = enabled;
		}
	}

	/**
	 * 设置是否显示地图底图文字，默认：显示。
	 * 
	 * @param enabled 是否显示地图底图文字
	 */
	public void showMapText(boolean enabled) {
		if (mCurMapStatus.m_textShow != enabled) {
			mapView.showMapText(enabled);
			mCurMapStatus.m_textShow = enabled;
		}
	}

	/**
	 * 设置是否显示天空背景，默认：显示。
	 * 
	 * @param enabled 是否显示天空背景
	 */
	public void showSkyBackground(boolean enabled) {
		if (mCurMapStatus.m_skyShow != enabled) {
			mapView.showSkyBackground(enabled);
			mCurMapStatus.m_skyShow = enabled;
		}
	}

	/**
	 * 设置实时交通是否可用
	 * 
	 * @param bEnable
	 *            是否显示实时交通
	 */
	public void setTrafficEnabled(boolean bEnable) {
	    if (bEnable != mCurMapStatus.m_bRTIC) {	        
	        RTICThread.GetInstance().SetRTICUse(bEnable);
	        mapGLSurfaceView.setTrafficEnabled(bEnable);
	        mCurMapStatus.m_bRTIC = bEnable;
	    }
	}
	
	public void setTrafficEventEnabled(boolean enabled) {
	    if (enabled != mCurMapStatus.m_bRTICEvent) {	        
	        RTICThread.GetInstance().setEventUse(enabled);
	        mapGLSurfaceView.setTrafficEventEnabled(enabled);
	        mCurMapStatus.m_bRTICEvent = enabled;
	    }
    }

	/**
	 * 终止地图camera动画
	 */
	@SuppressLint("NewApi")
	public void stopAnimation() {
		m_qCameraMoveReq.clear();

		if (m_animator != null) {
			if (m_animator.isRunning()) {
				m_animator.removeAllListeners();
				m_animator.cancel();
			}
			m_animator = null;
		}

		if (cancelableCallback != null) {
			cancelableCallback.onCancel();
			cancelableCallback = null;
		}

	}

	/**
	 * 设置启用外部定位源。
	 * 
	 * @param type
	 */
	public void setLocationSource(KLocationSource locationSource) {
		if (m_locationSource != null) {
			m_locationSource.deactivate();
			m_bLocationSourceEnable = false;
		}

		m_locationSource = locationSource;
		if (m_locationSource != null) {
			m_locationSource.activate(this);
			m_bLocationSourceEnable = true;
		}
	}

	// 公共方法完毕

	private void viewForMarker(final KMarker marker) {
		InfoView infoViews = new InfoView(GlobalSettings.getInstance()
				.getContext(), marker);

		if (marker.getMarkerOptions().isAnimatesDrop()) {
			// 执行marker的下落动画
			infoViews.playAnimatesDrop(marker);
		} else {
			infoViews.changeMarkerIcon();
		}
		synchronized (infoViewList) {
			infoViewList.add(infoViews);			
		}
	}

	/**
	 * 建立infowindow的View
	 * @return marekr的信息窗口view
	 */
	View constructInfoWindow(KMarker marker) {
		View view = null;
		if (m_infoWindowAdapter == null) {
			// DefaultView
			view = new KDefaultInfoWindowView(GlobalSettings.getInstance()
					.getContext(), marker);
		} else {
			if (marker != null) {
				view = m_infoWindowAdapter.getInfoWindow(marker);				
			}
			if (view == null) {
				View viewContent = null;
				if (marker != null) {
					viewContent = m_infoWindowAdapter.getInfoContents(marker);					
				}

				if (viewContent == null) {
					// 默认的view
					view = new KDefaultInfoWindowView(GlobalSettings
							.getInstance().getContext(), marker);
				} else {
					view = new KDefaultInfoWindowView(GlobalSettings
							.getInstance().getContext(), viewContent);
				}
			} else {
				// do nothing
			}
		}

		return view;
	}

	/**
	 * marker的顺序有改变，需要重新排序
	 */
	private void resortInfoView() {
		synchronized (infoViewList) {
			List<InfoView> temp = new ArrayList<InfoView>(infoViewList);
			Collections.sort(temp);
			infoViewList.clear();
			infoViewList.addAll(temp);
			mapView.reSortInfoView();			
		}
	}
	
	private void addShapeOverlays(KShapeOverlay overlay) {
		synchronized (shapeOverlays) {
			shapeOverlays.add(overlay);
			List<KShapeOverlay> temp = new ArrayList<KShapeOverlay>(shapeOverlays);
			Collections.sort(temp);
			shapeOverlays.clear();
			shapeOverlays.addAll(temp);
		}
	}

	CopyOnWriteArrayList<InfoView> getInfoViews() {
		return infoViewList;
	}
	
	CopyOnWriteArrayList<KShapeOverlay> getShapeOverlays() {
		return shapeOverlays;
	}

	/**
	 * marker多图片定时切换任务
	 */
	public class MarkerTask extends TimerTask {

		KMarker m_marker;

		public MarkerTask() {

		}

		void setMarker(KMarker m) {
			m_marker = m;
		}

		public void run() {
			m_marker.changeIndex();
		}
	}

	public class InfoView extends FrameLayout implements Comparable<InfoView> {
		public KMarker marker;
		public View infoView = null;
		private Timer m_iconAnimatorTimer = null;
		private MarkerTask m_iconAnimatorTask = null;
		ValueAnimator m_animateDrop = null;
		private static final float FLOAT_ACCURACY = 0.000001f;

		public InfoView(Context context) {
			super(context);
		}

		void releaseResource() {
			marker = null;
			hideInfoWindow();
			
			if (m_iconAnimatorTimer != null) {
				m_iconAnimatorTimer.cancel();
				m_iconAnimatorTimer = null;
			}
			m_iconAnimatorTask = null;
			
			stopAnimateDrop();
		}

		InfoView(Context context, KMarker marker) {
			super(context);
			this.marker = marker;

			FrameLayout.LayoutParams layoutParams = new FrameLayout.LayoutParams(
					FrameLayout.LayoutParams.WRAP_CONTENT,
					FrameLayout.LayoutParams.WRAP_CONTENT);
			layoutParams.bottomMargin = -2000;
			layoutParams.rightMargin = -1000;
			this.setLayoutParams(layoutParams);
		}

		@SuppressLint("NewApi")
		void playAnimatesDrop(final KMarker marker) {
			final boolean bGps = marker.getMarkerOptions().isGps();
			final KLatLng bakLatLng = marker.getPosition();
			KLatLng startLatlng = null;
			if (bGps) {
				Point point = projection.toScreenLocation(bakLatLng);
				startLatlng = projection.fromScreenLocation(new Point(point.x,
						0));
			}
			final KLatLng start = startLatlng;
			final Point bakPoint = new Point(marker.getMarkerOptions().getX(),
					marker.getMarkerOptions().getY());

			m_animateDrop = ValueAnimator.ofFloat(0, 1);
			m_animateDrop.setDuration(500);
			m_animateDrop.addUpdateListener(new AnimatorUpdateListener() {

				@Override
				public void onAnimationUpdate(ValueAnimator animation) {
					if (bGps) {
						// 经纬度
						double lat = (bakLatLng.latitude - start.latitude)
								* (Float) animation.getAnimatedValue()
								+ start.latitude;
						double lon = (bakLatLng.longitude - start.longitude)
								* (Float) animation.getAnimatedValue()
								+ start.longitude;
						marker.setPosition(new KLatLng(lat, lon));
					} else {
						// 屏幕坐标
						marker.setPositionByPixels(
								bakPoint.x,
								(int) ((Float) (animation.getAnimatedValue()) * bakPoint.y));
					}
				}
			});

			m_animateDrop.addListener(new AnimatorListener() {

				@Override
				public void onAnimationStart(Animator animation) {

				}

				@Override
				public void onAnimationRepeat(Animator animation) {

				}

				@Override
				public void onAnimationEnd(Animator animation) {
					changeMarkerIcon();
				}

				@Override
				public void onAnimationCancel(Animator animation) {
					changeMarkerIcon();
				}
			});
			m_animateDrop.start();
		}

		@SuppressLint("NewApi")
		public void stopAnimateDrop() {
			if (m_animateDrop != null && m_animateDrop.isRunning()) {
				m_animateDrop.cancel();
				m_animateDrop = null;
			}
		}

		/**
		 * 执行marker多图片定时切换
		 */
		public void changeMarkerIcon() {
			final ArrayList<BitmapDescriptor> iconList = marker.getIcons();
			if (iconList.size() > 1) {
				if (m_iconAnimatorTimer != null) {
					m_iconAnimatorTimer.cancel();
				}
				m_iconAnimatorTask = new MarkerTask();
				m_iconAnimatorTimer = new Timer();
				m_iconAnimatorTask.setMarker(marker);
				int count = marker.getPeriod();
				m_iconAnimatorTimer.schedule(m_iconAnimatorTask, 0, count);
			}
		}

		@Override
		public int compareTo(InfoView another) {
			// marker覆盖物比较，ZIndex越大则优先度越高，ZIndex相同时，后添加的优先度越高
			if (marker.getZIndex() - another.marker.getZIndex() > FLOAT_ACCURACY) {
				return -1;
			} else if (another.marker.getZIndex() - marker.getZIndex() > FLOAT_ACCURACY) {
				return 1;
			} else {
				int iRet = another.marker.getOverlayId() > marker
						.getOverlayId() ? 1 : -1;
				return iRet;
			}
		}

		/**
		 * 更新信息窗口view的位置
		 */
		public void recalcPos() {
			if (infoView != null && infoView.getVisibility() == View.VISIBLE) {
//				Log.i("xxx", "infowindow refresh!!!");
				int leftMargin = 0;
				int topMargin = 0;
				FrameLayout.LayoutParams params = (FrameLayout.LayoutParams) this.getLayoutParams();
				Rect infoRect = new Rect();
				infoView.getHitRect(infoRect);
				Rect markerRect = new Rect();
				getMarkerHitRect(markerRect);
				// 默认信息窗口显示与marker居中对齐
				leftMargin = markerRect.left - (infoRect.width() - markerRect.width()) / 2;
				topMargin = markerRect.top - infoRect.height();

				if (leftMargin != params.leftMargin
						|| topMargin != params.topMargin) {
					params.leftMargin = leftMargin + marker.getMarkerOptions().getInfoWindowOffsetX();
					params.topMargin = topMargin + marker.getMarkerOptions().getInfoWindowOffsetY();
					this.setLayoutParams(params);
				}
			}
		}

		public KLatLng calcSubtitudePos() {
			KLatLng pos = null;
			if (marker.isInfoWindowShown() && marker.getMarkerOptions().isGps()) {
				Point f = projection.toScreenLocation(marker.getPosition());
				Bitmap bitmap = marker.getIcons().get(0).getBitmap();
				if (bitmap != null) {
					int top = (int) (f.y - marker.getAnchorV() * bitmap.getHeight());
					int left = (int) (f.x - marker.getAnchorU() * bitmap.getWidth());
					Rect markerRect = new Rect(left, top, left + bitmap.getWidth(), top + bitmap.getHeight());

					Point cursorPoint = new Point((markerRect.left + markerRect.right) / 2 + marker.getMarkerOptions() .getInfoWindowOffsetX(),
							markerRect.top + marker.getMarkerOptions() .getInfoWindowOffsetY());
					pos = projection.fromScreenLocation(cursorPoint);
				}
			}
			return pos;
		}

		/**
		 * 获取marker可点击区域
		 * @param rect 可点击区域
		 */
		public void getMarkerHitRect(Rect rect) {
			if (marker.getMarkerOptions().isGps()) {
				Point f = projection.toScreenLocation(marker.getPosition());
				Bitmap bitmap = marker.getIcons().get(0).getBitmap();
				if (bitmap != null) {
					int top = (int) (f.y - marker.getAnchorV() * bitmap.getHeight());
					int left = (int) (f.x - marker.getAnchorU() * bitmap.getWidth());
					rect.set(left, top, left + bitmap.getWidth(), top + bitmap.getHeight());
					transByRotate(rect, marker);
				} else {
					rect.set(f.x, f.y, f.x, f.y);
				}
			} else {
				Bitmap bitmap = marker.getIcons().get(0).getBitmap();
				if (bitmap != null) {
					int top = (int) (marker.getMarkerOptions().getY() - marker.getAnchorV() * bitmap.getHeight());
					int left = (int) (marker.getMarkerOptions().getX() - marker.getAnchorU() * bitmap.getWidth());
					rect.set(left, top, left + bitmap.getWidth(), top + bitmap.getHeight());
					transByRotate(rect, marker);
				} else {
					rect.set(marker.getMarkerOptions().getX(), marker.getMarkerOptions().getY(), 
							marker.getMarkerOptions().getX(), marker.getMarkerOptions().getY());
				}
			}
		}

		private void transByRotate(Rect rect, KMarker marker) {
			RectF rectF = new RectF(rect);
			Matrix matrix = new Matrix();
			float px = rectF.left + rectF.width() * marker.getAnchorU();
			float py = rectF.top + rectF.height() * marker.getAnchorV();
			matrix.setRotate(360 - marker.getRotateAngle(), px, py);
			RectF dest = new RectF();
			matrix.mapRect(dest, rectF);
			rect.set((int) dest.left, (int) dest.top, (int) dest.right, (int) dest.bottom);
		}

		public void getInfoHitRect(Rect rect) {
			Rect infoRect = new Rect();
			infoView.getHitRect(infoRect);
			Rect layoutRect = new Rect();
			this.getHitRect(layoutRect);
			rect.set(infoRect.left + layoutRect.left, infoRect.top
					+ layoutRect.top, infoRect.right + layoutRect.left,
					infoRect.bottom + layoutRect.top);
		}

		public void showInfoWindow(View infoView) {
			this.removeAllViews();
			this.infoView = infoView;
			this.addView(infoView);

			this.infoView.setVisibility(View.VISIBLE);
			recalcPos();
		}

		public void hideInfoWindow() {
			if (infoView != null) {
				this.removeAllViews();
				infoView = null;
			}
		}

		public void setInfoWindowVisible(int visibility) {
			if (infoView != null && infoView.getVisibility() != visibility) {
				infoView.setVisibility(visibility);
				recalcPos();
				mapView.reFreshInfoOverlay();
			}
		}
	}

	public void onMarkerClick(KMarker marker) {
		if (m_markerClickListener != null && marker != null) {
			m_markerClickListener.onMarkerClick(marker);
		}
	}

	public void onMarkerDrag(KMarker marker) {
		if (m_markerDragListener != null && marker != null) {
			m_markerDragListener.onMarkerDrag(marker);
		}
	}

	public void onMarkerDragStart(KMarker marker) {
		if (m_markerDragListener != null && marker != null) {
			m_markerDragListener.onMarkerDragStart(marker);
		}
	}

	public void onMarkerDragEnd(KMarker marker) {
		if (m_markerDragListener != null && marker != null) {
			m_markerDragListener.onMarkerDragEnd(marker);
		}
	}
	
	public void onShapeOverlayClick(KShapeOverlay overlay, Point point) {
		if (overlay instanceof KArc) {
			if(m_mapArcClickListener != null && overlay != null) {
				m_mapArcClickListener.onArcClick((KArc)overlay, projection.fromScreenLocation(point));
			}
		} else if (overlay instanceof KCircle) {
			if(m_mapCircleClickListener != null && overlay != null) {
				m_mapCircleClickListener.onCircleClick((KCircle)overlay, projection.fromScreenLocation(point));
			}
		} else if (overlay instanceof KGroundOverlay) {
			if(m_mapGroundOverlayClickListener != null && overlay != null) {
				m_mapGroundOverlayClickListener.onGroundOverlayClick((KGroundOverlay)overlay, projection.fromScreenLocation(point));
			}
		} else if (overlay instanceof KPolyline) {
			if(m_mapPolylineClickListener != null && overlay != null) {
				m_mapPolylineClickListener.onPolylineClick((KPolyline)overlay, projection.fromScreenLocation(point));
			}
		} else if (overlay instanceof KPolygon) {
			if(m_mapPolygonClickListener != null && overlay != null) {
				m_mapPolygonClickListener.onPolygonClick((KPolygon)overlay, projection.fromScreenLocation(point));
			}
		}
	}
	
	public void onShapeOverlayDrag(KShapeOverlay overlay, Point point) {
		if (overlay instanceof KArc) {
			if(m_mapArcDragListener != null && overlay != null) {
				m_mapArcDragListener.onArcDrag((KArc)overlay, projection.fromScreenLocation(point));
			}
		} else if (overlay instanceof KCircle) {
			if(m_mapCircleDragListener != null && overlay != null) {
				m_mapCircleDragListener.onCircleDrag((KCircle)overlay, projection.fromScreenLocation(point));
			}
		} else if (overlay instanceof KGroundOverlay) {
			if(m_mapGroundOverlayDragListener != null && overlay != null) {
				m_mapGroundOverlayDragListener.onGroundOverlayDrag((KGroundOverlay)overlay, projection.fromScreenLocation(point));
			}
		} else if (overlay instanceof KPolyline) {
			if(m_mapPolylineDragListener != null && overlay != null) {
				m_mapPolylineDragListener.onPolylineDrag((KPolyline)overlay, projection.fromScreenLocation(point));
			}
		} else if (overlay instanceof KPolygon) {
			if(m_mapPolygonDragListener != null && overlay != null) {
				m_mapPolygonDragListener.onPolygonDrag((KPolygon)overlay, projection.fromScreenLocation(point));
			}
		}
	}
	
	public void onShapeOverlayDragStart(KShapeOverlay overlay, Point point) {
		if (overlay instanceof KArc) {
			if(m_mapArcDragListener != null && overlay != null) {
				m_mapArcDragListener.onArcDragStart((KArc)overlay, projection.fromScreenLocation(point));
			}
		} else if (overlay instanceof KCircle) {
			if(m_mapCircleDragListener != null && overlay != null) {
				m_mapCircleDragListener.onCircleDragStart((KCircle)overlay, projection.fromScreenLocation(point));
			}
		} else if (overlay instanceof KGroundOverlay) {
			if(m_mapGroundOverlayDragListener != null && overlay != null) {
				m_mapGroundOverlayDragListener.onGroundOverlayDragStart((KGroundOverlay)overlay, projection.fromScreenLocation(point));
			}
		} else if (overlay instanceof KPolyline) {
			if(m_mapPolylineDragListener != null && overlay != null) {
				m_mapPolylineDragListener.onPolylineDragStart((KPolyline)overlay, projection.fromScreenLocation(point));
			}
		} else if (overlay instanceof KPolygon) {
			if(m_mapPolygonDragListener != null && overlay != null) {
				m_mapPolygonDragListener.onPolygonDragStart((KPolygon)overlay, projection.fromScreenLocation(point));
			}
		}
	}

	public void onShapeOverlayDragEnd(KShapeOverlay overlay, Point point) {
		if (overlay instanceof KArc) {
			if(m_mapArcDragListener != null && overlay != null) {
				m_mapArcDragListener.onArcDragEnd((KArc)overlay, projection.fromScreenLocation(point));
			}
		} else if (overlay instanceof KCircle) {
			if(m_mapCircleDragListener != null && overlay != null) {
				m_mapCircleDragListener.onCircleDragEnd((KCircle)overlay, projection.fromScreenLocation(point));
			}
		} else if (overlay instanceof KGroundOverlay) {
			if(m_mapGroundOverlayDragListener != null && overlay != null) {
				m_mapGroundOverlayDragListener.onGroundOverlayDragEnd((KGroundOverlay)overlay, projection.fromScreenLocation(point));
			}
		} else if (overlay instanceof KPolyline) {
			if(m_mapPolylineDragListener != null && overlay != null) {
				m_mapPolylineDragListener.onPolylineDragEnd((KPolyline)overlay, projection.fromScreenLocation(point));
			}
		} else if (overlay instanceof KPolygon) {
			if(m_mapPolygonDragListener != null && overlay != null) {
				m_mapPolygonDragListener.onPolygonDragEnd((KPolygon)overlay, projection.fromScreenLocation(point));
			}
		}
	}

	/**
	 * 当信息窗口被点击时回调此方法 。
	 * 
	 * @param marker 被点击的信息窗口的标记覆盖物对象
	 */

	public void onInfoWindowClick(KMarker marker) {
		if (m_infoWindowClickListener != null && marker != null) {
			m_infoWindowClickListener.onInfoWindowClick(marker);
		}
	}

	/**
	 * 地图加载完成后回调此方法。
	 */
	public void onMapLoaded() {
		if (m_mapLoadedListener != null) {
			m_mapLoadedListener.onMapLoaded();
		}
	}

	public void onMapPropUpdate(String mapVersion) {
		if (m_mapPropUpdateListener != null) {
			m_mapPropUpdateListener.onMapVersionUpdate(mapVersion);
		}
	}

	// 该方法必须被Activity主动调用
	public void onPause() {
		// 备份当前地图状态，用于多mapview实例切换
		mCurMapStatus.m_Camera = new KCameraUpdate(getCameraPosition());
		mCurMapStatus.m_bRTIC = isTrafficEnabled();
		mCurMapStatus.m_bRTICEvent = isTrafficEventEnabled();
		m_bBackground = true;
		mapGLSurfaceView.onPause();
		MapCore.setRunType(1);
		RTICThread.GetInstance().endTimer();
		// 留给GL线程释放
		try {
			Thread.sleep(200);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		// Log.i("MapSysJNI", "Map onPause");

	}

	// 该方法必须被Activity主动调用
	public void onResume() {
		mapGLSurfaceView.setCurrentGestureHandler();
		mapGLSurfaceView.onResume();
		MapCore.setRunType(0);
		resetMapStatus();
		m_bBackground = false;
		RTICThread.GetInstance().startTimer();

		// Log.i("MapSysJNI", "Map onResume");
	}

	public void onCreate() {
		resetMapStatus();
	}

	public void onDestroy() {
	    synchronized (infoViewList) {            
	        if (m_infowindowSubstitute != null) {
	            m_infowindowSubstitute.remove();
	            m_infowindowSubstitute = null;
	        }
	        
	        if (m_infowindowDescriptor != null) {
	            m_infowindowDescriptor.recycle();
	            m_infowindowDescriptor = null;
	        }
        }
	}

	/**
	 * 重置地图状态
	 */
	private void resetMapStatus() {
		setTrafficEnabled(mCurMapStatus.m_bRTIC);
		setTrafficEventEnabled(mCurMapStatus.m_bRTICEvent);
		mapView.showBuildings(mCurMapStatus.m_buildingShow);
		mapView.showMapText(mCurMapStatus.m_textShow);
		mapView.showSkyBackground(mCurMapStatus.m_skyShow);
		mapView.setMapTextZIndex(mCurMapStatus.m_textZIndex);
		mapGLSurfaceView.setMapType(mCurMapStatus.m_MapType);
		mapGLSurfaceView.updateCamera(mCurMapStatus.m_Camera);
	}

	/**
	 * 设置卫片数据url
	 * 
	 * @param strTileUrl 卫片数据url
	 * @param xOrder 参数x的次序，取值范围0,1,2
	 * @param yOrder 参数y的次序，取值范围0,1,2
	 * @param zOrder 参数z的次序，取值范围0,1,2
	 */
	public void setTileUrl(String strTileUrl, int xOrder, int yOrder, int zOrder) {
		MapCore.setTileUrl(strTileUrl, xOrder, yOrder, zOrder);
	}

	/**
	 * 在地图快速变化（手势操作、camera动画）时，处理marker的信息窗口
	 * @param visible
	 */
	public void processInfowindowInQuickChange(int visible) {
	  synchronized (infoViewList) {
		if (m_infoViewWithwindow != null
				&& m_infoViewWithwindow.marker.isInfoWindowShown()
				&& m_infoViewWithwindow.marker.getMarkerOptions().isGps()) {
			if (visible == View.VISIBLE) {
				m_infoViewWithwindow.setInfoWindowVisible(visible);
			} else if (!m_bWaitingAddInfoWindow && m_infowindowSubstitute != null) {
				m_infowindowSubstitute.setVisible(true);
				mapView.refreshInfowindowSubtitude(m_infoViewWithwindow, m_infowindowSubstitute);
				mapGLSurfaceView.requestRender();
			}
			Message msg = Message.obtain();
			msg.arg1 = visible;
			msg.what = INFOWINDOW_MOVE;
			mHandler.sendMessageDelayed(msg, 100);
		}
	  }
	}

	/**
	 * 初始化配置项,可在map对象创建前提前初始化
	 * @param context
	 */
	public static void initConfig(Context context) {
		DataOperator.initialize(context);
	}

    public String getMapContentApprovalNumber() {
        return MapCore.getMapContentApprovalNumber();
    }

}
