package com.cmmap.internal.maps;

import com.cmmap.internal.driver.base.GlobalSettings;
import com.cmmap.internal.driver.base.Tools;
import com.cmmap.internal.mapcore.KInt;
import com.cmmap.internal.mapcore.Lg;
import com.cmmap.internal.mapcore.MapCore;


/**
 * 地图缓存管理类
 * 
 * @author liangz
 * 
 */
public class KMapCache {
	private static final KMapCache INSTANCE = new KMapCache();
	private boolean isEnable = true;
	private boolean isUseExternal = false;
	private long maxSize;
	private long totalSize;
	// [internal]内置path
	private static final String CACHE_PATH_INTERNAL = "/tminavi/";
	// [internal]外置path
	private static final String CACHE_PATH_EXTERNAL = "/tminavi/";

	/**
	 * 存储空间状态
	 * 
	 * @author liangz
	 * 
	 */
	public static enum StorageUsageHint {
		/** 正常使用 */
		NONE,
		/** 存储空间不足 */
		SHORTAGE,
		/** 系统故障不可食用 */
		SYSTEM_ERROR
	}

	/**
	 * MapCache对象
	 * 
	 * @return MapCache object
	 */
	public static KMapCache getInstance() {
		return INSTANCE;
	}

	/**
	 * 设置缓存功能是否有效
	 * 
	 * True为有效、false为无效。默认值为true。
	 * 
	 * @param enable
	 */
	public void setEnabled(boolean enable) {
		int i_return = MapCore.SetEnableCache(enable);
		Lg.e("MapCache", "setEnabled:" + i_return);
		if (i_return == 0) {
			this.isEnable = enable;
		}
	}

	/**
	 * 缓存是否有效
	 * 
	 * @return 缓存功能是否有效
	 */
	public boolean isEnabled() {
		boolean enable = MapCore.GetEnableCache();
		Lg.e("MapCache", "isEnabled:" + enable);
		isEnable = enable;
		return isEnable;
	}

//	boolean checkPermission(Context context,String permission)
//	{
//		PackageManager pm =context.getPackageManager();
//		boolean permissioned = (PackageManager.PERMISSION_GRANTED == 
//				pm.checkPermission("android.permission.RECORD_AUDIO", "packageName"));
//		return permissioned;
//	}
	/**
	 * 设置是否使用外部存储外部
	 * 
	 * true :使用外部存储
	 * 
	 * false:使用内部存储
	 * 
	 * 默认为false。
	 * 
	 * @param useExternalStorage
	 * @return 设置是否成功
	 */
	public boolean requestUseExternalStorage(boolean useExternalStorage) {
		KInt k_type = new KInt(0);
		int i_return = 1;
		boolean b_responce = false;
		if (useExternalStorage) {
			// [internal]检查sd卡是否存在以及写入权限
			if (Tools.ExistSDCard() /*&& PermissionManager.checkPermission(GlobalSettings.getInstance().getContext(), "android.permission.WRITE_EXTERNAL_STORAGE")*/) {
				k_type.PutVal(1);
				String s_path = Tools.getExternalStorageDirectory(CACHE_PATH_EXTERNAL);
				if (s_path != null) {
					i_return = MapCore.SetCachePath(s_path, k_type);
					Lg.e("MapCache", "SetCachePathExternal:" + i_return);
				}
				if (i_return == 0) {
					isUseExternal = true;
				}
			}
		} else {
			String s_path = Tools.getInternalStorageDirectory(GlobalSettings.getInstance().getContext(), CACHE_PATH_INTERNAL);
			if (s_path != null) {
				i_return = MapCore.SetCachePath(s_path, k_type);
				Lg.e("MapCache", "SetCachePathInternal:" + i_return);
			}
			if (i_return == 0) {
				isUseExternal = false;
			}
		}
		if (i_return == 0) {
			b_responce = true;
		}
		return b_responce;
	}

	/**
	 * 获取外部存储使用状态
	 * 
	 * @return 外部存储是否可以使用
	 */
	public boolean isUseExternalStorage() {
		KInt k_type = new KInt(0);
		MapCore.GetStorageType(k_type);
		if (k_type.IntVal() == 1) {
			isUseExternal = true;
		} else {
			isUseExternal = false;
		}
		return isUseExternal;
	}

	/**
	 * 设置缓存大小的最大值
	 * 
	 * 默认是10MB。
	 * 
	 * @param maxSize
	 *            Cashsize的最大値(単位:MB)
	 */
	public void setMaxSize(long maxSize) {
		int i_return = MapCore.SetMaxSize((int) (maxSize * 1024 * 1024));
//		Lg.e("MapCache", "setMaxSize:" + i_return);
	}

	/**
	 * 缓存大小的最大值
	 * 
	 * @return cashsize最大值（単位：MB）
	 */
	public long getMaxSize() {
		KInt k_size = new KInt(0);
		int i_return = MapCore.GetMaxSize(k_size);
//		Lg.e("MapCache", "getMaxSize:" + i_return);
		maxSize = (k_size.LongVal() / 1024 / 1024);
		return maxSize;
	}

	/**
	 * 清空缓存
	 * 
	 * 内部存储、外部存储全部清除
	 */
	public void clear() {
		int i_return = MapCore.ClearCache();
//		Log.e("MapCache", "Clear:" + i_return);
	}

	
	/**
	 * 获取当前缓存大小。单位：byte
	 * @return
	 */
	public long getTotalSize() {
		KInt k_size = new KInt(0);
		int i_return = MapCore.GetCurCacheSize(k_size);
//		Lg.e("MapCache", "getTotalSize:" + k_size.LongVal());
		totalSize = (k_size.LongVal());
		return totalSize;
	}


	/**
	 * 获取存储空间状态。
	 * 
	 * @return
	 */
	public StorageUsageHint getStorageUsageHint() {
		KInt k_hint = new KInt(0);
		int i_return = MapCore.GetCacheStorageUsageHint(k_hint);
		// Lg.e("MapCache", "getStorageUsageHint:"+i_return);
		StorageUsageHint hint = StorageUsageHint.NONE;
		switch (k_hint.IntVal()) {
		case 0:
			hint = StorageUsageHint.NONE;
			break;
		case 1:
			hint = StorageUsageHint.SHORTAGE;
			break;
		case 2:
			hint = StorageUsageHint.SYSTEM_ERROR;
			break;
		default:
			break;
		}
		return hint;
	}

}
