package com.cmmap.internal.maps;

import java.security.MessageDigest;
import java.util.Locale;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.pm.ApplicationInfo;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Bundle;
import android.os.IBinder;
import android.os.RemoteException;
import android.util.DisplayMetrics;
import android.util.Log;

import com.cmmap.api.maps.MapException;
import com.cmmap.internal.driver.base.DataOperator;
import com.cmmap.internal.driver.base.GlobalSettings;
import com.cmmap.internal.driver.base.HttpTool;
import com.cmmap.internal.driver.base.Tools;
import com.cmmap.internal.driver.broadcastReceiver.NetWorkMonitorReceiver;
import com.cmmap.internal.driver.broadcastReceiver.NetWorkMonitorReceiver.NET_TYPE;
import com.cmmap.internal.driver.service.DownloadInvoke;
import com.cmmap.internal.driver.service.HttpService;
import com.cmmap.internal.driver.service.IHttpService;
import com.cmmap.internal.mapcore.KHttpData;
import com.cmmap.internal.mapcore.MapCore;
import com.cmmap.internal.mapcore.MapJni;
import com.cmmap.internal.services.rtic.RTICThread;

/**
 * 地图sdk初始化类，不对外使用。
 * 
 * @author liangz
 * 
 */
public final class KMapInitializer {

	private final String Tag = "MapInitializer";
	private static final KMapInitializer INSTANCE = new KMapInitializer();
	private boolean isInitialized = false;
	private long endTime;
	private static final BlockingQueue<KHttpData> post = new LinkedBlockingQueue<KHttpData>();
	private static final BlockingQueue<KHttpData> close = new LinkedBlockingQueue<KHttpData>();
	
	public static float m_dpi;
	
	public static String STR_KEY;
	public static String STR_PACKAGE_NAME;
	public static String STR_SHA1;

    public static String strSDKVersion;
    public static String strURL;
    
    public static RTICThread rticThread;
    
    //默认服务器版本是0

    public static final int CITYLISTDAT = 100;
    
    public static final int DOWNLOAD_BUFFER = 1024*1024;
	
	private static final String TAG="KMapInitalizer";

    public static final String CONFIG_FLODER = "/tminavi/config";
    
    private DownloadInvoke downloadInvoke;
	public float getDPI() {
		return m_dpi;
	}
	
	/**
	 * 底层地图初始化，加载配置，
	 * 
	 * @param context app context
	 * @param timeout 初始化超时时间
	 * @throws ErrorException 初始化异常
	 */

	static void initialize(Context context, int timeout) throws MapException {
		INSTANCE.init(context, timeout);
	}
	
	void initAuth(Activity content) {
		String strAuth = null;
		try {
			// 鉴权key获取
			ApplicationInfo info = content.getPackageManager().getApplicationInfo(content.getPackageName(), PackageManager.GET_META_DATA);
			Bundle md = info.metaData;
			if(md != null) {
				STR_KEY = md.getString("com.cmmap.api.apikey");
			} else {
				STR_KEY = "com.cmmap.api.apikey not found";
			}

            final PackageInfo packageInfo = content.getPackageManager().getPackageInfo(content.getPackageName(), 64);
            
            final byte[] digest = MessageDigest.getInstance("SHA1").digest(packageInfo.signatures[0].toByteArray());
            final StringBuffer sb = new StringBuffer();
            for (int i = 0; i < digest.length; ++i) {
                final String upperCase = Integer.toHexString(0xFF & digest[i]).toUpperCase(Locale.US);
                if (upperCase.length() == 1) {
                    sb.append("0");
                }
                sb.append(upperCase);
                if (i < digest.length - 1) {
                    sb.append(":");
                }
            }
            STR_SHA1 = sb.toString();
            STR_PACKAGE_NAME = packageInfo.packageName;
			strAuth = String.format("{\"key\":\"%s\",\"service\":\"1\",\"user\":{\"type\":\"android\",\"android\":{\"package_name\":\"%s\",\"signature_sha1\":\"%s\"}},\"flow\":0}",
					STR_KEY, STR_PACKAGE_NAME, STR_SHA1);
		} catch (Exception e) {
			e.printStackTrace();
		}
		if (strAuth != null) {
			MapCore.setAuthInfo(strAuth);			
		}
	}
	
	private void setNetworkStatus(Context context) {
		ConnectivityManager connManager=(ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE);
		NetworkInfo mobNetInfo = connManager.getNetworkInfo(ConnectivityManager.TYPE_MOBILE);
		NetworkInfo wifiNetInfo = connManager.getNetworkInfo(ConnectivityManager.TYPE_WIFI);
		//网络不可用  没有WiFi 也没有移动数据
		if ((mobNetInfo == null && wifiNetInfo == null) || (mobNetInfo!= null && wifiNetInfo!= null && !mobNetInfo.isConnected() && !wifiNetInfo.isConnected())) {
			NetWorkMonitorReceiver.NetType = NET_TYPE.no_net;
			MapCore.setNetworkStatus(0);
		}
		//网络可用 
		else {
			// wifi连接
			if (wifiNetInfo!= null && wifiNetInfo.isConnected()) {
				NetWorkMonitorReceiver.NetType = NET_TYPE.wifi;
				Log.i("NetWorkMonitorService", "yes net is wifi ");
				MapCore.setNetworkStatus(1);
			}
			// 移动网络连接
			if (mobNetInfo!= null && mobNetInfo.isConnected()) {
				NetWorkMonitorReceiver.NetType = NET_TYPE.mobile;
				Log.i("NetWorkMonitorService", "yes net is mobile ");
				MapCore.setNetworkStatus(2);
			}
		}
	}

	private void init(Context context, int timeout) throws MapException {
		if (!isInitialized) {
			if(context == null){
				return ;
			}
			
			//传递DPI下去
			DisplayMetrics metric = new DisplayMetrics();
			Activity activity= (Activity)context;
			activity.getWindowManager().getDefaultDisplay().getMetrics(metric);
			initAuth(activity);
			m_dpi = metric.densityDpi;
			MapCore.SetDPI(metric.densityDpi);
			GlobalSettings globalSetting = GlobalSettings.getInstance();
			globalSetting.init(context);
			endTime = System.currentTimeMillis() + timeout * 1000;
			//获取数据缓存路径
			Tools tool = new Tools(context);
			String path = tool.getAppDataPath();
			if (path == null) {
				Log.e(TAG, "Data path unavailable");
				throw new MapException(MapException.ERROR_NOT_AVAILABLE);
			}
			Log.i(Tag, "MapInitializer() path = " + path);
			Log.i(Tag, "MapInitializer() MapCore.initialize before");
			
			Intent intent = new Intent(context,HttpService.class);
			context.getApplicationContext().startService(intent);
			Intent intent_bind = new Intent(context,HttpService.class);
			context.getApplicationContext().bindService(intent_bind, con, Context.BIND_AUTO_CREATE);
			
			downloadInvoke = new DownloadInvoke();
			
			//初始化加载本地数据
			DataOperator.initialize(context);
			
			int ret = MapCore.initialize(DataOperator.getFileDir(), path + "map", path);
			
			if (ret != 0) {
				Log.e(TAG, "MapEngine initialize failure!");
				throw new MapException(MapException.ERROR_UNKNOWN);
			}
			setNetworkStatus(context);
			String str = context.getFilesDir().getAbsolutePath();
			MapJni.init(str + CONFIG_FLODER);
			//init
			strSDKVersion = MapCore.getSDKVersion();
			strURL = MapJni.getOfflineURL(new byte[10]);
			
			StartRTIC();
			HttpTool.isHttpsInitialized();
			isInitialized = true;
			Log.i(TAG, "MapEngine initialize success!");
		}
	}

	private void StartRTIC()
	{		
		rticThread = RTICThread.GetInstance();
		if(!rticThread.isAlive()) {
			rticThread.start();			
		}
	}
	 
	public static KMapInitializer getInstance() {
		return INSTANCE;
	}

	public static boolean isIntialized() {
		return INSTANCE.isInitialized;
	}

	public void post(KHttpData data) {
		if (i_service != null) {
			try {
				i_service.post(data);
			} catch (RemoteException e) {
				Log.e("download", "i_service.post error");
				if(downloadInvoke != null)
				{
					downloadInvoke.invoke(data);
				}
				e.printStackTrace();
			}
		}else{
//			post.add(data);
			if(downloadInvoke != null)
			{
				downloadInvoke.invoke(data);
			}
		}
	}

	public void stop(KHttpData data) {
		if (i_service != null) {
			try {
				i_service.close(data);
			} catch (RemoteException e) {
				Log.e("download", "i_service.close error");
				if(downloadInvoke != null)
				{
					downloadInvoke.colseTask(data);
				}
				e.printStackTrace();
			}
		} else {
//			close.add(data);
			if(downloadInvoke != null)
			{
				downloadInvoke.colseTask(data);
			}
		}
	}
	private IHttpService i_service;
	private ServiceConnection con = new ServiceConnection() {

		@Override
		public void onServiceDisconnected(ComponentName name) {
			i_service = null;
		}

		@Override
		public void onServiceConnected(ComponentName name, IBinder service) {
			i_service = IHttpService.Stub.asInterface(service);
			while(!post.isEmpty()){
				KHttpData data = post.poll();
				post(data);
			}
			while(!close.isEmpty()){
				KHttpData data = close.poll();
				stop(data);
			}
		}
	};

}
