package com.cmmap.internal.maps;

import com.cmmap.api.maps.UiSettings;
import com.cmmap.internal.mapcore.KFloat;
import com.cmmap.internal.mapcore.KInt;
import com.cmmap.internal.mapcore.MapCore;
import com.cmmap.internal.mapcore.MapCore.NE_MapParam_Object;
import com.cmmap.internal.maps.model.KCameraPosition;

/**
 * 地図的option設定
 * 
 * @author liangz
 * 
 */
public class KMapOptions {
	
	private KUiSettings uiSettings;
	
	/**
	 * 设置了一个可视范围的初始化位置
	 * @param camera
	 * @return
	 */
	public KMapOptions camera(KCameraPosition camera) {
		return this;
	}
	
	/**
	 * 设置指南针是否可用
	 * @param enabled
	 * @return
	 */
	public KMapOptions compassEnabled(boolean enabled){
		uiSettings.setCompassEnabled(enabled);
		
		return this;
	}
	
	/**
	 * 获取可视范围的初始化位置
	 * @return
	 */
	public KCameraPosition getCamera() {
		return null;
	}
	
	
	/**
	 * 返回指南针功能是否可用
	 * @return
	 */
	public boolean getCompassEnabled() {
		return uiSettings.isCompassEnabled();
	}
	
	/**
	 * @return Logo的位置
	 */
	public int getLogoPosition(){
		return uiSettings.getLogoPosition();
	}
	
	/**
	 * @return 返回选项类里的地图类型
	 */
	public int getMapType() {
		//MapCore.get
		KFloat ki_theme = new KFloat(0);
		MapCore.GetConfiguration(NE_MapParam_Object.NE_Map_Theme.value(), ki_theme);
		return (int)(ki_theme.FloatVal());
	}
	/**
	 * @return 返回旋转手势是否被允许
	 */
	public boolean getRotateGesturesEnabled(){
		return uiSettings.isRotateGesturesEnabled();
	}
	
	/**
	 * @return 返回比例尺功能是否可用
	 */
	public boolean getScaleControlsEnabled() {
		return uiSettings.isScaleControlsEnabled();
	}
	
	/**
	 * 返回拖动手势是否被允许，如果返回false 则表示未设置
	 * @return
	 */
	public boolean getScrollGesturesEnabled() {
		return uiSettings.isScrollGesturesEnabled();
	}
	
	/**
	 * 返回地图倾斜手势（显示3D 楼块）是否被允许，如果返回false 则表示未设置
	 * @return
	 */
	public boolean getTiltGesturesEnabled() {
		return uiSettings.isTiltGesturesEnabled();
	}
	
	/**
	 * 返回地图是否允许缩放，如果返回false 则表示未设置
	 * @return
	 */
	public boolean getZoomControlsEnabled(){
		return uiSettings.isZoomControlsEnabled();
	}
	
	/**
	 * 返回缩放手势是否被支持，如果返回false 则表示未设置
	 * @return
	 */
	public boolean getZoomGesturesEnabled() {
		return uiSettings.isZoomGesturesEnabled();
	}
	
	/**
	 * @return 返回Z 轴排序是否被允许，如果返回false 则表示未设置
	 */
	public boolean getZOrderOnTop()  {
		return false;
	}
	
	/**
	 * Logo的位置
	 * @param position
	 * @return
	 */
	public KMapOptions logoPosition(int position){
		uiSettings.setLogoPosition(position);
		return this;
	}
	
	/**
	 * 设置一个地图显示类型以改变初始的类型
	 * @param mapType
	 * @return
	 */
	public KMapOptions mapType(int mapType) {
		MapCore.SetConfiguration(NE_MapParam_Object.NE_Map_Theme, (float)mapType);
		return this;
	}
	
	/**
	 * 设置地图是否可以通过手势进行旋转
	 * @param enabled
	 * @return
	 */
	public KMapOptions rotateGesturesEnabled(boolean enabled) {
		uiSettings.setRotateGesturesEnabled(enabled);
		return this;
	}
	
	/**
	 * 设置比例尺功能是否可用
	 * @param enabled
	 * @return
	 */
	public KMapOptions scaleControlsEnabled(boolean enabled){
		uiSettings.setScaleControlsEnabled(enabled);
		return this;
	}
	
	/**
	 * 设置地图是否可以通过手势滑动
	 * @param enabled
	 * @return
	 */
	public KMapOptions scrollGesturesEnabled(boolean enabled){
		uiSettings.setScrollGesturesEnabled(enabled);
		return this;
	}
	
	/**
	 * 设置地图是否可以通过手势倾斜（3D 楼块效果），默认为true
	 * @param enabled
	 * @return
	 */
	public KMapOptions tiltGesturesEnabled(boolean enabled){
		//this.tiltEnable = enabled;
		uiSettings.setTiltGesturesEnabled(enabled);
		return this;
	}
	
	/**
	 * 设置地图是否允许缩放
	 * @param enabled
	 * @return
	 */
	public KMapOptions zoomControlsEnabled(boolean enabled) {
		//this.pinchZoomEnable = enabled;
		uiSettings.setZoomControlsEnabled(enabled);
		return this;
	}
	
	/**
	 * 设置地图是否可以通过手势进行缩放
	 * @param enabled
	 * @return
	 */
	public KMapOptions zoomGesturesEnabled(boolean enabled) {
		uiSettings.setZoomGesturesEnabled(enabled);
		return this;
	}
	
	/**
	 * @param zOrderOnTop
	 * @return
	 */
	public KMapOptions zOrderOnTop(boolean zOrderOnTop) {
		
		return this;
	}

	
	public KMapOptions() {

	}

	public void setUiSettings(KUiSettings uiSettings) {
		this.uiSettings = uiSettings;
	}


	public boolean getAllGesturesEnabled() {
		return uiSettings.getAllGesturesEnabled();
	}

}
