package com.cmmap.internal.maps;

import com.cmmap.internal.mapcore.KFloat;
import com.cmmap.internal.mapcore.MapCore;
import com.cmmap.internal.maps.model.KLatLng;

/**
 * 定义了一个实现Map地图其他功能的类。
 */
public class KMapUtils {

	public KMapUtils() {
		
	}
	
	/**
	 * 计算地图上矩形区域的面积，单位平方米。
	 * @param leftTopLatlng 矩形区域左上角点坐标。
	 * @param rightBottomLatlng 矩形区域右下角点坐标。
	 * @return 返回地图上矩形区域的面积，单位平方米。
	 */
	public static float calculateArea(KLatLng leftTopLatlng,
            KLatLng rightBottomLatlng) {
		float height = calculateLineDistance(leftTopLatlng, new KLatLng(rightBottomLatlng.latitude, leftTopLatlng.longitude));
		float width = calculateLineDistance(leftTopLatlng, new KLatLng(leftTopLatlng.latitude, rightBottomLatlng.longitude));
		return height * width;
	}
	
	/**
	 * 根据用户的起点和终点经纬度计算两点间距离，此距离为相对较短的距离，单位米。
	 * @param startLatlng 起点的坐标。
	 * @param endLatlng 终点的坐标。
	 * @return 返回两点间相对较短的距离，单位米。
	 */
	public static float calculateLineDistance(KLatLng startLatlng,
            KLatLng endLatlng) {
		KFloat dist = new KFloat(0);
		MapCore.calcDistance(startLatlng, endLatlng, dist);
		return dist.FloatVal();
	}
}