package com.cmmap.internal.maps;

import android.content.Context;
import android.content.IntentFilter;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.PointF;
import android.os.Handler;
import android.os.Handler.Callback;
import android.os.Message;
import android.util.AttributeSet;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.WindowManager;
import android.widget.FrameLayout;

import com.cmmap.api.maps.Map;
import com.cmmap.api.maps.MapException;
import com.cmmap.api.maps.model.BitmapDescriptor;
import com.cmmap.api.maps.model.BitmapDescriptorFactory;
import com.cmmap.internal.driver.broadcastReceiver.NetWorkMonitorReceiver;
import com.cmmap.internal.driver.view.MapGLSurfaceView;
import com.cmmap.internal.driver.view.ViewContext;
import com.cmmap.internal.maps.KMap.InfoView;
import com.cmmap.internal.maps.model.KMarker;
import com.cmmap.internal.maps.model.KShapeOverlay;


/**
 * 地图视图类。
 * <p>地图视图负责从服务端获取并显示地图数据。
 * <br>当屏幕焦点在这个视图上时，它将会捕捉屏幕触控手势事件。
 * <br>使用地图视图类必须遵循Activity的生命周期进行调用。
 * <br>当地图视图初始化完成后，用户可以通过{@link #getMap}方法获得{@link Map}对象，如果初始化失败，将返回null。
 * </p>
 */
public final class KMapView extends FrameLayout {
	private static final String TAG = "MapView";
	private MapGLSurfaceView glSurfaceView;
	private KMap map;
	private KInfoOverlayView infoOverlayView;
	public Context context;
	private ViewContext viewContext;
	
	public KMapWidgetView widgetView;
	
	public static final int WHAT_SERVICE_NUMBER = 1001;
	NetWorkMonitorReceiver netReceiver = null;
	
	public Handler m_handler = new Handler(new Callback() {
		
		@Override
		public boolean handleMessage(Message msg) {
			switch (msg.what) {
			case WHAT_SERVICE_NUMBER:
				String mapVersion = (String) msg.obj;
				map.onMapPropUpdate(mapVersion);
				break;
			}
			return false;
		}
	});

	/**
	 * constructor
	 * 
	 * @param context
	 *            app context
	 */
	public KMapView(Context context) {
		super(context);
		init(context);
	}

	/**
	 * 构造函数。
	 * @param context 应用上下文
	 * @param attrs 视图属性
	 */
	public KMapView(Context context, AttributeSet attrs) {
		super(context, attrs);
		init(context);
	}


	/**
	 * 构造函数。
	 * @param context 应用上下文
	 * @param attrs 视图属性
	 * @param defStyleAttr 默认视图风格属性
	 */
	public KMapView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		init(context);
	}

	/**
	 * 获取map对象
	 * 
	 * @return Map  地图对象。如果这个地图视图没有初始化成功，则返回NULL
	 */
	public KMap getMap() {
		if (KMapInitializer.isIntialized()) {
			return map;
			
		}
		return null;
	}
	
	public void setMapRotateCenter(PointF pointF) {
		glSurfaceView.setMapRotateCenter(pointF);
	}
	
	@Override
	protected void onAttachedToWindow() {
		super.onAttachedToWindow();
		if (glSurfaceView.getParent() != null) {
			removeAllViews();
		}
		addView(glSurfaceView);

		addView(infoOverlayView);

		glSurfaceView.setOverlayInfoView(infoOverlayView);
		
		addView(widgetView);
	}

	static private int m_device_w = 800;
	static private int m_device_h = 480;
	static private float m_dpi_x = 320;
	static private float m_dpi_y = 320;
	
	static public int getDeviceWidth() {
		return m_device_w;
	}
	
	static public int getDeviceHeight() {
		return m_device_h;
	}
	
	static public float getDPIx() {
		return m_dpi_x;
	}
	
	static public float getDPIy() {
		return m_dpi_y;
	}
	
	public long getFreeMemorySize() {       
		 String dir = "/proc/meminfo";
		 try {
			 java.io.FileReader fr = new java.io.FileReader(dir);
			 java.io.BufferedReader br = new java.io.BufferedReader(fr, 2048);
			 br.readLine();
			 String memoryLine = br.readLine();
		     String subMemoryLine = memoryLine.substring(memoryLine.indexOf("MemFree:"));   
		     br.close(); 
		     return Integer.parseInt(subMemoryLine.replaceAll("\\D+", ""));// * 1024l;   
		 } catch (java.io.IOException e) {
			 e.printStackTrace();
		 } catch (Exception e) {
			 e.printStackTrace();
		 }  
		 return 0;   
	}
		
	private void init(Context context) {
		Log.i(TAG, "isInit");
		this.context = context;
		KMapOptions options = new KMapOptions();
		viewContext = new ViewContext(options);
		map = new KMap(viewContext, this);
		glSurfaceView = new MapGLSurfaceView(context, viewContext, map);
		infoOverlayView = new KInfoOverlayView(context, map, viewContext);
		// 初始设置同地图底色的背景颜色，避免activity切换到地图第一次绘制出来之间显示黑色
		infoOverlayView.setBackgroundColor(Color.argb(255, 246, 246, 246));
		
		WindowManager wm = (WindowManager) context.getSystemService(Context.WINDOW_SERVICE);		
		DisplayMetrics metric = new DisplayMetrics();
		wm.getDefaultDisplay().getMetrics(metric);
				
		int width = metric.widthPixels;
		int height = metric.heightPixels;
		m_device_w =  width;
		m_device_h =  height;
		m_dpi_x = metric.xdpi;
		m_dpi_y = metric.ydpi;
		
		try {
			if (!KMapInitializer.isIntialized()) {
				KMapInitializer.initialize(context, 10);
			}
			
		} catch (MapException e) {
			e.printStackTrace();
		}
		widgetView = new KMapWidgetView(context, map);
		options.setUiSettings(map.getUiSettings());
		map.setMapOperatorView(glSurfaceView);
		//注册
		IntentFilter filter = new IntentFilter();
		filter.addAction("android.net.conn.CONNECTIVITY_CHANGE");
		netReceiver = new NetWorkMonitorReceiver();
		netReceiver.setM_handler(m_handler);
		context.getApplicationContext().registerReceiver(netReceiver, filter);
	}
	
	/**
	 * [internal]添加图形图层
	 * @param overlay
	 */
	void addShapeOverlay(KShapeOverlay overlay) {
		glSurfaceView.addShapOverlay(overlay);
	}

	/**
	 * [internal]移除图形图层
	 * @param overlay
	 */
	void removeShapeOverlay(KShapeOverlay overlay){
		glSurfaceView.removeShapeOverlay(overlay);
	}
	/**
	 * [internal]更新图形图层
	 * @param overlay
	 */
	void updateShapeOverlay(KShapeOverlay overlay, int updateMask){
		glSurfaceView.updateShapeOverlay(overlay, updateMask);
	}
	
	/**
	 * [internal]更新图形图层的图片资源
	 * @param overlay
	 */
	void updateShapeOverlayBitmap(KShapeOverlay overlay) {
		glSurfaceView.updateShapeOverlayBitmap(overlay);
	}
	
	/**
	 * [internal]移除图形图层的图片资源
	 * @param overlay
	 */
	void removeShapeOverlayBitmap(KShapeOverlay overlay) {
		glSurfaceView.removeShapeOverlayBitmap(overlay);
	}
	
	/**
	 * [internal]添加信息图层
	 */
	void addInfoOverlay() {
		if (infoOverlayView != null && infoOverlayView.getParent() == null) {
			addView(infoOverlayView);
		}
		infoOverlayView.onRefresh();
	}
	
	/**
	 * [internal]刷新info图层
	 */
	public void reFreshInfoOverlay() {
		if (infoOverlayView != null && infoOverlayView.getParent() != null) {
			infoOverlayView.onRefresh();
		}
	}

	/**
	 * [internal]非主线程刷新infoOverlay
	 */
	public void postRefreshInfoOverlay(){
		if (infoOverlayView != null && infoOverlayView.getParent() != null) {
			infoOverlayView.onPostRefresh();
		}
	}
	
	public void refreshInfowindow(InfoView infoView){
		if(infoOverlayView != null && infoOverlayView.getParent() != null){
			infoOverlayView.onInfowindowRefresh(infoView);
		}
	}
	
	PointF getSize() {
		return glSurfaceView.getSize();
	}
	
	int getViewWidth(){
		return glSurfaceView.getViewWidth();
	}
	
	int getViewHeight(){
		return glSurfaceView.getViewHeight();
	}
	
	void updateWidget(){
		widgetView.updateWidget();
	}

	void reSortInfoView() {
		infoOverlayView.reSort();
	}
	
	/**
	 * 用户重写这个方法时必须调用父类的这个方法
	 * @param paramBundle 一个Bundle对象
	 */
	public final void onCreate(android.os.Bundle paramBundle){
		map.onCreate();
	}
	
	/**
	 * 用户必须重载{@link android.app.Activity#onResume}函数，调用地图视图的该函数。
	 */
	public final void onResume(){
		map.onResume();
		map.AddOverlayInMap();
	}
	
	/**
	 * 用户重写这个方法时必须调用父类的这个方法
	 */
	public final void onPause(){
		map.onPause();
		map.DelOverlayInMap();
		glSurfaceView.clearOverlayReq();
	}
	
	/**
	 * 用户重写这个方法时必须调用父类的这个方法
	 */
	public final void onDestroy(){

		if (context != null && netReceiver != null) {
			netReceiver.Release();
			context.getApplicationContext().unregisterReceiver(netReceiver);
		}
		map.onDestroy();
		widgetView.onDestroy();
		glSurfaceView.clearOverlayReq();
		glSurfaceView.onDestroy();
	}
	
	/**
	 * 用户重写这个方法时必须调用父类的这个方法
	 * @param paramBundle Bundle对象
	 */
	public final void onSaveInstanceState(android.os.Bundle paramBundle){
		
	}

	public void updateCompass() {
		widgetView.updateCompass();
	}

	public void showBuildings(boolean enabled) {
		glSurfaceView.showBuildings(enabled);
	}

	public void showMapText(boolean enabled) {
		glSurfaceView.showMapText(enabled);
	}

	public void clearTileCache(int _id) {
		glSurfaceView.clearTileCache(_id);
	}

	public void setMapTextZIndex(float index) {
		glSurfaceView.setMapTextZIndex(index);
	}

	public void showSkyBackground(boolean enabled) {
		glSurfaceView.showSkyBackground(enabled);
	}

	public void refreshInfowindowSubtitude(InfoView infoView,
			KMarker infowindowSubstitute) {
		glSurfaceView.setInfowindow(infoView, infowindowSubstitute);
		if (infoOverlayView != null && infoOverlayView.getParent() != null) {
			infoOverlayView.setInfowindowSubtitude(infowindowSubstitute);
		}
	}

	public void onMapRefreshFirstTime() {
		// 第一次地图刷新后，设置透明
		infoOverlayView.setBackgroundColor(Color.argb(0, 255, 255, 255));
	}

    public Bitmap processMapScreenShot(Bitmap mapBitmap) {
        if (mapBitmap == null) {
            return null;
        }
        
        Bitmap finalBitmap = Bitmap.createBitmap(mapBitmap.getWidth(), mapBitmap.getHeight(), Bitmap.Config.ARGB_8888);
        Canvas cv = new Canvas(finalBitmap);
        cv.drawBitmap(mapBitmap, 0, 0, null);
        // 在地图截图上叠加logo
        widgetView.processMapScreenShot(cv);
        // 在地图截图上叠加infowindow
        infoOverlayView.draw(cv);
        cv.save(Canvas.ALL_SAVE_FLAG);
        cv.restore();
        return finalBitmap;
    }
}
