package com.cmmap.internal.maps;

import java.io.IOException;
import java.io.InputStream;
import java.util.Vector;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.location.Location;
import android.os.Handler;
import android.os.Handler.Callback;
import android.os.Message;
import android.util.AttributeSet;
import android.util.Log;
import android.view.Gravity;
import android.view.MotionEvent;
import android.view.View;
import android.widget.ImageView;
import android.widget.ImageView.ScaleType;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.cmmap.api.maps.MapOptions;
import com.cmmap.api.maps.model.BitmapDescriptor;
import com.cmmap.api.maps.model.BitmapDescriptorFactory;
import com.cmmap.internal.driver.base.GlobalSettings;
import com.cmmap.internal.driver.base.WindowsTool;
import com.cmmap.internal.mapcore.KFloat;
import com.cmmap.internal.mapcore.MapCore;
import com.cmmap.internal.mapcore.NE_ScaleProp;
import com.cmmap.internal.maps.model.KCameraPosition;
import com.cmmap.internal.maps.model.KCircle;
import com.cmmap.internal.maps.model.KCircleOptions;
import com.cmmap.internal.maps.model.KLatLng;
import com.cmmap.internal.maps.model.KMarker;
import com.cmmap.internal.maps.model.KMarkerOptions;
import com.cmmap.internal.maps.model.MyLocationStyle;

// YuJun 定位说明:
// (1)初始化,转到LOCATION_TYPE_LOCATE模式.
// (2)定位不使能时,转到LOCATION_TYPE_LOCATE模式.
// (3)移动1次地图,转到LOCATION_TYPE_LOCATE模式.
// (4)在LOCATION_TYPE_LOCATE模式,点击一次定位按钮,就把地图中心移动到LOC位置,转到LOCATION_TYPE_FOLLOW模式,2D模式;
// (5)在LOCATION_TYPE_MAP_FOLLOW&ROTATE模式,收到LOC改变消息,就把地图中心移动到LOC位置;
// (6)在LOCATION_TYPE_MAP_FOLLOW模式,点击一次定位按钮,就设置LOCATION_TYPE_ROTATE模式,3D模式;
// (7)在LOCATION_TYPE_MAP_ROTATE模式,收到ROTATE改变消息,就旋转地图;
// (8)在LOCATION_TYPE_MAP_ROTATE模式,点击一次定位按钮,就设置LOCATION_TYPE_FOLLOW模式,2D模式;

/**
 * 地图控件层，外部不可见，不对外使用。
 * 
 */
public class KMapWidgetView extends RelativeLayout {
	public static final int MAP_WIDGET_LOGO_TEXT = 1;
	public static final int MAP_WIDGET_LOGO_ICON = 2;
	public static final int MAP_WIDGET_ZOOM_IN = 4;
	public static final int MAP_WIDGET_ZOOM_OUT = 5;
	public static final int MAP_WIDGET_COMPRESS = 6;
	public static final int MAP_WIDGET_SCALE_TEXT = 8;
	public static final int MAP_WIDGET_SCALE_ICON = 9;
	public static final int MAP_WIDGET_VERSION_TEXT = 10;
	public static final int MAP_WIDGET_DRAWMAP_DEBUG = 11;
	public static final int MAP_WIDGET_CONVERT_PARCEL_SHOW = 12;
	public static final int MAP_WIDGET_DRAW_ROADS = 13;
	public static final int MAP_WIDGET_DRAW_BACKAREA = 14;
	public static final int MAP_WIDGET_DRAW_FLOOR = 15;
	public static final int MAP_WIDGET_DRAW_RTIC = 16;
	public static final int MAP_WIDGET_DRAW_BKGLINE = 17;
	
	public static KMapWidgetView mapWidgetView;

	public static final int MAP_3D_TITL_DEFA = 90;	// 俯仰角
	public static final int MAP_2D_TITL_DEFA = 0;	// 俯仰角
	private static final int MAP_ZOOM_DEFA = 16;	// 100米地图比例尺

	/**
	 * 默认的控件相距父边框的距离
	 */
	private static int DEFAULT_MARGIN_X = 0;

	/**
	 * 标志图标和文字
	 */
	private ImageView logoImageView;

	/**
	 * 视角按钮
	 */
//	private ImageView upTypeButton;

	/**
	 * 放大按钮
	 */
	private ImageView zoomInButton;

	/**
	 * 缩小按钮
	 */
	private ImageView zoomOutButton;

	/**
	 * 指北针
	 */
	private ImageView compressButton;

	/**
	 * 定位按钮
	 */
//	private ImageView locationButton;
	
//	private Button m_cbConvertParcelShow;
//	private Button m_cbDrawRoads;
//	private Button m_cbDrawBackArea;
//	private Button m_cbDrawFloor;
//	private Button m_cbDrawRtic;
//	private Button m_cbDrawBkgLine;

	/**
	 * 比例尺文字和图标
	 */
	private TextView m_scaleText;
	private ImageView m_scaleIcon;
	
//	private TextView m_versionText;

//	private BitmapDescriptor m_icon_loc_once;
//	private BitmapDescriptor m_icon_loc_rotate;
//	private BitmapDescriptor m_icon_loc_gps;

	private RelativeLayout.LayoutParams logoImageLayoutParams;
	private RelativeLayout.LayoutParams upTypeLayoutParams;
	private RelativeLayout.LayoutParams locationLayoutParams;
	private RelativeLayout.LayoutParams zoomInLayoutParams;
	private RelativeLayout.LayoutParams zoomOutLayoutParams;
	private RelativeLayout.LayoutParams compressLayoutParams;
	private RelativeLayout.LayoutParams scaleLayoutParams;
	private RelativeLayout.LayoutParams logoTextLayoutParams;

	public MyLocationStyle m_location_style = null;

	public static final int USER_MOVE_MAP_MSG = 2000;
	public static final int SET_LOCATION_TYPE_MSG = 2002;
	public static final int SET_LOCATION_STYLE_MSG = 2003;
	public static final int SET_LOCATION_ENABLE_MSG = 2004;
	public static final int SET_LOCATION_CHANGED_MSG = 2005;

	/**
	 * 车标Marker
	 */
	private KMarker m_location_marker;
	private KCircle m_location_circle;
	private boolean m_my_location_is_displayed = false;

	int m_location_type;
	boolean m_bNeedLocaleOnce = true;

	private Vector<NE_ScaleProp> sceleProp;

	private KUiSettings settings;

	private Context context;

	private int logoPos;

	private int zoomPos;

	private KMap map;
	
	private static float DPIRATIO; 

	private KMapWidgetView(Context context, AttributeSet attrs, int defStyleAttr) {
		super(context, attrs, defStyleAttr);
		initView(context);
	}

	public KMapWidgetView(Context context, AttributeSet attrs) {
		super(context, attrs);
		initView(context);
	}

	public KMapWidgetView(Context context) {
		super(context);
		initView(context);
	}

	public KMapWidgetView(Context context, KMap map) {
		super(context);
		this.map = map;
		this.mapWidgetView = this;
		initView(context);
	}

	public Handler mHandler = new Handler(new Callback() {
		@Override
		public boolean handleMessage(Message msg) {

			if (msg.what == SET_LOCATION_ENABLE_MSG) {
				if (msg.arg1 == 1) {
					setMyLocationEnabled(true);
				} else {
					setMyLocationEnabled(false);
				}
			} else {
				if (msg.what == USER_MOVE_MAP_MSG) {
					setMyLocationType(KMap.LOCATION_TYPE_LOCATE);
				} else if (msg.what == SET_LOCATION_TYPE_MSG) {
					setMyLocationType(msg.arg1);
				} else if (msg.what == SET_LOCATION_STYLE_MSG) {
					if (m_my_location_is_displayed) {
						setMyLocationStyle();
					}
				} else if (msg.what == SET_LOCATION_CHANGED_MSG) {
					updateLocationMakerPosition();
					moveMapCenterToLocation();
				}
			}
			return false;
		}
	});

	private void updateLocationMakerPosition() {
		Location loc = map.getMyLocation();
		KLatLng pos = new KLatLng(loc.getLatitude(), loc.getLongitude());
		if (m_location_marker != null) {
			m_location_marker.setPosition(pos);
			if (m_location_type == KMap.LOCATION_TYPE_MAP_ROTATE)
			{
				m_location_marker.setFlat(false);
				m_location_marker.setRotateAngle(0);
			}else
			{
				m_location_marker.setFlat(true);
				m_location_marker.setRotateAngle(loc.getBearing());
			}
		}
		if (m_location_circle != null) {
			m_location_circle.setCenter(pos);
		}
	}

	private void setMyLocationStyle() {
		if (m_location_style == null) {
			return;
		}

		if (m_location_marker != null) {
			m_location_marker.remove();
			m_location_marker = null;
		}

		if (m_location_circle != null) {
			m_location_circle.remove();
			m_location_circle = null;
		}

		// add circle
		m_location_circle = map.addCircle(new KCircleOptions()
				.fillColor(m_location_style.getRadiusFillColor())
				// ARGB
				.strokeColor(m_location_style.getStrokeColor())
				.strokeWidth(m_location_style.getStrokeWidth())
				.center(new KLatLng(0, 0)).zIndex(0).visible(true).zIndex(0)
				.radius(200));

		// add marker
		KMarkerOptions Markeroptions = new KMarkerOptions();
		Markeroptions.anchor(m_location_style.getAnchorU(),
				m_location_style.getAnchorV());
		Markeroptions.draggable(false);
		java.util.ArrayList<BitmapDescriptor> icons = new java.util.ArrayList<BitmapDescriptor>();
		icons.add(m_location_style.getMyLocationIcon());
		Markeroptions.icons(icons);
		Markeroptions.period(0);
		Markeroptions.perspective(true);
		Markeroptions.setFlat(true);
		Markeroptions.setGps(true);
		Markeroptions.visible(true);
		Markeroptions.zIndex(0.0f);
		Markeroptions.position(new KLatLng(0, 0));
		Markeroptions.anchor(0.5f, 0.5f);

		m_location_marker = map.addMarker(Markeroptions);

	}

	private void setMyLocationEnabled(boolean enable) {
		m_my_location_is_displayed = enable;
		if (m_my_location_is_displayed) {
			if (m_location_marker != null) {
				m_location_marker.setVisible(true);
			}
		} else {
			if (m_location_marker != null) {
				m_location_marker.setVisible(false);
			}
		}
		updateWidget();
	}

	public boolean isMyLocationEnabled() {
		return m_my_location_is_displayed;
	}

	private void setMyLocationType(int type) {
		m_location_type = type;
		updateWidget();
	}
	
	Bitmap LoadBitmapFromAsset(java.lang.String assetName){
		InputStream is_bitmap;
		Bitmap _bitmap = null;
		try {
			is_bitmap = GlobalSettings.getInstance().getContext().getAssets().open(assetName);
			if(is_bitmap != null){
				_bitmap = BitmapFactory.decodeStream(is_bitmap);
			}
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
		
		return _bitmap;
	}

	Bitmap ResizeBitmap(Bitmap src, int dest_w, int dest_h)
	{  
        //获取这个图片的宽和高   
        int width = src.getWidth();   
        int height = src.getHeight();   
  
        //定义预转换成的图片的宽度和高度   
        int newWidth = dest_w;   
        int newHeight = dest_h;   
  
        //计算缩放率，新尺寸除原始尺寸   
        float scaleWidth = ((float) newWidth) / width;   
        float scaleHeight = ((float) newHeight) / height;   
 
        // 创建操作图片用的matrix对象   
       Matrix matrix = new Matrix();
  
        // 缩放图片动作   
        matrix.postScale(scaleWidth, scaleHeight);   
  	  
        // 创建新的图片   
        Bitmap resizedBitmap = Bitmap.createBitmap(src, 0, 0,   
                          width, height, matrix, true);   
 
        return resizedBitmap;
	}
	
	BitmapDescriptor logoDescriptor = BitmapDescriptorFactory.fromAsset("cmmap/mapres/logo.png");
	Bitmap logoBitmap = null;
	BitmapDescriptor scaleDescriptor = BitmapDescriptorFactory.fromAsset("cmmap/mapres/scale.png");
	BitmapDescriptor zoomoutDescriptor = BitmapDescriptorFactory.fromAsset("cmmap/mapres/zoom_out_icon.png");
	Bitmap zoomoutBitmap = null;
	BitmapDescriptor zoominDescriptor = BitmapDescriptorFactory.fromAsset("cmmap/mapres/zoom_in_icon.png");
	Bitmap zoominBitmap = null;
	BitmapDescriptor comDescriptor = BitmapDescriptorFactory.fromAsset("cmmap/mapres/map_compass.png");
	Bitmap comBitmap = null;
	BitmapDescriptor zoominPressedDescriptor = BitmapDescriptorFactory.fromAsset("cmmap/mapres/zoom_in_pressed_icon.png");
	Bitmap zoominPressedbitmap = null;
	BitmapDescriptor zoomoutPressedDescriptor = BitmapDescriptorFactory.fromAsset("cmmap/mapres/zoom_out_pressed_icon.png");
	Bitmap zoomoutPressedBitmap = null;
	BitmapDescriptor zoominDisabledDescriptor = BitmapDescriptorFactory.fromAsset("cmmap/mapres/zoom_in_disabled_icon.png");
	Bitmap zoominDisabledBitmap = null;
	BitmapDescriptor zoomoutDisabledDescriptor = BitmapDescriptorFactory.fromAsset("cmmap/mapres/zoom_out_disabled_icon.png");
	Bitmap zoomoutDisabledBitmap = null;
	 
	/**
	 * 初始化各view的布局位置到默认位置.
	 */
	private void initView(Context context) {
		this.context = context;
		if (context == null) {
			return;
		}
		
		DEFAULT_MARGIN_X = WindowsTool.dip2px(context, 2.0f);
		DPIRATIO = KMapInitializer.getInstance().getDPI() / 320;
		
		Bitmap icon = LoadBitmapFromAsset("cmmap/mapres/1.png");
		float size_new = icon.getWidth() * KMapInitializer.getInstance().getDPI() / 150 / 2;
		int i_size_new = (int)size_new;
		
//		m_icon_loc_once = BitmapDescriptorFactory.fromBitmap(
//				ResizeBitmap(LoadBitmapFromAsset("cmmap/mapres/1.png"), i_size_new, i_size_new));
//		m_icon_loc_gps =BitmapDescriptorFactory.fromBitmap(
//				ResizeBitmap(LoadBitmapFromAsset("cmmap/mapres/2.png"), i_size_new, i_size_new));
//		m_icon_loc_rotate = BitmapDescriptorFactory.fromBitmap(
//				ResizeBitmap(LoadBitmapFromAsset("cmmap/mapres/3.png"), i_size_new, i_size_new));
		logoBitmap = ResizeBitmap(logoDescriptor.getBitmap(),(int)(DPIRATIO*logoDescriptor.getWidth()),(int)(DPIRATIO*logoDescriptor.getHeight()));
		zoomoutBitmap = ResizeBitmap(zoomoutDescriptor.getBitmap(),(int) (DPIRATIO * zoomoutDescriptor.getWidth()),(int) (DPIRATIO * zoomoutDescriptor.getHeight()));
		zoominBitmap = ResizeBitmap(zoominDescriptor.getBitmap(),(int) (DPIRATIO * zoominDescriptor.getWidth()),(int) (DPIRATIO * zoominDescriptor.getHeight()));
		comBitmap = ResizeBitmap(comDescriptor.getBitmap(), (int) (DPIRATIO * comDescriptor.getWidth()), (int) (DPIRATIO  * comDescriptor.getHeight()));
		zoominPressedbitmap = ResizeBitmap(zoominPressedDescriptor.getBitmap(),(int) (DPIRATIO * zoominPressedDescriptor.getWidth()),(int) (DPIRATIO * zoominPressedDescriptor.getHeight()));
		zoomoutPressedBitmap = ResizeBitmap(zoomoutPressedDescriptor.getBitmap(),(int) (DPIRATIO * zoomoutPressedDescriptor.getWidth()),(int) (DPIRATIO * zoomoutPressedDescriptor.getHeight()));
		zoominDisabledBitmap = ResizeBitmap(zoominDisabledDescriptor.getBitmap(),(int) (DPIRATIO * zoominDisabledDescriptor.getWidth()),(int) (DPIRATIO * zoominDisabledDescriptor.getHeight()));
		zoomoutDisabledBitmap = ResizeBitmap(zoomoutDisabledDescriptor.getBitmap(),(int) (DPIRATIO * zoomoutDisabledDescriptor.getWidth()),(int) (DPIRATIO * zoomoutDisabledDescriptor.getHeight()));

		settings = new KUiSettings();
		settings.setOperationView(KMapWidgetView.this);
		logoPos = MapOptions.LOGO_POSITION_BOTTOM_LEFT;
		zoomPos = MapOptions.ZOOM_POSITION_RIGHT_BOTTOM;
		m_location_type = KMap.LOCATION_TYPE_LOCATE;
		m_bNeedLocaleOnce = true;

		LayoutParams params = new RelativeLayout.LayoutParams(
				LayoutParams.MATCH_PARENT, LayoutParams.MATCH_PARENT);

		setLayoutParams(params);
		initdefView();
		initListener();

		setMyLocationType(KMap.LOCATION_TYPE_LOCATE);
		
		//*
		// yujun 2016-5-3  打开此处就可以调试地图绘制性能
//		m_cbConvertParcelShow.setVisibility(View.INVISIBLE);
//		m_cbDrawRoads.setVisibility(View.INVISIBLE);
//		m_cbDrawBackArea.setVisibility(View.INVISIBLE);
//		m_cbDrawFloor.setVisibility(View.INVISIBLE);
//		m_cbDrawRtic.setVisibility(View.INVISIBLE);
//		m_cbDrawBkgLine.setVisibility(View.INVISIBLE);
		// */
	}

	private void initdefView() {

		try {

			// version text
//			{
//				m_versionText = new TextView(context);
//				RelativeLayout.LayoutParams versionLayoutParams = new RelativeLayout.LayoutParams(
//						LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
//				versionLayoutParams.addRule(ALIGN_PARENT_TOP);
//				versionLayoutParams.addRule(CENTER_HORIZONTAL);
//				versionLayoutParams.setMargins(0, 0, 0, 0);
//				m_versionText.setLayoutParams(versionLayoutParams);
//				m_versionText.setText("MapVersion:2016-04-08");
//				m_versionText.setTextSize(7);
//				m_versionText.setGravity(Gravity.CENTER);
//				m_versionText.setTextColor(Color.GRAY);
//				m_versionText.setId(MAP_WIDGET_VERSION_TEXT);
//				addView(m_versionText);
//			}
						
			// logo icon
			{
				logoImageView = new ImageView(context);

				logoImageLayoutParams = new RelativeLayout.LayoutParams(
						LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);

				logoImageLayoutParams.addRule(RelativeLayout.ALIGN_PARENT_BOTTOM);				
				logoImageLayoutParams.setMargins(DEFAULT_MARGIN_X, 0, 0, DEFAULT_MARGIN_X);
				logoImageView.setLayoutParams(logoImageLayoutParams);
				logoImageView.setImageBitmap(logoBitmap);
				logoImageView.setId(MAP_WIDGET_LOGO_ICON);
				addView(logoImageView);
			}

			// Location
//			{
//				locationButton = new ImageView(context);
//				locationLayoutParams = new RelativeLayout.LayoutParams(
//						LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
//				locationLayoutParams.addRule(RelativeLayout.ALIGN_LEFT,
//						MAP_WIDGET_LOGO_ICON);
//				locationLayoutParams.addRule(RelativeLayout.ABOVE,
//						MAP_WIDGET_LOGO_ICON);
//				locationButton.setLayoutParams(locationLayoutParams);
//				locationButton.setImageBitmap(m_icon_loc_once.getBitmap());
//				locationButton.setId(MAP_WIDGET_LOCATION);
//				addView(locationButton);
//			}

			// scale icon
			{
				m_scaleIcon = new ImageView(context);

				LayoutParams params = new RelativeLayout.LayoutParams(
						LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
				params.setMargins(DEFAULT_MARGIN_X, 0, 0, DEFAULT_MARGIN_X);
				params.addRule(RelativeLayout.ABOVE, MAP_WIDGET_LOGO_ICON);
				m_scaleIcon.setLayoutParams(params);

				m_scaleIcon.setScaleType(ScaleType.MATRIX);
				m_scaleIcon.setImageBitmap(scaleDescriptor.getBitmap());
				m_scaleIcon.setId(MAP_WIDGET_SCALE_ICON);

				addView(m_scaleIcon);
			}
			
			// scale text
			{
				
				m_scaleText = new TextView(context);
				scaleLayoutParams = new RelativeLayout.LayoutParams(
						LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
				scaleLayoutParams.setMargins(DEFAULT_MARGIN_X, 0, 0, DEFAULT_MARGIN_X);
				scaleLayoutParams.addRule(RelativeLayout.ABOVE, MAP_WIDGET_LOGO_ICON);
				m_scaleText.setLayoutParams(scaleLayoutParams);
				m_scaleText.setText("50m");
				m_scaleText.setGravity(Gravity.CENTER);
				m_scaleText.setTextColor(Color.BLACK);
				if (KMapInitializer.getInstance().getDPI() <= 320) {
					m_scaleText.setTextSize(12);
				} else {
					m_scaleText.setTextSize(14);
				}
				//(KMapInitializer.m_dpi / 320)*11     (KMapInitializer.m_dpi / 480)*14
				//m_scaleText.setTextScaleX();
				m_scaleText.setId(MAP_WIDGET_SCALE_TEXT);
				addView(m_scaleText);
			}

			// zoomOut
			{
				zoomOutButton = new ImageView(context);
				zoomOutLayoutParams = new RelativeLayout.LayoutParams(
						LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
				zoomOutLayoutParams.addRule(RelativeLayout.ALIGN_PARENT_RIGHT);
//				zoomOutLayoutParams.addRule(RelativeLayout.CENTER_IN_PARENT);
				zoomOutLayoutParams.addRule(RelativeLayout.ABOVE,MAP_WIDGET_LOGO_ICON);
				zoomOutLayoutParams.setMargins(0, 0, DEFAULT_MARGIN_X, 0);
				zoomOutButton.setLayoutParams(zoomOutLayoutParams);
				zoomOutButton.setImageBitmap(zoomoutBitmap);

				zoomOutButton.setId(MAP_WIDGET_ZOOM_OUT);
				zoomOutButton.setClickable(true);
				addView(zoomOutButton);
			}

			// zoomIn
			{
				zoomInButton = new ImageView(context);
				zoomInLayoutParams = new RelativeLayout.LayoutParams(
						LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
				zoomInLayoutParams.addRule(RelativeLayout.ALIGN_LEFT,
						MAP_WIDGET_ZOOM_OUT);
				zoomInLayoutParams.addRule(RelativeLayout.ABOVE,
						MAP_WIDGET_ZOOM_OUT);
				zoomInLayoutParams.bottomMargin = 0;
				zoomInButton.setLayoutParams(zoomInLayoutParams);
				zoomInButton.setImageBitmap(zoominBitmap);

				zoomInButton.setId(MAP_WIDGET_ZOOM_IN);
				zoomInButton.setClickable(true);
				addView(zoomInButton);
			}

			// compress view
			compressButton = new ImageView(context);
			compressLayoutParams = new RelativeLayout.LayoutParams(
					LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
			compressLayoutParams.addRule(ALIGN_PARENT_TOP);
			compressLayoutParams.addRule(ALIGN_PARENT_LEFT);
			compressLayoutParams.setMargins(DEFAULT_MARGIN_X, DEFAULT_MARGIN_X,
					0, 0);
			compressButton.setLayoutParams(compressLayoutParams);
			compressButton.setImageBitmap(comBitmap);
			compressButton.setScaleType(ScaleType.MATRIX);
			addView(compressButton);

			updateWidget();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	private void moveMapCenterToLocation() {
		if (m_location_type == KMap.LOCATION_TYPE_LOCATE) {
			
			if (!m_bNeedLocaleOnce) {
				return;
			}
		}
		m_bNeedLocaleOnce = false;

		Location loc = map.getMyLocation();
		if (loc.getAccuracy() < 500)
		{
			// 如果定位精度高,就移动地图
			KCameraPosition c = map.getCameraPosition();
			c.target.longitude = loc.getLongitude();
			c.target.latitude = loc.getLatitude();

			if (m_location_type == KMap.LOCATION_TYPE_MAP_ROTATE) {
				map.animateCamera(KCameraUpdateFactory
						.newCameraPosition(new KCameraPosition(c.target, c.zoom, c.tilt,
							loc.getBearing())));
			}
			else
			{
				map.animateCamera(KCameraUpdateFactory.changeLatLng(c.target));
			}
		}
	}
	
	private void initListener() {
////		locationButton.setOnClickListener(new OnClickListener() {
//
//			@Override
//			public void onClick(View v) {
//				Location loc = map.getMyLocation();
//				if (loc == null) {
//					return;
//				}
//
//				KCameraPosition c = map.getCameraPosition();
//				c.target.longitude = loc.getLongitude();
//				c.target.latitude = loc.getLatitude();
//				if (m_location_type == KMap.LOCATION_TYPE_LOCATE)
//				{
//					setMyLocationType(KMap.LOCATION_TYPE_MAP_FOLLOW);
//					map.animateCamera(KCameraUpdateFactory
//							.newCameraPosition(new KCameraPosition(c.target,
//									MAP_ZOOM_DEFA, 0, c.bearing)));
//				}else if (m_location_type == KMap.LOCATION_TYPE_MAP_FOLLOW) {
//					setMyLocationType(KMap.LOCATION_TYPE_MAP_ROTATE);
//					map.animateCamera(KCameraUpdateFactory
//							.newCameraPosition(new KCameraPosition(c.target,
//									MAP_ZOOM_DEFA, MAP_3D_TITL_DEFA, loc.getBearing())));
//				}else if (m_location_type == KMap.LOCATION_TYPE_MAP_ROTATE) {
//					setMyLocationType(KMap.LOCATION_TYPE_MAP_FOLLOW);
//					map.animateCamera(KCameraUpdateFactory
//							.newCameraPosition(new KCameraPosition(c.target,
//									MAP_ZOOM_DEFA, 0, 0)));
//				}
//				
//				updateLocationMakerPosition();
//			}
//		});

		// 切换2D/3D地图
		compressButton.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				KCameraPosition c = map.getCameraPosition();
				if (map.is2DMode()) {
					map.animateCamera(KCameraUpdateFactory
							.newCameraPosition(new KCameraPosition(c.target,
									c.zoom, MAP_3D_TITL_DEFA, 0)));
				} else {
					map.animateCamera(KCameraUpdateFactory
							.newCameraPosition(new KCameraPosition(c.target,
									c.zoom, MAP_2D_TITL_DEFA, 0)));
				}
			}
		});
		zoomInButton.setOnTouchListener(new OnTouchListener() {
			
			@Override
			public boolean onTouch(View v, MotionEvent event) {
				switch (event.getAction()) {
				case MotionEvent.ACTION_DOWN:
					zoomInButton.setImageBitmap(zoominPressedbitmap);
					break;
				case MotionEvent.ACTION_UP:
					zoomInButton.setImageBitmap(zoominBitmap);
					map.animateCamera(KCameraUpdateFactory.zoomIn());
					break;
				}
				return true;
			}
		});

		zoomOutButton.setOnTouchListener(new OnTouchListener() {
			
			@Override
			public boolean onTouch(View v, MotionEvent event) {
				switch (event.getAction()) {
				case MotionEvent.ACTION_DOWN:
					zoomOutButton.setImageBitmap(zoomoutPressedBitmap);
					break;
				case MotionEvent.ACTION_UP:
					zoomOutButton.setImageBitmap(zoomoutBitmap);
					map.animateCamera(KCameraUpdateFactory.zoomOut());
					break;
				}
				return true;
			}
		});
	}

	@Override
	protected void onLayout(boolean changed, int l, int t, int r, int b) {
		super.onLayout(changed, l, t, r, b);
	}

	public int getZoomPos(){
		return zoomPos;
	}
	
	public int getLogoPos() {
		return logoPos;
	}

	@SuppressLint("NewApi")
	public void setLogoPos(int pos) {

		if (pos == logoPos) {
			return;
		}
		switch (pos) {
		case MapOptions.LOGO_POSITION_BOTTOM_CENTER:
			logoPos = pos;
			logoImageLayoutParams.removeRule(RelativeLayout.ALIGN_PARENT_LEFT);
			logoImageLayoutParams.removeRule(RelativeLayout.ALIGN_PARENT_RIGHT);
			logoImageLayoutParams.addRule(RelativeLayout.CENTER_IN_PARENT);
			logoImageView.setLayoutParams(logoImageLayoutParams);
			break;
		case MapOptions.LOGO_POSITION_BOTTOM_LEFT:
			logoPos = pos;
			logoImageLayoutParams.removeRule(RelativeLayout.CENTER_IN_PARENT);
			logoImageLayoutParams.removeRule(RelativeLayout.ALIGN_PARENT_RIGHT);
			logoImageLayoutParams.addRule(RelativeLayout.ALIGN_PARENT_LEFT);
			logoImageView.setLayoutParams(logoImageLayoutParams);
			break;
		case MapOptions.LOGO_POSITION_BOTTOM_RIGHT:
			logoPos = pos;
			logoImageLayoutParams.removeRule(RelativeLayout.CENTER_IN_PARENT);
			logoImageLayoutParams.removeRule(RelativeLayout.ALIGN_PARENT_LEFT);
			logoImageLayoutParams.addRule(RelativeLayout.ALIGN_PARENT_RIGHT);
			logoImageView.setLayoutParams(logoImageLayoutParams);
			break;
		}

		requestLayout();
	}

	public boolean isCompassEnabled() {
		return compressButton.isEnabled();
	}

	public boolean isScaleControlEnable() {
		return m_scaleText.isEnabled();
	}

	public boolean isZoomControlEnable() {
		return zoomInButton.isEnabled();
	}

	public void setCompassEnabled(boolean enable) {
		compressButton.setEnabled(enable);
		if (enable) {
			compressButton.setVisibility(View.VISIBLE);
		} else {
			compressButton.setVisibility(View.INVISIBLE);
		}
	}

	public void setScaleControlsEnabled(boolean enabled) {
		m_scaleText.setEnabled(enabled);
		m_scaleIcon.setEnabled(enabled);
		if (enabled) {
			m_scaleText.setVisibility(View.VISIBLE);
			m_scaleIcon.setVisibility(View.VISIBLE);
		} else {
			m_scaleText.setVisibility(View.INVISIBLE);
			m_scaleIcon.setVisibility(View.INVISIBLE);
		}
	}

	public void setZoomControlEnable(boolean enabled) {
		zoomInButton.setEnabled(enabled);
		zoomOutButton.setEnabled(enabled);

		if (enabled) {
			zoomInButton.setVisibility(View.VISIBLE);
			zoomOutButton.setVisibility(View.VISIBLE);
		} else {
			zoomInButton.setVisibility(View.INVISIBLE);
			zoomOutButton.setVisibility(View.INVISIBLE);
		}
	}
	
	public void setLogoEnable(boolean enabled) {
		logoImageView.setEnabled(enabled);

		if (enabled) {
			logoImageView.setVisibility(View.VISIBLE);
		} else {
			logoImageView.setVisibility(View.INVISIBLE);
		}
	}

	@SuppressLint("NewApi")
	public void setzoomPos(int pos) {
		if (pos == zoomPos) {
			return;
		}

		switch (pos) {
		case MapOptions.ZOOM_POSITION_RIGHT_BOTTOM:
			zoomOutLayoutParams.removeRule(RelativeLayout.CENTER_IN_PARENT);
			zoomOutLayoutParams.addRule(RelativeLayout.ABOVE,MAP_WIDGET_LOGO_ICON);
			zoomPos = pos;
			break;
		case MapOptions.ZOOM_POSITION_RIGHT_CENTER:
			zoomPos = pos;
			zoomOutLayoutParams.removeRule(RelativeLayout.ABOVE);
			zoomOutLayoutParams.addRule(RelativeLayout.CENTER_IN_PARENT);
			break;
		}
		requestLayout();
	}

	public View getViewByType(int which) {

		View view = null;
		switch (which) {
 		case MAP_WIDGET_LOGO_ICON:
			view = logoImageView;
			break;
		case MAP_WIDGET_ZOOM_IN:
			view = zoomInButton;
			break;
		case MAP_WIDGET_ZOOM_OUT:
			view = zoomOutButton;
			break;
		case MAP_WIDGET_COMPRESS:
			view = compressButton;
			break;
		case MAP_WIDGET_SCALE_TEXT:
			view = m_scaleText;
			break;
		}
		return view;
	}

	private void updateScaleIcon() {
		float orgin_w = m_scaleIcon.getDrawable().getBounds().width();
		float rate = MapCore.GetScalePerPixel();
		float rule_len;
		final float[] scales = { 10, 25, 50, 100, 200, 500, 1000, 2000, 5000,
				10000, 20000, 30000, 50000, 100000, 200000, 500000, 1000000 };
		KCameraPosition c = map.getCameraPosition();

		int index = 19 - (int) c.zoom;
		rule_len = scales[index];

		float change_w = rule_len / rate;

		Matrix matrix = new Matrix();
//		matrix.postScale(change_w / orgin_w, 1.0f * 1.0f, m_scaleIcon
//				.getDrawable().getBounds().width(), m_scaleIcon.getDrawable()
//				.getBounds().height() * 2);
		matrix.postScale(change_w / orgin_w, 1.0f * 1.0f, 0, m_scaleIcon.getDrawable()
				.getBounds().height() * 2);
		m_scaleIcon.setImageMatrix(matrix);
	}

	protected void updateCompass() {
		KCameraPosition c = map.getCameraPosition();
		Matrix matrix = new Matrix();
		matrix.postRotate(360 - c.bearing, compressButton.getDrawable()
				.getBounds().width() / 2, compressButton.getDrawable()
				.getBounds().height() / 2);
		//此注释可能用得着
//			matrix.postScale(1.0f, (float) Math.cos(c.tilt * Math.PI / 180.f),
//					compressButton.getDrawable().getBounds().width() / 2,
//					compressButton.getDrawable().getBounds().height() / 2);
		compressButton.setImageMatrix(matrix);
	}

	private void updateLocation() {
//		if (m_my_location_is_displayed) {
//			switch (m_location_type) {
//			case KMap.LOCATION_TYPE_LOCATE:
//				locationButton.setImageBitmap(m_icon_loc_once.getBitmap());
//				break;
//			case KMap.LOCATION_TYPE_MAP_FOLLOW:
//				locationButton.setImageBitmap(m_icon_loc_gps.getBitmap());
//				break;
//			case KMap.LOCATION_TYPE_MAP_ROTATE:
////				locationButton.setImageBitmap(m_icon_loc_rotate.getBitmap());
//				break;
//			}
//		}
	}

	private void updateScaleText() {
		KFloat fLevel = new KFloat(0);
		MapCore.GetMapScaleLevel(fLevel);
		int curScale = (int) fLevel.FloatVal();
		String strScale = "unknown";
		if (sceleProp == null) {
			sceleProp = new Vector<NE_ScaleProp>();
		}
		if (sceleProp.isEmpty()) {
			MapCore.GetMapScaleLevelMapping(sceleProp);
		}
		int realScale = 19 - curScale;
		if (realScale < 0 || realScale >= sceleProp.size()) {
			return;
		}
		strScale = sceleProp.get(realScale).m_scale_Name;
		CharSequence s = m_scaleText.getText();
		s = strScale;
		m_scaleText.setText(s);
	}

	public void updateWidget() {
		updateScaleIcon();
		updateScaleText();
//		updateLocation();
		updateCompass();
		updateZoom();
	}

	void updateZoom() {
		KCameraPosition c = map.getCameraPosition();
		int maxScale;
		Log.i("updateZoom", "updateZoom:" + c.zoom);
		maxScale = MapCore.GetMaxScale();
		Log.i("updateZoom", "updateZoom:maxScale:" + maxScale);
		if (c.zoom == maxScale) {
			zoomInButton.setImageBitmap(zoominDisabledBitmap);
			zoomInButton.setEnabled(false);
			return;
		} else {
			zoomInButton.setImageBitmap(zoominBitmap);
			zoomInButton.setEnabled(true);
		}
		int minScale;
		minScale = MapCore.GetMinScale();
		Log.i("updateZoom", "updateZoom:minScale:" + minScale);
		if (c.zoom == minScale) {
			zoomOutButton.setImageBitmap(zoomoutDisabledBitmap);
			zoomOutButton.setEnabled(false);
			return ;
		} else {
			zoomOutButton.setImageBitmap(zoomoutBitmap);
			zoomOutButton.setEnabled(true);
		}
	}

	public KUiSettings getSettings() {
		return settings;
	}

	public void onDestroy() {
		if (m_location_marker != null) {
			m_location_marker.remove();
			m_location_marker = null;
		}
		if (logoDescriptor != null) {
			logoDescriptor.recycle();
			logoDescriptor = null;
		}
		if (logoBitmap != null) {
			logoBitmap.recycle();
			logoBitmap = null;
		}
		if (scaleDescriptor != null) {
			scaleDescriptor.recycle();
			scaleDescriptor = null;
		}
		if (zoomoutDescriptor != null) {
			zoomoutDescriptor.recycle();
			zoomoutDescriptor = null;
		}
		if (zoomoutBitmap != null) {
			zoomoutBitmap.recycle();
			zoomoutBitmap = null;
		}
		if (zoominDescriptor != null) {
			zoominDescriptor.recycle();
			zoominDescriptor = null;
		}
		if (zoominBitmap != null) {
			zoominBitmap.recycle();
			zoominBitmap = null;
		}
		if (comDescriptor != null) {
			comDescriptor.recycle();
			comDescriptor = null;
		}
		if (comBitmap != null) {
			comBitmap.recycle();
			comBitmap = null;
		}
		if (zoominPressedDescriptor != null) {
			zoominPressedDescriptor.recycle();
			zoominPressedDescriptor = null;
		}
		if (zoominPressedbitmap != null) {
			zoominPressedbitmap.recycle();
			zoominPressedbitmap = null;
		}
		if (zoomoutPressedDescriptor != null) {
			zoomoutPressedDescriptor.recycle();
			zoomoutPressedDescriptor = null;
		}
		if (zoomoutPressedBitmap != null) {
			zoomoutPressedBitmap.recycle();
			zoomoutPressedBitmap = null;
		}
		if (zoominDisabledDescriptor != null) {
			zoominDisabledDescriptor.recycle();
			zoominDisabledDescriptor = null;
		}
		if (zoominDisabledBitmap != null) {
			zoominDisabledBitmap.recycle();
			zoominDisabledBitmap = null;
		}
		if (zoomoutDisabledDescriptor != null) {
			zoomoutDisabledDescriptor.recycle();
			zoomoutDisabledDescriptor = null;
		}
		if (zoomoutDisabledBitmap != null) {
			zoomoutDisabledBitmap.recycle();
			zoomoutDisabledBitmap = null;
		}
	}
	

    public void processMapScreenShot(Canvas cv) {
        // 在地图截图上叠加logo
        if (logoImageView != null
                && logoImageView.getVisibility() == View.VISIBLE) {
            cv.drawBitmap(logoBitmap, logoImageView.getLeft(), logoImageView.getTop(), null);
        }
    }
}
