package com.cmmap.internal.maps;

import android.graphics.Point;
import android.graphics.PointF;

import com.cmmap.internal.driver.base.MapTool;
import com.cmmap.internal.maps.model.KLatLng;

/**
 * 地图坐标映射类。
 * <p>地图坐标映射类实现了屏幕位置和地理坐标（经纬度）之间的进行转换，
 * 屏幕位置是相对于地图视图的左上角的位置，并不一定是相对于屏幕左上角的位置。</p>
 */
public class KProjection {
	
	private PointF size;
	
	private static final KProjection INSTANCE = new KProjection();
	
	/**
	 * 内部构造函数，不公开提供
	 */
	private KProjection(){
		size = new PointF();
	}
	
	public void setSize(PointF pointF){
		INSTANCE.size = pointF;
	}
	
	public PointF getSize() {
		return INSTANCE.size;
	}
	
	public static KProjection getInstance(){
		return INSTANCE;
	}
	/**
	 * 根据输入的屏幕位置（屏幕像素）返回一个地图位置（经纬度）。
	 * @param point 屏幕位置（屏幕像素）
	 * @return 地图位置（经纬度）
	 */
	public KLatLng fromScreenLocation(Point point){
		Point p_screen = new Point(point.x, (int)size.y - point.y);
		return MapTool.pointToLatlon(p_screen);
	}

	/**
	 * 返回一个从地图位置转换来的屏幕位置。
	 * 
	 * @param location
	 *            
	 * @return 屏幕坐标
	 */
	public Point toScreenLocation(KLatLng location){
		Point p_screen = MapTool.latlonToPoint(location);
		return new Point(p_screen.x, (int) (size.y - p_screen.y));
	}
	
	/**
	 * 返回一个在屏幕坐标与地理经纬度坐标之间锥形视图的映射
	 * @return
	 */
	public KVisibleRegion getVisibleRegion(){
		return null;
	}
}
