package com.cmmap.internal.maps;

import com.cmmap.api.maps.Map;
import com.cmmap.api.maps.MapOptions;


/**
 * 地图用户界面设置类。
 * <p>该实例需要通过调用{@link Map#getUiSettings}获取。</p>
 */
public class KUiSettings {
	
	private KMapWidgetView mapWidgetView;
	
	public KMapWidgetView getMapWidgetView() {
		return mapWidgetView;
	}

	public void setMapWidgetView(KMapWidgetView mapWidgetView) {
		this.mapWidgetView = mapWidgetView;
	}

	private boolean rotateGestureEnable = true;
	
	private boolean scollGestureEnable = true;
	
	private boolean tiltGesturesEnable = true;
	
	private boolean zoomGestureEnable = true;
	
	private boolean allGestureEnable = true;
	
	 public KUiSettings()
	{
		
	}
	
	void setOperationView(KMapWidgetView view){
		this.mapWidgetView = view;
	}
	
	/**
	 * 获取地图自带Logo的显示位置。
	 * @return 地图自带Logo的显示位置，参见：
	 *         <br>{@link MapOptions#LOGO_POSITION_BOTTOM_LEFT}（默认值）
	 *         <br>{@link MapOptions#LOGO_POSITION_BOTTOM_CENTER}
	 *         <br>{@link MapOptions#LOGO_POSITION_BOTTOM_RIGHT}
	 */
	public int getLogoPosition() {
		if (mapWidgetView != null) {
			return mapWidgetView.getLogoPos();
		}
		return  0;
	}
	
	/**
	 * 返回地图自带指南针控件是否可用。
	 * @return 地图自带指南针控件是否可用，默认可用
	 */
	public boolean isCompassEnabled(){
		
		if (mapWidgetView != null) {
			return mapWidgetView.isCompassEnabled();
		}
		return false;
	}
	
	/**
	 * 返回是否允许用户手势旋转地图。
	 * @return 是否允许用户手势旋转地图，默认：允许
	 */
	public boolean isRotateGesturesEnabled() {
		return rotateGestureEnable;
	}
	
	/**
	 * 返回地图自带比例尺控件是否可用。
	 * @return 地图自带比例尺是否可用，默认为可用
	 */
	public boolean isScaleControlsEnabled() {
		return mapWidgetView.isScaleControlEnable();
	}
	
	
	/**
	 * 返回是否允许用户手势移动地图。
	 * @return 是否允许用户手势移动地图，默认：允许
	 */
	public boolean isScrollGesturesEnabled(){
		return scollGestureEnable;
	}
	
	/**
	 * 返回是否允许用户手势倾斜地图。
	 * @return 是否允许用户手势倾斜地图， 默认：允许
	 */
	public boolean isTiltGesturesEnabled(){
		return tiltGesturesEnable;
	}
	
	/**
	 * 返回地图自带缩放按钮是否可用。
	 * @return 地图自带缩放按钮是否可用，默认为可用
	 */
	public boolean isZoomControlsEnabled(){
		return mapWidgetView.isZoomControlEnable();
	}
	
	
	/**
	 * 返回是否允许用户手势缩放地图。
	 * @return 是否允许用户手势缩放地图，默认：允许
	 */
	public boolean isZoomGesturesEnabled() {
		return zoomGestureEnable;
	}
	
	public boolean getAllGesturesEnabled() {
		return allGestureEnable;
	}
	/**
	 * 设置地图自带指南针控件是否可用。
	 * @param enabled 地图自带指南针控件是否可用
	 */
	public void setCompassEnabled(boolean enabled) {
		mapWidgetView.setCompassEnabled(enabled);
	}
	
	
	/**
	 * 设置地图自带Logo的显示位置。
	 * @param position 地图自带Logo的显示位置，参见：
	 *        <br>{@link MapOptions#LOGO_POSITION_BOTTOM_LEFT}
	 *        <br>{@link MapOptions#LOGO_POSITION_BOTTOM_CENTER}
	 *        <br>{@link MapOptions#LOGO_POSITION_BOTTOM_RIGHT}
	 */
	public void setLogoPosition(int position) {
		mapWidgetView.setLogoPos(position);
	}
	
	/**
	 * 获取地图自带缩放按钮的显示位置。
	 * @return 地图自带缩放按钮的显示位置：
	 *         <br>{@link KMapOptions#ZOOM_POSITION_RIGHT_CENTER}
	 *         <br>{@link KMapOptions#ZOOM_POSITION_RIGHT_BOTTOM}（默认值）
	 */
	public int getZoomPosition() {
		return mapWidgetView.getZoomPos();
	}
	
	/**
	 * 设置logo是否显示
	 * @param enabled
	 */
	public void setMyLogoEnabled(boolean enabled) {
		mapWidgetView.setLogoEnable(enabled);
	}
	
	/**
	 * 设置是否允许用户手势旋转地图。
	 * <br>
	 * 这个设置不会影响用户在程序里对地图的旋转。
	 * @param enabled 是否允许用户手势旋转地图
	 */
	public void setRotateGesturesEnabled(boolean enabled){
		rotateGestureEnable = enabled;
	}
	
	/**
	 * 设置地图自带比例尺控件是否可用。
	 * @param enabled 地图自带比例尺控件是否可用
	 */
	public void setScaleControlsEnabled(boolean enabled) {
		mapWidgetView.setScaleControlsEnabled(enabled);
	}
	
	/**
	 * 设置是否允许用户手势移动地图。
	 * <br>
	 * 这个设置不会影响用户通过程序接口移动地图。
	 * @param enabled 是否允许用户手势移动地图
	 */
	public void setScrollGesturesEnabled(boolean enabled) {
		scollGestureEnable = enabled;
	}
	
	/**
	 * 设置是否允许用户手势倾斜地图。
	 * <br>
	 * 这个设置不会影响用户通过程序接口旋转地图。
	 * @param enabled 是否允许用户手势倾斜地图
	 */
	public void setTiltGesturesEnabled(boolean enabled) {
		tiltGesturesEnable = enabled;
	}
	
	/**
	 * 设置地图自带缩放按钮是否可用。
	 * @param enabled 地图自带缩放按钮是否可用
	 */
	public void setZoomControlsEnabled(boolean enabled){
		mapWidgetView.setZoomControlEnable(enabled);
	}
	
	/**
	 * 设置是否允许用户手势缩放地图。
	 * <br>
	 * 这些手势包括通过双指捏合和绽放地图、单指双击放大地图及双指单击缩小地图。
	 * <br>
	 * 这个设置不会影响缩放按钮的功能， 也不会影响程序对地图的操控。
	 * @param enabled 是否允许用户手势缩放地图
	 */
	public void setZoomGesturesEnabled(boolean enabled){
		zoomGestureEnable = enabled;
	}
	
	/**
	 * 设置地图自带缩放按钮的显示位置。
	 * @param position 地图自带缩放按钮的显示位置：
	 *        <br>{@link MapOptions#ZOOM_POSITION_RIGHT_CENTER}
	 *        <br>{@link MapOptions#ZOOM_POSITION_RIGHT_BOTTOM}
	 */
	public void setZoomPosition(int position){
		mapWidgetView.setzoomPos(position);
	}
}
