package com.cmmap.internal.maps;

import com.cmmap.api.maps.model.LatLngBounds;
import com.cmmap.internal.maps.model.KLatLng;

/**
 * 在地图的可视区域中定义多边形四边的四个点坐标包含的区域是可见的。
 * 这个多边形是不规则四边形， 而不是矩形，因为可视区域是可以倾斜的。
 * 如果可视区域正好在可视区域中心的上方，形状就是矩形；
 * 如果可视区域倾斜，那么形状就是梯形，距离视野最近的边最短
 * 
 *
 */
public class KVisibleRegion {

	/**
	 * 区域离视野远的左顶点
	 */
	public KLatLng farLeft;
	
	/**
	 * 区域离视野远的右顶点
	 */
	public KLatLng farRight;
	
	/**
	 * 区域离视野近的左顶点
	 */
	public KLatLng nearLeft;
	
	/**
	 * 区域离视野近的右顶点
	 */
	public KLatLng nearRight;
	
	/**
	 * 最小边界框，包含在这个类里定义的可见区域
	 */
	public LatLngBounds latLngBounds;
	
	
	/**
	 * 给出可视区域四个角的坐标，创建VisibleRegion对象
	 * @param nearLeft
	 * @param nearRight
	 * @param farLeft
	 * @param farRight
	 * @param latLngBounds
	 */
	public KVisibleRegion(KLatLng nearLeft, KLatLng nearRight, KLatLng farLeft, KLatLng farRight, LatLngBounds latLngBounds){
		
		this.nearLeft = nearLeft;
		this.nearRight = nearRight;
		this.farLeft = farLeft;
		this.farRight = farRight;
		this.latLngBounds = latLngBounds;
	}
	
}
