package com.cmmap.internal.maps.model;

import java.util.ArrayList;
import java.util.List;

import android.graphics.Color;

import com.cmmap.api.maps.model.ArcOptions;


public class KArcOptions{

	private int strokeColor = Color.BLACK;
	private float strokeWidth = 10;
	private boolean draggable = false;
	private int level = 0;
	private float zindex = 0;
	private boolean bVisible = false;
	
	private List<KLatLng> points = new ArrayList<KLatLng>();
	
	public KArcOptions(KArcOptions options)
	{
		this.strokeColor = options.strokeColor;
		this.strokeWidth = options.strokeWidth;
		this.draggable = options.draggable;
		this.level = options.level;
		this.zindex = options.zindex;
		this.bVisible = options.bVisible;
		for (int i = 0; i < options.points.size(); i++) {
			this.points.add(new KLatLng(options.points.get(i)));
		}
	}
	
	public KArcOptions () {
		
	}
	
	public KArcOptions(ArcOptions options) {
		strokeColor = options.getStrokeColor();
		strokeWidth = options.getStrokeWidth();
		draggable = options.isDraggable();
		level = options.getLevel();
		zindex = options.getZIndex();
		bVisible = options.isVisible();

		points.clear();
		points.add(new KLatLng(options.getStart().latitude, options.getStart().longitude));
		points.add(new KLatLng(options.getPassed().latitude, options.getPassed().longitude));
		points.add(new KLatLng(options.getEnd().latitude, options.getEnd().longitude));
	}
	
	/**
	 * 返回起点坐标。
	 * @return 起点坐标
	 */
	public KLatLng getStart() {
		if (points.size() > 0) {
			return new KLatLng(points.get(0));
		}
		return null;
	}

	/**
	 * 返回途经点坐标。
	 * @return 途经点坐标
	 */
	public KLatLng getPassed() {
		if (points.size() > 1) {
			return new KLatLng(points.get(1));
		}
		return null;
	}

	/**
	 * 返回终点坐标。
	 * @return 终点坐标
	 */
	public KLatLng getEnd() {
		if (points.size() > 2) {
			return new KLatLng(points.get(2));
		}
		return null;
	}
	
	/**
	 * 返回边框颜色。
	 * @return 边框颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getStrokeColor() {
		return this.strokeColor;
	}
	
	/**
	 * 返回边框宽度。
	 * @return 边框宽度，单位：像素，默认：10像素
	 */
	public float getStrokeWidth(){
		return this.strokeWidth;
	}
	
	
	/**
	 * 返回level
	 * @return 
	 */
	public int getLevel() {
		return this.level;
	}
	
	/**
	 * 返回Z轴索引。
	 * @return Z轴索引，默认：1.0
	 */
	public float getZIndex() {
		return this.zindex;
	}
	

	/**
	 * 返回是否可见。
	 * @return 是否可见，默认：可见
	 */ 
	public boolean isVisible() {
		return this.bVisible;
	}
	
	/**
	 * 设置弧线的起终点和途径点
	 * @param startpoint
	 * @param passpoint
	 * @param endpoint
	 * @return
	 */
	public KArcOptions points(KLatLng startpoint, KLatLng passpoint, KLatLng endpoint) {
		if (points != null) {
			points.clear();
			this.points.add( new KLatLng(startpoint));
			this.points.add( new KLatLng(passpoint));
			this.points.add( new KLatLng(endpoint));
		}
		return this;
	}
	
	public List<KLatLng> getPoints() {
		return this.points;
	}
	/**
	 * 设置边框颜色。
	 * @param color ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 * @return 该对象本身
	 */
	public KArcOptions strokeColor(int color) {
		this.strokeColor = color;
		return this;
	}
	
	/**
	 * 设置边框宽度。
	 * @param width 边框宽度，单位：像素，默认：10像素
	 * @return 该对象本身
	 */
	public KArcOptions strokeWidth(float width){
		this.strokeWidth = width;
		return this;
	}
	
	/**
	 * 返回是否可拖拽。
	 * @return 是否可拖拽，默认：否
	 */
	public boolean isDraggable() {
		return draggable;
	}

	/**
	 * 设置是否可拖拽。
	 * @param enable 是否可拖拽，默认：否
	 * @return 该对象本身
	 */
	public KArcOptions draggable(boolean enable) {
		draggable = enable;
		return this;
	}
	
	/**
	 * 设置是否可见。
	 * @param visible 是否可见，默认：可见
	 * @return 该对象本身
	 */
	public KArcOptions visible(boolean visible) {
		this.bVisible = visible;
		return this;
	}
	
	
	/**
	 * 设置level。
	 * @param
	 * @return 该对象本身
	 */
	public KArcOptions level(int level){
		this.level = level;
		return this;
	}

	/**
	 * 设置Z轴索引。
	 * @param zIndex Z轴索引，默认：1.0
	 * @return 该对象本身
	 */
	public KArcOptions zIndex(float zIndex){
		this.zindex = zIndex;
		return this;
	}

	void releaseResource() {
		points.clear();
		points = null;
	}
}
