package com.cmmap.internal.maps.model;

import com.cmmap.api.maps.model.LatLng;

import android.os.Parcel;
import android.os.Parcelable;


/**
 * 存储经纬度坐标值的类，单位角度。
 */
public class KLatLng implements Parcelable,Cloneable {
	/** 纬度 (水平方向)，单位角度 */
	public double latitude;
	/** 经度 (垂直方向)，单位角度 */
	public double longitude;
	
	public KLatLng() {
		this.longitude = 0;
		this.latitude = 0;
	}
	
	/**
	 * 使用传入的经纬度构造LatLng 对象，一对经纬度值代表地球上一个地点。
	 * @param latitude 地点的纬度，在-90 与90 之间的double 型数值
	 * @param longitude 地点的经度，在-180 与180 之间的double 型数值
	 */
	public KLatLng(double latitude, double longitude) {
		this.longitude = longitude;
		this.latitude = latitude;
	}
	
	public KLatLng(KLatLng target) {
		this.longitude = target.longitude;
		this.latitude = target.latitude;
	}
	
	public KLatLng(LatLng target) {
		this.longitude = target.longitude;
		this.latitude = target.latitude;
	}
	
	public void setLongitude(double longitude){
		this.longitude = longitude;
	}
	public void setLatitude(double latitude){
		this.latitude = latitude;
	}
	
	@Override
	public KLatLng clone() {
		return new KLatLng(latitude, longitude);
		
	}
	

	@Override
	public int hashCode() {
		// TODO
		return 0;
	}
	
	/* (non-Javadoc)
	 * 判断是否与另一个LatLng 对象相等的方法。 如果两个地点的经纬度全部相同，则返回true。否则返回false。
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object o) {
		if (o == null) {
			return false;
		}

		if (!(o instanceof KLatLng)) {
			return false;
		}
		KLatLng latLon = (KLatLng)o;
		return (Double.compare(longitude, latLon.longitude)==0)&&(Double.compare(latitude, latLon.latitude)==0);
	}

	public boolean isValidLatlon() {
		return false;
	}

	public int compareTo(Object another) {

		KLatLng rhsLatLon = (KLatLng)another;
		
		if (this.equals(rhsLatLon)) {
			return 0;
		}
		if (this.longitude < rhsLatLon.longitude) {
			return -1;
		}
		else if (this.longitude == rhsLatLon.longitude) {
			if (this.latitude < rhsLatLon.latitude) {
				return -1;
			}
			else {
				return 1;
			}
		}
		else {
			return 1;
		}
		//return 0;
	}

	/* (non-Javadoc)
	 * @see android.os.Parcelable#describeContents()
	 */
	@Override
	public int describeContents() {
		return 0;
	}

	/* (non-Javadoc)
	 * @see android.os.Parcelable#writeToParcel(android.os.Parcel, int)
	 */
	@Override
	public void writeToParcel(Parcel dest, int flags) {
		dest.writeDouble(latitude);
		dest.writeDouble(longitude);
	}
	
	public static double meterToGeoUnit(KLatLng point, double meter, boolean is_lon)
	{
		double dist = 1.0;
		
		if (is_lon)
		{
			KLatLng point2 = new KLatLng(point.latitude, point.longitude + dist);
			double meter_std = calculateSphericalDistance(point, point2);
			double rate = meter / meter_std;
			return rate;
		}
		
		KLatLng point2 = new KLatLng(point.latitude + dist, point.longitude);
		double meter_std = calculateSphericalDistance(point, point2);
		double rate = meter / meter_std;
		return rate;
	}
	
	private static double calculateSphericalDistance(KLatLng point1, KLatLng point2)
	{
		if (point1 == point2)
		{
			return 0;
		}
		
		double DE2RA = 0.01745329252;
		double FLATTENING = 1.000000/298.257223563; // Earth flattening (WGS84)
		double ERAD = 6378.137;
		
		double lat1 = DE2RA * point1.latitude;
		double lon1 = -DE2RA * point1.longitude;
		double lat2 = DE2RA * point2.latitude;
		double lon2 = -DE2RA * point2.longitude;

		double F = (lat1 + lat2) / 2.0;
		double G = (lat1 - lat2) / 2.0;
		double L = (lon1 - lon2) / 2.0;
	
		double sing = Math.sin(G);
		double cosl = Math.cos(L);
		double cosf = Math.cos(F);
		double sinl = Math.sin(L);
		double sinf = Math.sin(F);
		double cosg = Math.cos(G);
	
		double S = sing*sing*cosl*cosl + cosf*cosf*sinl*sinl;
		double C = cosg*cosg*cosl*cosl + sinf*sinf*sinl*sinl;
		double W = Math.atan2(Math.sqrt(S),Math.sqrt(C));
		double R = Math.sqrt((S*C))/W;
		double H1 = (3 * R - 1.0) / (2.0 * C);
		double H2 = (3 * R + 1.0) / (2.0 * S);
		double D = 2 * W * ERAD;
		
		double r = (1.0 * 1000.0 * D * (1 + FLATTENING * H1 * sinf*sinf*cosg*cosg -
			FLATTENING*H2*cosf*cosf*sing*sing));
		
		return r;
	}
	
	public LatLng toLatLng() {
		return new LatLng(latitude, longitude);
	}
}
