package com.cmmap.internal.maps.model;

import java.util.ArrayList;
import java.util.List;

import android.graphics.Point;

import com.cmmap.api.maps.model.BitmapDescriptor;
import com.cmmap.api.maps.model.MarkerOptions;
import com.cmmap.internal.driver.base.MapTool;
import com.cmmap.internal.maps.KProjection;
/**
 * 线段覆盖物选项类
 */
public class KMarkerOptions{

	/**
	 * 锚点x分量
	 */
	private float anchorX = 0.5f;
	
	/**
	 * 锚点y分量
	 */
	private float anchorY = 1.0f;
		
	/**
	 * 是否允许拖动
	 */
	private boolean bDragable = false;
	

	/**
	 * 是否允许点击
	 */
	private boolean bClickable = false;
	
	/**
	 * marker的动画图片列表
	 */
	private ArrayList<BitmapDescriptor> m_bitmapDescriptors;
	
	/**
	 * 是否贴合地图
	 */
	private boolean m_is_flat = false;
	
	/**
	 * 是否支持下落动画
	 */
	private boolean m_is_animatesDrop = false;
	
	/**
	 * 是否是GPS坐标
	 */
	private boolean m_is_gps = false;
	
	/**
	 * 是否是近大远小效果
	 */
	private boolean m_is_perspective = false; // no used
		
	/**
	 * 是否显示
	 */
	private boolean m_is_visible = true;
	
	/**
	 * 设置多少帧刷新一次图片资源，Marker动画的间隔时间，值越小动画越快
	 */
	private int period = 50;
	
	/**
	 * Marker的位置
	 */
	private int m_x = 0;
	
	private int m_y = 0;
	
	/**
	 * infowindow相对marker的offset
	 */
	private int windowOffsetX = 0;
	
	/**
	 * infowindow相对marker的offset
	 */
	private int windowOffsetY = 0;
	
	/**
	 * Marker的描述文字
	 */
	private String snippet = null;
	
	/**
	 * marker的标题
	 */
	private String title = null;
	
	/**
	 * level的值
	 */
	private int level = 0;
	
	/**
	 * z轴Index
	 */
	private float zIndex = 0;
	
	/**
	 * 设置Marker图片旋转的角度，从正北开始，逆时针计算
	 */
	private float rotateAngle = 0;
	
	//protected boolean m_bMoveWithMap = true;
	
	/**
	 * 默认构造函数
	 */
	public KMarkerOptions(){
		icon(KOverlayManager.getInstance().getDefaultMarkerIcon());	// 设置默认图片，避免外部不设置图片
	}
	
	/**
	 * 复制构造函数
	 * @param options
	 */
	public KMarkerOptions(KMarkerOptions options){
		anchor(options.anchorX, options.anchorY);
		this.bDragable = options.bDragable;
		this.bClickable = options.bClickable;
		this.m_is_flat = options.m_is_flat;
		this.m_is_animatesDrop = options.m_is_animatesDrop;
		this.m_is_gps = options.m_is_gps;
		this.m_is_perspective = options.m_is_perspective;
		this.m_is_visible = options.m_is_visible;
		period(options.period);
		this.m_x = options.m_x;
		this.m_y = options.m_y;
		this.windowOffsetX = options.windowOffsetX;
		this.windowOffsetY = options.windowOffsetY;
		this.snippet = options.snippet;
		this.title = options.title;
		this.level = options.level;
		this.zIndex = options.zIndex;
		if (options.m_bitmapDescriptors == null) {
			icon(KOverlayManager.getInstance().getDefaultMarkerIcon());	// 设置默认图标				
		}else
		{			
			icons(options.getIcons());	// 复制图标
		}
		this.rotateAngle = options.rotateAngle;
	}
	
	public KMarkerOptions(MarkerOptions options) {
		anchor(options.getAnchorU(), options.getAnchorV());
		bDragable = options.isDraggable();
		bClickable = options.isClickable();
		m_is_flat = options.isFlat();
		m_is_animatesDrop = options.isAnimatesDrop();
		m_is_gps = true;
		position(new KLatLng(options.getPosition().latitude, options.getPosition().longitude));
		m_is_visible = options.isVisible();
		period(options.getPeriod());
		windowOffsetX = options.getInfoWindowOffsetX();
		windowOffsetY = options.getInfoWindowOffsetY();
		snippet = options.getSnippet();
		title = options.getTitle();
		level = options.getLevel();
		zIndex = options.getZIndex();
		if (options.getIcons() == null || options.getIcons().size() <= 0) {
			icon(KOverlayManager.getInstance().getDefaultMarkerIcon());	// 设置默认图标
		} else {
			icons(options.getIcons());
		}
		rotateAngle = options.getRotateAngle();
	}
		
	/**
	 * 定义marker 图标的锚点
	 * 锚点是marker 图标接触地图平面的点。
	 * 图标的左顶点为（0,0）点，右底点为（1,1）点。默认情况下，锚点为（0.5,1.0）
	 * @param u 锚点水平范围的比例，必须传入0 到1 之间的数值
	 * @param v 锚点垂直范围的比例，必须传入0 到1 之间的数值
	 * @return 设置了锚点的MarkerOptions 对象
	 */
	public KMarkerOptions anchor(float u, float v) {
		if (u < 0) {
			anchorX = 0;
		} else if (u > 1) {
			anchorX = 1;
		} else {
			anchorX = u;
		}
		
		if (v < 0) {
			anchorY = 0;
		} else if (v > 1) {
			anchorY = 1;
		} else {
			anchorY = v;
		}
		return this;
	}
	
	/**
	 * 返回图片旋转角度
	 * @return 图片旋转角度，默认：0
	 */
	public float getRotateAngle() {
		return rotateAngle;
	}
	
	/**
	 * 设置图片旋转角度。
	 * @param angle 图片旋转角度，从正北开始，逆时针计算
	 * @return 该对象本身
	 */
	public KMarkerOptions rotateAngle(float angle) {
		rotateAngle = angle;
		return this;
	}
	
	/**
	 * 是否允许拖动
	 * @param draggable
	 * @return
	 */
	public KMarkerOptions draggable(boolean draggable){
		this.bDragable = draggable;
		return this;
	}
	
	/**
	 * @return 返回锚点在水平范围的比例，是一个0 到1 之间的float 数值
	 */
	public float getAnchorU() {
		return this.anchorX;
	}
	
	/**
	 * @return 返回锚点在垂直范围的比例，是一个0 到1 之间的float 数值
	 */
	public float getAnchorV(){
		return this.anchorY;
	}
	
	/**
	 * @return marker的图片资源
	 */
	public BitmapDescriptor getIcon(){
		if (m_bitmapDescriptors == null || m_bitmapDescriptors.size() <= 0) return null;
		return this.m_bitmapDescriptors.get(0);
	}
	
	/**
	 * @return Marker的动画帧列表，动画的描点和大小以第一帧为准
	 */
	public ArrayList<BitmapDescriptor> getIcons() {
		return this.m_bitmapDescriptors;
	}
	
	/**
	 * 得到多少帧刷新一次图片资源，值越小动画越快
	 * @return
	 */
	public int getPeriod(){
		return this.period;
	}
	
	/**
	 * 获取Marker的经纬度坐标
	 * @return
	 */
	public KLatLng getPosition() {
		if (m_is_gps) {
			return new KLatLng(MapTool.longToDouble(this.m_y), MapTool.longToDouble(this.m_x));
		} else {
			return KProjection.getInstance().fromScreenLocation(new Point(m_x, m_y));
		}
	}
	
	/**
	 * @return 获取Marker设置的文字片段
	 */
	public String getSnippet() {
		return this.snippet;
	}
	
	/**
	 * @return 获取标题
	 */
	public String getTitle(){
		return this.title;
	}
	
	
	/**
	 * 返回level
	 * @return 
	 */
	public int getLevel() {
		return this.level;
	}
	
	/**
	 * 获取zIndex
	 * @return
	 */
	public float getZIndex() {
		return this.zIndex;
	}
	
	/**
	 * 设置MarkerOptions 对象的自定义图标
	 * @param icon
	 * @return
	 */
	public KMarkerOptions icon(BitmapDescriptor icon) {
		if(icon != null) {
			if (m_bitmapDescriptors == null) {
				m_bitmapDescriptors = new ArrayList<BitmapDescriptor>();
			}
			else {
				m_bitmapDescriptors.clear();
			}
			m_bitmapDescriptors.add(icon);
		}
		return this;
	}
	
	/**
	 * 设置Marker的动画帧列表
	 * @param icons
	 * @return
	 */
	public KMarkerOptions icons(List<BitmapDescriptor> icons) {
		if(icons != null && icons.size() >= 1) {
			if (m_bitmapDescriptors == null) {
				m_bitmapDescriptors = new ArrayList<BitmapDescriptor>();
			}
			else {
				m_bitmapDescriptors.clear();
			}
			this.m_bitmapDescriptors.addAll(icons);
		}
		return this;
	}
	
	void recycleIcon() {
		if (m_bitmapDescriptors != null) {
			m_bitmapDescriptors.clear();
		}
	}
	
	/**
	 * marker是否允许拖动
	 * @return
	 */
	public boolean isDraggable() {
		return this.bDragable;
	}
	
	/**
	 * 返回marker是否平贴地图
	 * @return
	 */
	public boolean isFlat(){
		return this.m_is_flat;
	}
	
	/**
	 * Marker 坐标是否是Gps，默认为false
	 * @return
	 */
	public boolean isGps(){
		return this.m_is_gps;
	}
	
	/**
	 * 返回标记是否设置为近大远小效果
	 * @return
	 */
	public boolean isPerspective(){
		return this.m_is_perspective;
	}
	
	/**
	 * @return Marker可见性
	 */
	public boolean isVisible() {
		return this.m_is_visible;
	}
	
	/**
	 * 设置多少帧刷新一次图片资源，Marker动画的间隔时间，值越小动画越快
	 * @param period
	 * @return
	 */
	public KMarkerOptions period(int period) {
		if (period > 0) {
			this.period = period;
		}
		return this;
	}
	
	/**
	 * 设置标记的近大远小效果，在marker初始化时使用。
	 * @param perspective
	 * @return
	 */
	public KMarkerOptions perspective(boolean perspective) {
		this.m_is_perspective = perspective;
		return this;
	}
	
	/**
	 * 设置Marker的位置
	 * @param position
	 * @return
	 */
	public KMarkerOptions position(KLatLng position){
		setGps(true);
		this.m_x = (int)MapTool.doubleToLong(position.longitude);
		this.m_y = (int)MapTool.doubleToLong(position.latitude);
		this.m_is_gps = true;
		return this;
	}

	
	public KMarkerOptions setX(int x) {
		this.m_x = x;
		this.m_is_gps = false;
		return this;
	}
	
	public KMarkerOptions setY(int y) {
		this.m_y = y;
		this.m_is_gps = false;
		return this;
	}
	
	public int getX() {
		return this.m_x;
	}
	
	public int getY() {
		return this.m_y;
	}
	
	/**
	 * 设置Marker是否平贴地图
	 * @param flat
	 * @return
	 */
	public KMarkerOptions setFlat(boolean flat) {
		this.m_is_flat = flat;
		return this;
	}
	
	/**
	 * 设置marker的坐标是否是Gps，默认为false
	 * @param isGps
	 * @return
	 */
	public KMarkerOptions setGps(boolean isGps){
		this.m_is_gps = isGps;
		return this;
	}
	
	/**
	 * 设置当前标记的InfoWindow相对marker的偏移
	 * @param offsetX
	 * @param offsetY
	 * @return
	 */
	public KMarkerOptions setInfoWindowOffset(int offsetX, int offsetY){
		this.windowOffsetX = offsetX;
		this.windowOffsetY = offsetY;
		return this;
	}
	
	/**
	 * Marker上的文字描述
	 * @param snippet
	 * @return
	 */
	public KMarkerOptions snippet(java.lang.String snippet){
		this.snippet = snippet;
		return this;
	}
	
	/**
	 * 获取marker的水平偏移距离。
	 * @return 信息窗口相对原点（marker的中上点）的横向像素偏移量，默认：0
	 */
	public int getInfoWindowOffsetX() {
		return windowOffsetX;
	}

	/**
	 * 获取marker的垂直偏移距离。
	 * @return 信息窗口相对原点（marker的中上点）的纵向像素偏移量，默认：0
	 */
	public int getInfoWindowOffsetY() {
		return windowOffsetY;
	}
	
	/**
	 * 设置Marker的标题
	 * @param title
	 * @return
	 */
	public KMarkerOptions title(String title) {
		this.title = title;
		return this;
	}
	
	/**
	 * 设置Marker是否显示
	 * @param visible
	 * @return
	 */
	public KMarkerOptions visible(boolean visible) {
		this.m_is_visible = visible;
		return this;
	}
	
	
	/**
	 * 设置level。
	 * @param
	 * @return 该对象本身
	 */
	public KMarkerOptions level(int level){
		this.level = level;
		return this;
	}
	
	/**
	 * 设置zIndex
	 * @param zIndex
	 * @return
	 */
	public KMarkerOptions zIndex(float zIndex) {
		this.zIndex = zIndex;
		return this;
	}
	
	/**
	 * 返回是否支持下落动画效果。
	 * @return 是否支持下落动画效果，默认：否
	 */
	public boolean isAnimatesDrop() {
		return m_is_animatesDrop;
	}

	/**
	 * 设置是否支持下落动画效果。
	 * @param animatesDrop 是否支持下落动画效果，默认：否
	 * @return 该对象本身
	 */
	public KMarkerOptions setAnimatesDrop(boolean animatesDrop) {
		m_is_animatesDrop = animatesDrop;
		return this;
	}

	/**
	 * 是否允许点击
	 * @param clickable
	 * @return
	 */
	public KMarkerOptions clickable(boolean clickable) {
		this.bClickable  = clickable;
		return this;	
	}
	
	/**
	 * marker是否允许点击
	 * @return
	 */
	public boolean isClickable() {
		return this.bClickable;
	}
	
}
