package com.cmmap.internal.maps.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.cmmap.api.maps.model.LatLng;
import com.cmmap.api.maps.model.NavigateArrowOptions;

import android.graphics.Color;
import android.os.Parcel;
import android.os.Parcelable;

/**
 * 箭头覆盖物选项类。
 */
public class KNavigateArrowOptions{

	private int sideColor = Color.BLACK;
	private int topColor = Color.BLACK;
	private float width = 10;
	private int level = 0;
	private float zindex = 0;
	private boolean bVisible = true;
	
	private List<KLatLng> points = new ArrayList<KLatLng>();
	
	public KNavigateArrowOptions(KNavigateArrowOptions options)
	{
		this.sideColor = options.sideColor;
		this.topColor = options.topColor;
		this.width = options.width;
		this.level = options.level;
		this.zindex = options.zindex;
		this.bVisible = options.bVisible;
		for (int i = 0; i < options.points.size(); i++) {
			this.points.add(new KLatLng(options.points.get(i)));
		}
	}
	
	public KNavigateArrowOptions(NavigateArrowOptions options) {
		sideColor = options.getColor();
		topColor = options.getColor();
		width = options.getWidth();
		level = options.getLevel();
		zindex = options.getZIndex();
		bVisible = options.isVisible();
		for (LatLng point : options.getPoints()) {
			points.add(new KLatLng(point.latitude, point.longitude));
		}
	}
	
	public KNavigateArrowOptions () {
		
	}
	
	/**
	 * 返回 Options对象的侧边颜色，ARGB格式
	 * @return
	 */
	public int getSideColor() {
		return this.sideColor;
	}
	
	/**
	 * 返回 Options对象的顶颜色，ARGB格式
	 * @return
	 */
	public int getTopColor() {
		return this.topColor;
	}
	
	/**
	 * 返回箭头宽度，单位：像素
	 * @return
	 */
	public float getWidth(){
		return this.width;
	}
	
	
	/**
	 * 返回level
	 * @return 
	 */
	public int getLevel() {
		return this.level;
	}
	
	/**
	 * 返回箭头Z轴的数值
	 * @return
	 */
	public float getZIndex() {
		return this.zindex;
	}
	

	/**
	 * @return 箭头是否可见
	 */
	public boolean isVisible() {
		return this.bVisible;
	}
	
	/**
	 * 添加一批顶点到顶点坐标列表中。
	 * @param points 要添加的顶点集合
	 * @return 该对象本身
	 */
	public KNavigateArrowOptions add(KLatLng... points) {
		for (int i = 0; i < points.length; i++) {
			this.points.add(points[i]);
		}
		return this;
	}
	
	
	/**
	 * 追加一个顶点到箭头的终点
	 * @param point
	 * @return 该对象本身
	 */
	public KNavigateArrowOptions add(KLatLng  point) {
		this.points.add(point);
		return this;
	}
	
	/**
	 * 添加一批顶点到顶点坐标列表中。
	 * @param points 要添加的顶点集合
	 * @return 该对象本身
	 */
	public KNavigateArrowOptions addAll(java.lang.Iterable<KLatLng> points)  {
		this.points.clear();
		Iterator<KLatLng> iter = points.iterator(); 
		while(iter.hasNext()) { 
			KLatLng point = (KLatLng) iter.next(); 
			this.points.add(point);
		}
		return this;
	}
	
	/**
	 * 返回 Options对象的顶点坐标列表
	 * @return
	 */
	public List<KLatLng> getPoints() {
		return this.points;
	}
	/**
	 * 设置箭头的侧边颜色，需要传入32位的ARGB格式
	 * @param color
	 * @return
	 */
	public KNavigateArrowOptions sideColor(int color) {
		this.sideColor = color;
		return this;
	}
	
	/**
	 * 设置箭头的顶颜色，需要传入32位的ARGB格式
	 * @param color
	 * @return
	 */
	public KNavigateArrowOptions topColor(int color) {
		this.topColor = color;
		return this;
	}
	
	/**
	 * 设置箭头的宽度，单位像素
	 * @param width
	 * @return
	 */
	public KNavigateArrowOptions width(float width){
		this.width = width;
		return this;
	}
	
	/**
	 * 返回箭头颜色。
	 * @return 箭头颜色，ARGB格式，参见{@link android.graphics.Color}，默认：黑色
	 */
	public int getColor() {             //**************未完成**************//
		// TODO
		return 0;
	}

	/**
	 * 设置箭头颜色。
	 * @param color 箭头颜色，ARGB格式，参见{@link android.graphics.Color}
	 * @return 该对象本身
	 */
	public KNavigateArrowOptions color(int color) {	             //**************未完成**************//
		// TODO
		return null;
	}

	
	/**
	 * 设置箭头的可见性
	 * 默认为可见
	 * @param visible
	 * @return
	 */
	public KNavigateArrowOptions visible(boolean visible) {
		this.bVisible = visible;
		return this;
	}
	
	/**
	 * 设置level。
	 * @param
	 * @return 该对象本身
	 */
	public KNavigateArrowOptions level(int level){
		this.level = level;
		return this;
	}
	
	
	/**
	 * 设置箭头Z轴的值
	 * @param zIndex
	 * @return
	 */
	public KNavigateArrowOptions zIndex(float zIndex){
		this.zindex = zIndex;
		return this;
	}

	void releaseResource() {
		points.clear();
		points = null;
	}
}
