package com.cmmap.internal.maps.model;

import java.util.List;

import android.graphics.Color;

import com.cmmap.api.maps.model.LatLng;


public class KOverlayPoints extends KShapeOverlay{
	KOverlayPointsOptions m_options;
	
	public KOverlayPoints(KOverlayPointsOptions options) {
		setOverlayId(assignedId());
		m_options = new KOverlayPointsOptions(options);
		setVisible(options.isVisible());
		setLevel(options.getLevel());
		setZIndex(options.getZIndex());
	}
	
	/**
	 * 设置点的颜色
	 * @param color 颜色值
	 */
	public void setColor(int color) {
		if (m_options != null) {			
			m_options.color(color);
			firePropertyChange(UPDATE_ALL);
		}
	}
	
	/**
	 * 设置点的半径
	 * @param radius 半径
	 */
	public void setRadius(float radius) {
		if (m_options != null) {			
			m_options.radius(radius);
			firePropertyChange(UPDATE_ALL);
		}
	}
	
	/**
	 * 添加点
	 * @param point 点的经纬度
	 */
	public void addPoint(LatLng point) {
		if (m_options != null) {			
			m_options.addPoint(point);
			firePropertyChange(UPDATE_ADD_POINT);
		}
	}
	
	/**
	 * 获取点颜色
	 * @return 点颜色
	 */
	public int getColor() {
		if (m_options != null) {			
			return m_options.getColor();
		}
		return KOverlayPointsOptions.DEFAULT_COLOR;
	}
	
	/**
	 * 获取点半径
	 * @return 点半径
	 */
	public float getRadius() {
		if (m_options != null) {
			return m_options.getRadius();
		}
		return KOverlayPointsOptions.DEFAULT_RADIUS;
	}
	
	/**
	 * 获取点集合
	 * @return 点集合
	 */
	public List<LatLng> getPoints() {
		if (m_options != null) {
			return m_options.getPoints();
		}
		return null;
	}
	
	/**
	 * 获取圆的ID
	 * @return
	 */
	public String getId() {
		return String.valueOf(getOverlayId());
	}
	
	/**
	 * 移除
	 */
	public void remove() {
		if (m_options != null) {
			m_options.clear();
			m_options = null;
		}
		super.remove();
	}
}
