package com.cmmap.internal.maps.model;

import java.util.ArrayList;
import java.util.List;

import com.cmmap.api.maps.model.LatLng;

import android.graphics.Color;

public class KOverlayPointsOptions {
	static final int DEFAULT_COLOR = Color.BLUE;
	static final float DEFAULT_RADIUS = 5;
	/**
	 * 颜色
	 */
	int color = DEFAULT_COLOR;
	/**
	 * 半径
	 */
	float radius = DEFAULT_RADIUS;
	/**
	 * 点集合
	 */
	List<LatLng> points  = new ArrayList<LatLng>();
	
	private int level = 0;
	/**
	 * Z轴的值
	 */
	private float zIndex = 0;
	/**
	 * 是否可见
	 */
	private boolean bVisible = true;
	
	public KOverlayPointsOptions() {
		
	}
	
	public KOverlayPointsOptions(KOverlayPointsOptions options) {
		color = options.getColor();
		radius = options.getRadius();
		points.addAll(options.getPoints());
		level = options.getLevel();
		zIndex = options.getZIndex();
		bVisible = options.isVisible();
	}

	/**
	 * 设置点颜色
	 * @param color 颜色值
	 * @return 该对象本身
	 */
	public KOverlayPointsOptions color(int color) {
		this.color = color;
		return this;
	}
	
	/**
	 * 设置点半径
	 * @param radius 半径
	 * @return 该对象本身
	 */
	public KOverlayPointsOptions radius(float radius) {
		this.radius = radius;
		return this;
	}
	
	/**
	 * 设置点集合
	 * @param lstPoint 点集合
	 * @return 该对象本身
	 */
	public KOverlayPointsOptions points(List<LatLng> lstPoint) {
		points.addAll(lstPoint);
		return this;
	}
	
	/**
	 * 设置level。
	 * @param
	 * @return 该对象本身
	 */
	public KOverlayPointsOptions level(int level){
		this.level = level;
		return this;
	}
	
	
	/**
	 * 设置z轴序列
	 * @param zIndex z轴序列
	 * @return 对象本身
	 */
	public KOverlayPointsOptions zIndex(float zIndex) {
		this.zIndex = zIndex;
		return this;
	}
	
	/**
	 * 设置是否可见
	 * @param visible 是否可见
	 * @return 对象本身
	 */
	public KOverlayPointsOptions visible(boolean visible) {
		this.bVisible = visible;
		return this;
	}
	
	/**
	 * 获取颜色值
	 * @return 颜色值，默认蓝色
	 */
	public int getColor() {
		return color;
	}
	
	/**
	 * 获取半径
	 * @return 半径，默认5
	 */
	public float getRadius() {
		return radius;
	}
	
	/**
	 * 获取点集合
	 * @return 点集合
	 */
	public List<LatLng> getPoints() {
		return points;
	}

	/**
	 * 获取是否可见
	 * @return 是否可见
	 */
	public boolean isVisible() {
		return bVisible;
	}
	
	/**
	 * 返回level
	 * @return 
	 */
	public int getLevel() {
		return this.level;
	}

	/**
	 * 获取Z轴序列
	 * @return Z轴序列
	 */
	public float getZIndex() {
		return zIndex;
	}

	/**
	 * 增加一个点
	 * @param point 点的经纬度
	 */
	void addPoint(LatLng point) {
		points.add(point);
	}

	/**
	 * 清空
	 */
	void clear() {
		points.clear();
	}
}
