package com.cmmap.internal.maps.model;

import java.util.concurrent.atomic.AtomicInteger;

import com.cmmap.api.maps.model.Event;
import com.cmmap.api.maps.model.Poi;


/**
 * 地图底图POI点类。 
 * <p>地图底图POI点包含如下属性：</p>
 * <ul>
 * <li>名称<br>POI点在底图上显示的名称。</li>
 * <li>坐标<br>POI点在底图上的经纬度坐标。</li>
 * <li>ID<br>POI点唯一标识。</li>
 * </ul>
 */
public class KPoi{
	/** 地图POI点的坐标 */
	private KLatLng coordinate;
	
	private String name;
	
	private String info;
	
	private boolean bEvent;
	
	private static final AtomicInteger poiIdAssigner = new AtomicInteger();
	/** 地图POI点ID */
	public  String poiid;
	
	private int icon_type = 0;
	
	public KPoi(){
		poiid = ((Integer)poiIdAssigner.decrementAndGet()).toString();
	}
	
	public KPoi(KPoi poi){
		this(poi.name, poi.coordinate, poi.poiid);
	}
	/**
	 * 构造函数
	 * @param name 地图POI点的名称
	 * @param coordinate 地图POI点的坐标
	 * @param poiid 地图POI点ID
	 */
	public KPoi(String name, KLatLng coordinate, String poiid) {
		this.name = name;
		this.coordinate = new KLatLng(coordinate);
		this.poiid = poiid;
	}
	
	public KPoi(KLatLng coordnate, String name) {
		this.coordinate = new KLatLng(coordnate);
		this.name = name;
		this.poiid = ((Integer)poiIdAssigner.decrementAndGet()).toString();
	}
	
	/**
	 * @return 获取POI坐标
	 */
	public KLatLng getCoordinate()
	{
		return coordinate;
	}
	
	/**
	 * 获取POI名称
	 * @return
	 */
	public String getName()
	{
		return name;
	}
	
	/**
	 * 获取POI详细信息
	 * @return
	 */
	public String getInfo()
	{
		return info;
	}
	
	/**
	 * 获取POI是否为事件
	 * @return
	 */
	public boolean getIsEvent()
	{
		return bEvent;
	}
	
	/**
	 * 设置坐标
	 * @param latLng
	 */
	public void SetCoordnate(KLatLng latLng){
		this.coordinate = latLng;
	}
	
	/**
	 * 设置名称
	 * @param strName
	 */
	public void SetName(String strName) {
		this.name = strName;
	}
	
	/**
	 * 设置名称
	 * @param strName
	 */
	public void SetInfo(String strInfo) {
		this.info = strInfo;
	}
	
	/**
	 * 设置名称
	 * @param strName
	 */
	public void SetIsEvent(boolean bEvent) {
		this.bEvent = bEvent;
	}
	
	/**
     * 设置图标类型（路况事件专用）
     * @param type
     */
    public void setIconType(int type) {
        this.icon_type = type;
    }

	public Poi toPoi(){
		return new Poi(name, coordinate.toLatLng(), poiid);
	};
	
	public Event toEvent(){
		return new Event(name, info, coordinate.toLatLng(), poiid, icon_type);
	};
	
	public KPoi(Poi poi) {
		this(poi.name, new KLatLng(poi.coordinate.latitude, poi.coordinate.longitude), poi.poiid);
	}
}
