package com.cmmap.internal.maps.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.cmmap.api.maps.model.LatLng;
import com.cmmap.api.maps.model.PolygonOptions;

import android.graphics.Color;
import android.os.Parcel;
import android.os.Parcelable;

/**
 * 多边形覆盖物选项类
 */
public class KPolygonOptions{
	
	int fillColor = Color.BLACK;
	List<KLatLng> latLngs = new ArrayList<KLatLng>();
	int strokeColor = Color.BLACK;
	float strokeWidth = 10;
	boolean draggable = false;
	int level = 0;
	float zIndex;
	boolean isVisible = true;
	
	public KPolygonOptions(KPolygonOptions options)
	{
		this.strokeColor = options.strokeColor;
		this.strokeWidth = options.strokeWidth;
		this.level = options.level;
		this.zIndex = options.zIndex;
		this.isVisible = options.isVisible;
		this.fillColor = options.fillColor;
		this.draggable = options.draggable;
		for (int i = 0; i < options.latLngs.size(); i++) {
			this.latLngs.add(new KLatLng(options.latLngs.get(i))) ;
		}
	}
	
	public KPolygonOptions(PolygonOptions options) {
		strokeColor = options.getStrokeColor();
		strokeWidth = options.getStrokeWidth();
		level = options.getLevel();
		zIndex = options.getZIndex();
		isVisible = options.isVisible();
		fillColor = options.getFillColor();
		draggable = options.isDraggable();
		for (LatLng point : options.getPoints()) {
			latLngs.add(new KLatLng(point.latitude, point.longitude));
		}
	}
	
	public KPolygonOptions () {
		
	}
	
	/**
	 * 添加一批顶点到顶点坐标列表中。
	 * @param points 要添加的顶点集合
	 * @return 该对象本身
	 */
	public KPolygonOptions add(KLatLng... paramArrayOfLatLng)  {
		for (int i = 0; i < paramArrayOfLatLng.length; i++) {
			this.latLngs.add(paramArrayOfLatLng[i]);
		}
		return this;
	}
	
	/**
	 * 添加一个多边形边框的顶点
	 * @param paramLatLng 要添加的顶点
	 * @return 该对象本身
	 */
	public KPolygonOptions add(KLatLng paramLatLng)  {
		this.latLngs.add(paramLatLng);
		return this;
	}
	
	/**
	 * 添加一批顶点到顶点坐标列表中。
	 * @param points 要添加的顶点集合
	 * @return 该对象本身
	 */
	public KPolygonOptions  addAll(java.lang.Iterable<KLatLng> paramIterable)  {
		latLngs.clear();
		Iterator<KLatLng> iter = paramIterable.iterator(); 
		while(iter.hasNext()) { 
			KLatLng point = (KLatLng) iter.next(); 
			this.latLngs.add(point);
		}
		return this;
	}
	
	/**
	 * 设置多边形的填充颜色，32位ARGB格式，默认为黑色
	 * @param color
	 * @return
	 */
	public KPolygonOptions fillColor(int color)   {
		this.fillColor = color;
		return this;
	}
	
	/**
	 * 返回多边形填充颜色
	 * @return
	 */
	public int getFillColor()   {
		return fillColor;
	}
	
	/**
	 * @return 多边形顶点坐标列表
	 */
	public List<KLatLng> getPoints()  {
		return this.latLngs;
	}
	
	/**
	 * @return 边框颜色
	 */
	public int getStrokeColor() {
		return this.strokeColor;
	}
	
	/**
	 * 获取边框宽度
	 * @return
	 */
	public float getStrokeWidth()  {
		return this.strokeWidth;
	}
	
	/**
	 * 返回level
	 * @return 
	 */
	public int getLevel() {
		return this.level;
	}
	
	public float getZIndex()  {
		return zIndex;
	}
	
	/**
	 * @return 返回多边形是否可见
	 */
	public boolean isVisible()  {
		return isVisible;
	}
	
	/**
	 * 设置多边形边框颜色，32位 ARGB格式，默认为黑色
	 * @param color
	 * @return
	 */
	public KPolygonOptions strokeColor(int color)  {
		this.strokeColor = color;
		return this;
		
	}
	
	
	/**
	 * 设置多边形边框宽度，单位：像素。默认为10
	 * @param paramFloat
	 * @return
	 */
	public KPolygonOptions strokeWidth(float paramFloat)  {
		this.strokeWidth = paramFloat;
		return this;
	}
	
	/**
	 * 返回是否可拖拽。
	 * @return 是否可拖拽，默认：否
	 */
	public boolean isDraggable() {
		return draggable;
	}

	/**
	 * 设置是否可拖拽。
	 * @param enable 是否可拖拽，默认：否
	 * @return 该对象本身
	 */
	public KPolygonOptions draggable(boolean enable) {
		draggable = enable;
		return this;
	}
	
	/**
	 * 设置多边形的是否可见。默认为可见
	 * @param visible
	 * @return
	 */
	public KPolygonOptions visible(boolean visible)  {
		this.isVisible = visible;
		return this;
	}
	
	
	/**
	 * 设置level。
	 * @param
	 * @return 该对象本身
	 */
	public KPolygonOptions level(int level){
		this.level = level;
		return this;
	}
	
	/**
	 * 设置Z轴数值
	 * @param zIndex
	 * @return
	 */
	public KPolygonOptions zIndex(float zIndex)  {
		this.zIndex = zIndex;
		return this;
	}

	void releaseResource() {
		latLngs.clear();
		latLngs = null;
	}
}
