package com.cmmap.internal.maps.model;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import android.graphics.Color;

import com.cmmap.api.maps.model.BitmapDescriptor;
import com.cmmap.api.maps.model.LatLng;
import com.cmmap.api.maps.model.PolylineOptions;


public class KPolylineOptions{

	private List<KLatLng> points = new ArrayList<KLatLng>();
	
	private int color = Color.WHITE; //颜色
	
	private boolean geodesic = false; //是否大地曲线
	
	private BitmapDescriptor bitmapDescriptor;
	
	private float width = 10;//宽度
	
	private int level = 0;
	
	private float zIndex;
	
	private boolean bDotLine = false; //是否虚线
	
	private boolean bUseTexture;
	
	private boolean bVisible = true;
	
	private boolean draggable = false;
	
	private boolean bLineNameVisible = false;
	
	private String m_LineName = "";
	
	private int m_name_color = Color.BLACK;
	
	private int m_name_edge_color = Color.WHITE;
	
	private int m_name_font_size = 22;
	
	
    private List<Integer> m_lst_colors = new ArrayList<Integer>();
    
    private List<BitmapDescriptor> m_lst_bitmapDescriptors = new ArrayList<BitmapDescriptor>();
    
    private List<Integer> m_lst_texture_index = new ArrayList<Integer>();
    
    private List<String> m_lst_line_name = new ArrayList<String>();
    
    private List<Integer> m_lst_name_index = new ArrayList<Integer>();
	
	
	public KPolylineOptions() {
		this.color = Color.WHITE;
		this.width = 1;
		this.geodesic = false;
		this.bitmapDescriptor = null;
		this.level = 0;
		this.zIndex = 0;
		this.bVisible = true;
		this.bDotLine = false;
		this.bUseTexture = false;
		this.bLineNameVisible = false;
		this.m_LineName = "";
		this.m_name_color = Color.BLACK;
		this.m_name_edge_color = Color.WHITE;
		this.m_name_font_size = 22;
		this.m_lst_colors.clear();
		this.m_lst_bitmapDescriptors.clear();
		this.m_lst_texture_index.clear();
		this.m_lst_line_name.clear();
		this.m_lst_name_index.clear();
	}
		
	public KPolylineOptions (KPolylineOptions options) {
		this.color = options.color;
		this.width = options.width;
		this.geodesic = options.geodesic;
		this.bitmapDescriptor = options.bitmapDescriptor;
		this.level = options.level;
		this.zIndex = options.zIndex;
		this.bVisible = options.bVisible;
		this.bDotLine = options.bDotLine;
		this.bUseTexture = options.bUseTexture;
		this.draggable = options.draggable;
		this.bLineNameVisible = options.bLineNameVisible;
		this.m_LineName = options.m_LineName;
		this.m_name_color = options.m_name_color;
		this.m_name_edge_color = options.m_name_edge_color;
		this.m_name_font_size = options.m_name_font_size;
		
		points.clear();
		for (KLatLng point : options.getPoints()) {
			points.add(new KLatLng(point.latitude, point.longitude));
		}
		
		m_lst_colors.clear();
		for (Integer color : options.getColorValues()) {
			m_lst_colors.add(color);
		}
		
		m_lst_bitmapDescriptors.clear();
		for (BitmapDescriptor bitmap : options.getCustomTextureList()) {
			m_lst_bitmapDescriptors.add(bitmap);
		}
		
		m_lst_texture_index.clear();
		for (Integer index : options.getCustomTextureIndex()) {
			m_lst_texture_index.add(index);
		}
		
		m_lst_line_name.clear();
		for (String name : options.getLineNameList()) {
			m_lst_line_name.add(name);
		}
		
		m_lst_name_index.clear();
		for (Integer index : options.getLineNameIndex()) {
			m_lst_name_index.add(index);
		}
	}
	
	public KPolylineOptions (PolylineOptions options) {
		color = options.getColor();
		width = options.getWidth();
		geodesic = options.isGeodesic();
		bitmapDescriptor = options.getCustomTexture();
		level = options.getLevel();
		zIndex = options.getZIndex();
		bVisible = options.isVisible();
		bDotLine = options.isDottedLine();
		bUseTexture = options.isUseTexture();
		draggable = options.isDraggable();
		bLineNameVisible = options.getLineNameVisible();
		m_LineName = options.getLineName();
		m_name_color = options.getNameColor();
		m_name_edge_color = options.getNameEdgeColor();
		m_name_font_size = options.getNameFontSize();
		
		points.clear();
		for (LatLng point : options.getPoints()) {
			points.add(new KLatLng(point.latitude, point.longitude));
		}
		
		m_lst_colors.clear();
		for (Integer color : options.getColorValues()) {
			m_lst_colors.add(color);
		}
		
		m_lst_bitmapDescriptors.clear();
		for (BitmapDescriptor bitmap : options.getCustomTextureList()) {
			m_lst_bitmapDescriptors.add(bitmap);
		}
		
		m_lst_texture_index.clear();
		for (Integer index : options.getCustomTextureIndex()) {
			m_lst_texture_index.add(index);
		}
		
		m_lst_line_name.clear();
		for (String name : options.getLineNameList()) {
			m_lst_line_name.add(name);
		}
		
		m_lst_name_index.clear();
		for (Integer index : options.getLineNameIndex()) {
			m_lst_name_index.add(index);
		}
	}
	
	/**
	 * 追加一批顶点到线段终点
	 * @param points
	 * @return
	 */
	public KPolylineOptions add(KLatLng... points) {
		for (int i = 0; i < points.length; i++) {
			this.points.add(points[i]);
		}
		return this;
	}
	
	
	/**
	 * 追加一批顶点到线段终点
	 * @param point
	 * @return
	 */
	public KPolylineOptions add(KLatLng  point) {
		this.points.add(point);
		return this;
	}
	
	/**
	 * 追加一批顶点到线段终点
	 * @param points
	 * @return
	 */
	public KPolylineOptions addAll(java.lang.Iterable<KLatLng> points)  {
		this.points.clear();
		Iterator<KLatLng> iter = points.iterator(); 
		while(iter.hasNext()) { 
			KLatLng point = (KLatLng) iter.next(); 
			this.points.add(point);
		}
		return this;
	}
	
	
	/**
	 * 设置线段的颜色，需要传入32位的ARGB格式
	 * @param color
	 * @return
	 */
	public KPolylineOptions color(int color)  {
		this.color = color;
		return this;
	}
	
	/**
	 * 设置线段是否是大地曲线
	 * @param geodesic
	 * @return
	 */
	public KPolylineOptions geodesic(boolean geodesic) {
		this.geodesic = geodesic;
		return this;
	}
	
	/**
	 * 获取线段颜色
	 * @return
	 */
	public int getColor()  {
		return this.color;
	}
	
	/**
	 * 返回线段的纹理图
	 * @return
	 */
	public BitmapDescriptor getCustomTexture() {
		return this.bitmapDescriptor;
	}
	
	void ReleaseResource() {
		bitmapDescriptor = null;
		
		if (points != null) {			
			points.clear();
			points = null;
		}
		
		if (m_lst_colors != null) {
			m_lst_colors.clear();
			m_lst_colors = null;
		}
		
		if (m_lst_bitmapDescriptors != null) {
			m_lst_bitmapDescriptors.clear();
			m_lst_bitmapDescriptors = null;
		}
		
		if (m_lst_texture_index != null) {
			m_lst_texture_index.clear();
			m_lst_texture_index = null;
		}
		
		if (m_lst_line_name != null) {
			m_lst_line_name.clear();
			m_lst_line_name = null;
		}
		
		if (m_lst_name_index != null) {
			m_lst_name_index.clear();
			m_lst_name_index = null;
		}
	}
	
	/**
	 * 返回Options对象的顶点坐标列表
	 * @return
	 */
	public List<KLatLng> getPoints() {
		return this.points;
	}
	
	/**
	 * 线段宽度
	 * @return
	 */
	public float getWidth()  {
		return this.width;
	}
	
	/**
	 * 返回level
	 * @return 
	 */
	public int getLevel() {
		return this.level;
	}
	
	
	public float getZIndex()  {
		return this.zIndex;
	}
	
	/**
	 * 是否画虚线，默认为false，画实线
	 * @return
	 */
	public boolean isDottedLine()  {
		return this.bDotLine;
	}
	
	/**
	 * 是否大地曲线
	 * @return
	 */
	public boolean isGeodesic()  {
		return this.geodesic;
				
	}
	
	/**
	 * 是否使用纹理
	 * @return
	 */
	public boolean isUseTexture()  {
		return bUseTexture;
	}
	
	public boolean isVisible()  {
		return bVisible;
	}
	
	/**
	 * 设置线段的纹理图，图片长宽为2的n次方
	 * @param customTexture
	 * @return
	 */
	public KPolylineOptions setCustomTexture(BitmapDescriptor customTexture) {
		this.bUseTexture = true;
		this.bitmapDescriptor = customTexture;
		return this;
	}
	
	/**
	 * 设置是否画虚线，默认为false，画实线
	 * @param isDottedLine
	 * @return
	 */
	public KPolylineOptions setDottedLine(boolean isDottedLine) {
		this.bDotLine = isDottedLine;
		return this;
	}
	
	/**
	 * 设置是否使用纹理贴图画线
	 * @param useTexture
	 * @return
	 */
	public KPolylineOptions setUseTexture(boolean useTexture) {
		bUseTexture = useTexture;
		return this;
	}
	
	/**
	 * 设置线段的可见性
	 * @param isVisible
	 * @return
	 */
	public KPolylineOptions visible(boolean isVisible) {
		this.bVisible = isVisible;
		return this;
	}
	
	/**
	 * 设置线段的宽度，单位像素
	 * @param width
	 * @return
	 */
	public KPolylineOptions  width(float width) {
		this.width = width;
		return this;
	}
	
	/**
	 * 设置level。
	 * @param
	 * @return 该对象本身
	 */
	public KPolylineOptions level(int level){
		this.level = level;
		return this;
	}
	
	public KPolylineOptions zIndex(float zIndex) {
		this.zIndex = zIndex;
		return this;
	}
	
	/**
	 * 返回是否可拖拽。
	 * @return 是否可拖拽，默认：否
	 */
	public boolean isDraggable() {
		return draggable;
	}

	/**
	 * 设置是否可拖拽。
	 * @param enable 是否可拖拽，默认：否
	 * @return 该对象本身
	 */
	public KPolylineOptions draggable(boolean enable) {
		draggable = enable;
		return this;
	}
	
	/**
	 * 设置名称。
	 * @param String 名称
	 */
	public KPolylineOptions setlineName(String lineName) {
		m_LineName = lineName;
		return this;
	}
	
	/**
	 * 返回名称。
	 * @param String 名称
	 */
	public String getlineName() {
		return m_LineName;
	}
	
	/**
	 * 设置名称是否可见。
	 * @param visible 是否可见
	 */
	public KPolylineOptions setLineNameVisible(boolean visible) {
		this.bLineNameVisible = visible;
		return this;
	}
	
	/**
	 * 返回名称是否可见。
	 * @return visible 是否可见
	 */
	public boolean getLineNameVisible() {
		return this.bLineNameVisible;
	}
	
	/**
	 * 返回名称前景色。
	 * @return 颜色，默认：黑色
	 */
	public int getNameColor() {
		return m_name_color;
	}
	
	/**
	 * 设置名称前景色。
	 * @param 颜色，默认：黑色
	 * @return 该对象本身
	 */
	public KPolylineOptions nameColor(int color) {
		m_name_color = color;
		return this;
	}
	
	/**
	 * 返回名称边缘色。
	 * @return 颜色，默认：不色
	 */
	public int getNameEdgeColor() {
		return m_name_edge_color;
	}
	
	/**
	 * 设置名称背景色。
	 * @param 颜色，默认：黑色
	 * @return 该对象本身
	 */
	public KPolylineOptions nameEdgeColor(int color) {
		m_name_edge_color = color;
		return this;
	}
	
	/**
	 * 返回名称大小。
	 * @return 字体大小，默认：22
	 */
	public int getNameFontSize() {
		return m_name_font_size;
	}
	
	/**
	 * 设置名称大小。
	 * @param 字体大小，默认：22
	 * @return 该对象本身
	 */
	public KPolylineOptions nameFontSize(int size) {
		m_name_font_size = size;
		return this;
	}
	
	/**
	 * 获取线段的颜色列表
	 * @return 颜色列表
	 */
	public List<Integer> getColorValues() {
	    return m_lst_colors;
	}
	
	/**
	 * 获取线段纹理列表
	 * @return 纹理列表
	 */
	public List<BitmapDescriptor> getCustomTextureList(){
        return m_lst_bitmapDescriptors;
    }
	
	/**
	 * 获取线段纹理index列表
	 * @return 分段纹理index列表
	 */
	public List<java.lang.Integer> getCustomTextureIndex(){
        return m_lst_texture_index;
    }
	
	/**
	 * 获取线段名称列表
	 * @return 名称列表
	 */
	public List<String> getLineNameList(){
        return m_lst_line_name;
    }
	
	/**
	 * 获取线段名称index列表
	 * @return 分段名称index列表
	 */
	public List<java.lang.Integer> getLineNameIndex(){
        return m_lst_name_index;
    }
	
	/**
	 * 设置线段的颜色
	 * @param colors 颜色列表
	 * @return PolylineOptions 对象
	 */
	public KPolylineOptions colorValues(List<Integer> colors) {
		m_lst_colors = colors;
	    return this;
	}
	
	/**
	 * 设置线段纹理list
	 * @param bitmapDescriptors 纹理列表
	 * @return PolylineOptions 对象
	 */
	public KPolylineOptions setCustomTextureList(List<BitmapDescriptor> bitmapDescriptors) {
        this.m_lst_bitmapDescriptors = bitmapDescriptors;
        return this;
    }
	
	/**
	 * 设置线段纹理index数组
	 * @param integers 每一段对应的纹理，用纹理列表的index来对应
	 * @return PolylineOptions 对象
	 */
	public KPolylineOptions setCustomTextureIndex(List<Integer> integers) {
        this.m_lst_texture_index = integers;
        return this;
    }
	
	/**
	 * 设置线段名称list
	 * @param bitmapDescriptors 名称列表
	 * @return PolylineOptions 对象
	 */
	public KPolylineOptions setLineNameList(List<String> lineNames) {
        this.m_lst_line_name = lineNames;
        return this;
    }
	
	/**
	 * 设置线段名称index数组
	 * @param integers 每一段对应的名称，用名称列表的index来对应
	 * @return PolylineOptions 对象
	 */
	public KPolylineOptions setLineNameIndex(List<Integer> integers) {
        this.m_lst_name_index = integers;
        return this;
    }
}
