package com.cmmap.internal.maps.model;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.concurrent.BlockingQueue;

import android.annotation.SuppressLint;

import com.cmmap.api.maps.model.BitmapDescriptor;
import com.cmmap.api.maps.model.LatLngBounds;
import com.cmmap.internal.driver.base.MapTool;
import com.cmmap.internal.mapcore.KGEOCOORD;
import com.cmmap.internal.mapcore.KRECOLOR;
import com.cmmap.internal.mapcore.KRect;
import com.cmmap.internal.mapcore.Lg;
import com.cmmap.internal.mapcore.MapOverLayShape;
import com.cmmap.internal.mapcore.MapOverLayShape.NE_MapOverLayShape_Type;
import com.cmmap.internal.mapcore.MapOverLayShapeArc;
import com.cmmap.internal.mapcore.MapOverLayShapeCircle;
import com.cmmap.internal.mapcore.MapOverLayShapeGround;
import com.cmmap.internal.mapcore.MapOverLayShapeMarker;
import com.cmmap.internal.mapcore.MapOverLayShapeNavigateArrow;
import com.cmmap.internal.mapcore.MapOverLayShapePoints;
import com.cmmap.internal.mapcore.MapOverLayShapePolyLine;
import com.cmmap.internal.mapcore.MapOverLayShapePolygon;
import com.cmmap.internal.mapcore.MapOverlayBitmap;
import com.cmmap.internal.mapcore.MapOverlayShapeTile;

public class KShapeOverlayFactory {

	/**
	 * 创建Circle、Polyline、Polygon对应的MapOverLayShape
	 * @param overlay
	 * @return
	 */
	public static MapOverLayShape createShapeOverlay(KShapeOverlay overlay) {
		if (overlay == null) {
			return null;
		}
		
		if (overlay instanceof KCircle) {
			return createMapCircle((KCircle) overlay);
		} else if (overlay instanceof KPolyline) {
			return createMapPolyLine((KPolyline) overlay);
		} else if (overlay instanceof KPolygon) {
			return createMapPolygon((KPolygon) overlay);
		} else if (overlay instanceof KArc) {
			return createMapArc((KArc) overlay);
		}else if (overlay instanceof KGroundOverlay) {
			return createMapGround((KGroundOverlay) overlay);
		}else if(overlay instanceof KNavigateArrow) {
			return createNavigateArrow((KNavigateArrow) overlay);
		}else if(overlay instanceof KMarker) {
			return createMarker((KMarker) overlay);
		} else if (overlay instanceof KTileOverlay) {
			return createTile((KTileOverlay) overlay);
		} else if (overlay instanceof KOverlayPoints) {
			return createOverlayPoints((KOverlayPoints) overlay);
		}
		return null;
	}
	
	private static MapOverLayShape createOverlayPoints(KOverlayPoints overlay) {
		MapOverLayShapePoints overlayPoints = new MapOverLayShapePoints();
		// [internal] 父类的属性设置
		overlayPoints.m_i_shapid = overlay.getOverlayId();
		overlayPoints.m_i_level = overlay.getLevel();
		overlayPoints.m_i_zindex = overlay.getZIndex();	
		overlayPoints.m_b_visible = overlay.isVisible();
		overlayPoints.m_locations.addAll(overlay.getPoints());

		KGEOCOORD k_coord = new KGEOCOORD();
		// 最后一个点为新增点
		int iLastLocationIndex = overlayPoints.m_locations.size() - 1;
		if (iLastLocationIndex >= 0) {
			k_coord.lLatitude = MapTool.doubleToLong(overlayPoints.m_locations.get(iLastLocationIndex).latitude);
			k_coord.lLongitude = MapTool.doubleToLong(overlayPoints.m_locations.get(iLastLocationIndex).longitude);
			overlayPoints.m_location = k_coord;
		}
		overlayPoints.m_radius = (float) overlay.getRadius();
		overlayPoints.m_color = parseColor(overlay.getColor());
		
		return overlayPoints;
	}

	private static MapOverLayShape createTile(KTileOverlay tile) {
		if (tile == null) {
			return null;
		}
		
		MapOverlayShapeTile shapeTile = new MapOverlayShapeTile(tile.getOverlayId(), tile.m_Options);
		shapeTile.m_b_visible = tile.isVisible();
		shapeTile.m_i_level = tile.getLevel();
		shapeTile.m_i_zindex = tile.getZIndex();
		return shapeTile;
	}
	
	private static MapOverLayShape createMarker(KMarker marker) {
		if (marker == null) {
			return null;
		}
		
		MapOverLayShapeMarker m_io_marker = new MapOverLayShapeMarker(marker.getOverlayId(),
				marker.getMarkerOptions(), marker.getBitmapIndex(), marker.getRotateAngle());
		m_io_marker.m_b_visible = marker.isVisible();
		m_io_marker.m_i_level = marker.getLevel();
		m_io_marker.m_i_zindex = marker.getZIndex();
		
		return m_io_marker;
	}

	// 创建jni箭头的实体类
	private static MapOverLayShape createNavigateArrow(KNavigateArrow arrow) {
		MapOverLayShapeNavigateArrow m_arrow = new MapOverLayShapeNavigateArrow();
		m_arrow.m_i_shapid = arrow.getOverlayId();
		m_arrow.m_i_level = arrow.getLevel();
		m_arrow.m_i_zindex = arrow.getZIndex();
		m_arrow.m_b_visible = arrow.isVisible();
		m_arrow.m_v_points  = latlonToKCoord(arrow.getPoints());
		m_arrow.m_f_width = arrow.getWidth();
		m_arrow.m_sideColor = parseColor(arrow.getSideColor());
		m_arrow.m_topColor = parseColor(arrow.getTopColor());
		
		return m_arrow;
	}

	// [internal] 创建jni圆的实体类
	private static MapOverLayShapeCircle createMapCircle(KCircle circle) {
		MapOverLayShapeCircle m_circle = new MapOverLayShapeCircle();
		// [internal] 父类的属性设置
		m_circle.m_i_shapid = circle.getOverlayId();
		m_circle.m_i_level = circle.getLevel();
		m_circle.m_i_zindex = circle.getZIndex();
		m_circle.m_b_visible = circle.isVisible();

		KGEOCOORD k_coord = new KGEOCOORD();
		k_coord.lLatitude = MapTool.doubleToLong(circle.getCenter().latitude);
		k_coord.lLongitude = MapTool.doubleToLong(circle.getCenter().longitude);
		m_circle.m_location = k_coord;
		m_circle.m_radius = (float) circle.getRadius();
		m_circle.m_border_width = circle.getStrokeWidth();
		m_circle.m_border_color = parseColor(circle.getStrokeColor());
		m_circle.m_color = parseColor(circle.getFillColor());
		
		m_circle.m_is_dot_line = circle.isDottedLine();
		return m_circle;
	}

	private static MapOverLayShapePolyLine createMapPolyLine(KPolyline polyline) {
		MapOverLayShapePolyLine m_polyline = new MapOverLayShapePolyLine();
		m_polyline.m_i_shapid = polyline.getOverlayId();
		m_polyline.m_i_level = polyline.getLevel();
		m_polyline.m_i_zindex = polyline.getZIndex();
		m_polyline.m_b_visible = polyline.isVisible();
		m_polyline.m_options = new KPolylineOptions(polyline.getPolyLineOptions());
		return m_polyline;
	}

	private static MapOverLayShapePolygon createMapPolygon(KPolygon polygon) {
		MapOverLayShapePolygon m_polygon = new MapOverLayShapePolygon();
		// [internal] 父类属性设置
		m_polygon.m_i_shapid = polygon.getOverlayId();
		m_polygon.m_i_level = polygon.getLevel();
		m_polygon.m_i_zindex = polygon.getZIndex();
		m_polygon.m_b_visible = polygon.isVisible();

		m_polygon.m_color = parseColor(polygon.getFillColor()); // [internal] 颜色
		m_polygon.m_v_points = latlonToKCoord(polygon.getPoints());// [internal] 面的坐标数组
		m_polygon.m_borderWidth = polygon.getStrokeWidth(); // [internal] 边线的宽度
		m_polygon.m_borderColor = parseColor(polygon.getStrokeColor()); // [internal] 边线的颜色
//		m_polygon.m_cap = transformCap(polygon.getCap());
//		m_polygon.m_join = transformJoin(polygon.getJoin()); // [internal] 线的连接点
		return m_polygon;
	}

	@SuppressLint("NewApi")
	private static MapOverLayShapeGround createMapGround(KGroundOverlay groundOverlay) {
		if (groundOverlay == null) {
			return null;
		}
		//long l_memory = Runtime.getRuntime().freeMemory();
		//System.out.println("memory:" + l_memory);
		//Bitmap bitmap1 = groundOverlay.getImage().getBitmap();
		// if(bitmap1.getWidth() > 512 || bitmap1.getHeight() > 512){
		// Lg.e("createMapPoint", "out of width or height");
		// return null;
		// }
		//if (l_memory < bitmap1.getByteCount()) {
		//	Lg.e("createMapPoint", "low memory");
		//	return null;
		//}
		//Bitmap bitmap = bitmap1.copy(Bitmap.Config.ARGB_8888, true);
		//ByteBuffer buffer = ByteBuffer.allocate(bitmap.getByteCount());
		//Lg.e("zhuliang", "=================== ShapeOverlayFactory createMapPoint buffer.capacity():"+buffer.capacity());
		//bitmap.copyPixelsToBuffer(buffer);
		
		MapOverLayShapeGround m_point = new MapOverLayShapeGround(groundOverlay.getImage());
		
		// [internal] 父类属性设置
		m_point.m_i_shapid = groundOverlay.getOverlayId();
		m_point.m_i_level = groundOverlay.getLevel();
		m_point.m_i_zindex = groundOverlay.getZIndex();
		m_point.m_b_visible = groundOverlay.isVisible();

		// [internal] 位置
		KGroundOverlayOptions opt = groundOverlay.getOptions(); 
		KLatLng loc = opt.getLocation();
		if (loc != null)
		{
			m_point.m_location = new KGEOCOORD(MapTool.doubleToLong(loc.longitude),
					MapTool.doubleToLong(loc.latitude));
			m_point.m_width = MapTool.doubleToLong(KLatLng.meterToGeoUnit(loc, opt.getWidth(), true));
			m_point.m_height = MapTool.doubleToLong(KLatLng.meterToGeoUnit(loc, opt.getHeight(), false));
		}else
		{
			m_point.m_location = null;
			m_point.m_width = 0;
			m_point.m_height = 0;
		}
		
		m_point.m_bearing = groundOverlay.getBearing();

		m_point.m_anchor_u = groundOverlay.getOptions().getAnchorU();
		m_point.m_anchor_v = groundOverlay.getOptions().getAnchorV();
		
		LatLngBounds bounds = groundOverlay.getOptions().getBounds();
		if (bounds != null)
		{
			m_point.m_bounds = new KRect();
			m_point.m_bounds.iLeft = MapTool.doubleToLong(bounds.southwest.longitude);
			m_point.m_bounds.iRight = MapTool.doubleToLong(bounds.northeast.longitude);
			m_point.m_bounds.iBottom = MapTool.doubleToLong(bounds.southwest.latitude);
			m_point.m_bounds.iTop = MapTool.doubleToLong(bounds.northeast.latitude);
		}else
		{
			m_point.m_bounds = null;
		}
		
		m_point.m_transparent = groundOverlay.getOptions().getTransparency();
		
		return m_point;
	}

	@SuppressLint("NewApi")
	private static MapOverLayShapeArc createMapArc(KArc arc) {
		if (arc == null) {
			return null;
		}
		MapOverLayShapeArc  m_arc = new MapOverLayShapeArc();
		
		
		//MapOverLayShapeGround m_point = new MapOverLayShapeGround(buffer.capacity());
		// [internal] 父类属性设置
		m_arc.m_i_shapid = arc.getOverlayId();
		m_arc.m_i_level = arc.getLevel();
		m_arc.m_i_zindex = arc.getZIndex();
		m_arc.m_b_visible = arc.isVisible();
		m_arc.m_v_points  = latlonToKCoord(arc.getPoints());
		m_arc.m_f_width = arc.getStrokeWidth();
		m_arc.m_strokeColor = parseColor(arc.getStrokeColor());
		return m_arc;
	}
	
	// [internal] int型arbg颜色转换为KRECOLOR
	public static KRECOLOR parseColor(int i_color) {

		KRECOLOR k_color = new KRECOLOR();
		int[] pBuf = new int[4];
		pBuf[0] = (i_color >> 24) & 0xff;
		pBuf[1] = (i_color >> 16) & 0xff;
		pBuf[2] = (i_color >> 8) & 0xff;
		pBuf[3] = (i_color >> 0) & 0xff;
		k_color.a = pBuf[0];
		k_color.r = pBuf[1];
		k_color.g = pBuf[2];
		k_color.b = pBuf[3];

		return k_color;
	}

	// [internal] LatLon转换为KGEOCOORD
	private static Vector<KGEOCOORD> latlonToKCoord(List<KLatLng> latlons) {
		synchronized (latlons) {
			Vector<KGEOCOORD> v_coord = new Vector<KGEOCOORD>();
			Iterator<KLatLng> itor_latlons = latlons.iterator();
			KLatLng latlon = null;
			KGEOCOORD k_coord = null;
			KGEOCOORD pre_crd = null;
			while (itor_latlons.hasNext()) {
				latlon = itor_latlons.next();
				k_coord = new KGEOCOORD(MapTool.doubleToLong(latlon.longitude), MapTool.doubleToLong(latlon.latitude));
				if (pre_crd == null
						|| !k_coord.equals(pre_crd)) {
					v_coord.add(k_coord);
				}
				pre_crd = new KGEOCOORD(k_coord.lLongitude, k_coord.lLongitude);
			}
			return v_coord;
		}
	}

	public static int[] hexStringToBytes(String hexString) {
		if (hexString == null || hexString.equals("")) {
			return null;
		}
		hexString = hexString.toUpperCase();
		int length = (int) Math.ceil((hexString.length() / 2.0));
		Lg.e("zhuliang", length + "....");
		char[] hexChars = hexString.toCharArray();
		int[] d = new int[length];
		for (int i = 0; i < length; i++) {
			int pos = i * 2;
			d[i] = (int) (charToByte(hexChars[pos]) << 4 | charToByte(hexChars[pos + 1]));
		}

		return d;
	}

	private static byte charToByte(char c) {
		return (byte) "0123456789ABCDEF".indexOf(c);
	}
	
	public static void createShapeBitmap(KShapeOverlay overlay, BlockingQueue<MapOverlayBitmap> qBitmapQueue) {
		if (overlay == null) {
			return;
		}
		
		if (overlay instanceof KPolyline) {
			createMapPolyLineBitmap((KPolyline) overlay, qBitmapQueue);
		} else if (overlay instanceof KGroundOverlay) {
			createMapGroundBitmap((KGroundOverlay) overlay, qBitmapQueue);
		} else if(overlay instanceof KMarker) {
			createMarkerBitmap((KMarker) overlay, qBitmapQueue);
		}
	}

	private static void createMarkerBitmap(KMarker overlay, BlockingQueue<MapOverlayBitmap> qBitmapQueue) {
		for (int i = 0; i < overlay.getIcons().size(); i++) {
			BitmapDescriptor bitmapDescriptor = overlay.getIcons().get(i);
			qBitmapQueue.offer(new MapOverlayBitmap(NE_MapOverLayShape_Type.MapOverLay_Shape_Marker, bitmapDescriptor.getBitmap(), bitmapDescriptor.describeContents()));
		}
	}

	private static void createMapGroundBitmap(
			KGroundOverlay overlay, BlockingQueue<MapOverlayBitmap> qBitmapQueue) {
		BitmapDescriptor bitmapDescriptor = overlay.getImage();
		qBitmapQueue.offer(new MapOverlayBitmap(NE_MapOverLayShape_Type.MapOverLay_Shape_Ground, bitmapDescriptor.getBitmap(), bitmapDescriptor.describeContents()));
	}

	private static void createMapPolyLineBitmap(
			KPolyline overlay, BlockingQueue<MapOverlayBitmap> qBitmapQueue) {
		BitmapDescriptor bitmapDescriptor = overlay.getPolyLineOptions().getCustomTexture();
		if (bitmapDescriptor != null) {
			qBitmapQueue.offer(new MapOverlayBitmap(NE_MapOverLayShape_Type.MapOverLay_Shape_PolyLine, bitmapDescriptor.getBitmap(), bitmapDescriptor.describeContents()));
		}
		
		List<BitmapDescriptor> lstBitmapDescriptor = overlay.getPolyLineOptions().getCustomTextureList();
		for(int i = 0; i < lstBitmapDescriptor.size(); ++i)
		{
			BitmapDescriptor bitmapDescriptorTemp = lstBitmapDescriptor.get(i);
			if (bitmapDescriptorTemp != null) {
				qBitmapQueue.offer(new MapOverlayBitmap(NE_MapOverLayShape_Type.MapOverLay_Shape_PolyLine, bitmapDescriptorTemp.getBitmap(), bitmapDescriptorTemp.describeContents()));
			}
		}
	}
}
