package com.cmmap.internal.maps.model;

/**
 * 栅格图层类。
 * <p>栅格图层是显示在基本地图上的一组图像。栅格图层具有以下属性：</p>
 * <ul> 
 * <li>栅格提供者<br>提供者提供在栅格图层上使用的图像。
 * 您必须在栅格添加到地图上之前指定提供者。
 * 一旦栅格添加到地图之后，提供者将不能更改。
 * 如果栅格提供者发生变化，您必须使用{@link #clearTileCache}方法确保以前的栅格图不再出现。</li>
 * <li>Z轴索引<br>Z轴索引是用来控制地图上的各个覆盖物或图层的绘制顺序的。Z轴索引值较大的覆盖物或图层会绘制在Z轴索引值较小的覆盖物或图层上面。如果两个覆盖物或图层的Z轴索引数值相同，则先后绘制顺序是随机的。</li>
 * <li>可见性<br>表示该图层是否可见，如果不可见，则不会被绘制。</li>
 * </ul>
 */
public class KTileOverlay  extends KShapeOverlay{
	public KTileOverlayOptions m_Options = null;
	
	public KTileOverlay() {
		setOverlayId(assignedId());
	}
	
	public void setTileOverlayOptions(KTileOverlayOptions options) {
		m_Options = new KTileOverlayOptions(options);
		setLevel(options.getLevel());
		setZIndex(options.getZIndex());
		setVisible(options.isVisible());
	}

	/**
	 * 清空栅格图层的缓存。
	 */
	public void clearTileCache() {
		clearCache();
	}

	/**
	 * 返回图层ID。
	 * <br>ID是该图层在所属的地图对象里的唯一标识。
	 * @return ID 图层ID
	 */
	public String getId() {
		return String.valueOf(getOverlayId());
	}

	@Override
	public void remove() {
		if (m_Options != null) {
			m_Options.releaseResource();
			m_Options = null;
		}
		super.remove();
	}
}
