package com.cmmap.internal.maps.model;

import android.graphics.Color;

import com.cmmap.api.maps.model.BitmapDescriptor;
import com.cmmap.api.maps.model.BitmapDescriptorFactory;

/**
 * 定位（当前位置）的绘制样式类
 *
 */
public class MyLocationStyle {
	
	private BitmapDescriptor m_icon;
	private float m_anchor_u;
	private float m_anchor_v;
	private int m_radius_fill_color;
	private int m_stroke_color;
	private float m_stroke_width;
	
	public MyLocationStyle() {
		m_anchor_u = 0.5f;
		m_anchor_v = 1.0f;
		m_radius_fill_color = Color.argb(75, 131, 182, 222);
		m_icon = BitmapDescriptorFactory.fromAsset("cmmap/mapres/location_up.png");
		m_stroke_color =  Color.argb(225, 131, 182, 222);
		m_stroke_width = 1;
	}
	
	//设置定位图标的锚点。
	public MyLocationStyle anchor(float u, float v) {
		m_anchor_u = u;
		m_anchor_v = v;
		return this;
	}	
	 
	public int describeContents() {
		return 0;
	}
		
	//得到锚点横坐标方向的偏移量。
	public float getAnchorU() {
		return m_anchor_u;
	}
	
	//得到锚点纵坐标方向的偏移量。 
	public float getAnchorV() {
		return m_anchor_v;
	}
	
	//得到当前位置的图标。 
	public BitmapDescriptor getMyLocationIcon() {
		return m_icon;
	}
	
	//得到圆形区域（以定位位置为圆心，定位半径的圆形区域）的填充颜色值。	
	public int getRadiusFillColor() {
		return m_radius_fill_color;
	}

	//得到圆形区域（以定位位置为圆心，定位半径的圆形区域）边框的颜色值。
	public int getStrokeColor() {
		return m_stroke_color;
	}
	
	//得到圆形区域（以定位位置为圆心，定位半径的圆形区域）边框的宽度。	
	public float getStrokeWidth() {
		return m_stroke_width;
	}
	
	//设置定位（当前位置）的icon图标。
	public MyLocationStyle myLocationIcon(BitmapDescriptor myLocationIcon)  {
		m_icon = myLocationIcon;
		return this;
	}
	
	//设置圆形区域（以定位位置为圆心，定位半径的圆形区域）的填充颜色。	
	public MyLocationStyle radiusFillColor(int color) {
		m_radius_fill_color = color;
		return this;
	}

	//设置圆形区域（以定位位置为圆心，定位半径的圆形区域）的边框颜色。
	public MyLocationStyle strokeColor(int color) {
		m_stroke_color = color;
		return this;
	}
	
	//设置圆形区域（以定位位置为圆心，定位半径的圆形区域）的边框宽度。
	public MyLocationStyle strokeWidth(float width) {
		m_stroke_width = width;
		return this;
	}
	
}
