package com.cmmap.internal.services.rtic;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import javax.net.ssl.HttpsURLConnection;

import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.util.Log;

import com.cmmap.internal.driver.base.HttpTool;
import com.cmmap.internal.driver.base.HttpsPost;
import com.cmmap.internal.mapcore.ByteUlit;
import com.cmmap.internal.mapcore.KInt;
import com.cmmap.internal.mapcore.MapCore;
import com.cmmap.internal.mapcore.MapJni;

public class RTICThread extends Thread {
	
    static final int RTIC_BASE_BUF_SIZE = 16;
    
	//	构造函数
	private RTICThread() {
	}
	
	//	获取实例
	public static RTICThread GetInstance() {
		if(null == m_this) {
			m_this = new RTICThread();
			m_this.Initialize();
		}
		return m_this;
	}
	//	初始化
	void Initialize() {
		m_strRTICServer = MapJni.getRTICServer();
		m_strEventServer = MapJni.getEventServer();
		m_bRTICUse = MapCore.GetRTICUse();
		m_bEventUse = MapCore.GetEventUse();
	}
	
	Timer mRTICTmr = null;
	//	start timer
	public void startTimer() {
		if (mRTICTmr != null) {
			return;
		}
		mRTICTmr = new Timer();
		TimerTask taskUpdate = new TimerTask() {
			public void run() {
				Handler hdl = getHandler();
				Message msg = hdl.obtainMessage(RTICThread.RTIC_UPDATE_TIME);
				hdl.sendMessage(msg);
			}
		};
		TimerTask taskCoordRefresh = new TimerTask() {
			public void run() {
				Handler hdl = getHandler();
				Message msg = hdl.obtainMessage(RTICThread.RTIC_UPDATE_COORD);
				hdl.sendMessage(msg);
			}
		};
		mRTICTmr.scheduleAtFixedRate(taskCoordRefresh, 0, 1000);
		mRTICTmr.scheduleAtFixedRate(taskUpdate, 0 , 1000 * 60);
	}
	// end timer
	public void endTimer() {
		if (mRTICTmr != null) {
			mRTICTmr.cancel();
			mRTICTmr = null;
		}
	}
	
	private static RTICThread m_this;
	
	public static final int RTIC_UPDATE_COORD = 1;
	public static final int RTIC_UPDATE_TIME =2;
	HashMap<String, String> m_mapAllCity = new HashMap<String, String>();
	HashMap<String, String> m_map_errorCityCode = new HashMap<String, String>();
	List<String> m_lstCurCity = new ArrayList<String>();
	boolean m_bRTICUse = false;
	boolean m_bEventUse = false;
	static String m_strRTICServer = "";
	static String m_strEventServer = "";
	
	public void SetRTICUse(boolean bUse) {
		m_bRTICUse = bUse;
	}
	
	public void setEventUse(boolean enabled) {
        m_bEventUse = enabled;
    }
	 
	public boolean GetRTICUse() {
		return m_bRTICUse;
	}
	
	public boolean GetRTICEventUse() {
	    return m_bEventUse;
	}
	 
	//	URL包含IP,端口以及Key.如"http://61.189.20.114:9988/tmc_server?key=5f6a25331224009bb93a195"
	public static void SetServerURL(String strURL) {
		if(strURL != null) {
			m_strRTICServer = strURL;
			MapJni.setRTICServer(strURL);
		}
	}
	
	public static void SetEventURL(String strURL) {
		if(strURL != null) {
			m_strEventServer = strURL;
			MapJni.setEventServer(strURL);
		}
	}
	
	public byte[] GetCityRTICData(String strCity)
	{
	    if (!m_bRTICUse)
	    {
	        return null;
	    }
	    
		if (m_strRTICServer.startsWith("https")) {
			return GetHttpsCityRTICData(strCity);
		}
		byte[] data = null;
		InputStream in = null;
		try {
			String strDate = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
			String strURL = m_strRTICServer + "&sn=" + strDate + "&type=3&city_code=" + strCity;
			URL url = new URL(strURL);
			HttpURLConnection urlConn = (HttpURLConnection) url.openConnection();
			urlConn.setConnectTimeout(1000 * 3);
			urlConn.setReadTimeout(1000 * 30);
			//urlConn.setDoOutput( true );
			urlConn.setRequestMethod("GET"); 

			//urlConn.addRequestProperty("User-Agent","Mozilla/5.0 (Windows NT 6.1; rv:40.0) Gecko/20100101 Firefox/40.0");
			//urlConn.setRequestProperty("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8");
			//urlConn.setRequestProperty("Accept-Encoding", "gzip,deflate");
			//urlConn.setRequestProperty("Accept-Language", "zh-CN,zh;q=0.8,en-US;q=0.5,en;q=0.3");

			//urlConn.addRequestProperty("DNT","1");
			//urlConn.addRequestProperty("Connection","keep-alive");
			urlConn.addRequestProperty("request_sn",strDate);
			String strCurResultTag = m_mapAllCity.get(strCity);
			if(null != strCurResultTag) {
				urlConn.addRequestProperty("result_tag",strCurResultTag);
			}
			
			int iReponseCode = urlConn.getResponseCode();
			Log.w("jfh", "UpdateRTICData ResponseCode="+iReponseCode + "|URL=" + strURL);
			in = urlConn.getInputStream();
			if(iReponseCode == HttpURLConnection.HTTP_OK)
			{
				data = HttpTool.readFromInputStream(urlConn.getInputStream(), urlConn.getContentEncoding());
				String strResultTag = urlConn.getHeaderField("result_tag");
				m_mapAllCity.put(strCity, strResultTag);
			}
			else {
				byte[] bufError = HttpTool.readFromInputStream(urlConn.getErrorStream(), urlConn.getContentEncoding());
				String strError = new String(bufError, "UTF-8");
				Log.e("Network", "RTIC Http Code="+iReponseCode + "!Erro="+strError);
			}		
			in.close();
		}
		catch (Exception e) {
			Log.w("jfh", "UpdateRTICData Exception=" + e.toString());
		}
		return data;
	}
	
	public byte[] GetCityEventData(String strCity)
	{
	    if (!m_bEventUse)
	    {
	        return null;
	    }
	    
		if (m_strEventServer.startsWith("https")) {
			return GetHttpsCityEventData(strCity);
		}
		byte[] data = null;
		InputStream in = null;
		try {
			String strDate = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
			String strURL = m_strEventServer + "city_code=" + strCity;
			URL url = new URL(strURL);
			HttpURLConnection urlConn = (HttpURLConnection) url.openConnection();
			urlConn.setConnectTimeout(1000 * 3);
			urlConn.setReadTimeout(1000 * 30);
			//urlConn.setDoOutput( true );
			urlConn.setRequestMethod("GET"); 

			//urlConn.addRequestProperty("User-Agent","Mozilla/5.0 (Windows NT 6.1; rv:40.0) Gecko/20100101 Firefox/40.0");
			//urlConn.setRequestProperty("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8");
			//urlConn.setRequestProperty("Accept-Encoding", "gzip,deflate");
			//urlConn.setRequestProperty("Accept-Language", "zh-CN,zh;q=0.8,en-US;q=0.5,en;q=0.3");

			//urlConn.addRequestProperty("DNT","1");
			//urlConn.addRequestProperty("Connection","keep-alive");
			urlConn.addRequestProperty("request_sn",strDate);
			String strCurResultTag = m_mapAllCity.get(strCity);
			if(null != strCurResultTag) {
				urlConn.addRequestProperty("result_tag",strCurResultTag);
			}
			
			int iReponseCode = urlConn.getResponseCode();
			Log.w("jfh", "UpdateRTICData ResponseCode="+iReponseCode + "|URL=" + strURL);
			in = urlConn.getInputStream();
			if(iReponseCode == HttpURLConnection.HTTP_OK)
			{
				data = HttpTool.readFromInputStream(urlConn.getInputStream(), urlConn.getContentEncoding());
				String strResultTag = urlConn.getHeaderField("result_tag");
				m_mapAllCity.put(strCity, strResultTag);
			}
			else {
				byte[] bufError = HttpTool.readFromInputStream(urlConn.getErrorStream(), urlConn.getContentEncoding());
				String strError = new String(bufError, "UTF-8");
				Log.e("Network", "Event Http Code="+iReponseCode + "!Erro="+strError);
			}		
			in.close();
		}
		catch (Exception e) {
			Log.w("jfh", "UpdateRTICData Exception=" + e.toString());
		}
		return data;
	}

	public byte[] GetHttpsCityRTICData(String strCity)
	{
		byte[] data = null;
		InputStream in = null;
		try {
			String strDate = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
			String strURL = m_strRTICServer + "&sn=" + strDate + "&type=3&city_code=" + strCity;
			URL url = new URL(strURL);
			HttpsPost.initHttpsURLConnection();
			HttpsURLConnection urlConn = (HttpsURLConnection) url.openConnection();
			urlConn.setConnectTimeout(1000 * 3);
			urlConn.setReadTimeout(1000 * 30);
//			urlConn.setDoOutput( true );
			urlConn.setRequestMethod("GET"); 

			urlConn.addRequestProperty("request_sn",strDate);
			String strCurResultTag = m_mapAllCity.get(strCity);
			if(null != strCurResultTag) {
				urlConn.addRequestProperty("result_tag",strCurResultTag);
			}
			
			int iReponseCode = urlConn.getResponseCode();
			Log.w("jfh", "UpdateRTICData ResponseCode="+iReponseCode + "|URL=" + strURL);
			in = urlConn.getInputStream();
			if(iReponseCode == HttpURLConnection.HTTP_OK)
			{
				data = HttpTool.readFromInputStream(urlConn.getInputStream(), urlConn.getContentEncoding());
				String strResultTag = urlConn.getHeaderField("result_tag");
				m_mapAllCity.put(strCity, strResultTag);
			}
			else {
				byte[] bufError = HttpTool.readFromInputStream(urlConn.getErrorStream(), urlConn.getContentEncoding());
				String strError = new String(bufError, "UTF-8");
				Log.e("Network", "RTIC Https Code="+iReponseCode + "!Erro="+strError);
			}		
			in.close();
		}
		catch (Exception e) {
			Log.w("jfh", "UpdateRTICData Exception=" + e.toString());
		}
		return data;
	}
	
	public byte[] GetHttpsCityEventData(String strCity)
	{
		byte[] data = null;
		InputStream in = null;
		try {
			String strDate = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());
			String strURL = m_strEventServer + "city_code=" + strCity;
			URL url = new URL(strURL);
			HttpsPost.initHttpsURLConnection();
			HttpsURLConnection urlConn = (HttpsURLConnection) url.openConnection();
			urlConn.setConnectTimeout(1000 * 3);
			urlConn.setReadTimeout(1000 * 30);
//			urlConn.setDoOutput( true );
			urlConn.setRequestMethod("GET"); 

			urlConn.addRequestProperty("request_sn",strDate);
			String strCurResultTag = m_mapAllCity.get(strCity);
			if(null != strCurResultTag) {
				urlConn.addRequestProperty("result_tag",strCurResultTag);
			}
			
			int iReponseCode = urlConn.getResponseCode();
			Log.w("jfh", "UpdateRTICData ResponseCode="+iReponseCode + "|URL=" + strURL);
			in = urlConn.getInputStream();
			if(iReponseCode == HttpURLConnection.HTTP_OK)
			{
				data = HttpTool.readFromInputStream(urlConn.getInputStream(), urlConn.getContentEncoding());
				String strResultTag = urlConn.getHeaderField("result_tag");
				m_mapAllCity.put(strCity, strResultTag);
			}
			else {
				byte[] bufError = HttpTool.readFromInputStream(urlConn.getErrorStream(), urlConn.getContentEncoding());
				String strError = new String(bufError, "UTF-8");
				Log.e("Network", "RTIC Https Code="+iReponseCode + "!Erro="+strError);
			}		
			in.close();
		}
		catch (Exception e) {
			Log.w("jfh", "UpdateRTICData Exception=" + e.toString());
		}
		return data;
	}
	
	void RefreshCityDataByTime() {
		UpdateRTICData(m_lstCurCity);
	}
	
	void RefreshCityDataByCity(Collection<String> lstCity) {
		//	过滤已经下载的数据
		List<String> lstLeft = new ArrayList<String>();
		for (String strCity : lstCity) {
			int city = Integer.parseInt(strCity);
			if (!MapJni.isRTICCityExist(city)) {
				m_lstCurCity.remove(strCity);
				m_mapAllCity.remove(strCity);
			}
			
			if (m_lstCurCity.contains(strCity)) {
				continue;
			}
			lstLeft.add(strCity);
		}
		UpdateRTICData(lstLeft);

		//	再次过滤:因为UpdateRTICData中新增了error code
		m_lstCurCity.clear();
		m_lstCurCity.addAll(lstCity);
		for (String strCity : lstCity) {
			if(m_map_errorCityCode.containsKey(strCity)) {
				continue;
			}
			m_lstCurCity.add(strCity);
		}
	}
	
	void UpdateRTICData(Collection<String> listCity) {
		for (String strCity : listCity) {
			byte[] data = GetCityRTICData(strCity);
			byte[] data2 = GetCityEventData(strCity);
			int iTotalSize = 0;
			int iRTICSize = 0;
			int iEventSize = 0;
			byte[] RTICbuf = null;
			if(null != data) {
				//	数据校验(目前异常信息不是二进制，所以用该值校验.正常的字符串长度小于255)
				int iCheck = ByteHelper.byteArray2Int(data, 0);
				if(iCheck > 255) {
					continue;
				}
				RTICCity city = new RTICCity();		
				int iRet = city.LoadFromByte(data, strCity);
				if(iRet != 0) {
					if(1 == iRet) {
						if (!m_map_errorCityCode.containsKey(strCity)) {
							m_map_errorCityCode.put(strCity, null);
						}
					}
					continue;
				}
				
				//	数据是否更新
				if(city.GetDateTime().length() == 0) {
					continue;
				}
				iRTICSize = city.Size();
				RTICbuf = new byte[iRTICSize];
				city.WriteToBuf(RTICbuf, 0);
			} else {
			    iRTICSize = RTIC_BASE_BUF_SIZE;
			    RTICbuf = new byte[iRTICSize];
			    // city
			    ByteHelper.intToByteArray(Integer.parseInt(strCity), RTICbuf, 0);
			    // date
	            Calendar calendar = Calendar.getInstance();
	            int idate = calendar.get(Calendar.YEAR) * 10000 + (calendar.get(Calendar.MONTH) + 1) * 100 + calendar.get(Calendar.DAY_OF_MONTH);
	            ByteHelper.intToByteArray(idate, RTICbuf, 4);
	            // time
	            int itime = calendar.get(Calendar.HOUR_OF_DAY) *10000 + calendar.get(Calendar.MINUTE) + calendar.get(Calendar.SECOND);
	            ByteHelper.intToByteArray(itime, RTICbuf, 8);
	            // num
	            ByteHelper.intToByteArray(0, RTICbuf, 12);
			}
				
			if (data2 != null){
				iEventSize = data2.length;
			}
			// 将事件与路况合并
			if(iEventSize > 0 || iRTICSize > RTIC_BASE_BUF_SIZE)
			{
				int HEAD_SIZE = 16;
				iTotalSize = iEventSize + iRTICSize + HEAD_SIZE;
				byte[] pDataBuff = new byte[iTotalSize];
				int iAddr = HEAD_SIZE;
				int iSize = iRTICSize;
				int ofs = 0;
				ByteHelper.intToByteArray(iAddr, pDataBuff, ofs);
				ofs += 4;
				ByteHelper.intToByteArray(iSize, pDataBuff, ofs);
				ofs += 4;
				iAddr = HEAD_SIZE + iRTICSize;
				iSize = iEventSize;
				ByteHelper.intToByteArray(iAddr, pDataBuff, ofs);
				ofs += 4;
				ByteHelper.intToByteArray(iSize, pDataBuff, ofs);
				ofs += 4;

				if(iRTICSize > 0)
				{
					System.arraycopy(RTICbuf, 0, pDataBuff, ofs, iRTICSize);
				}

				if(iEventSize > 0)
				{
					System.arraycopy(data2, 0, pDataBuff, ofs + iRTICSize, iEventSize);
				}
				MapCore.UpdateRTICData(pDataBuff);
			}
		}
	}

	Handler handler = null;
	
    @Override
    public void run() {
    	Looper.prepare();
        // handler类接收数据  
    	handler = new ThreadHandler(Looper.myLooper()); 
    	Looper.loop();
    }
    
    public Handler getHandler() {
    	while(null == handler){
    		try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
    	}
    	return handler;
    }
    
    class ThreadHandler extends Handler{
    	public ThreadHandler(Looper looper) {
    		super(looper);
    	}
		boolean m_bProcessing = false;
    	
    	public void handleMessage(Message msg) {
    		//这里对该线程中的MessageQueue中的Message进行处理  
    		if(!(m_bRTICUse || m_bEventUse) || MapCore.getNetworkStatus() == 0 || m_bProcessing) {
    			//Log.i("RTIC", "handleMessage RTICUse="+m_bRTICUse+ "!NetStatus="+MapCore.getNetworkStatus()+"|Processing="+m_bProcessing);
    			return;
    		}
    		m_bProcessing = true;
            switch (msg.what) {
            case RTIC_UPDATE_TIME: {
            	RefreshCityDataByTime();
            	}
            break;
            case RTIC_UPDATE_COORD: {
            	short[] codes = new short[20];
            	MapCore.GetAddressCode(codes);
            	List<String> lstCity = new ArrayList<String>();
            	//	过滤无效的Code
            	for(int i = 0; i < codes.length; ++i) {
            		int iCode = codes[i];
            		if(iCode == 0) {
            			continue;
            		}
                	String str = String.format("%04d", iCode);
                	//	过滤无效的City
                	if(m_map_errorCityCode.containsKey(str)) {
                		continue;
                	}
                	lstCity.add(str);
            	}
            	
            	if(!lstCity.isEmpty()) {
            		RefreshCityDataByCity(lstCity);
            	}
            	}
            break;
            }
            m_bProcessing = false;
    	}
    }
    
}
